procedure camila8(sequence, root, ims, filter, flatim, refimag)

#xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
# This script is a new version (camila8) of camila reduction program for 
# version 2.11 (new) of IRAF and new version of CAMILA sequencing.
# Sequences recognized are: bianca, cruzgal, cruz, mycruz, sky, cuadrog,
#       cruzgal2, cuadro, square, recta, migcruz, galchica 
#
# Main change from the previous version is that the "xyshift" parameter
# now is split into "xshi" and "yshi" which are "real". Also the
# parameter "pos[*]" defining the sequence is also split into "xpos[*]"
# and "ypos[*]" and is "real". The new ypos & ypos parameters are the sum of
# xshi & yshi parameters and the intrinsic shifts defining the sequence.
# Another change is the introduction of imalign "boxsize=7" and "bigbox=11"
# parameters. If the program aborts because of imalign centering errors, one 
# or more of the parameters "xshi, yshi, boxsize and bigbox" should be changed. 
# 
# If photometry is required: digiphot/apphot should have been loaded
# Make sure to "unlearn" & edit the phot pset parameters to appropriate values.
# Last modified 7 Aug 1999 (renamed from camila6 to camila7 on 1/8/99)
# Last modified 16 Dec 1999 at SPM (renamed from camila7 to camila8) 
#
#xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx

string     sequence = "cuadrog"           {prompt="Sequence Name"}
string     root     = "fs28"              {prompt="Root Image Name"}
int        ims      = "1"                 {prompt="First Image Number"}
string     filter   = "j1"                {prompt="Filter code"}
string     flatim   = "flatjn.sep"        {prompt="Flat field frame Name"}
string     refimag  = "fs28002ref"        {prompt="Reference Image name"}
string     skyframe = "stack"         {prompt="Sky frame stack|null|name|0345"}
string     imframe  = "sequence"         {prompt="Image frames sequence|all"}
bool       rfits    = "yes"               {prompt="Input is a Fits file?"}
bool       skyedit  = "yes"               {prompt="Clean the sky using imedit?"}
bool       doflat   = "yes"               {prompt="Do flat-fielding?"}
bool       submed   = "no"                {prompt="Remove the horizontal band?"}
bool       immos    = "no"                {prompt="Mosaic the images?"}
bool       disp     = "no"                {prompt="Display before imcombine? "}
bool       dophot   = "no"                {prompt="Do photometry?"}
real       xshift  = "0"              {prompt="X-shift[=Xref-Xin] wrt refimag"}
real       yshift  = "0"              {prompt="Y-shift[=Yref-Yin] wrt refimag"}
real       boxsize = "7"              {prompt="imalign boxsize"}
real       bigbox  = "11"             {prompt="imalign bigbox"}

begin
        string  froot, seq, logfil, skyfile, fil, flat, skyim,cns,imfr
        string  cm[10], cmn, photfil, pos[10], xyshi, irred1 
	string  coofil, xyfil, refim, mosfil, shiftfil, ans
	string  tmpfits, tmpimh, tmpobjs, tmpobjsf, tmpal, tmpsky 
	string  tmpobj, tmpobjsfa, tmpobjsfb, tmpalb 
	int     i,ins,iseq,n,m,im1,imn,npass,j,nobj,nsky,ns[10],nob[10] 
	bool    skysta
	real    xshi, yshi, xpos[10], ypos[10]

        seq = sequence
        froot  = root  
	im1 = ims
	fil = filter
	flat = flatim
	xshi  = xshift
	yshi  = yshift
	skyim = skyframe
	imfr  = imframe
	xyshi = "0  0"

	cache datapars
	cache centerpars
	cache fitskypars
	cache photpars
	
	iseq = 0
	if (seq=="bianca") iseq = 1 
	if (seq=="cruzgal") iseq = 2 
	if (seq=="cruz") iseq = 2 
	if (seq=="mycruz") iseq = 2 
	if (seq=="migcruz") iseq = 2 
	if (seq=="sky") iseq = 3
	if (seq=="cuadrog") iseq = 4 
	if (seq=="cruzgal2") iseq = 5 
	if (seq=="cuadro") iseq = 6 
	if (seq=="square") iseq = 7 
	if (seq=="recta") iseq = 8 
	if (seq=="mosgal") iseq = 2 
	if (seq=="medgal") iseq = 2 
	if (seq=="newrecta") iseq = 9 
	if (seq=="smagal") iseq = 10
	if (seq=="galchica") iseq = 11
	if (seq=="bianca2") iseq = 12 
        #print("seq no ",  seq, " ",iseq)

	switch (iseq) {
	  case 1 :  
		{
	    	npass = 6
	  	nobj  = 6
		nsky  = 4
	  	for (i=1; i <= nobj; i +=1) 
	    		nob[i] = i

			ns[1] = 3
			ns[2] = 4
			ns[3] = 5
			ns[4] = 6
                xpos[1] =   0 + xshi; ypos[1] =   0 + yshi
                xpos[2] =   0 + xshi; ypos[2] =   0 + yshi
                xpos[3] = 118 + xshi; ypos[3] =   9 + yshi
                xpos[4] = 107 + xshi; ypos[4] = 128 + yshi
                xpos[5] = -11 + xshi; ypos[5] = 118 + yshi
                xpos[6] =   0 + xshi; ypos[6] =   0 + yshi
		}
	  case 2 :
		{
	    	npass = 10
	  	nobj  = 6
		nsky  = 4
			j = 1
	    		nob[j] = 1
	  	for (i=2; i <= npass; i +=2) {
			j = j + 1
	    		nob[j] = i
	    		if (i < npass) ns[j-1] = i+1
			}
	    for (i=1; i <= nobj; i +=1) {
	    	xpos[i] = xshi
	    	ypos[i] = yshi
	    	#pos[i] = xyshi
		#print(" positions....", pos[i], i) 
		}
		}
	  case 3 :
		{
	    	npass = 6
	  	nobj  = 4
		nsky  = 3
			j = 1
	    		nob[j] = 1
	  	for (i=2; i <= npass; i +=2) {
			j = j + 1
	    		nob[j] = i
	    		if (i < npass) ns[j-1] = i+1
		}
			ns[3] = 4
	    for (i=1; i <= nobj; i +=1) 
	    	xpos[i] = xshi
	    	ypos[i] = yshi
	    	#pos[i] = xyshi
		}
	  case 4 : 
		{
	    	npass = 5
	  	nobj  = 5
		nsky  = 3
	  	for (i=1; i <= nobj; i +=1) 
	    		nob[i] = i

			ns[1] = 2
			ns[2] = 3
			ns[3] = 4
                xpos[1] =   0 + xshi; ypos[1] =   0 + yshi
                xpos[2] =   8 + xshi; ypos[2] =-127 + yshi
                xpos[3] =-120 + xshi; ypos[3] =-134 + yshi
                xpos[4] =-126 + xshi; ypos[4] =  -6 + yshi
                xpos[5] =   0 + xshi; ypos[5] =   0 + yshi
		}
	  case 5 :
		{
	    	npass = 10
	  	nobj  = 6
		nsky  = 4
			j = 1
	    		nob[j] = 1
	  	for (i=2; i <= npass; i +=2) {
			j = j + 1
	    		nob[j] = i
	    		if (i < npass) ns[j-1] = i+1
			}
                xpos[1] = -13 + xshi; ypos[1] = -26 + yshi
                xpos[2] = -13 + xshi; ypos[2] = -26 + yshi
                xpos[3] = -13 + xshi; ypos[3] = -13 + yshi
                xpos[4] = -13 + xshi; ypos[4] =   0 + yshi
                xpos[5] =   0 + xshi; ypos[5] =   0 + yshi
                xpos[6] =  13 + xshi; ypos[6] =   0 + yshi
		}
	  case 6 : 
		{
	    	npass = 6
	  	nobj  = 6
		nsky  = 4
	  	for (i=1; i <= nobj; i +=1) 
	    		nob[i] = i

			ns[1] = 2
			ns[2] = 3
			ns[3] = 4
			ns[4] = 5
                xpos[1] =   0 + xshi; ypos[1] =   0 + yshi
                xpos[2] =   0 + xshi; ypos[2] =   0 + yshi
                xpos[3] =   0 + xshi; ypos[3] = -70 + yshi
                xpos[4] = -70 + xshi; ypos[4] = -70 + yshi
                xpos[5] = -70 + xshi; ypos[5] =   0 + yshi
                xpos[6] =   0 + xshi; ypos[6] =   0 + yshi
		}
          case 7 :
                {
                npass = 6
                nobj  = 6
                nsky  = 4
                for (i=1; i <= nobj; i +=1)
                        nob[i] = i

                        ns[1] = 3
                        ns[2] = 4
                        ns[3] = 5
                        ns[4] = 6
	  	xpos[1] = 1 + xshi  ; ypos[1] = 1 + yshi
	  	xpos[2] = 0 + xshi  ; ypos[2] = 0 + yshi
	  	xpos[3] = 0 + xshi  ; ypos[3] = 112 + yshi
	  	xpos[4] = 112 + xshi; ypos[4] = 112 + yshi
	  	xpos[5] = 112 + xshi; ypos[5] =   0 + yshi
	  	xpos[6] =   0 + xshi; ypos[6] =   0 + yshi
                }
          case 8 :
                {
                npass = 8
                nobj  = 6
                nsky  = 4
                        nob[1] = 1
                        nob[2] = 2
                        nob[3] = 3
                        nob[4] = 5
                        nob[5] = 6
                        nob[6] = 8

                        ns[1] = 3
                        ns[2] = 4
                        ns[3] = 5
                        ns[4] = 7
                xpos[1] = 1 + xshi  ; ypos[1] = 1 + yshi
                xpos[2] = 0 + xshi  ; ypos[2] = 0 + yshi
                xpos[3] = 0 + xshi  ; ypos[3] = 100 + yshi
                xpos[4] = 82 + xshi ; ypos[4] = 100 + yshi
                xpos[5] = 82 + xshi ; ypos[5] =   0 + yshi
		#Following 3 shifts are for nov98 n7171 j[001-016] only
                #xpos[3] = 0 + xshi  ; ypos[3] = 110 + yshi
                #xpos[4] =110 + xshi ; ypos[4] = 110 + yshi
                #xpos[5] =110 + xshi ; ypos[5] =   0 + yshi
                xpos[6] =   0 + xshi; ypos[6] =   0 + yshi
                }
          case 9 :
                {
                npass = 8
                nobj  = 6
                nsky  = 4
                        nob[1] = 1
                        nob[2] = 2
                        nob[3] = 4
                        nob[4] = 5
                        nob[5] = 7
                        nob[6] = 8

                        ns[1] = 3
                        ns[2] = 4
                        ns[3] = 6
                        ns[4] = 7
                xpos[1] =  1 + xshi  ; ypos[1] =   1 + yshi
                xpos[2] =  0 + xshi  ; ypos[2] =   0 + yshi
                xpos[3] = 82 + xshi  ; ypos[3] =   0 + yshi
                xpos[4] = 82 + xshi  ; ypos[4] =-100 + yshi
                xpos[5] =  0 + xshi  ; ypos[5] =-100 + yshi
                xpos[6] =  0 + xshi  ; ypos[6] =   0 + yshi
                }
          case 10 :
                {
                npass = 10
                nobj  = 6
                nsky  = 4
			j = 1
	    		nob[j] = 1
	  	for (i=2; i <= npass; i +=2) {
			j = j + 1
	    		nob[j] = i
	    		if (i < npass) ns[j-1] = i+1
			}
                xpos[1] =  1 + xshi  ; ypos[1] =   1 + yshi
                xpos[2] =  0 + xshi  ; ypos[2] =   0 + yshi
                xpos[3] = 70 + xshi  ; ypos[3] =   0 + yshi
                xpos[4] = 70 + xshi  ; ypos[4] = -70  + yshi
                xpos[5] =  0 + xshi  ; ypos[5] = -70 + yshi
                xpos[6] =  0 + xshi  ; ypos[6] =   0 + yshi
                }
          case 11 :
                {
                npass = 10
                nobj  = 8
                nsky  = 5
                        nob[1] = 1
                        nob[2] = 2
                        nob[3] = 4
                        nob[4] = 5
                        nob[5] = 6
                        nob[6] = 8
                        nob[7] = 9
                        nob[8] = 10

                        ns[1] = 3
                        ns[2] = 5
                        ns[3] = 6
                        ns[4] = 7
                        ns[5] = 9

                xpos[1] =  1 + xshi  ; ypos[1] =   1 + yshi
                xpos[2] =  0 + xshi  ; ypos[2] =   0 + yshi
                xpos[3] = -70 + xshi ; ypos[3] =   0 + yshi
                xpos[4] = -70 + xshi ; ypos[4] = -70  + yshi
                xpos[5] =  0 + xshi  ; ypos[5] = -70 + yshi
                xpos[6] =  0 + xshi  ; ypos[6] = -70 + yshi
                xpos[7] = -70 + xshi ; ypos[7] =   0 + yshi
                xpos[8] =  0 + xshi  ; ypos[8] =   0 + yshi
                }
          case 12 :
                {
                npass = 6
                nobj  = 6
                nsky  = 4 
                for (i=1; i <= nobj; i +=1)
                        nob[i] = i

                        ns[1] = 3
                        ns[2] = 4
                        ns[3] = 5
                        ns[4] = 6
                xpos[1] =   0 + xshi; ypos[1] =   0 + yshi
                xpos[2] =   0 + xshi; ypos[2] =   0 + yshi
                xpos[3] = 118 + xshi; ypos[3] =   0 + yshi
                xpos[4] = 118 + xshi; ypos[4] = 118 + yshi
                xpos[5] =   0 + xshi; ypos[5] = 118 + yshi
                xpos[6] =   0 + xshi; ypos[6] =   0 + yshi
                }
	  default:
		goto exit
	}
	
	if (imfr=="all") {
	    	#npass = 10
	  	#nobj  = 10
		#nsky  = 4
	  	 nobj  = npass
	  	for (i=1; i <= nobj; i +=1) {
	    		nob[i] = i
	    	        #pos[i] = xyshi
			}
		}

	if (skyim == "stack")
	  skysta = yes
	else if (skyim == "null")
	  skysta = no
	else {
	    ins = strlen(skyim)
	    cns = substr(skyim,1,1)
	    if (cns == "0") {
	      skysta = yes
	      nsky = ins - 1
	      for (i=1; i <= nsky; i +=1) 
	    	ns[i] = int(substr(skyim,i+1,i+1))
	     } else {
	      skysta = no
	    } 
	}


        #print("npass nobj nsky ", npass, " ",nobj, " ", nsky)
	#  	for (i=1; i <= nsky; i +=1) 
        #print("Sky images ", ns[i])

          shiftfil = mktemp ("tmp$ia_tmp.")

	n = 0
loop:
	n = n + 1
	imn = im1 + n - 1
	cmn = froot//str(imn)
	if (imn < 100) cmn = froot//"0"//str(imn)
	if (imn < 10) cmn = froot//"00"//str(imn)
	  cm[n] = cmn
	if (n < npass )goto loop

	refim = refimag
	coofil = refim//".coo"
	xyfil  = refim//".xy"
	mosfil = froot//fil
	logfil = froot//"-camila.log"
	photfil = refim//".mag"

        tmpfits = mktemp ("tmp$ia_tmp.")
        tmpimh = mktemp ("tmp$ia_tmp.")
        tmpobj = mktemp ("tmp$ia_tmp.")
        tmpobjs = mktemp ("tmp$ia_tmp.")
        tmpobjsf = mktemp ("tmp$ia_tmp.")
        tmpobjsfb = mktemp ("tmp$ia_tmp.")
        tmpal = mktemp ("tmp$ia_tmp.")
        tmpsky = mktemp ("tmp$ia_tmp.")

	for (i=1; i <= npass; i +=1) {
          print(cm[i]//".d.fits",  >> tmpfits)
          print(cm[i],  >> tmpimh)
	}
	for (i=1; i <= nobj; i +=1) {
          print(cm[nob[i]],  >> tmpobj)
          print(cm[nob[i]]//"s",  >> tmpobjs)
          print(cm[nob[i]]//"sf",  >> tmpobjsf)
          print(cm[nob[i]]//"sfm",  >> tmpobjsfb)
	}
	for (i=1; i <= nsky; i +=1) 
          print(cm[ns[i]],  >> tmpsky)
	

	skyfile = froot//fil//"-sky"

	if (fil=="test")goto exit
	
if (rfits) { 

	print("Reading fits files ")

	imdelete("@"//tmpimh, ver-, >& "dev$null") 

rfits ("@"//tmpfits, "", "@"//tmpimh, make_image=yes, long_header=no, 
short_header=yes, datatype="", blank=0., scale=yes, oldirafname=no, offset=0)

}

	print ("                                     ", >> froot//"-camila.log")
	print (" --- CAMILA REDUCTION PARAMETERS --- ", >> froot//"-camila.log")
	print (" Object: ",froot," Sequence: ",seq, >> froot//"-camila.log")
	print (" Reference Image: ",refim," coordinates in ",coofil, >> froot//"-camila.log")
	print (" xyshift: ",xshi," ",yshi," imalign.boxsize=",boxsize," imalign.bigbox=",bigbox, >> froot//"-camila.log")
	print (" ----------------------------------- ", >> froot//"-camila.log")

if (skysta) {

	print("Preparing sky file from ")
	type (tmpsky)
	print("Stacked sky frame is ",skyfile)

	imdelete(skyfile, ver-, >& "dev$null") 

imcombine ("@"//tmpsky, skyfile,
sigma="", logfile=logfil, combine="median",
reject="none", project=no, outtype="real", offsets="none", masktype="none",
maskvalue=0., blank=0., scale="none", zero="median", weight="none",
statsec="", expname="", lthreshold=INDEF, hthreshold=INDEF, nlow=1, nhigh=1,
nkeep=1, mclip=yes, lsigma=3., hsigma=3., rdnoise="0.", gain="1.",
snoise="0.", sigscale=0.1, pclip=-0.5, grow=0)

if (skyedit) {

	print("Displaying sky for interactive cleaning --- uses imedit")

imedit (skyfile, "", 
cursor="", logfile="", display=yes, autodisplay=yes, autosurface=no,
aperture="circular", radius=2., search=2., buffer=1., width=2., xorder=2,
yorder=2, value=0., sigma=INDEF, angh=-33., angv=25.,
command="display $image 1 erase=$erase fill=yes order=0 >& dev$null",
graphics="stdgraph", default="b", fixpix=no)

}
} else {
	      skyfile = skyim
#	 print ("Sky image name ")
#	 scan (skyfile)
}



	print("Subtracting sky frame ", skyfile," from ")
	type (tmpobj)

	imdelete("@"//tmpobjs, ver-, >& "dev$null") 

imarith ("@"//tmpobj, "-", skyfile, "@"//tmpobjs, 
title="sky subtracted "//skyfile, divzero=0., hparams="",
pixtype="", calctype="", verbose=no, noact=no)

	imdelete("@"//tmpimh, ver-, >& "dev$null") 
        delete (tmpimh, ver-, >& "dev$null")


if (doflat) {

	print("Statistics of the flat image: ", flat) 
imstat (flat, fields = "image,mean,stddev,min,max,midpt", format=yes)
imstat (flat, fields = "image,mean,stddev,min,max,midpt", format=yes, >> froot//"-camila.log")
	print("Flat fielding the following images using ", flat) 
	type (tmpobjs)

	imdelete("@"//tmpobjsf, ver-, >& "dev$null") 

imarith ("@"//tmpobjs, "/", flat, "@"//tmpobjsf, 
title="flat fielded "//flat, divzero=0., hparams="",
pixtype="", calctype="", verbose=no, noact=no)

	imdelete("@"//tmpobjs, ver-, >& "dev$null") 

}

if (submed) {

print ("Subtracting a 255x1 median image by itself to remove the horizontal bands") 

	imdelete("@"//tmpobjsfb, ver-, >& "dev$null") 

median ("@"//tmpobjsf, "@"//tmpobjsfb, 255, 1, zloreject=INDEF, 
zhireject=INDEF, boundary="wrap", constant=0., verbose=yes)

imarith ("@"//tmpobjsf, "-", "@"//tmpobjsfb, "@"//tmpobjsf, 
title="sky subtracted "//skyfile, divzero=0., hparams="",
pixtype="", calctype="", verbose=no, noact=no)

	imdelete("@"//tmpobjsfb, ver-, >& "dev$null") 

}

irred1 = froot//" SKY="//skyfile//" FLAT="//flat//" 255x1 MEDIAN SUB: "//submed

print('hedit: ',irred1)
print('hedit: ',irred1, >> froot//"-camila.log")

#hedit ("@"//tmpobjsf, field="IRRED1", value=irred1, add=yes, del=no, 
#       veri=no, show=no, update=yes)

if (immos) {

          tmpalb = mktemp ("tmp$ia_tmp.")
	if (disp) {
	  print("Check the image quality/sequence before stacking") 
          tmpobjsfa = mktemp ("tmp$ia_tmp.")

	  ans = "yes"

	  for (i=1; i <= nobj; i +=1) {
	      display (cm[nob[i]]//"sf", xmag=1.5, ymag=1.5)

	print ("Is the image ",cm[nob[i]]//"sf", " OK? [yes,no,pos] [",ans,"] ")
	    scan (ans)

	    if (ans=="no") {
		imdelete(cm[nob[i]]//"sf", ver-, >& "dev$null") 
	        } else {
		m = i
	      if (ans=="pos") {
	    	print ("On-field position number for ",m,"=",cm[nob[i]]//"sf")
	    	print ("[to assign shifts]")
	    	scan (m)
		}
		print(cm[nob[i]]//"sf",  >> tmpobjsfa)
		print("rg"//cm[nob[i]]//"sf",  >> tmpal)
		print("rg"//cm[nob[i]]//"sf.mag.1",  >> tmpalb)
	   	print(xpos[m]," ",ypos[m], >> shiftfil) 
	    }
	  }
        delete (tmpobjsf, ver-, >& "dev$null")
	tmpobjsf = tmpobjsfa

	} else 
	  for (i=1; i <= nobj; i +=1) {
	   	#print(pos[i], >> shiftfil) 
	   	print(xpos[i]," ",ypos[i], >> shiftfil) 
          	print("rg"//cm[nob[i]]//"sf",  >> tmpal)
              	print("rg"//cm[nob[i]]//"sf.mag.1",  >> tmpalb)
	  }

	print("Aligning images ")  
	type (tmpobjsf)
	type (shiftfil)

	imdelete("@"//tmpal, ver-, >& "dev$null") 

imalign ("@"//tmpobjsf, refim, coofil, "@"//tmpal, shifts=shiftfil, 
boxsize=boxsize, bigbox=bigbox, negative=no, background=INDEF, lower=INDEF, 
upper=INDEF, niterate=3, tolerance=0, shiftimages=yes, interp_type="linear", 
boundary_typ="constant", consta=0., trimimages=no, verbose=yes, list="") 

	imdelete(mosfil, ver-, >& "dev$null") 

	print("Mossaicing images ")  
	type (tmpal)

imcombine ("@"//tmpal, mosfil,
sigma="", logfile=logfil, combine="median",
reject="none", project=no, outtype="real", offsets="none", masktype="none",
maskvalue=0., blank=0., scale="none", zero="median", weight="none",
statsec="", expname="", lthreshold=INDEF, hthreshold=INDEF, nlow=1, nhigh=1,
nkeep=1, mclip=yes, lsigma=3., hsigma=3., rdnoise="0.", gain="1.",
snoise="0.", sigscale=0.1, pclip=-0.5, grow=0)

	#if (delflat) 
	imdelete("@"//tmpobjsf, ver-, >& "dev$null") 

}

if (dophot) {

	print("Doing aperture photometry on aligned images")  


datapars.scale=1.0
datapars.fwhmpsf=2.5
datapars.epadu=15.0
datapars.readnoise=40.0
datapars.filter="FILTER"
datapars.exposur=""
datapars.airmass=""
#datapars.=""
datapars.xairmas=1.0
datapars.itime=1.0
#datapars.=""
fitskypars.salgorithm="median"
fitskypars.annulus =15
fitskypars.dannulus =10
centerpars.cbox=5
centerpars.calgorithm="centroid"
photpars.apertures="3,5,8,10,12"
photpars.zmag=26.

	print(mosfil,  >> tmpal)
	print(mosfil//".mag.1",  >> tmpalb)
        delete ("@"//tmpalb, ver-, >& "dev$null")

phot ("@"//tmpal, "", coords=xyfil, output="default", plotfile="", 
interactive=no, radplots=no,
verify=no, update=no, verbose=no, graphics="stdgraph", display="stdimage",
icommands="", gcommands="")

txdump ("@"//tmpalb,
"MERR[4],MAG[1-4],XCEN,YCEN,IFIL,IMAGE", "ID=1", headers=no,
parameters=yes) | cat >> refim//".mag"

txdump ("@"//tmpalb, "MERR[4],MAG[1-4],XCEN,YCEN,IFIL,IMAGE", 
"ID=1", headers=no, parameters=yes) 

txdump ("@"//tmpalb,
"MERR[4],MAG[1-4],XCEN,YCEN,IFIL,IMAGE", "ID=2", headers=no,
parameters=yes) | cat >> refim//".mag"
#"MERR[4],MAG[2],MAG[4-5],XCEN,YCEN,MSKY,IFIL,IMAGE", "ID=2", headers=no,

#txdump ("@"//tmpalb, "MERR[4],MAG[2],MAG[4-5],XCEN,YCEN,MSKY,IFIL,IMAGE", 
txdump ("@"//tmpalb, "MERR[4],MAG[1-4],XCEN,YCEN,IFIL,IMAGE", 
"ID=2", headers=no, parameters=yes) 

txdump ("@"//tmpalb,
"MERR[4],MAG[1-4],XCEN,YCEN,IFIL,IMAGE", "ID=3", headers=no,
parameters=yes) | cat >> refim//".mag"
#"MERR[4],MAG[2],MAG[4-5],XCEN,YCEN,MSKY,IFIL,IMAGE", "ID=3", headers=no,

#txdump ("@"//tmpalb, "MERR[4],MAG[2],MAG[4-5],XCEN,YCEN,MSKY,IFIL,IMAGE", 
txdump ("@"//tmpalb, "MERR[4],MAG[1-4],XCEN,YCEN,IFIL,IMAGE", 
"ID=3", headers=no, parameters=yes) 

txdump ("@"//tmpalb,
"MERR[4],MAG[1-4],XCEN,YCEN,IFIL,IMAGE", "ID=4", headers=no,
parameters=yes) | cat >> refim//".mag"
#"MERR[4],MAG[2],MAG[4-5],XCEN,YCEN,MSKY,IFIL,IMAGE", "ID=4", headers=no,

#txdump ("@"//tmpalb, "MERR[4],MAG[2],MAG[4-5],XCEN,YCEN,MSKY,IFIL,IMAGE", 
txdump ("@"//tmpalb, "MERR[4],MAG[1-4],XCEN,YCEN,IFIL,IMAGE", 
"ID=4", headers=no, parameters=yes) 

print (" ----------------- ", >> refim//".mag")
#type (refim//".mag")
#print (" ----------------- ", >> "camila-photIR.log")
#print (" ----------------- ", >> "camila-phot.log")

}

exit:

        delete (tmpfits, ver-, >& "dev$null")
        delete (tmpimh, ver-, >& "dev$null")
	delete (tmpobj, ver-, >& "dev$null")
        delete (tmpobjs, ver-, >& "dev$null")
        delete (tmpobjsf, ver-, >& "dev$null")
        delete (tmpobjsfb, ver-, >& "dev$null")
        delete (tmpal, ver-, >& "dev$null")
        if (disp) delete (tmpobjsfa, ver-, >& "dev$null")
        if (immos) delete (tmpalb, ver-, >& "dev$null")
        delete (shiftfil, ver-, >& "dev$null")
        delete (tmpsky, ver-, >& "dev$null")

        print("***DONE***")

end

