procedure canmag(image, catroot)

#packages to load:
#  digiphot apphot astutil stsdas analysis fitting
#add the command "gv" to login
#task    $gv                                                     = "$foreign"
#This task reads a 2MASS catalog and associates the catalog RA, DEC and MAG 
#to a star identified interactively on a CANICA image
#The first 11 columns of the 2MASS catalog is assumed as follows (without the begining #).  
#All lines stating with "#" are ignored
#It is also assumed that the "calibration star" is among the first "nmark" stars. 
#_RAJ2000        _DEJ2000        RAJ2000          DEJ2000          2MASS                  Jmag   e_Jmag  Hmag   e_Hmag   Kmag    e_Kmag  
#--------        --------        -------          -------           -----                 ----    -----  ----   ------   ----    ------  
#149.253196      +68.901779      149.253196      +68.901779      09570076+6854064         7.413   0.019  6.911   0.018   6.801   0.017 
#148.767531      +68.901566      148.767531      +68.901566      09550420+6854056         7.592   0.020  7.392   0.020   7.342   0.016
#148.323737      +69.046722      148.323737      +69.046722      09531769+6902481         8.327   0.026  7.840   0.057   7.646   9.998
#Tips to use
#The script allows you to display a 2MASS image (previously downloaded) and displays the catalog stars on that.
#Note the identification number of the "calibration star"
#Identify the corresponding star on the CANICA image
#The RADEC coordinates as well as the magnitude of the "calibration star" are used
#The CRVAL1, CRVAL2, CRPIX1, and CRPIX2 as well as ZEROMAG are written into the image header.
#The default value of CDELT1 and CDELT2 correspond to the CANICA images (East to the right of North).
#Oct 2010: A non-interactive mode is implemented, where the RADEC and XY are read file a file.
#
string  image    = "as40k"             	{prompt="Image Name"}
string  catroot  = "m81_2mass"     	{prompt="2MASS catalog root name [default extn is .cat]"}
#string  catal    = "m81_2mass.cat"     	{prompt="2MASS catalog name"}
#string  starlist = "inter"         	{prompt="Interactively identify [inter] or give a [ra dec x y] file of a star"}
bool    intera   = "yes"         	{prompt="Interactively identify [inter] or give a [ra dec x y] file of a star"}
int     starno   = 1		      	{prompt="Ref star number in the 2MASS catalog"}
int     nmark    = 50		      	{prompt="Number of stars to mark"}
real    cdelt1   = 0.32   		{prompt="Image scale in RA (arcsec/pix)"}
real    cdelt2   = 0.32   		{prompt="Image scale in Dec (arcsec/pix)"}
real    radlim   = 130.    		{prompt="Radius of GOOD field of view in arcsec"}
real    maglim   = 15.5   		{prompt="Magnitud lim of 2MASS to find zeropoint"}
real    fwhm     = 5.5     		{prompt="FWHM of the images"}
real    optxcen  = 0.0    		{prompt="Optical X center of the image"}
real    optycen  = 0.0    		{prompt="Optical Y center of the image"}
string  plotx    = "all"      		{prompt="X-axis of magplot [mag/rad/jhcol/jkcol]"}
#string  filter = "J"                  	{prompt="Filter Name"}
#real    cdelt1   = "8.974722e-5"   	{prompt="Pixel scale in RA (in degrees)"}
#real    cdelt2   = "8.974722e-5"   	{prompt="Pixel scale in Dec (in degrees)"}

begin

	string	imfil, mosfil, catfil, c1, regfil, imexfil, filt, creg,cid[999], c2, c3, c4
	string	starid,photcoo1,photcoo,photmag,radecfil,cdata[999],radecfil1,filthd, jd
	string	photmag1, idcom, plottyp, plotfil, obsdate, instru, igifil, epsfile, comm, fitfil,objnam
	string	catal
	real    ra,dec,ra2,dec2,jmag,ejmag,hmag,ehmag,kmag,ekmag, rad, zpold, zpmag, errmag
	real	err_jmag[999],err_hmag[999],err_kmag[999], rad_lim, mag_lim, mag_lo, mag_hi
	real    a_ra[999],a_dec[999],a_jmag[999],a_hmag[999],a_kmag[999],jkcol,jhcol
        real    rapix,decpix,xx, yy, ddd, amag, raref, decref, obsmag, refmag
	real	rain,decin,x1,x2,y1,y2,xin,yin,fwhmpsf, xc, yc, raddist, efftime, exptime
	real	a_obsmag[999],a_refmag[999], magfinal, magdif, a_errmag[999],eobsmag
	real	magcor,magcor_av,magcor_rms, fitrms, fitc1, fitc2
	int	i1, i2, i3, imark, sid, twomid1,twomid2,nx,ny,ns,index[999], nadd, agnid,len,nrows
	bool	staridint

	struct  line

	#starid = starlist; imfil = image; catfil = catal; imark = nmark; fwhmpsf=fwhm
	imfil = image; catal = catroot 
	starid = catal//".id"; catfil = catal//".cat"; imark = nmark; fwhmpsf=fwhm
	staridint = intera  
	rapix  = cdelt1/3600.; decpix = cdelt2/3600.
	plottyp = plotx; xc = optxcen; yc = optycen
	#filt  = filter

	#radlim = 400.; maglim = 15.5
	rad_lim = radlim/cdelt1; mag_lim = maglim; sid = starno
	rad = 3.0; zpmag=0.0
	regfil = imfil//"_canwcshead.reg"
        radecfil = imfil//"_radec.cat"
        radecfil1 = imfil//"_radec1.cat"
	imexfil = "canwcshead.imexam"
	delete(regfil, ver-, >& "dev$null")
	delete(imexfil, ver-, >& "dev$null")
        delete(radecfil, ver-, >& "dev$null")
        delete(radecfil1, ver-, >& "dev$null")

	hselect (imfil, "INSTRUME DATE-OBS", yes) | scan (instru, obsdate)
        #instru = "OSIRIS"
	c1 = substr(obsdate,1,4)
        if (instru == "CANICA"){
	if (int(c1) >= 2005) {
	print (obsdate, " ", c1)
        hselect (imfil, "i_naxis1, i_naxis2, EXPTIME, COADDS, FILTER2, DATE-OBS", yes) | scan (nx, ny, exptime, nadd, filthd, obsdate)
	} else {
	hselect (imfil, "i_naxis1, i_naxis2, FILTER2, DATE-OBS", yes) | scan (nx, ny, filthd, obsdate)
	nadd = 1; exptime = 1.0
	}
	efftime = exptime*nadd
        if (filthd == "1"){filt = "J"} else if (filthd == "2") {filt = "H"}
                                       else if (filthd == "3") {filt = "K"} else { filt = "X" }
        if (xc == 0.0 || yc == 0.0) {xc = real(nx)/2.0; yc = real(ny)/2.0}
        if (xc < 0.0 || xc > real(nx)) xc = real(nx)/2.0
        if (yc < 0.0 || yc > real(nx)) yc = real(ny)/2.0
	}
	print (efftime, " ", exptime, " ", nadd, " ", filt, " ", obsdate)

	#jd calculation
        len = strlen(obsdate); c3 = substr(obsdate,len,len); c4 = substr(obsdate,len-1,len)
        c2 = obsdate; delete("cmds.tempjd", ver-, >& "dev$null")
        i1 = strlstr("-",obsdate); len = strlen(obsdate); i2 = strlstr("T",obsdate)
        if (i1 == 7) {c2 = substr(obsdate,1,5); c1=substr(obsdate,6,len); c2=c2//"0"//c1}
        if (i2-i1 == 2) {c1=substr(c2,9,len); c2=substr(c2,1,8); c2=c2//"0"//c1}
        obsdate = c2
        len = strlen(obsdate); if (len==18) {obsdate=obsdate//c3}; if (len==17) {obsdate=obsdate//c4}
        #print (i1, " ", i2, " ", len, " ", c2, obsdate )
        c2 = 'jd = julday ("'//obsdate//'")'
        print(c2, > "cmds.tempjd")
        asthedit (imfil, "cmds.tempjd", table="", colname="", verbose-)
        hselect (imfil, "jd", yes) | scan (jd)
        print("Julian Date=",jd)
        c1 = substr(obsdate,1,4); 
	#jd calculation end

	print("# Region file format: DS9 version 4.0", >> regfil)
	print("# Filename: ", imfil, >> regfil)
	print("global color=green font=\"helvetica 10 normal\" select=1 highlite=1 edit=1 move=1 delete=1 include=1 fixed=0 source \nfk5", >> regfil)
	i2 = 0 
	i3 = 0 
	list = catfil
        while( fscan(list,line) != EOF ) {
	c1 = substr(line,1,3)
	print(line, >> radecfil)
	if (c1=="---") {
        while( fscan(list,line) != EOF ) {
        i2 = fscan(line,ra,dec,ra2,dec2,twomid1,jmag,ejmag,hmag,ehmag,kmag,ekmag) 
	  i3 = i3 + 1
	  cdata[i3] = line
	  cid[i3] = str(i3)
	  a_ra[i3] = ra; a_dec[i3] = dec; a_jmag[i3] = jmag; a_hmag[i3] = hmag; a_kmag[i3] = kmag
	  err_jmag[i3] = ejmag; err_hmag[i3] = ehmag; err_kmag[i3] = ekmag
	if (i3 <= imark) {
	creg = "# color=red text={"//cid[i3]//"}"
	printf("circle(%10f,%10f,%4f\") %s\n",a_ra[i3],a_dec[i3],rad, creg, >> regfil)
        printf("%15.7f %15.7f %8.3f %8.3f %8.3f %s\n",a_ra[i3],a_dec[i3],a_jmag[i3],a_hmag[i3],a_kmag[i3],cid[i3], >> radecfil1)
	}
	}
	}
	}

	zpold = rimexam.magzero
	#print (zpold)
	rimexam.magzero = 0.0

#	if (starid == "inter") {
	if (staridint) {
	#print ("Display 2MASS image on DS9 frame 1 using file option and overlay the region file ", regfil, " and note the ID of the calibration star")
	#print ("When ready press ENTER")
	#scan (c1)

	print ("Press letter a on the calibration star and Quit with q")
	display (imfil, 2, fill+, zs+)
	imexamin (frame=2, keeplog+, logfile=imexfil)
	#print ("Star identification number")
	#scan (sid)
 	print(cid[sid], " RA=", a_ra[sid], " Dec=", a_dec[sid], " J=",a_jmag[sid]," H=", a_hmag[sid]," K=",a_kmag[sid] )
	rimexam.magzero = zpold
	
	list = imexfil
        while( fscan(list,line) != EOF ) {
	c1 = substr(line,1,1)
	if (c1!="#") {
	  i2 = fscan(line,xx,yy,ddd,ddd,ddd,amag)
	if (filt=="J"){zpmag = a_jmag[sid] - amag}
	if (filt=="H"){zpmag = a_hmag[sid] - amag}
	if (filt=="K"){zpmag = a_kmag[sid] - amag}
	}
	}
	print(xx,yy,amag, zpmag)
	#creg = "# color=red text={"//cid[9]//"}"
	raref = a_ra[sid]; decref = a_dec[sid]
	zpmag = 0.0
	} else {
	zpmag = 0.0
	list = starid 
        while( fscan(list,line) != EOF ) {
	c1 = substr(line,1,1)
	if (c1!="#") {
	  i2 = fscan(line,raref, decref, xx,yy)
	  print (raref, " ", decref, " ", xx," ", yy, " ", filt)
		goto endfile
	             }
	}
	endfile:
	}

	mosfil = imfil
    hedit (mosfil, fields="WAT0_001", add-, delete+, verify-, show-, update+)
    hedit (mosfil, fields="WAT1_001", add-, delete+, verify-, show-, update+)
    hedit (mosfil, fields="WAT2_001", add-, delete+, verify-, show-, update+)

    hedit (mosfil, fields="CTYPE1", value="RA---SIN", add+, delete-, verify-, show-, update+)
    hedit (mosfil, fields="CTYPE2", value="DEC--SIN", add+, delete-, verify-, show-, update+)
    hedit (mosfil, fields="CRVAL1", value=raref, add+, delete-, verify-, show-, update+)
    hedit (mosfil, fields="CRVAL2", value=decref, add+, delete-, verify-, show-, update+)
    hedit (mosfil, fields="CRPIX1", value=xx, add+, delete-, verify-, show-, update+)
    hedit (mosfil, fields="CRPIX2", value=yy, add+, delete-, verify-, show-, update+)
    hedit (mosfil, fields="CDELT1", value=rapix, add+, delete-, verify-, show-, update+)
    hedit (mosfil, fields="CDELT2", value=decpix, add+, delete-, verify-, show-, update+)
    hedit (mosfil, fields="EQUINOX", value="2000", add+, delete-, verify-, show-, update+)
    hedit (mosfil, fields="ZEROMAG", value=zpmag, add+, delete-, verify-, show-, update+)
    hedit (mosfil, fields="FILTNAM", value=filt, add+, delete-, verify-, show-, update+)

	#prepare the radec list of stars in the field of view

        photcoo1 = imfil//"_phot1.coo"; delete(photcoo1, ver-, >& "dev$null")

        photcoo = imfil//"_phot.coo"; delete(photcoo, ver-, >& "dev$null")

#skyxymatch (imfil, imfil, photcoo1, coords=catfil, xmin=INDEF, xmax=INDEF, ymin=INDEF,
skyxymatch (imfil, imfil, photcoo1, coords=radecfil1, xmin=INDEF, xmax=INDEF, ymin=INDEF,
ymax=INDEF, nx=10, ny=10, wcs="world", xcolumn=1, ycolumn=2, xunits="",
yunits="", xformat="%10.3f", yformat="%10.3f", rwxformat="%-15.7f", rwyformat="%-15.7f",
wxformat="", wyformat="", min_sigdigit=7, verbose=no)

        delete(radecfil1, ver-, >& "dev$null")
        #hselect (imfil, "i_naxis1, i_naxis2", yes) | scan (nx, ny)

	x1 = 10; x2=nx-10; y1 = 10; y2 = ny-10
        i3 = 0; ns = 0
        list = photcoo1
        for (j=1; j <= 18; j +=1) { i1 = fscan(list,line) }
        while( fscan(list,line) != EOF ) {
        #c1 = substr(line,1,3)
        #if (c1=="---") {
        #while( fscan(list,line) != EOF ) {
        i1 = fscan(line,xin,yin,rain,decin,rain,decin)
	  i3 = i3 + 1
	#if (xin > x1 ) {
	#print (rain," ",decin," ",xin, " ",yin, >> photcoo) #}
	#if (xin > x1 && xin < x2 && yin > y1 && yin < y2) {print (a_ra[i3]," ",a_dec[i3]," ",xin, " ",yin, " ", i3, >> photcoo)}
	if (xin > x1 && xin < x2 && yin > y1 && yin < y2) {print (xin, " ",yin, " ", i3, >> photcoo)}
	if (xin > x1 && xin < x2 && yin > y1 && yin < y2) {ns = ns + 1; index[ns]=i3; print (cdata[i3]," ",i3, >> radecfil)}
        }
        delete(photcoo1, ver-, >& "dev$null")

	#carry out photometry at the expected positions of the stars. get the zeropt using 2mass mags. plot the zeropts

        photmag = imfil//"_phot.mag"; delete(photmag, ver-, >& "dev$null")
        photmag1 = imfil//"_phot1.mag"; delete(photmag1, ver-, >& "dev$null")
        #daomag = imfil//"_dao.mag"; delete(daomag, ver-, >& "dev$null")
        plotfil = imfil//"_plot_zeropt.mag"; delete(plotfil, ver-, >& "dev$null")
        igifil = imfil//"_plot_zeropt.igi"; delete(igifil, ver-, >& "dev$null")
        fitfil = imfil//"_plot_zeropt.dat"; delete(fitfil, ver-, >& "dev$null")

        phot (imfil, skyfile="", coords=photcoo, output=photmag1, plotfile="",
                scale=1.0, fwhmpsf=fwhmpsf, epadu=10.0, readnoise=3.0, filter="",
                exposur="", airmass="", xairmas=1.0, itime=1.0, salgorithm="median",
                annulus =20, dannulus =10, cbox=5, calgorithm="centroid",
                apertures="5,10,14,17,20", zmag=zpmag, interactive=no, radplots=no,
                verify=no, update=no, verbose=no, graphics="stdgraph", display="stdimage",
                icommands="", gcommands="")

	
        delete(photcoo, ver-, >& "dev$null")
	pselect (photmag1,photmag,"MAG[5]!=INDEF && MAG[4]!=INDEF && MAG[3]!=INDEF && MAG[2]!=INDEF && MAG[1]!=INDEF")
        txdump (photmag, "XCENTER YCENTER MAG[3]  ID", "MAG[3]!=INDEF && MAG[2]!=INDEF && MAG[1]!=INDEF ", headers-, paramet-, >> photcoo)
        #txdump (photmag, "XCENTER YCENTER MAG[1-5]  ID", "MAG[5]!=INDEF && MAG[3]!=INDEF && MAG[2]!=INDEF && MAG[1]!=INDEF", headers+, paramet+, >> daomag)
        pcalc (photmag, "MAG[5]", "MAG[3]-MAG[5]")
        tstat.median=0.05; tstat.stddev = 0.05
        for (j=1; j <= 4; j +=1) { 
	if (tstat.stddev < 0.03) tstat.stddev = 0.03
        mag_lo = 0.0; mag_hi = tstat.median+1.0*tstat.stddev
        txdump (photmag,"MAG[5]","yes") | tstat (lowlim=mag_lo, highlim=mag_hi)
        }
	magcor_av = tstat.median; magcor_rms = tstat.stddev

	for (j=1; j <= ns; j +=1) {
	idcom = "ID="//str(j)
	txdump (photmag, "XCENTER YCENTER MAG[3] MAG[5]", idcom, headers-, paramet-) | scan (xin, yin, obsmag, magcor)
	i3 = index[j]
	a_obsmag[j] = obsmag
	zpmag = 0.0
        if (obsmag!=INDEF) {
	if (filt=="J"){refmag = a_jmag[i3]; errmag=err_jmag[i3]}; if (filt=="H"){refmag = a_hmag[i3]; errmag=err_hmag[i3]}; if (filt=="K"){refmag = a_kmag[i3]; errmag=err_kmag[i3]}
	jkcol = a_jmag[i3] - a_kmag[i3]; jhcol = a_jmag[i3] - a_hmag[i3]
	zpmag = refmag - obsmag - 2.5*log10(efftime)
	a_refmag[j] = refmag; a_errmag[j] = errmag
        raddist = sqrt((xin - xc)**2 + (yin - yc)**2)
	if (raddist < rad_lim && refmag < mag_lim) { printf("%8.3f %8.3f %8.3f \n", refmag, zpmag, errmag, >> fitfil) }
        if (plottyp=="mag") {printf("%8.3f %8.3f %8.3f \n", refmag, zpmag, jkcol, >> plotfil)}
        if (plottyp=="rad") {printf("%8.3f %8.3f %8.3f \n", raddist, zpmag, jkcol, >> plotfil)}
        if (plottyp=="jhcol") {printf("%8.3f %8.3f  \n", jhcol, zpmag, >> plotfil)}
        if (plottyp=="jkcol") {printf("%8.3f %8.3f  \n", jkcol, zpmag, >> plotfil)}
	if (plottyp=="all") {printf("%8.3f %8.3f %8.3f %8.3f %8.3f %8.3f \n", refmag, zpmag, raddist, jkcol, jhcol, magcor, >> plotfil)}
	#print(j, i3, xin, yin, obsmag, refmag, zpmag, a_ra[i3], a_dec[i3], >> "tempppp")
	}
	}

	#graph (plotfil, wx1=0., wx2=0., wy1=0., wy2=0., pointmo+, marker="box", szmarke=0.01, xlabel="Reference MAG", ylabel="ZEROPOINT")
	#delete(plotfil, ver-, >& "dev$null"); 
	print ("data ",plotfil, >> igifil)
	#print ("location .1 .95 .1 .5; xcolumn 1;  ycolumn 2; ecolumn 3; limits; box; color 3; points; color 1; number ", >> igifil)
	#print ("location .1 .95 .1 .45; xcolumn 1;  ycolumn 2; limits; box; color 3; points; color 1; number ", >> igifil)
	print ("location .1 .95 .1 .45; xcolumn 1;  ycolumn 2; limits 9 17 18 23; box; color 3; points; color 1; number ", >> igifil)
	print ('xlabel "2MASS MAG"; ylabel "Zeropoint"', >> igifil)
	print ("location .1 .95 .45 .65; xcolumn 1;  ycolumn 6; limits 9 17 -0.05 0.15; box 0 2; color 3; points; color 1; number ", >> igifil)
	#printf ("ltype 2; relocate 9 %8.4f ",magcor_rms,"; draw 17 %8.4f",magcor_rms,"; ltype 0 \n", >> igifil)
	printf ("ltype 2; relocate 9 %8.4f \n",magcor_av, >> igifil)
	printf ("draw 17 %8.4f \n",magcor_av, >> igifil)
	print ('ltype 0 ylabel "[MAG3]-MAG[5]"', >> igifil)
	#print ('ylabel "Zeropoint"', >> igifil)
	print ("location .1 .45 .75 .95; xcolumn 3;  ycolumn 2; limits; box; color 3; points; color 1; number ", >> igifil)
	print ('xlabel "Radial distance"; ylabel "Zeropoint"', >> igifil)
	print ("location .55 .95 .75 .95; xcolumn 4;  ycolumn 2; limits; box; color 3; points; color 1; number ", >> igifil)
	print ('xlabel "J-K"; ylabel "Zeropoint"', >> igifil)
	#print (igifil) | igi dev=eps
	#type (igifil) | igi dev=eps,  >& "dev$null"
	type (igifil) | igi dev=eps
	#type (igifil) | igi dev=eps
	type (igifil) | igi 

	comm = " -t sgi*.eps "
	ls comm//" " | grep sgi | scan (epsfile)
	comm = epsfile//" &"
	#print (comm)
	gv comm//" "

	#print ("When ready press ENTER")
	#scan (c1)

        display (imfil, 1, zs+, fill+, >& "dev$null")
        tvmark (1, photcoo, mark="circle", radii=10, number+, nxoff=3, nyoff=3, txsize=2)
	
	tstat.median=20.3; tstat.stddev = 0.3
        for (j=1; j <= 4; j +=1) { 
	mag_lo = tstat.median-3.0*tstat.stddev; mag_hi = tstat.median+3.0*tstat.stddev
	tstat (fitfil, 2, lowlim=mag_lo, highlim=mag_hi) 
	}
	nfit1d (fitfil, "test.nfit1d.tab", functio="user", userpar.functio="c1+c2*x", c1=tstat.median, c2=0.0, v1=yes, v2=yes, ltype="boxes", int+)
	#delete(fitfil, ver-, >& "dev$null")
	zpmag = tstat.median

	#tprint test.nfit1d.tab columns="rms coeff1 coeff2" row=65 showhdr- showuni- showrow- | wc | scan (nrows)
	ttools.tprint ("test.nfit1d.tab", prparam=no, prdata=yes, pwidth=80, plength=0, showrow=no, orig_row=yes,
	showhdr=no, showunits=no, columns="rms coeff1 coeff2", rows="-", option="plain", align=yes,
	sp_col="", lgroup=0) | wc | scan (nrows)
	ttools.tprint ("test.nfit1d.tab", prparam=no, prdata=yes, pwidth=80, plength=0, showrow=no, orig_row=yes,
	showhdr=no, showunits=no, columns="rms coeff1 coeff2", rows=nrows, option="plain", align=yes,
	sp_col="", lgroup=0) | scan (fitrms, fitc1, fitc2)

	print ("Zeropont from the fit = ", fitc1, "+/- ",fitrms)
	zpmag = fitc1

	#print("Enter the best fit ZP and RMS")
	#scan (zpfit, rmsfit)

	print("Enter ID number of the variable source")
	scan (agnid)

	magfinal = zpmag + a_obsmag[agnid] + 2.5*log10(efftime)
	magdif = a_refmag[agnid] - magfinal
	#zpmag = refmag - obsmag - 2.5*log10(efftime)
        #printf("%8.3f %8.3f %8.3f %8.3f \n", "Magnitude : ",magfinal, "+/-",tstat.stddev, " 2MASS = ",a_refmag[agnid], " dm=",magdif)
	idcom = "ID="//str(agnid)
	#print(idcom)

	txdump (photmag, "MERR[3]", idcom, headers-, paramet-) | scan (eobsmag)
	#hselect (imfil, "OBJECT $I DATE-OBS JD FILTNAM", yes) 
	hselect (imfil, "OBJECT", yes) | scan (objnam)
	#printf("IMAGE=%12s OBJECT=%10s DATE=%18s JD=%16s FILTER=%3s \n", imfil, objnam, obsdate, str(jd), filt)
	print("---------------------------------------------------------------------------")
	printf("%9s OBJECT=%10s %18s JD=%16s FILTER=%3s \n", imfil, objnam, obsdate, str(jd), filt)
        #printf("Magnitude=(%7.3f+/-%6.3f)+/-%6.3f 2MASS=%7.3f+/-%6.3f dm= %7.3f \n", magfinal, magcor_rms, tstat.stddev, a_refmag[agnid], a_errmag[agnid],magdif)
        printf("Magnitude=(%7.3f+/-%6.3f)+/-%6.3f 2MASS=%7.3f+/-%6.3f dm= %7.3f \n", magfinal, magcor_rms, fitrms, a_refmag[agnid], a_errmag[agnid],magdif)
	print("---------------------------------------------------------------------------")

	end
