#--------------------------------------------------------------------
#CANICA REDUCTION SCRIPT  ---- Version 30 Dec 2003     ydm@inaoep.mx
#Changes: 1. Can read upto 99 images (instead of 20)
#2. Doesn't reset phot pset parameters
#3. A new parameter "jhkzp" to hold JHK zeropoints is introduced. Automatically
#   identifies the JHK filters, and uses the corresponding zeropoints. For
#   other filters uses a zeropoint of 20.
#4. After aligning the images, images are combined using 
#   imcombine combine=average reject=minmax nlow=1 nhigh=1 
#   the photometric accuracy is found to be better with this option than
#   with combine=median reject=none
#5. Keywords "ZEROPT" and "REDUCED" are introduced to hold zeropoint and 
#   the version of canobj. Script parameter "version" should be updated after
#   every new version.
#   08 Jan 2004
#6. Keyword "EXPTIME" is compared in different images and exits if not equal in 
#   all. Also writes the value of EXPTIME in the log file.
#5  iref: should be removed
#--------------------------------------------------------------------
procedure canobj(root, imbeg, imend)

string  root     = "as40k"             {prompt="Root Image Name"}
int     imbeg    = 1                 {prompt="First Image Number"}
int     imend    = 1                 {prompt="Last Image Number"}
string  sequence = "preview"           {prompt="Sequence file [file.seq/preview]"}
string  skyframe = "stack"             {prompt="Sky frame [stack/all/name]"}
string  flattyp  = "null"              {prompt="Flat-fielding strategy [null/flat/optflat]"}
string  flatim   = " "                 {prompt="Flat field frame Name"}
string  refimag  = "as40j002ref"       {prompt="Reference Image name"}
string  skymult  = "1 11"              {prompt="Image number intervals for multiple sky"}
bool    centroid = yes               {prompt="Find centroids in images?"}
int     iref     = 1                 {prompt="ref star image number"}
int     seqbeg   = 1                 {prompt="Image number of the first position in sequence file"}
bool    skyedit  = yes               {prompt="Clean the sky using imedit?"}
bool    disp     = yes               {prompt="Display before imcombine? "}
int     idisp    = 1                 {prompt="Display Frame No [1-4] o 0 for cycle thru all"}
#bool    submed   = "no"                {prompt="Remove the horizontal band?"}
bool    remhole  = yes               {prompt="Remove the holes left by bright stars?"}
bool    immos    = yes               {prompt="Mosaic the images?"}
bool    dophot   = yes               {prompt="Do photometry?"}
real    xshift   = 0              {prompt="X-shift[=Xref-Xin] wrt refimag"}
real    yshift   = 0              {prompt="Y-shift[=Yref-Yin] wrt refimag"}
real    skyout   = 600       {prompt="Minimum Distance of the off-field sky"}
real    skyin    = 150       {prompt="Minimum Distance of the on-field sky"}
string  skysect  = "[551:575,551:575]" {prompt="Section for skyvalue estimation"}
string  flatsect = "[551:575,551:575]" {prompt="Section for flat normalization"}
real    boxsize  = 25              {prompt="imalign boxsize"}
real    bigbox   = 25             {prompt="imalign bigbox"}
real    crval1   = 0.0           {prompt="RA of the header (in degrees)"}
real    crval2   = 0.0           {prompt="Dec of the header (in degrees)"}
real    crpix1   = 512.5        {prompt="X-Pixel corresponding to the RA-Dec"}
real    crpix2   = 512.5        {prompt="Y-Pixel corresponding to the RA-Dec"}
real    cdelt1   = 8.88889e-5   {prompt="Pixel scale in RA (in degrees)"}
real    cdelt2   = 8.88889e-5   {prompt="Pixel scale in Dec (in degrees)"}
real    epoch    = 2000.0       {prompt="Epoch of RA-Dec"}
bool    dowcs    = yes          {prompt="Transform the image to N_up E_left?"} 
real    xymag    = 2.0          {prompt="XY reduction factor if dowcs=yes"} 
string  jhkzp    = "20.60 20.40 20.20"   {prompt="JHK Zeropoints"} 
string  rakey    = "RA"           {prompt="Right Ascension Keyword name in the input image"}
string  suffix   = "_I.fits"      {prompt="Image ending name (after the numbers)"}

begin 
	string  froot, seq, logfil, skyfile, flat, skyim, cns, ra_key
        string  cm[99], cmn, pos[99], xyshi, irred1, cfilter2, cfilter1
        string  coofil, xyfil, refim, mosfil, shiftfil, ans
        string  tmpobjs, tmpobjsf, tmpal, tmpsky,filt01,filt02
	string  zpjhk, version, flatstrat, secsky, secflat
        string  tmpobj, tmpobjsfa, tmpobjsfb, tmpalb, tmp
        int     i,ins,iseq,n,m,im1,im2,imn,i1, i2, i3, i4, iref0, itv,ntv
	int	npass,j,nobj,nsky,ns[99],nob[99],kref,nx,ny,dskyind
        int     skyind[10], s1, s2, s3, s4, s5, s6, s7, s8, s9, s10
	int     numsky,numsky1,k,skyindk,kk,isky,nsabs,nobabs,im0,in
        bool    skysta,find_centroid,clean 
        real    xshi, yshi, xpos[99], ypos[99], xdif[99],ydif[99],dist[99]
        real    chipsiz, galsiz, scal, xref, yref, xdiff,ydiff
	real    diff,diff1,diff2,diff3,diff4,flatval,zmode
	real    raval,decval,xradec,yradec,rapix,decpix,obsyr,mag
	real    zpj, zph, zpk, zpmag, expt01, cexptime, skyval
	real    xmin, xmax, ymin, ymax,exptime
	string  skyfil[10],skygrp,objsf,irred,skytype
	string  suffix1, suffix2, suffix3, q1, q2, q3, q4
        struct  line
        #string  dummy
 
	#version = "using CANOBJ version 30_Dec_2003 by ydm@inaoep.mx"
	#version = "using CANOBJ version 27_May_2004 by ydm@inaoep.mx"
	version = "using CANOBJ version 07_Mar_2005 by ydm@inaoep.mx"

        seq = sequence; froot  = root; flat = flatim; skyim = skyframe
	im1 = imbeg; im2 = imend; im0 = seqbeg; iref0  = iref; kref = iref0 - im0 + 1
        xshi  = xshift; yshi  = yshift; xyshi = "0  0"
	chipsiz = skyout;  galsiz  = skyin; flatstrat = flattyp
	raval  = crval1; decval = crval2; xradec = crpix1; yradec = crpix2
	rapix  = cdelt1; decpix = cdelt2; obsyr  = epoch; mag = xymag; zpjhk = jhkzp
	secsky    = skysect    #"[551:575,551:575]"
	secflat   = flatsect   #"[551:575,551:575]"
        skygrp = skymult; find_centroid = centroid
	ntv = idisp; itv = ntv
	clean = remhole
	ra_key = rakey; suffix1 = suffix

#------------------------------------------------------------------------
#	file definitions 
#------------------------------------------------------------------------

        refim = refimag; coofil = refim//".coo"; xyfil  = refim//".xy"
        logfil = froot//"-canica.log"; mosfil = froot//"_"//iref0

#------------------------------------------------------
#	Read the sequence coordinate file
#------------------------------------------------------

	  npass = im2 - im1 + 1

	if (seq == "preview") {
          skysta = yes
	} else {
        list = coofil
        i1 = fscan(list,xref,yref)
        i1=0
        list = seq
        while( fscan(list, line) != EOF ) {
	    tmp = substr (line, 1, 1)
	    if (tmp != "#") {
		i2 = fscan(line,x,y)
                i1=i1+1; in = i1-im1+im0
		if (in > 0){ xpos[in]=x; ypos[in]=y }
                }
                }
	if (i1 < npass) npass = i1
	}

	i1 = fscan(zpjhk,zpj,zph,zpk)
	print("JHK ZPs ", zpj,zph,zpk,i1)


#------------------------------------------------------------------------
#	Decide whether use an already prepared sky image, or prepare it
#	by stacking or don't subtract sky, or obtain the sequence
#	number of sky images for stacking.
#------------------------------------------------------------------------
#	if (skyim == "stack") obtain sky from the present sequence by stacking off-source frames
#       if (skyim == "all") obtain sky from the present sequence by stacking all frames
#       if (skyim == "name") implies you are giving a previously prepared sky frame "name"
#       if (skyim == "0345") obtain sky from the present sequence by stacking fames 003,004 and 005: outdated

        if (skyim == "stack"){ skysta = yes } 
	else if (skyim == "all") { skysta = yes } 
	else { skysta = no; numsky = 2; skyfil[1] = skyim; skyind[1]=im1; skyind[2]=im2 }
	#else { skysta = no; skyfile = skyim; numsky = 2; skyfil[1] = skyim; skyind[1]=im1, skyind[2]=im2 }

        if (seq == "preview" ) {skysta = yes; skyim = "all"}
        #if (seq == "preview" && skysta == yes ) { skyim = "all" }

	if (skysta == yes ) {
        numsky = fscanf (skygrp, "%d %d %d %d %d %d %d %d %d %d", s1, s2, s3, s4, s5, s6, s7, s8, s9, s10)
        skyind[1] = s1 ; skyind[2] = s2 ; skyind[3] = s3; skyind[4] = s4
        skyind[5] = s5 ; skyind[6] = s6 ; skyind[7] = s7; skyind[8] = s8
        skyind[9] = s9 ; skyind[10] = s10 
	if (seq == "preview" ){numsky=1 }
        if (numsky <= 2 ) {
                numsky = 2
                skyind[1] = im1
                skyind[2] = im2
        	}
        if (skyind[1] <= 0)skyind[1] = im1
        if (skyind[numsky] < im2)skyind[numsky] = im2
        if (numsky > 2) {
        if (skyind[numsky] > im2 || skyind[1] < im1 ) {
	print("Check skymult parameter. Image range for multiple sky should be between ",im1,"& ",im2)
		goto exit
		}
        for (i=2;  i <= numsky;  i += 1) { 
		dskyind = skyind[i] - skyind[i-1]
          	if (dskyind < 5) {
        print ("Image number interval for multiple sky should be greater than 4")
                goto exit 
		}
        	}
	} else { numsky = 2; skyfil[1] = skyim }
	}

	skytype = "["
        for (i=1;  i <= numsky;  i += 1) { 
	  skytype = skytype//" "//str(skyind[i])
	}

#------------------------------------------------------------------------
# 	Write useful reduction parameters into a logfile
#------------------------------------------------------------------------
 
      print (" ___________________________________ ", >> froot//"-canica.log")
      print (" ooooooooooooooooooooooooooooooooooo ", >> froot//"-canica.log")
      print (" <<<<<<<<<<<<<<< CANICA >>>>>>>>>>>> ", >> froot//"-canica.log")
      print (" ooooooooooooooooooooooooooooooooooo ", >> froot//"-canica.log")
      print ("                                     ", >> froot//"-canica.log")
      print (" --- CANOBJ[",seq,"] REDUCTION PARAMETERS --- ", >> froot//"-canica.log")
      print (" Object: ",froot," Sequence: ",seq, >> froot//"-canica.log")
      print (" Reference Image: ",refim," coordinates in ",coofil, >> froot//"-canica.log")
#------------------------------------------------------------------------
#	Identify sky and objects frames automatically (needs to be done
#	even if skyim = "name")
#------------------------------------------------------------------------
#	if (skyim == "all") { use all the images for sky as well as object; suitable only for
#				point source dithered observations
#	} else {
#		1. skyframes are those which are dithered by more than "skyin"
#		2. object frames are those which are dithered less than "skyout"
#	}

    #if (!skysta) {goto skip_sky_stack}

	if (skyim == "all"){
           nsky = npass
	   nobj = npass
           for (i=1; i <= npass; i +=1) {
              ns[i] = i 
              nob[i] = i
	   }
	} else {

        i2 = 0
        i3 = 0
        i4 = 0
 
        for (i=1; i <= npass ; i +=1) {
                xdif[i] = (xref - xpos[i]) + xshi
                ydif[i] = (yref - ypos[i]) + yshi
                xdiff = (xpos[kref] - xpos[i]) 
                ydiff = (ypos[kref] - ypos[i]) 
                dist[i] = sqrt(xdiff**2 + ydiff**2)
	if (xpos[i] < 1.0 || ypos[i] < 1.0 || xpos[i] > 1024 || ypos[i] > 1024) dist[i] = chipsiz + 1.0
  printf("%c %8.3f %8.3f %8.3f \n",froot//str(im1+i-1), xdif[i],ydif[i], dist[i], >> froot//"-canica.log" )
                if (dist[i] > chipsiz) {
                        i2=i2+1
                        ns[i2] = i
                } else if (dist[i] > galsiz) {
                        i2=i2+1
                        i3=i3+1
                        ns[i2] = i
                        nob[i3] = i
                } else {
                        i3=i3+1
                        i4 = i4 + 1
                        #if (i4 == 1){
                        #        ns[i2 + 1] = i
                        #        i2 = i2 + 1
                        #        }
                        nob[i3] = i
                        }
                }
 
		nsky = i2
		nobj = i3

	if (nsky > nobj) nsky = nobj

        if (skysta) {print("Nsky = ",nsky, " NObject = ",nobj)}
        if (!skysta) {print(" NObject = ",nobj)}
 
                for (i=1; i <= nsky; i +=1) {
    if (skysta) {
	print("sky_",i, ns[i], froot//str(im1+ns[i]-1) )
    print("sky_",i, ns[i], froot//str(im1+ns[i]-1), >> froot//"-canica.log")
		}
                        }
                for (i=1; i <= nobj; i +=1) {
                       print("Object_",i, nob[i], froot//str(im1+nob[i]-1) )
print("Object_",i, nob[i], froot//str(im1+nob[i]-1), >> froot//"-canica.log")
                        }
	}    #skyim = all end

	if (nsky <= 2 && skysta == yes){
	  print ("Not enough images for sky, change skyin/skyout and re-run canica")
	goto exit
	}
 
skip_sky_stack:

#------------------------------------------------------------------------
#	Form the filename using rootname and imbeg
#------------------------------------------------------------------------

        n = 0
loop:
        n = n + 1
        imn = im1 + n - 1
        cmn = froot//str(imn)
        if (imn < 100) cmn = froot//"0"//str(imn)
        if (imn < 10) cmn = froot//"00"//str(imn)
          cm[n] = cmn
	#imgets (cm[n]//"_I.fits", "FILTER1")
	imgets (cm[n]//suffix1, "FILTER1")
	cfilter1  =(imgets.value)
	#imgets (cm[n]//"_I.fits", "EXPTIME")
	imgets (cm[n]//suffix1, "EXPTIME")
	cexptime  =real(imgets.value)
	if (n == 1) filt01 = cfilter1
	if (n == 1) expt01 = cexptime
	if (filt01 != cfilter1) {
	print("All the images are not in the same filter")
	print("Image1_Filter1=", filt01, " Image",cm[n]," Filter1=",cfilter1)
	goto exit
	}
        if (expt01 != cexptime) {
        print("All the images are not of the same exposure time")
        print("Image1_EXPTIME=", expt01, " Image",cm[n]," EXPTIME=",cexptime)
        goto exit
	}

	#imgets (cm[n]//"_I.fits", "FILTER2")
	imgets (cm[n]//suffix1, "FILTER2")
	cfilter2  =(imgets.value)
	if (n == 1) filt02 = cfilter2
	if (filt02 != cfilter2) {
	print("All the images are not in the same filter")
	print("Image1_Filter2=", filt02, " Image",cm[n]," Filter2=",cfilter2)
	goto exit
	}
	#print("FILTER2 ", cfilter2)
        if (n < npass )goto loop

	if (filt02 == "1"){
		zpmag = zpj
	} else if (filt02 == "2") {
                zpmag = zph
	} else if (filt02 == "3") {
                zpmag = zpk
	} else {
	        zpmag = 20.0
	}
 
	print("FILTER1=",cfilter1, " FILTER2=", cfilter2, " EXPTIME=", expt01, >> froot//"-canica.log")
        print (" xyshift: ",xshi," ",yshi," imalign.boxsize=",boxsize," imalign.bigbox=",bigbox, >> froot//"-canica.log")
        print (" ----------------------------------- ", >> froot//"-canica.log")
 
#------------------------------------------------------------------------
#	divide by flat first
#------------------------------------------------------------------------
	if (flatstrat == "flat" || flatstrat == "optflat") {
	  imdelete(flat//"N", ver-, >& "dev$null")
	  imcopy (flat, flat//"N", verb-)
	  normalize (flat//"N", sample_=secflat)
        print("Statistics of the flat image: ", flat//"N")
	imstat (flat//"N"//secflat, fields = "image,mean,stddev,min,max,midpt", format=yes)
	imstat (flat//"N"//secflat, fields = "image,mean,stddev,min,max,midpt", 
		format=yes) | cat >> froot//"-canica.log")
        #print("Flat fielding the following images using ", flat//"N")
	}

	#suffix1 = "_I.fits"
        suffix2 = "ps"
        suffix3 = "sf"
        #suffix4 = "fs"
	#if (flatstrat == "optflat") suffix1 = "f"    #suffix2
	#if (flatstrat == "optflat") suffix3 = "fs"   #suffix4
	
	#for (i=1; i <= npass ; i +=1) {
	#  objf[i] = cm[i]//suffix1
	#}
	if (ra_key == "RA") {
	for (i=1; i <= npass ; i +=1) {
	  imdelete(cm[i]//suffix2, ver-, >& "dev$null")
	  imarith (cm[i]//suffix1, "/", cexptime, cm[i]//suffix2, ver-)
	}
	suffix1 = "ps"
	}

	if (flatstrat == "optflat") {
          suffix2 = "f"
        for (i=1; i <= npass ; i +=1) {
	  imdelete(cm[i]//suffix2, ver-, >& "dev$null")
	  imarith (cm[i]//suffix1, "/", flat//"N", cm[i]//suffix2, ver-)
        }
	  suffix1 = suffix2; suffix3 = "fs"
        } 
#---------------------------------------------------------------------
#	plot the sky values
#------------------------------------------------------------------------

	ymax = 0.0
	ymin = 0.0
	xmin = real(im1) - 1.0
	xmax = real(im2) + 1.0
	delete("sky_mode.tbl", ver-, >& "dev$null")
	for (i=1; i <= npass ; i +=1) {
	  imstat (cm[i]//suffix1, fields="mode", format- ) | scan (zmode)
	  k = im1+i-1
	  print (k, zmode, >> "sky_mode.tbl") 
	  if (i == 1) ymin = zmode
	  if (zmode < ymin) {ymin = zmode}
	  if (zmode > ymax) {ymax = zmode}
	}
	ymin = ymin*0.9
	ymax = ymax*1.1
	graph ("sky_mode.tbl", wx1=xmin, wx2=xmax, wy1=ymin, wy2=ymax, pointmo+, marker="box", szmarke=0.01, xlabel="image number", ylabel="Sky counts/second", overplo-, app- ) 

#------------------------------------------------------------------------
#	Prepare sky image if asked to do so
#------------------------------------------------------------------------

if (skysta) {
        tmpsky = mktemp ("tmp$ia_tmp.")

        for (k=1; k <= numsky-1; k +=1) {
          if (k == 1)skyindk = skyind[k]
          if (k > 1)skyindk = skyind[k] + 1
	  isky = 0
	  if (skyindk < im1 || skyind[k+1] > im2) 
		break
          for (kk=skyindk; kk <= skyind[k+1]; kk +=1) {
            for (i=1; i <= nsky; i +=1) {
		nsabs = im1 + ns[i] - 1
          	if (nsabs == kk) {
	 	  isky = isky + 1
            	  print(cm[ns[i]]//suffix1,  >> tmpsky)
            	  print(i,cm[ns[i]]//suffix1 )
            	  #print(cm[ns[i]]//"_I.fits",  >> tmpsky)
            	  #print(i,cm[ns[i]]//"_I.fits")
          	}
            }
          }
	  if (isky > 2) {
                   skyfil[k] = froot//"_sky"//k
           print("Preparing sky file ",skyfil[k], " using images ", skyindk," to ",skyind[k+1])
	   print("   -----------------------------------------")
           imdelete(skyfil[k], ver-, >& "dev$null")
     imcombine ("@"//tmpsky, skyfil[k], combine="median", scale="none", zero="median", logfile=logfil)
        delete (tmpsky, ver-, >& "dev$null")
        tmpsky = mktemp ("tmp$ia_tmp.")
	  } else {
	print("There should be at least 4 frames for sky. Nsky=",isky,":",skyind[k],"-->",skyind[k+1])
		type(tmpsky)
		#goto exit
	  }
        }


#------------------------------------------------------------------------
#	Edit the sky image if asked to do so
#------------------------------------------------------------------------

if (seq != "preview" && skyedit) {
 
	for (k=1; k <= numsky-1; k +=1) {

        print("Displaying sky image ",skyfil[k]," for interactive cleaning --- uses imedit")
 
imedit (skyfil[k], "",
cursor="", logfile="", display=yes, autodisplay=yes, autosurface=no,
aperture="circular", radius=2., search=2., buffer=1., width=2., xorder=2,
yorder=2, value=0., sigma=INDEF, angh=-33., angv=25.,
command="display $image 1 erase=$erase fill=yes order=0 >& dev$null",
graphics="stdgraph", default="b", fixpix=no)
 
        #delete (tmpsky, ver-, >& "dev$null")

	} 
	}
	} #skysta endif
#--------------------------------------------
 
	#if (skyedit) goto exit

#------------------------------------------------------------------------
#	Subtract sky image if asked to do so
#------------------------------------------------------------------------

        if (seq == "preview" && disp == no)nobj= 2

        tmpobj = mktemp ("tmp$ia_tmp.")
        tmpobjs = mktemp ("tmp$ia_tmp.")
        tmpobjsf = mktemp ("tmp$ia_tmp.")
        for (i=1; i <= nobj; i +=1) {
          print(cm[nob[i]]//suffix1,  >> tmpobj)
          #print(cm[nob[i]]//"_I.fits",  >> tmpobj)
          #print(cm[nob[i]]//"s",  >> tmpobjs)
          #print(cm[nob[i]]//"sf",  >> tmpobjsf)
          print(cm[nob[i]]//"s",  >> tmpobjs)
          print(cm[nob[i]]//suffix3,  >> tmpobjsf)
	}

        if (flatstrat != "optflat") imdelete("@"//tmpobjs, ver-, >& "dev$null")

#------------------------------------------------------------------------------
#	if (flatstrat == "skynew") {
#		1. subtract the interchip horizontal and vertical bands 
#		2. normalize the above sky "skyfile" and use it as flat
#		3. subtract sky value secsky[551:575,551:575] from the sky image "skyfile" obtained above
#		}
#       if (flatstrat == "skyold") {
#		1. subtract the interchip horizontal and vertical bands 
#		   (if skyim=stack uses new skyfile, else uses "flatim")
#               2. normalize the image supplied by the parameter "flatim" and use it as flat
#		3. subtract sky value [551:575,551:575] from the sky image obtained above
#		  (if skyim=stack uses new skyfile, else uses "flatim")
#               }
#       if (flatstrat == "flat") {
#               1. subtract sky image obtained above
#               2. devide by normalized flat supplied by the parameter "flatim"
#               }
#       if (flatstrat == "null") {
#		1. subtract sky image obtained above
#		2. No correction for flat field
#               } 
#------------------------------------------------------------------------------

	if (flatstrat == "skynew" || flatstrat == "skyold") {
	  print("flattyp = ",flatstrat," is no more available")
	}
#flatstrat = skynew or skyold end; 

        q1 = "[1:512,1:512]"
        q2 = "[513:1024,1:512]"
        q3 = "[513:1024,513:1024]"
        q4 = "[1:512,513:1024]"

   if (flatstrat == "null" || flatstrat == "flat" || flatstrat == "optflat") {
        for (k=1; k <= numsky-1; k +=1) {
          if (k == 1)skyindk = skyind[k]
          if (k > 1)skyindk = skyind[k] + 1
	  if (skyindk >= im1 && skyind[k+1] <= im2){
          print("Subtracting SKY ", skyfil[k]," from OBJECT frames ",skyindk,"->",skyind[k+1])
	#if (flatstrat == "flat")print("Dividing OBJECT frames ",skyindk,"->",skyind[k+1], " by normalized flat ",flat//"N")
	  }
        for (i=1; i <= nobj; i +=1) {
	  nobabs = nob[i] + im1 - 1
	  #if (nob[i] >= skyindk && nob[i] <= skyind[k+1]) {
	  if (nobabs >= skyindk && nobabs <= skyind[k+1]) {
	  #objsf = cm[nob[i]]//"sf"
	  objsf = cm[nob[i]]//suffix3
          imdelete(objsf, ver-, >& "dev$null")
	  imdelete(objsf//"q*", ver-, >& "dev$null")
	  #imarith (cm[nob[i]]//"_I.fits", "-", skyfil[k], objsf, ver-)
	  imarith (cm[nob[i]]//suffix1, "-", skyfil[k], objsf, ver-)

	  #imstat (objsf//q1, fields="mode", format- ) | scan (zmode) 
	  #imexpr (repl(zmode, 512), objsf//"q1", dim="512,512")
	  #imstat (objsf//q2, fields="mode", format- ) | scan (zmode) 
	  #imexpr (repl(zmode, 512), objsf//"q2", dim="512,512")
	  #imstat (objsf//q3, fields="mode", format- ) | scan (zmode) 
	  #imexpr (repl(zmode, 512), objsf//"q3", dim="512,512")
	  #imstat (objsf//q4, fields="mode", format- ) | scan (zmode) 
	  #imexpr (repl(zmode, 512), objsf//"q4", dim="512,512")
	  #imexpr ("repl(0.0, 1024)", objsf//"q", dim="1024,1024")

	  imstat (objsf, fields="mode", format- ) | scan (zmode) 
	  if (flatstrat == "null")print ("Subtracting residual sky offset ", zmode, " from ",nob[i]//"s")
	  if (flatstrat == "flat")print ("Subtracting offset sky ", zmode, " from image #",nob[i]," and flat fielding")
	  #imarith (objsf, "-", objsf//"q", objsf, ver-)
	  imarith (objsf, "-", zmode, objsf, ver-)
	  if (flatstrat == "flat") imarith (objsf, "/", flat//"N", objsf, ver-)
#	  print (i, zmode, >> "sky_mode.tbl") 
	  }
	}
	#irred1 = froot//" SKY="//skyfil[k]//" FLAT="//flatstrat//" MEDIAN SUB: "//submed
	irred1 = froot//" SKY="//skyfil[k]//" FLAT="//flatstrat//" CLEAN: "//clean
	print(irred1, >> froot//"-canica.log")
	}
	}
	irred = froot//" SKY="//froot//"_sky"//skytype//"] FLAT="//flatstrat 
        for (i=1; i <= nobj; i +=1) {
	  imdelete(cm[i]//suffix2, ver-, >& "dev$null")
	}

#------------------------------------------------------------------------
#	Subtract the interchip horizontal band if asked to do so
#------------------------------------------------------------------------

#if (submed) {
# 
#        tmpobjsfb = mktemp ("tmp$ia_tmp.")
#        for (i=1; i <= nobj; i +=1) 
#          print(cm[nob[i]]//"sfm",  >> tmpobjsfb)
#
#print ("Subtracting a 1023x1 median image by itself to remove the horizontal bands")
# 
#        imdelete("@"//tmpobjsfb, ver-, >& "dev$null")
# 
#median ("@"//tmpobjsf, "@"//tmpobjsfb, 1023, 1, zloreject=INDEF,
#zhireject=INDEF, boundary="wrap", constant=0., verbose=yes)
# 
#imarith ("@"//tmpobjsf, "-", "@"//tmpobjsfb, "@"//tmpobjsf,
#title="sky subtracted ", divzero=0., hparams="",
#pixtype="", calctype="", verbose=no, noact=no)
###title="sky subtracted "//skyfile, divzero=0., hparams="",
# 
#        imdelete("@"//tmpobjsfb, ver-, >& "dev$null")
#        delete (tmpobjsfb, ver-, >& "dev$null")
# 
#}
	if (seq == "preview") {
	#ndisp = nobj
	#if (disp == no)ndisp = 2
print("Displaying images for examining --- ")
print("-----to write xy pixels of a star, put the cursor on a star and type a ")
print("-----to quit, type q ")
	#imexamin ("@"//tmpobjsf, 1)
          for (i=1; i <= nobj; i +=1) {
		if (ntv == 0){itv = mod(i,4)};if (itv == 0){itv = 4}
                #display (cm[nob[i]]//"sf", zs+, fill+)
                display (cm[nob[i]]//suffix3, itv, zs+, fill+)
		imexam (logfile="imexam.log", keeplog+)
	  }
	goto exit
	}

if (clean) {
	print("Cleaning the images for stellar -ve residuals --- ")
        imdelete ("tclean", ver-, >& "dev$null")
          for (i=1; i <= nobj; i +=1) {
        #canclean (cm[nob[i]]//suffix3, "tclean", xlef=1, xrig=512, ybot=1, ytop=512)
        canhole (cm[nob[i]]//suffix3, "tclean")
        imdelete (cm[nob[i]]//suffix3, ver-, >& "dev$null")
        imrename ("tclean", cm[nob[i]]//suffix3, ver-)
					}
		}

#------------------------------------------------------------------------
#	combine the aligned images
#------------------------------------------------------------------------
 
if (immos) {
 
          shiftfil = mktemp ("tmp$ia_tmp.")
 
        #---------------------------------------------------------------
	#Combine step 1: display sky substracted images and examine
        #---------------------------------------------------------------

        tmpal = mktemp ("tmp$ia_tmp.")
        tmpalb = mktemp ("tmp$ia_tmp.")
        if (disp) {
          print("Check the image quality/sequence before stacking")
          tmpobjsfa = mktemp ("tmp$ia_tmp.")
 
          ans = "yes"
 
          for (i=1; i <= nobj; i +=1) {
              #display (cm[nob[i]]//"sf", zs+, fill+)
		if (ntv == 0){itv = mod(i,4)};if (itv == 0){itv = 4}
              display (cm[nob[i]]//suffix3, itv, zs+, fill+)
 
        #print ("Is the image ",cm[nob[i]]//"sf", " OK? [yes,no] [",ans,"] ")
        print ("Is the image ",cm[nob[i]]//suffix3, " OK? [yes,no] [",ans,"] ")
            scan (ans)
 
            if (ans=="no") {
                #imdelete(cm[nob[i]]//"sf", ver-, >& "dev$null")
                imdelete(cm[nob[i]]//suffix3, ver-, >& "dev$null")
                } else {
                m = i
#              if (ans=="pos") {
#	print ("So there is positioning error. To which position of the ")
#	print ("sequence the image ",m,"=",cm[nob[i]]//"sf corresponds?")
                #print ("On-field position number for ",m,"=",cm[nob[i]]//"sf")
                #print ("[to assign shifts]")
#                scan (m)
#                }   #ans==pos endif
                #print(cm[nob[i]]//"sf",  >> tmpobjsfa)
                #print("rg"//cm[nob[i]]//"sf",  >> tmpal)
                #print("rg"//cm[nob[i]]//"sf.mag.1",  >> tmpalb)
                print(cm[nob[i]]//suffix3,  >> tmpobjsfa)
                print("rg"//cm[nob[i]]//suffix3,  >> tmpal)
                print("rg"//cm[nob[i]]//suffix3//".mag.1",  >> tmpalb)
                print(xdif[nob[m]]," ",ydif[nob[m]], >> shiftfil)
            }   #ans==no endif
	    }   #end for for all objects

        delete (tmpobjsf, ver-, >& "dev$null")
        tmpobjsf = tmpobjsfa
 
        } else
          for (i=1; i <= nobj; i +=1) {
                print(xdif[nob[i]]," ",ydif[nob[i]], >> shiftfil)
                #print("rg"//cm[nob[i]]//"sf",  >> tmpal)
                #print("rg"//cm[nob[i]]//"sf.mag.1",  >> tmpalb)
                print("rg"//cm[nob[i]]//suffix3,  >> tmpal)
                print("rg"//cm[nob[i]]//suffix3//".mag.1",  >> tmpalb)
          } #end display
 
        #imdelete ("tclean", ver-, >& "dev$null")
          for (i=1; i <= nobj; i +=1) {
	#canclean (cm[nob[i]]//suffix3, "tclean", xl=1, xr=512, yb=1, yt=512, ver-)
	#imdelete (cm[nob[i]]//suffix3, ver-, >& "dev$null")
	#imrename ("tclean", cm[nob[i]]//suffix3, ver-)
printf("%c %8.3f %8.3f \n", cm[nob[i]]//suffix3//" ", xdif[nob[i]],ydif[nob[i]]) | cat >> froot//"-canica.log")
	}

        #---------------------------------------------------------------
	#Combine step 2: align the images
        #---------------------------------------------------------------

        print("Aligning images ")
        #type (tmpobjsf)
 
        #type (shiftfil)
 
        imdelete("@"//tmpal, ver-, >& "dev$null")

	delete("imalign.log", ver-, >& "dev$null")
 
#imalign ("@"//tmpobjsf, refim, coofil, "@"//tmpal, 
cimalign ("@"//tmpobjsf, refim, coofil, "@"//tmpal, find_centroid, 
shifts=shiftfil, boxsize=boxsize, bigbox=bigbox, negative=no, background=INDEF, 
lower=INDEF, upper=INDEF, niterate=3, tolerance=0, shiftimages=yes, 
interp_type="linear", boundary_typ="constant", consta=0., trimimages=no, 
verbose=yes, list="") 

#        imdelete("@"//tmpal, ver-, >& "dev$null")
#
#cimalign ("@"//tmpobjsf, refim, coofil, "@"//tmpal, find_centroid, 
#shifts=shiftfil, boxsize=boxsize, bigbox=bigbox, negative=no, background=INDEF, 
#lower=INDEF, upper=INDEF, niterate=3, tolerance=0, shiftimages=yes, 
#interp_type="linear", boundary_typ="constant", consta=0., trimimages=no, 
#verbose=yes, list="") | cat > "imalign.log"
 
#	type ("imalign.log") | cat >> froot//"-canica.log"

        imdelete(mosfil, ver-, >& "dev$null")
 
        #---------------------------------------------------------------
	#Combine step 3: align the images
        #---------------------------------------------------------------

        print("Combining the aligned images ")
        type (tmpal)
 
#imcombine ("@"//tmpal, mosfil, combine="median", zero="median", logfile=logfil)
imcombine ("@"//tmpal, mosfil, combine="average", scale="none", zero="median", reject="minmax", nlow=1, nhigh=1, logfile=logfil)

        imdelete("@"//tmpobjsf, ver-, >& "dev$null")
}
        delete (tmpobjsf, ver-, >& "dev$null")
 
#------------------------------------------------------------------------
#	Do photometry of the reference star(s) is asked to do so
#------------------------------------------------------------------------

 if (immos) {

if (dophot) {
 
        print("Doing aperture photometry on aligned images")
 
        print(mosfil,  >> tmpal)
        print(mosfil//".mag.1",  >> tmpalb)
        delete ("@"//tmpalb, ver-, >& "dev$null")
 
	exptime = 1.0
	#if (ra_key == "RA") exptime=cexptime    #New version with total counts

phot ("@"//tmpal, "", coords=xyfil, output="default", plotfile="",
scale=1.0, fwhmpsf=5.0, epadu=10.0, readnoise=3.0, filter="FILTER2",
exposur="", airmass="", xairmas=1.0, itime=exptime, salgorithm="median",
annulus =20, dannulus =10, cbox=5, calgorithm="centroid",
apertures="5,10,14,17,20", zmag=zpmag, interactive=no, radplots=no, 
verify=no, update=no, verbose=no, graphics="stdgraph", display="stdimage", 
icommands="", gcommands="")
 
txdump ("@"//tmpalb,
"MERR[4],MAG[2-5],XCEN,YCEN,IFIL,IMAGE", "ID=1", headers=no,
parameters=yes) | cat >> refim//".mag"
 
txdump ("@"//tmpalb, "MERR[4],MAG[2-5],XCEN,YCEN,IFIL,IMAGE",
"ID=1", headers=no, parameters=yes)
 
txdump ("@"//tmpalb,
"MERR[4],MAG[2-5],XCEN,YCEN,IFIL,IMAGE", "ID=2", headers=no,
parameters=yes) | cat >> refim//".mag"
 
txdump ("@"//tmpalb, "MERR[4],MAG[2-5],XCEN,YCEN,IFIL,IMAGE",
"ID=2", headers=no, parameters=yes)
 
txdump ("@"//tmpalb,
"MERR[4],MAG[2-5],XCEN,YCEN,IFIL,IMAGE", "ID=3", headers=no,
parameters=yes) | cat >> refim//".mag"
 
txdump ("@"//tmpalb, "MERR[4],MAG[2-5],XCEN,YCEN,IFIL,IMAGE",
"ID=3", headers=no, parameters=yes)
 
txdump ("@"//tmpalb,
"MERR[4],MAG[2-5],XCEN,YCEN,IFIL,IMAGE", "ID=4", headers=no,
parameters=yes) | cat >> refim//".mag"

txdump ("@"//tmpalb, "MERR[4],MAG[2-5],XCEN,YCEN,IFIL,IMAGE",
"ID=4", headers=no, parameters=yes)
 
txdump ("@"//tmpalb,
"MERR[4],MAG[2-5],XCEN,YCEN,IFIL,IMAGE", "ID=5", headers=no,
parameters=yes) | cat >> refim//".mag"

txdump ("@"//tmpalb, "MERR[4],MAG[2-5],XCEN,YCEN,IFIL,IMAGE",
"ID=5", headers=no, parameters=yes)

        delete ("phot_graph.mag", ver-, >& "dev$null")
txdump ("@"//tmpalb, "MAG[3]", "ID=1", >> "phot_graph.mag") 
cat "phot_graph.mag" | scan(ymax)
ymin = ymax - 0.25; ymax = ymax + 0.25; nobj = nobj + 2
graph ("phot_graph.mag", wx1=0, wx2=nobj, wy1=ymin, wy2=ymax, pointmo+, marker="box", szmarke=0.01, xlabel="image number", ylabel="mag")
 
print (" ----------------- ", >> refim//".mag")

#copy (mosfil//".mag.1", mosfil//".mag")
delete ("@"//tmpalb, ver-, >& "dev$null")

} #phot

        delete (tmpal, ver-, >& "dev$null")
        if (immos) delete (tmpalb, ver-, >& "dev$null")
        delete (shiftfil, ver-, >& "dev$null")
        #delete (dummy, ver-, >& "dev$null")

    #hedit (mosfil, fields="COMMENT1", value=irred1, add+, delete-, verify-, show-, update+)
    hedit (mosfil, fields="ZEROPT", value=zpmag, add+, delete-, verify-, show-, update+)
    hedit (mosfil, fields="REDUCED", value=version , add+, delete-, verify-, show-, update+)
    hedit (mosfil, fields="RED_PAR1", value=irred, add+, delete-, verify-, show-, update+)
    if (find_centroid) {
    hedit (mosfil, fields="RED_PAR2", value="CENTROID found before aligning", add+, delete-, verify-, show-, update+)
	} else {
    hedit (mosfil, fields="RED_PAR2", value="Images aligned using SEQ file shifts", add+, delete-, verify-, show-, update+)
	}
    hedit (mosfil, fields="WAT0_001", add-, delete+, verify-, show-, update+)
    hedit (mosfil, fields="WAT1_001", add-, delete+, verify-, show-, update+)
    hedit (mosfil, fields="WAT2_001", add-, delete+, verify-, show-, update+)

    hedit (mosfil, fields="CTYPE1", value="RA---SIN", add+, delete-, verify-, show-, update+)
    hedit (mosfil, fields="CTYPE2", value="DEC--SIN", add+, delete-, verify-, show-, update+)
    hedit (mosfil, fields="CRVAL1", value=raval, add+, delete-, verify-, show-, update+)
    hedit (mosfil, fields="CRVAL2", value=decval, add+, delete-, verify-, show-, update+)
    hedit (mosfil, fields="CRPIX1", value=xradec, add+, delete-, verify-, show-, update+)
    hedit (mosfil, fields="CRPIX2", value=yradec, add+, delete-, verify-, show-, update+)
    hedit (mosfil, fields="CDELT1", value=rapix, add+, delete-, verify-, show-, update+)
    hedit (mosfil, fields="CDELT2", value=decpix, add+, delete-, verify-, show-, update+)
    hedit (mosfil, fields="EQUINOX", value=obsyr, add+, delete-, verify-, show-, update+)
    #hedit (mosfil, fields="", value="", add+, delete-, verify-, show-, update+)

if (dowcs) {
		if (itv == 0){itv = 4}
        imdelete(mosfil//"a", ver-, >& "dev$null")
        imgets (mosfil, "i_naxis1")
        nx = int(imgets.value)
        imgets (mosfil, "i_naxis2")
        ny = int(imgets.value)
	xradec = (nx - xradec + 1)/mag
	yradec = yradec/mag
	nx = nx/mag
	ny = ny/mag
	imlintran (mosfil, mosfil//"a", xrotatio=180.0, yrotatio=0.0, xmag=mag, ymag=mag, ncols=nx, nlines=ny)
    hedit (mosfil//"a", fields="CRPIX1", value=xradec, add+, delete-, verify-, show-, update+)
    hedit (mosfil//"a", fields="CRPIX2", value=yradec, add+, delete-, verify-, show-, update+)
	wlpars.axis1_title = "RA ("//str(obsyr)//")"
	wlpars.axis2_title = "Dec ("//str(obsyr)//")"
	display (mosfil//"a", itv, fill-, zs+ , xmag=0.75, ymag=0.75)
	wcslab (mosfil//"a", itv, usewcs-)
	display (mosfil//"a", itv, fill-, zs+ , xmag=0.75, ymag=0.75)
	wcslab (mosfil//"a", itv, usewcs-)
}
} #immos

exit:
        canobj.root= froot
        canobj.imbeg=im1
        canobj.imend=im1 + npass - 1
	canobj.xshi = 0.0
	canobj.yshi = 0.0
	canobj.rakey = ra_key
	#canobj.crval1   = 0.0
	#canobj.crval2   = 0.0
	#canobj.epoch    = 2000.0
	#canobj.iref = 1

        print("***DONE***")
 
end
