procedure catimreg(image, catal)

#digiphot apphot stsdas

#   1. read the reference catalog: 2mass/gsc   (ra and dec)
#   2. optionally display the reference stars on a "REFERENCE" image
#   3. run "daofind" on the input image
#   4. sort the daophot catalog
#   5. optionally display the detected stars on the input image
#   6. match stars using ccxymatch
#   7. transform and update the header of the input image
#   8. optionally display the reference stars on the "INPUT" image
#   9. run "phot" on the sorted daophot catalog
#   10. plot (mag_dao - mag_dao[1]) vs mag_dao for all the daophot stars
#   11. Also plot (mag_dao - mag_ref) vs mag_ref for all the daophot stars

#This task reads a 2MASS catalog and associates the catalog RA, DEC and MAG 
#to a star identified interactively on a CANICA image
#The first 11 coulms of the 2MASS catalog is assumed as follows (without the begining #).  
#All lines stating with "#" are ignored
#It is also assumed that the "calibration star" is among the first "nmark" stars. 
#_RAJ2000        _DEJ2000        RAJ2000          DEJ2000          2MASS                  Jmag   e_Jmag  Hmag   e_Hmag   Kmag    e_Kmag  
#--------        --------        -------          -------           -----                 ----    -----  ----   ------   ----    ------  
#149.253196      +68.901779      149.253196      +68.901779      09570076+6854064         7.413   0.019  6.911   0.018   6.801   0.017 
#148.767531      +68.901566      148.767531      +68.901566      09550420+6854056         7.592   0.020  7.392   0.020   7.342   0.016
#148.323737      +69.046722      148.323737      +69.046722      09531769+6902481         8.327   0.026  7.840   0.057   7.646   9.998
#Tips to use
#The script allows you to display a 2MASS image (previously downloaded) and displays the catalog stars on that.
#Note the identification number of the "calibration star"
#Identify the corresponding star on the CANICA image
#The RADEC coordinates as well as the magnitude of the "calibration star" are used
#The CRVAL1, CRVAL2, CRPIX1, and CRPIX2 as well as ZEROMAG are written into the image header.
#The default value of CDELT1 and CDELT2 correspond to the CANICA images (East to the right of North).
#
string  image  = "sHa3141"              {prompt="Image Name"}
string  catal  = "ds9_2mass.cat"      {prompt="Star catalog name"}
string  catbase= "2MASS"      		{prompt="Database of catalog [2MASS/GSC/SDSS]"}
int     nmark  = 120                  {prompt="Number of stars to mark"}
string  starlis= "daofind"      {prompt="daofind/Sextractor/file name"}
bool    sextract = no        {prompt="Use Sextractor for star search?[if NO uses daofind]"}
real    datamaxim   = 20000.0   {prompt="Maximum good data"}
real    kclip       = 20.0    {prompt="k-sigma clipping "}
real    fwhm        = 5.5     {prompt="FWHM of the images"}
#real    zeromag     = 0.0     {prompt="Zeropoint"}
string  imsect = "[*,*]"      {prompt="Image section to search stars using daofind"}
bool    display     = yes     {prompt="Display the identified stars in each image?"}
bool    fitint      = no      {prompt="Fit coordinates [cctmap] interactively?"}
bool    wcsupdate   = yes     {prompt="Update the image headers with WCS?"}
real    optxcen     = 0.0    {prompt="Optical X center of the image"}
real    optycen     = 0.0    {prompt="Optical Y center of the image"}
string  plotx	    = "mag"      {prompt="X-axis of magplot [mag/rad/jhcol/jkcol]"}
#real    cdelt1   = "8.974722e-5"   {prompt="Pixel scale in RA (in degrees)"}
#real    cdelt2   = "8.974722e-5"   {prompt="Pixel scale in Dec (in degrees)"}


begin

	string	imfil, catfil, c1, regfil, filt, creg,cid[9999], starlist
	int	i1, i2, i3, imark, sid, id[9999],twomid1,twomid2, nstar
	real    ra,dec,ra2,dec2,jmag,ejmag,hmag,ehmag,kmag,ekmag, rad
	real    a_ra[9999],a_dec[9999],a_jmag[9999],a_hmag[9999],a_kmag[9999]
        real    rapix,decpix,xx, yy, ddd, amag,inmag[9999],apmag,efftime
	string	radecfil,radecmagfil,regmagfil, obsdate, filthd, plotfil
        string  daocoo,daomag,photmag,ccxymat,ccmapdb,ccmapres,logfile
        string  sexcmd, cattype, instru, imsec,plottyp
        real    skymed, skyrms, datamax, ksigma, fwhmpsf, zpmag, grad, xc, yc
	real	raref,decref,xin,yin,mag_dif,exptime, nadd, refmag, jkcol, jhcol, raddist
	int     itv, nx, ny, nref,nin
        bool    disp, sextractor, update, inter

	struct  line

	cattype = catbase    #"GSC"   #"2MASS"  #
	plottyp = plotx; inter = fitint
	if (cattype != "2MASS") {cattype = "GSC"}
	imfil = image
	#filt  = filter
	catfil = catal
	imark = nmark
	imsec = imsect      #"[*,*]"
        datamax = datamaxim; ksigma = kclip; fwhmpsf = fwhm; zpmag = 0.0
	xc = optxcen; yc = optycen
	starlist = starlis

        disp = display; sextractor=sextract; update = wcsupdate
	radecfil = imfil//"_cat.radec"
	radecmagfil = imfil//"_radec_mag.res"
	rad = 3.0
	regfil = radecfil//".reg"
	plotfil = imfil//"_plot_zeropt.mag"
	regmagfil = imfil//"_radec_mag.reg"
	delete(radecfil, ver-, >& "dev$null")
	delete(radecmagfil, ver-, >& "dev$null")
	delete(regfil, ver-, >& "dev$null")
	delete(regmagfil, ver-, >& "dev$null")
	delete(plotfil, ver-, >& "dev$null")

	print("# Region file format: DS9 version 4.0", >> regfil)
	print("# Filename: ", imfil, >> regfil)
	print("global color=green font=\"helvetica 10 normal\" select=1 highlite=1 edit=1 move=1 delete=1 include=1 fixed=0 source \nfk5", >> regfil)

	i1 = 0 
	i2 = 0 
	i3 = 0 
	list = catfil
        while( fscan(list,line) != EOF ) {
	c1 = substr(line,1,3)
	if (c1=="---") {
        while( fscan(list,line) != EOF ) {
	if (cattype == "2MASS") {
        i2 = fscan(line,ra,dec,ra2,dec2,twomid1,jmag,ejmag,hmag,ehmag,kmag,ekmag) 
	  i3 = i3 + 1
	  cid[i3] = str(i3)
	  a_ra[i3] = ra; a_dec[i3] = dec; a_jmag[i3] = jmag; a_hmag[i3] = hmag; a_kmag[i3] = kmag
	#printf("%12.7f %12.7f %8.3f %8.3f %8.3f %s\n",a_ra[i3],a_dec[i3],a_jmag[i3],a_hmag[i3],a_kmag[i3],cid[i3], >> radecfil)
	printf("%15.3H %15.2h %8.3f %8.3f %8.3f %s\n",a_ra[i3],a_dec[i3],a_jmag[i3],a_hmag[i3],a_kmag[i3],cid[i3], >> radecfil)
	}
	if (cattype == "GSC") {
        i2 = fscan(line,ra,dec) 
	  i3 = i3 + 1
	  cid[i3] = str(i3)
	  a_ra[i3] = ra; a_dec[i3] = dec
	#printf("%12.7f %12.7f  %s\n",a_ra[i3],a_dec[i3], cid[i3], >> radecfil)
	printf("%15.3H %15.2h  %s\n",a_ra[i3],a_dec[i3], cid[i3], >> radecfil)
	}

	if (i3 <= imark) {
	creg = "# color=red text={"//cid[i3]//"}"
	printf("circle(%10f,%10f,%4f\") %s\n",a_ra[i3],a_dec[i3],rad, creg, >> regfil)
	}
	}
	}
	}

	instru = "LFOSC"
	if (instru == "LFOSC") {
	obsdate = "2009-11-08T12:30:52"
	efftime = 60.0
	goto skip_lfosc
	}
	#hselect (imfil, "INSTRUME", yes) | scan (instru)
        hselect (imfil, "INSTRUME DATE-OBS", yes) | scan (instru, obsdate)
        #instru = "OSIRIS"
        c1 = substr(obsdate,1,4)
	if (instru == "CANICA"){
        if (int(c1) >= 2005) {
        print (obsdate, " ", c1)
        hselect (imfil, "i_naxis1, i_naxis2, EXPTIME, COADDS, FILTER2, DATE-OBS", yes) | scan (nx, ny, exptime, nadd, filthd, obsdate)
        } else {
        hselect (imfil, "i_naxis1, i_naxis2, FILTER2, DATE-OBS", yes) | scan (nx, ny, filthd, obsdate)
        nadd = 1; exptime = 1.0
        }
	#hselect (imfil, "i_naxis1, i_naxis2, EXPTIME, COADDS, FILTER2, DATE-OBS", yes) | scan (nx, ny, exptime, nadd, filthd, obsdate) 
	if (xc == 0.0 || yc == 0.0) {xc = real(nx)/2.0; yc = real(ny)/2.0}
        if (xc < 0.0 || xc > real(nx)) xc = real(nx)/2.0
        if (yc < 0.0 || yc > real(nx)) yc = real(ny)/2.0
	#xc = real(nx)/2.0; yc = real(ny)/2.0; grad = 0.45*min(nx, ny)
        if (filthd == "1"){filt = "J"} else if (filthd == "2") {filt = "H"} 
                                       else if (filthd == "3") {filt = "K"} else { filt = "X" }
	efftime = exptime*nadd
	print("Image size in ",imfil," is =",nx, ny, exptime, nadd, filthd,"=",filt, " ", obsdate)
	}
	if (instru == "OSIRIS"){
	#imsec = "[1:1950,1:4000]"
	#imsec = "[501:1950,10:4000]"
	#hselect (imfil, "i_naxis1, i_naxis2, EXPTIME, DATE", yes) | scan (nx, ny, exptime, obsdate) 
	}

	skip_lfosc:
        daocoo = imfil//"_dao.coo"; delete(daocoo, ver-, >& "dev$null")
        daomag = imfil//"_dao.mag"; delete(daomag, ver-, >& "dev$null")
        photmag = imfil//"_phot.mag"; delete(photmag, ver-, >& "dev$null")
        ccxymat = imfil//"_ccxy.mat"; delete(ccxymat, ver-, >& "dev$null")
        ccmapdb = imfil//"_ccmap.db"; delete(ccmapdb, ver-, >& "dev$null")
        ccmapres = imfil//"_ccmap.res"; delete(ccmapres, ver-, >& "dev$null")

        imstat (imfil, fields="midpt,stddev", nclip=10, lsigma=3, usigma=3, format- ) | scan (skymed,skyrms)
	print("Sky median and rms in ",imfil," is =",skymed,skyrms)

	if (starlist == "daofind"){
        daofind(imfil//imsec, output=daocoo, fwhmpsf=fwhmpsf, sigma=skyrms, datamin=skymed,
                datamax=datamax, thresho=ksigma, verify-, int-, >& "dev$null")
                #datamax=datamax, thresho=ksigma, verify-, int-)
        txdump (daocoo, "XCENTER YCENTER MAG SHARPNESS SROUND GROUND ID", "SHARPNESS!=INDEF && SROUND>0.1", headers+, paramet+, >> daomag)
        txdump (daocoo, "XCENTER YCENTER MAG SHARPNESS SROUND GROUND ID", "SHARPNESS!=INDEF && SROUND>0.1", headers+, paramet+) | wc
	delete(daocoo, ver-, >& "dev$null")
	rename (daomag, daocoo)
                #scale=1.0, fwhmpsf=fwhmpsf, epadu=10.0, readnoise=3.0, filter="FILTER2",
	} else {
	daocoo = starlist
	}
        phot (imfil, skyfile="", coords=daocoo, output=photmag, plotfile="",
                scale=1.0, fwhmpsf=fwhmpsf, epadu=10.0, readnoise=3.0, filter="",
                exposur="", airmass="", xairmas=1.0, itime=1.0, salgorithm="median",
                annulus =20, dannulus =10, cbox=5, calgorithm="centroid",
                apertures="5,10,14,17,20", zmag=zpmag, interactive=no, radplots=no,
                verify=no, update=no, verbose=no, graphics="stdgraph", display="stdimage",
                icommands="", gcommands="")

        #delete(daocoo, ver-, >& "dev$null")
        txsort (photmag, field="MAG[3]", ascend+)
        txdump (photmag, "MAG[3]", "MAG[3]!=INDEF") | wc | scan (nstar)
	print("Number of stars identified in ",imfil," by DAOPHOT=",nstar)
        txdump (photmag, "XCENTER YCENTER MAG[3]  ID", "MAG[3]!=INDEF && MAG[2]!=INDEF && MAG[1]!=INDEF", headers-, paramet-, >> daomag)
        #txdump (photmag, "XCENTER YCENTER MAG[3]  ID", "MAG[3]!=INDEF && MAG[2]!=INDEF && MAG[1]!=INDEF && MAG[3]<-2.5*log10(sqrt(615.44)*10.0*STDEV)", headers-, paramet-, >> daomag)
        #txdump (photmag, "XCENTER YCENTER MAG[3]  ID", "MAG[3]!=INDEF && MAG[3]<-2.5*log10(sqrt(615.44)*10.0*STDEV)", headers-, paramet-, >> daomag)
	#print (obsdate)
	itv = 1
        if (disp){
                display (imfil, itv, zs+, fill+, >& "dev$null")
                tvmark (itv, daomag, mark="circle", radii=10, number+, nxoff=3, nyoff=3, txsize=2)
                }


#ccxymatch (daomag,
#radecfil, ccxymat, 1., 8., refpoints="", xin=537.22, yin=549.694,
#xmag=1.00, ymag=1.00, xrotation=90.0, yrotation=270.0, projection="tnx",
#lngref=07:21:33.366, latref=71:19:21.06, lngcolumn=1, latcolumn=2, xcolumn=1, ycolumn=2,
#lngunits="hours", latunits="degrees", separation=3., pseparation=9.,
#matching="triangles", nmatch=imark, ratio=10., nreject=50, lngformat="",
#latformat="", xformat="%13.3f", yformat="%13.3f", verbose=yes)

ccxymatch (daomag,
radecfil, ccxymat, 1., 8., refpoints="", xin=INDEF, yin=INDEF,
xmag=INDEF, ymag=INDEF, xrotation=INDEF, yrotation=INDEF, projection="tnx",
lngref=INDEF, latref=INDEF, lngcolumn=1, latcolumn=2, xcolumn=1, ycolumn=2,
lngunits="hours", latunits="degrees", separation=3., pseparation=9.,
matching="triangles", nmatch=imark, ratio=10., nreject=50, lngformat="",
latformat="", xformat="%13.3f", yformat="%13.3f", verbose=yes)


ccmap (ccxymat,
"ccmap.db", solutions="", images=imfil, results=ccmapres, xcolumn=3, ycolumn=4,
lngcolumn=1, latcolumn=2, xmin=INDEF, xmax=INDEF, ymin=INDEF, ymax=INDEF,
lngunits="", latunits="", insystem="j2000", refpoint="user", lngref="INDEF",
latref="INDEF", refsystem="INDEF", lngrefunits="", latrefunits="",
projection="tnx", fitgeometry="general", function="polynomial", xxorder=3,
xyorder=3, xxterms="half", yxorder=3, yyorder=3, yxterms="half", maxiter=0,
reject=3., update=update, pixsystem="logical", verbose=no , interactive=inter,
graphics="stdgraph", cursor="")


	if (instru == "CANICA"){
  #-------------------------
	print("# Region file format: DS9 version 4.0", >> regmagfil)
	print("# Filename: ", imfil, >> regmagfil)
	print("global color=green font=\"helvetica 10 normal\" select=1 highlite=1 edit=1 move=1 delete=1 include=1 fixed=0 source \nfk5", >> regmagfil)
	#first read daomag file

        i1 = 0; i2 = 0; i3 = 0; list = daomag
        while( fscan(list,line) != EOF ) {
        i2 = fscan(line,xin,yin,apmag)
          i3 = i3 + 1
	inmag[i3] = apmag + 2.5*log10(efftime)
        }

  #-------------------------
        i1 = 0; i2 = 0; i3 = 0
        list = ccxymat
        for (i=1; i <= 15; i +=1) { i2 = fscan(list,line) }
        c1 = substr(line,1,1)
	if (c1 == "#") {for (i=1; i <= 5; i +=1) { i2 = fscan(list,line) }}
        #while( fscan(list,line) != EOF ) {
        #c1 = substr(line,22,25)
        #if (c1=="line") {
        #i2 = fscan(list,line)

        while( fscan(list,line) != EOF ) {
	#print(line)
        i2 = fscan(line,raref,decref,xin,yin,nref,nin)
          i3 = i3 + 1
	if (filt == "J") {refmag = a_jmag[nref]}
	if (filt == "H") {refmag = a_hmag[nref]}
	if (filt == "K") {refmag = a_kmag[nref]}
	mag_dif = refmag - inmag[nin]
	jkcol = a_jmag[nref] - a_kmag[nref]
        jhcol = a_jmag[nref] - a_hmag[nref]
        raddist = sqrt((xin - xc)**2 + (yin - yc)**2)
        creg = "# color=green text={"//str(nref)//"}"
        printf("circle(%10f,%10f,%4f\") %s\n",a_ra[nref], a_dec[nref], rad, creg, >> regmagfil)
        printf("%15.3H %15.2h %8.3f %8.3f %8.3f %8.3f %3d %3d %8.3f %8.3f %8.3f\n",a_ra[nref], a_dec[nref],a_jmag[nref],jhcol,jkcol,mag_dif,nref,nin, xin,yin,raddist, >> radecmagfil)
        #printf("%15.3H %15.2h %8.3f %8.3f %8.3f %8.3f %d %d\n",a_ra[nref], a_dec[nref],a_jmag[nref],a_hmag[nref],a_kmag[nref],mag_dif,nref,nin, >> radecmagfil)
	if (plottyp=="mag") {printf("%8.3f %8.3f %8.3f \n", refmag, mag_dif, jkcol, >> plotfil)}
	if (plottyp=="rad") {printf("%8.3f %8.3f %8.3f \n", raddist, mag_dif, jkcol, >> plotfil)}
	if (plottyp=="jhcol") {printf("%8.3f %8.3f  \n", jhcol, mag_dif, >> plotfil)}
	if (plottyp=="jkcol") {printf("%8.3f %8.3f  \n", jkcol, mag_dif, >> plotfil)}
	#printf("%8.3f %8.3f \n", jkcol, mag_dif, >> plotfil)
        #}
        #}
        }

graph (plotfil, wx1=0., wx2=0., wy1=0., wy2=0., pointmo+, marker="box", szmarke=0.01, xlabel="Reference MAG", ylabel="ZEROPOINT")
	}

	end
