## frei3.cl: A task to obtain azimuthally averaged radial profiles of optical
## images of galaxies. The center, PA and ellipticity can be fed using epar.
## Uses stsdas.ellipse. Before ellipse is run, the counts are converted into
## standard magnitude scale using the zeropoints in the header. Subtracts
## sky value taken from the header. The output results are written into a
## file named (ngc)#pr (eg 2903pr), which becomes the input file to "tdgrad"
# 
##Last Modified: 11 July 1995
#

procedure frei3(ngcno)

string  ngcno                           {prompt="NGC no of the galaxy"}
string  band   =  "g"                   {prompt="Band Name"}
real    x0     =  "360."                {prompt="Image x_center"}
real    y0     =  "360."                {prompt="Image y_center"}
#real    sigm   =  "1.50"                {prompt="Sigma of smoothing gaussian"}
real    sigm   =  "1.00"                {prompt="Sigma of smoothing gaussian"}
real    nsigm  =  "4.00"                {prompt="Sigma of smoothing gaussian"}
real    eps0   =  "0.15"                {prompt="Trial ellipticity"}
real    teta0  =  "30.0"                {prompt="Trial position angle"}
real    a0     =  "5.0"                 {prompt="Starting major axis"}
real    mina   =  "5.0"                 {prompt="Minimum major axis"}
real    maxa   =  "300."                {prompt="Minimum major axis"}
real    astep  =  "10.0"                {prompt="Interval along major axis"}
bool    hcen   =  "yes"                 {prompt="Hold Center"}
bool    heps   =  "yes"                 {prompt="Hold ellipticity"}
bool    hteta  =  "yes"                 {prompt="Hold Position angle"}

begin

string    tele, filt, gal1, ok, ngc, gal
real      const, skyval, const1, scale, zeromag, sigma, dnatobv, nsigma
int  blksiz

sigma = sigm
nsigma = nsigm
ok = "yes"
ngc  = ngcno
gal = "p"//ngc//band

	if (ngc == "2403") {
	  eps0  = 1.0-0.562
	  teta0 = -53.0 }
	else if (ngc == "4258") {
	  eps0  = 1.0-0.389
	  teta0 = -30.0 }
	else if (ngc == "628") {
	  eps0  = 1.0-0.912
	  teta0 =  25.0  }
	else if (ngc == "2841") {
	  eps0  = 1.0-0.437
	  teta0 = -33.0  }
	else if (ngc == "3627") {
	  eps0  = 1.0-0.457
	  teta0 = -7.0  }
	else if (ngc == "4736") {
	  eps0  = 1.0-0.813
	  teta0 = -75.0  }
	else if (ngc == "6503") {
	  eps0  = 1.0-0.339
	  teta0 = -57.0  }
	else if (ngc == "7331") {
	  eps0  = 1.0-0.355
	  teta0 = -9.0  }
	else if (ngc == "5033") {
	  eps0  = 1.0-0.468
	  teta0 = -10.0  }
	else if (ngc == "4192") {
	  eps0  = 1.0-0.282
	  teta0 = -25.0  }
	else if (ngc == "4254") {
	  eps0  = 1.0-0.871
	  teta0 =  0.0  }
	else if (ngc == "4501") {
	  eps0  = 1.0-0.537
	  teta0 = -40.0  }
	else if (ngc == "4535") {
	  eps0  = 1.0-0.708
	  teta0 = 0.0  }
	else if (ngc == "4569") {
	  eps0  = 1.0-0.457
	  teta0 = 23.0  }
	else if (ngc == "5055") {
	  eps0  = 1.0-0.575
	  teta0 = -75.0  }
	else if (ngc == "2903") {
	  eps0  = 1.0-0.479
	  teta0 =  17.0  }
	else if (ngc == "3079") {
	  eps0  = 1.0-0.180
	  teta0 = -15.0  }
	else if (ngc == "3198") {
	  eps0  = 1.0-0.390
	  teta0 =  35.0  }
	else if (ngc == "3628") {
	  eps0  = 1.0-0.200
	  teta0 = -76.0  }
	else if (ngc == "4303") {
	  eps0  = 1.0-0.890
	  teta0 =   0.0  }
	else if (ngc == "4321") {
	  eps0  = 1.0-0.850
	  teta0 =  30.0  }
	else if (ngc == "4656") {
	  eps0  = 1.0-0.200
	  teta0 =  33.0  }
	else if (ngc == "6946") {
	  eps0  = 1.0-0.850
	  teta0 =  32.0  }
	else {
	  print("x0 y0 eps0 teta0 ",x0,y0,eps0,teta0 )
	  print ("Ellipse parameters ok? (yes|no) ") 
	  scan (ok)
	  if (ok == "no")goto exit
	}

imgets (gal, "TELESCOP")   
tele = (imgets.value)
imgets (gal, "FILTER1")   
filt = (imgets.value)
imgets (gal, "SKY")   
 = (imgets.value)
skyval = real(imgets.value)
imgets (gal, "DNAT0_BV")   #, value="")
dnatobv = real(imgets.value)
if (tele == "Lowell") {
# if (filt == "J") const = 9.094E+07/real(imgets.value)
  if (filt == "J") const1 = 9.094E+07
  if (filt == "R") const1 = 6.879E+07
scale = 1.35
 blksiz = 11
  }
  else {
#if (filt == "g") const = 11.587E+07/real(imgets.value)
if (filt == "g") const1 = 11.587E+07
if (filt == "r") const1 = 8.838E+07
if (filt == "i") const1 = 7.210E+07
#const = const1/real(imgets.value)
scale = 1.19
 blksiz = 12
}
imgets (gal, "G_CENT_X")   
x0   = real(imgets.value)/blksiz
imgets (gal, "G_CENT_Y")   
y0   = real(imgets.value)/blksiz

	maxa = 0.90*x0

print(" x0, y0 maxa ", x0, y0, maxa)

const = const1/dnatobv
#const = blksiz*blksiz*const1/dnatobv
#const = const1/real(imgets.value)
print(gal, "Sky: ", skyval, "Const: ", const, " ", tele, " ", filt)

zeromag = 2.5*log10(const1) + 5.0*log10(scale)

gal1 = gal//"t1"
imdelete (gal1, ver-)

imarith (gal, "-", skyval, gal1, title="MJy/sr", divzero=0., hparams="", pixtype="", calctype="", verbose=no, noact=no)

blkavg (gal1, gal1, blksiz, blksiz, 1, 1, 1, 1, 1, option="average")

imarith (gal1, "*", const, gal1, title="Optical Intensity in MJy/sr", divzero=0., hparams="", pixtype="", calctype="", verbose=no, noact=no)

gauss (gal1, gal1, sigma , ratio=1., theta=0., nsigma=nsigma, bilinear=yes,
boundary="nearest", constant=0.)

	  print ("Continue with Ellipse fit? (yes|no) ") 
	  scan (ok)
	  if (ok == "no")goto exit

delete (gal//"el*", ver-)

ellipse (gal1, gal//"el", a0=a0, mina=mina, maxa=maxa,\
sdas=no, interactive=no, device="stdgraph", imcur="",\
gcur="", initfile="", samplef="", badpixel="", harmonics="", x0=x0, y0=y0,\
eps0=eps0, teta0=teta0, hcenter=hcen, heps=heps, hteta=hteta, astep=astep,\
linear=yes, minit=8, maxit=50, lslope=0., conver=0., clip=0., maxrit=INDEF,\
integrmode="bi-linear", mag0=zeromag, refer=1., backgr=0., star=0.)

	  print ("Continue with tdgrad? (yes|no) ") 
	  scan (ok)
	  if (ok == "no")goto exit

delete (gal//"pr", ver-)
delete (gal//"pr1", ver-)
copy (gal//"el1", gal//"pr1", ver-)
copy (gal//"el4", gal//"pr", ver-)

tdgrad (gal//"pr", scale, "0.0")      #choose  X_UV=0.0  ====> LISRF

tdgrad (gal//"pr", scale, "3.0")      #Choose  X_UV=3.0  ====> LISRF+3xLisrf_uv

#imdelete (gal1, ver-)
delete (gal//"el*", ver-)

#tsort (gal1, "a", ascend=yes, casesens=yes)
#tprint (gal1,\ 
#prparam=no, prdata=yes, pwidth=80, plength=0, showrow=yes, showhdr=yes,\
#columns="", rows="-", option="plain", align=yes, sp_col="", lgroup=0) \
#| cat > gal1//"el"

exit:

	  print("===> BYE <==== ")

end
