procedure galasym(xylist)
#  uses scripts inscale, instat, intrim and inasym
#----------------------------------------------------------------------------------------------------
#Step 1: returns a first-guess automatically determined values of ellipse parameters
#Step 2: reads the ellipse parameters from step#1 (or from its edited version) and reruns ellipse
#        From the new ellipse parameters, constructs a BMODEL and RESIDUAL image
#step 3: Runs asym_par for the entire list and adds an assymmetric index column to the list from step#1
#
# zp = zmag - 2.5*log10(exptime) where exptime is the value of the KEYWORD expkey if (expkey!=null) else inttime
# outtab = DEFAULT creates infil//_geom#.par, .reg and .coo output files, with "#" taking the next available
#       integer. .par, .reg and .coo files are not over-written, and instead a new name is created.
#       outtab value is reset to the last created fileroot name while exiting in step#1.
#----------------------------------------------------------------------------------------------------

string  xylist  = "instat.lis"  {prompt="Input list file containing XY of object"}
string  infil   = "ocoaddJ_align"    	{prompt="Input image"}
int 	step    = 1   		{prompt="Step number of the run"}
string  outdir  = "data"    	{prompt="Directory for output files"}
string  outtab  = "DEFAULT"    	{prompt="Output file containing geometric parameters"}
int  	xtrim   = 500		{prompt="Trim size in x-axis"}
int 	ytrim   = 500		{prompt="Trim size in y-axis"}
real	zmag 	= 26.97		{prompt="Zeropoint mag"}
real	krmsav	= 10.0 		{prompt="Clipping radius in rms/bin units"}
real	inttime = 1.0  		{prompt="Integration time in seconds"}
string  zpkey   = ""    	{prompt="Image header Keyword containing Zeromag"}
string  expkey  = ""     	{prompt="Image header Keyword containing EXPTIME"}
string  scale   = ""            {prompt="Image scale arc/pix"}
string  gainkey = "GAIN"     	{prompt="Image header Keyword containing GAIN"}
real    gain    = 1.0        	{prompt="Gain value in e-/ADU"}
bool	plotell = no  		{prompt="Plot ellipse parameters?"}
real	smastep = 0.1           {prompt="sma step between successive ellipses"}
bool	linear  = no            {prompt="linear sma step?"}
real 	fflux   = 0.90		{prompt="Fractional flux at which PA and ELL are averaged"}
int  	nflux   = 2    		{prompt="Number of SMA for averaging for PA and ELL (2*nflux+1)"}
real    rmspix = 3.0            {prompt="Clipping radius in rms/pix units"}
int     intunit = 1             {prompt="Intensity units (1=counts; 2 = mag/arcsec2)"}
string  low_int = "RFIT"        {prompt="Lower limit of image  [value, RFIT, par]"}
string  hi_int  = "REFF"        {prompt="Higher limit of image [value, REFF, par]"}
bool    verbose = no            {prompt="Verbose each operation"}
bool    savergb = no            {prompt="Save RGB [R=image G=res B=good_pix]?"}

begin
	string	outfil, imfil, xyfil, id, c1, ddir, trimfil
	string	clow_lim, chi_lim, extn, froot, comm1, comm2
	real	low_lim, hi_lim, kkrmspix, bg
	int	j, i1, ieff, ifit, iunit, istep, neff, nfit
	bool	talk, head, makepng
        struct  line

	imfil = infil; xyfil = xylist; ddir = outdir; outfil = outtab
	talk = verbose; istep = step; kkrmspix = rmspix
	clow_lim = low_int; chi_lim = hi_int; iunit = intunit; makepng = savergb

        low_lim = 0.1; hi_lim = 10000.0
        if (strupr(clow_lim) != "RFIT" && strupr(clow_lim) != "PAR"){low_lim = real(clow_lim)}
        if (strupr(chi_lim) != "REFF" && strupr(chi_lim) != "PAR"){hi_lim = real(chi_lim)}

	if (istep < 3) {
	ingeom(xylist=xyfil, infil=imfil, step=istep, outdir=ddir, outtab=outfil,
xtrim=xtrim, ytrim=ytrim, zmag=zmag, krmsav=krmsav, inttime=inttime, zpkey=zpkey, expkey=expkey,
scale=scale, gainkey=gainkey, gain=gain, plotell=plotell, smastep=smastep, linear=linear,
fflux=fflux, nflux=nflux, verbose=talk)
#rmspix=rmspix, intunit=intunit, low_int=low_int, hi_int=hi_int,
	galasym.outtab = ingeom.outtab
	} else {


	#step2:
	#j = 99
	j = 1
        froot = outfil; if (strupr(froot) == "DEFAULT"){froot = imfil//"_geom"}
        outfil = froot//str(j)
	while (!access(outfil//".par")){print ("par file= ", outfil, " from step1 should exist before running step2")}
        while (access(outfil//".par")){j = j + 1; outfil = froot//str(j)}
	outfil = froot//str(j-1)
        #while (!access(outfil//".par") && j>0){j = j - 1; outfil = froot//str(j)}
	#if (strupr(outfil) == "DEFAULT"){outfil = imfil//"_geom"//str(j)}
	#while (!access(outfil//".par") && j>0){j = j - 1; outfil = imfil//"_geom"//str(j)}
	print(outfil)
	galasym.outtab = froot
	#if (istep > 2){goto step3}

	xyfil = outfil//".par"
        if (!access(xyfil)){
                print("Error*** Ellipse parameter list ", xyfil, " not found.")
                print("Enter correct filename: ")
                scan(xyfil)
                }
        list = xyfil
        id = "#"
        i1 = 0
        while( fscan(list,line) != EOF ) {
        c1 = substr(line,1,1)
        if (c1!="#") {
          i1 = i1 + 1
	#print(line) | fields (fields="1,5,7,10", line=1) | scan (id, nfit, neff, bg)
	print(line) | fields (fields="1,5-7,10", line=1) | scan (id, nfit, reff, neff, bg)
	if (talk) print("=========================================================")
	if (talk) {print(id, nfit, neff, bg)}

        trimfil = ddir//"/"//imfil//"_"//id

step3:
        imdelete(trimfil//"_mod", ver-, >& "dev$null")
        imdelete(trimfil//"_res", ver-, >& "dev$null")

        extn = ".fits"
        if (imaccess(trimfil//"_mask.pl")){
                #print(" *** Using mask file for ", id, " ***")
                extn = ".fit"
                imrename(trimfil//".fits", trimfil//".fit", verb-);
                imrename(trimfil//"_mask.pl", trimfil//".pl", verb-);
                }

	bmodel (trimfil//"el1", trimfil//"_mod", fulltab+, minsma=1.0, maxsma=1.0, backgr=bg, interp="spline", highar-, verbose-)
        if (imaccess(trimfil//".pl")){imrename(trimfil//".fit", trimfil//".fits", verb-); imrename(trimfil//".pl", trimfil//"_mask.pl", verb-)}

	imexpr("a-b", trimfil//"_res", trimfil, trimfil//"_mod", verb-)

	if (talk) {tprint(trimfil//"el1", columns="SMA INTENS", rows=str(neff)//","//str(nfit), showrow+, showhdr-) }
	tprint(trimfil//"el1", columns="INTENS", rows=neff, showrow-, showhdr-) | scan(ieff)
	tprint(trimfil//"el1", columns="INTENS", rows=nfit, showrow-, showhdr-) | scan(ifit)

	if (strupr(clow_lim) == "RFIT"){low_lim = ifit; iunit=1}
	if (strupr(chi_lim ) == "REFF"){hi_lim = ieff; iunit=1}
	#print(low_lim, hi_lim, iunit)

	if (i1 == 1){head = yes} else {head = no}
	inasym (imfil, objnam="_"//id, datadir=ddir, intunit=iunit, low_int=low_lim, hi_int=hi_lim, rmspix=kkrmspix, verb=talk, head=head)

        #comm1 = "b<136? a : 0"
        #imdelete (imfil//"_res0", ver-, >& "dev$null")
        #imexpr (comm1, imfil//"_res0", imfil, modfil, maskim, verb-)

	#makepng = yes
	if (makepng){
        comm1 = "-view colorbar no -scale linear -scale mode zscale -zoom 2 -height 400 -width 400 "
	comm2 = "-rgb -red "//trimfil//".fits -green "//trimfil//"_res.fits -blue "//trimfil//"_good_pix.fits -regions load "//outfil//".reg -saveimage "//trimfil//".png "
	delete("temp.com", ver-, >& "dev$null")
        print("ds9 ", comm1//comm2, " -exit", >> "temp.com")
        if (!deftask("source")){task $source = "$foreign"}
        source "temp.com"
	}

        } }
	}

	#print("Done")

end
