procedure inasym(obsim)

	#1: Read the trimmed image and set to zero all pixels with I > low_int
	#2: Make sure the object is at the center of the image
	#3: Rotate the galaxy by 180 deg
	#4: Use the formula for finding rotational asymmetry to find the index value

string	obsim 	= "ocoaddJ_align"	{prompt="Input image"}
string  objnam	= ""       		{prompt="Object name in the input image"}
string  modim 	= "_mod"      		{prompt="Output of bmodel [_mod, empty or complete filename]"}
string  datadir = "data"        	{prompt="Directory for output files"}
int     intunit = 1             	{prompt="Intensity units (1=counts; 2 = mag/arcsec2)"}
string	bgkey   = "_bg"			{prompt="Background level of image [_bg, keySKY, value]"}
string  rmskey	= "_rms"   		{prompt="rms of image [_rms, keyRMS, value]"}
real	low_int	= 0.12    		{prompt="Lower limit of image in image counts"}
real	hi_int	= 10000.0      		{prompt="Higher limit of image in image counts"}
real    zmag    = 26.97         	{prompt="Zeropoint mag"}
real    rmspix  = 3.0          		{prompt="Clipping radius in rms/pix units"}
real    inttime = 1.0           	{prompt="Integration time in seconds"}
string  zpkey   = ""            	{prompt="Image header Keyword containing Zeromag"}
string  expkey  = ""     		{prompt="Image header Keyword containing EXPTIME"}
string  scale 	= ""       		{prompt="Image scale arc/pix"}
bool    header  = yes      		{prompt="Print a header row for the output file?"}
bool    verbose = yes      		{prompt="Verbose each operation"}

begin
        string  imfil, modfil, ddir, zpmagkey, exptkey, bgstr, rmstr, comm1
	string	zerofil, id, fnam, outfil, cscale, head_com, maskim
	real	int1, int2, ksig, mu1, mu2, zp, imscale, zpmag, zeromag, exptime
	real	bg, rms, low_lim, hi_lim, xc, yc, mu_lim, nvalue, dvalue, rot_asy_par, area_asy_par
	real	mu_lo, mu_hi, rot_asy_par2, rot_asy_err, evalue, area_asy_err
	int	iunit, npix, npix1, npix2, err_flag, nasym, ntot
	bool	talk, head

	ddir = datadir; zpmagkey = zpkey; exptkey = expkey; bgstr = bgkey; rmstr = rmskey
	fnam = obsim; id = fnam//objnam; imfil = ddir//"/"//id; modfil = modim  
	head = header
	#id = objnam; imfil = ddir//"/"//obsim//id; modfil = modim  
	if (modfil != "" && !imaccess(modfil) && imaccess(imfil//modfil)){modfil = imfil//modfil}
	
	iunit = intunit; zpmag = zmag; exptime = inttime; talk = verbose; err_flag = 0
	cscale = scale  #; xc = xcen; yc = ycen
        if (cscale != ""){imscale = real(cscale)} else {inscale(imfil, verbose-); imscale = inscale.scale}

	if (strlwr(substr(bgstr, 1, 3)) == "key"){hselect (imfil, substr(bgstr, 4, strlen(bgstr)), yes) | scan(bg)} 
		else if (imaccess(imfil//bgstr)){imstat (imfil//bgstr, fields="midpt", nclip=3, format-) | scan (bg) } else {bg = real(bgstr)}
	if (strlwr(substr(rmstr, 1, 3)) == "key"){hselect (imfil, substr(rmstr, 4, strlen(rmstr)), yes) | scan(rms)} 
		else if (imaccess(imfil//rmstr)){imstat (imfil//rmstr, fields="midpt", nclip=3, format-) | scan (rms) } else {rms = real(rmstr)}

        if (exptkey != "") {hselect (imfil, exptkey, yes) | scan(exptime)}
        if (zpmagkey != "") {hselect (imfil, zpmagkey, yes) | scan(zpmag)}
        zeromag = zpmag - 2.5*log10(exptime) + 5.0*log10(imscale)

        int1 = max(hi_int, 1.0E-30); int2 = max(low_int, 1.0E-30)   #; dint=int_bin; 
	ksig = rmspix*rms
	if (iunit > 1){int1 = 10**(-0.4*(int1 - zeromag)) + bg; int2 = 10**(-0.4*(int2 - zeromag)) + bg}  #; dint = 10**(-0.4*dint)
        mu_lim = -2.5*log10(ksig) + zeromag 

        if (talk) print("################################################################################################")
	if (talk) print(bgstr, " ", bg, " ", rmstr, " ", rms, " ZP=", zeromag, " Scale=", imscale, "mu_lim = ", mu_lim, " mag/arcsec**2")

	imstat (imfil, fields="max", nclip=0, format-) | scan (hi_lim)
	low_lim = max(int2, ksig+bg)   #make 0 those pixels that are below ksig or above max
	hi_lim  = min(int1, hi_lim*1.01)
	mu_lo =  -2.5*log10(low_lim) + zeromag
	mu_hi =  -2.5*log10(hi_lim) + zeromag
        comm1 = "(a>="//str(low_lim)//" && a <="//str(hi_lim)//") ? a : 0"
	if (modfil != ""){comm1 = "(a>="//str(low_lim)//" && a <="//str(hi_lim)//" && b >="//str(bg + ksig)//") ? a : 0"}
	if (hi_lim > low_lim){
	if (talk) print(comm)
	zerofil = imfil//"_zero"
	imdelete (zerofil, ver-, >& "dev$null")
        imexpr (comm1, zerofil, imfil, modfil, verb-)

	#hselect (imfil, "NAXIS1 NAXIS2 EXPTIME", yes) | scan(npix1, npix2, exptime)
	#if (xc == 0.0) {xc = real(npix1)/2.0 + 1.0} 
	#if (yc == 0.0) {xc = real(npix2)/2.0 + 1.0} 

	if (talk) print("*** Rotational asymmetry calculations ")

        imdelete (imfil//"_180", ver-, >& "dev$null")
        imdelete (imfil//"_0", ver-, >& "dev$null")
	imcopy(zerofil, imfil//"_0", verbo-)

	rotate (imfil//"_0", imfil//"_180", 180., xin=INDEF, yin=INDEF, xout=INDEF, yout=INDEF, ncols=INDEF, nlines=INDEF, interpolant="linear", boundary="constant", constant=0., verbose-)

        imdelete (imfil//"_n", ver-, >& "dev$null")
        imdelete (imfil//"_err", ver-, >& "dev$null")

	if (talk) print("Numerator: 0.5*(a-b)**2")
	#imexpr ("abs(a-b) > c ? 0.5*(a-b)**2 : 0.0", imfil//"_n", imfil//"_0", imfil//"_180", ksig, verb-)
	imexpr ("(a>c && b>c) ? 0.5*(a-b)**2 : 0.0", imfil//"_n", imfil//"_0", imfil//"_180", ksig, verb-)
	#count error in each pixel = sqrt(2*rms**2)
	imexpr ("(a>c && b>c) ? 0.5*2.0*c**2 : 0.0", imfil//"_err", imfil//"_0", imfil//"_180", rms, verb-)

        imdelete (imfil//"_d", ver-, >& "dev$null")

	if (talk) print("Denominator: a**2")
	imexpr ("a**2", imfil//"_d", imfil//"_0", ver-)
	#imexpr ("(a**2+b**2)/2.0", imfil//"_d", imfil//"_0", imfil//"_180", ver-)

	imstat (imfil//"_n", fields="mean, npix", lower=INDEF, upper=INDEF, nclip=0, binwidth=0.1, format-) | scan(nvalue, npix)
	imstat (imfil//"_d", fields="mean", lower=INDEF, upper=INDEF, nclip=0, binwidth=0.1, format-) | scan(dvalue)
	imstat (imfil//"_err", fields="mean", lower=INDEF, upper=INDEF, nclip=0, binwidth=0.1, format-) | scan(evalue)
	rot_asy_par = sqrt(nvalue/dvalue)
	rot_asy_err = sqrt(evalue/dvalue)
	if (talk) print("Asym = Sqrt(Sum of Numerator/Sum of Denominator) = sqrt(", nvalue*real(npix), "/", dvalue*real(npix), " ) = ", rot_asy_par , "+/-", rot_asy_err)
	#printf("%8.3f\n", rot_asy_par)

	#Area asymmetry
	#1. Prepare an image from residual image that satisfies the ksig criteria
	#2. Count the number of pixels in "residual" image
	#3. Count the number of pixels in "zero" image
	#area_asym = #1/#3

	if (talk) print("*** Area asymmetry calculations ")

	maskim = imfil//"_mask.pl"
        if (!imaccess(maskim)){maskim = imfil//"_mask0.pl"; imdelete (maskim, ver-, >& "dev$null"); imexpr ("a*0", maskim, imfil, verb-)}
	#imstat(maskim)

        comm1 = "(a>="//str(low_lim)//" && a <="//str(hi_lim)//" && b=0) ? 1 : 0"
	if (modfil != ""){comm1 = "(a>="//str(low_lim)//" && a <="//str(hi_lim)//"  && b=0 && c >="//str(bg + ksig)//") ? 1 : 0"}
        imdelete (imfil//"_good_pix", ver-, >& "dev$null")
        imexpr (comm1, imfil//"_good_pix", imfil, maskim, modfil, verb-)
        #imexpr (comm1, imfil//"_good_pix", modfil, verb-)
	imstat (imfil//"_good_pix", fields="mean, npix", lower=INDEF, upper=INDEF, nclip=0, binwidth=0.1, format-) | scan(dvalue, npix2)

        comm1 = "(a-b>="//str(low_lim)//" && a-b <="//str(hi_lim)//" && c=0) ? 1 : 0"
	#if (modfil != ""){comm1 = "(a-b>="//str(low_lim)//" && a-b <="//str(hi_lim)//" && b >="//str(bg + ksig)//") ? 1 : 0"}
	if (modfil != ""){comm1 = "(a-b>="//str(low_lim)//" && a-b <="//str(hi_lim)//" && b >="//str(low_lim)//" && b <="//str(hi_lim)//" && c=0) ? 1 : 0"}
	if (talk) print(comm)
        imdelete (imfil//"_asym_pix", ver-, >& "dev$null")
        imexpr (comm1, imfil//"_asym_pix", imfil, modfil, maskim, verb-)
	imstat (imfil//"_asym_pix", fields="mean, npix", lower=INDEF, upper=INDEF, nclip=0, binwidth=0.1, format-) | scan(nvalue, npix1)
	area_asy_par = nvalue*npix1/(dvalue*npix2)
	nasym = int(nvalue*real(npix1)); ntot = int(dvalue*real(npix2))
	area_asy_err = 1.0/sqrt(ntot)
	if (talk) print("Asym = No of good asymmetric pixels/Total no of good  pixels = ", nasym, "/", ntot, "=", area_asy_par, "+/-", area_asy_err )
	} else {if (talk){print("Error*** ", comm)}; rot_asy_par=0.0; area_asy_par = 0.0; err_flag=1; nasym=0; ntot=0}
	#printf("%8.3f %8.3f %8.3f %8.3f %8.3f %8.3f %d %d %d %s\n", rot_asy_par, area_asy_par, low_lim, hi_lim, mu_lo, mu_hi, nasym, ntot, err_flag, id)
	outfil = fnam//"_asym.par"
	imdelete (imfil//"_mask0.pl", ver-, >& "dev$null")
	if (head){
	head_com = "#rot_ind          area_ind         low_int  hi_int    low_mu   hi_mu   Nasy  Ntot flag    ID" 
	#head_com = "#rot_ind rot_ind2   area_ind   low_int  hi_int   low_mu    hi_mu  Nasy  Ntot flag    ID" 
	#head_com = "#rot_ind area_ind   low_int  hi_int   low_mu    hi_mu  Nasy  Ntot flag    ID" 
	#print("################################################################################################", >> outfil)
	print(head_com)
	print(head_com, >> outfil)
	}
        #delete(outfil, ver-, >& "dev$null")

	#residual image with center masked
        imdelete (imfil//"_res0", ver-, >& "dev$null")
        imexpr ("a>0.0 ? b*c : b", imfil//"_res0", modfil, imfil//"_res", imfil//"_good_pix", verb-)

	#Rotational asymmetry of only "good" pixels

        imdelete (imfil//"_nn", ver-, >& "dev$null")
        #imexpr ("abs(a-b) > c && d=1.0 ? 0.5*(a-b)**2 : 0.0", imfil//"_nn", imfil//"_0", imfil//"_180", ksig, imfil//"_good_pix", verb-)
        imexpr ("(a>c && b>c && d=1.0) ? 0.5*(a-b)**2 : 0.0", imfil//"_nn", imfil//"_0", imfil//"_180", ksig, imfil//"_good_pix", verb-)

        imdelete (imfil//"_dd", ver-, >& "dev$null")
        imexpr ("b=1.0 ? a**2 : 0.0", imfil//"_dd", imfil//"_0", imfil//"_good_pix", ver-)
        imstat (imfil//"_nn", fields="mean, npix", lower=INDEF, upper=INDEF, nclip=0, binwidth=0.1, format-) | scan(nvalue, npix)
        imstat (imfil//"_dd", fields="mean", lower=INDEF, upper=INDEF, nclip=0, binwidth=0.1, format-) | scan(dvalue)
        rot_asy_par2 = sqrt(nvalue/dvalue)

	#printf("%8.3f %8.3f %8.3f %8.3f %8.3f %8.3f %8.3f %5d %5d %4d %24s\n", rot_asy_par, rot_asy_par2, area_asy_par, low_lim, hi_lim, mu_lo, mu_hi, nasym, ntot, err_flag, id)
	#printf("%8.3f %8.3f %8.3f %8.3f %8.3f %8.3f %8.3f %5d %5d %4d %24s\n", rot_asy_par, rot_asy_par2, area_asy_par, low_lim, hi_lim, mu_lo, mu_hi, nasym, ntot, err_flag, id, >> outfil)
	printf("%6.3f +/- %5.3f %6.3f +/- %5.3f %8.3f %8.3f %8.3f %8.3f %5d %5d %4d %24s\n", rot_asy_par, rot_asy_err, area_asy_par, area_asy_err, low_lim, hi_lim, mu_lo, mu_hi, nasym, ntot, err_flag, id)
	printf("%6.3f +/- %5.3f %6.3f +/- %5.3f %8.3f %8.3f %8.3f %8.3f %5d %5d %4d %24s\n", rot_asy_par, rot_asy_err, area_asy_par, area_asy_err, low_lim, hi_lim, mu_lo, mu_hi, nasym, ntot, err_flag, id, >> outfil)
	#print(rot_asy_par2)

	#print("DONE")

end

