procedure incmdprof(filbig, xcoo, ycoo)

string  filbig   = "Disk200_300pc"        {prompt="Input cluster catalog"}
real    xcoo     = 0.0      		{prompt="X coordinate (RA) in pixel (or degrees)"}
real    ycoo     = 0.0      		{prompt="Y coordinate (Dec) in pixel (or degrees)"}
real    radius   = 5.0      		{prompt="Radius of the circle for CMD (pixel or arcsec)"}
real    indisk   = 5.0      		{prompt="Inner radius of the background circle (pixel or arcsec)"}
real    outdisk  = 10.0      		{prompt="Outer radius of the background circle (pixel or arcsec)"}
string  filhead  = "Cluster.head"        {prompt="Header description file"}
string  xhead    = "RA"        		{prompt="X column header name in filbig"}
string  yhead    = "DEC"        	{prompt="Y column header name in filbig"}
string  mhead    = "IMAG"        	{prompt="Magnitude column header name in filbig for CMD"}
string  cform    = "BMAG-IMAG"        	{prompt="Color formula using column header names in filbig for CMD"}
string  xyunit   = "degree"        	{prompt="Units of XY [pixel/degree]"}
string  runit    = "arcsec"       	{prompt="Units of radius, indisk and outdisk [pixel/arcsec/degree]"}
string  outdat   = "incmdprof.dat"       {prompt="Disk-subtracted CMD stars"}
string  outreg   = "incmdprof.reg"       {prompt="Disk-subtracted CMD star coordinates in ds9 reg format"}
real    mag1     = 18.0      		{prompt="mag1"}
real    mag2     = 28.0    		{prompt="mag2"}
real    col1     = -0.6    		{prompt="col1"}
real    col2     =  1.0    		{prompt="col2"}
real    magint   =  1.0    		{prompt="col2"}
real    colint   =  0.2    		{prompt="col2"}

begin
	string	disk_fil, head_fil, comm1, rahead, dechead, maghead, colhead, comm2
	string  ofildat, ofilreg, coounit, radunit
        real    mbin, cbin, bmag, vmag, imag, x0, y0, bimag
        real    xx[999], yy[999], bm[999], vm[999], im[999]
	int	i1, nmbin, ncbin, i, j, ndisk, nclus
	real	magbin, colbin, mag_max, mag_min, col_max, col_min, scale
	real	mag_lo, mag_hi, col_lo, col_hi, xra, ydec, rad0, rad1, rad2, area_fac

	disk_fil = filbig  #"Disk200_300pc"
	#clus_fil = filsma  #"ExtCluster"
	head_fil = filhead  #"Cluster.head"
	ofildat = outdat; ofilreg = outreg
        delete(ofildat, ver-, >& "dev$null")
        delete(ofilreg, ver-, >& "dev$null")

	rahead = xhead; dechead = yhead; maghead = mhead; colhead = cform
	magbin = magint; colbin = colint
	mag_max = mag2; mag_min = mag1; col_max = col2; col_min = col1
	nmbin = (mag_max - mag_min)/magbin
	ncbin = (col_max - col_min)/colbin
	xra = xcoo; ydec = ycoo
	coounit = xyunit; radunit = runit
	scale = 1.; if (substr(radunit, 1, 1) == "a") {scale = 1./3600.}
	rad0 = radius*scale; rad1 = indisk*scale; rad2 = outdisk*scale

	print(nmbin, ncbin)

	if (!defpac("ttools")) {tables; ttools}
	if (!access(disk_fil//".tab")){
        delete(disk_fil//".tab", ver-, >& "dev$null")
	tcreate (disk_fil//".tab", head_fil, disk_fil//".dat")
	tcalc (disk_fil//".tab", "COLOR", colhead)

	#comm1 = "sqrt(("//rahead//" - "//xra//")**2 + ("//dechead//" - "//ydec//")**2)"
	comm1 = "sqrt((("//rahead//" - "//xra//")*cos("//ydec//"/57.295779513082))**2 + ("//dechead//" - "//ydec//")**2)"
	print(comm1)
	tcalc (disk_fil//".tab", "RADIUS", comm1)
	tsort (disk_fil//".tab", "RADIUS", ascend+)
	}
	area_fac = (rad2**2-rad1**2)/rad0**2
	print(area_fac)

	i1 = 100
        for (i=1; i <= nmbin; i +=1) {
	mag_lo = mag_min + real(i-1)*magbin
	mag_hi = mag_lo  + magbin
        for (j=1; j <= ncbin; j +=1) {
	col_lo = col_min + real(j-1)*colbin
	col_hi = col_lo  + colbin
	#comm1 = "IMAG < "//str(mag_hi)//" && IMAG >= "//str(mag_lo)//" && COLOR < "//str(col_hi)//" && COLOR >= "//str(col_lo)
	comm1 = maghead//" < "//str(mag_hi)//" && "//maghead//" >= "//str(mag_lo)//" && COLOR < "//str(col_hi)//" && COLOR >= "//str(col_lo)
	comm2 = " && RADIUS > "//str(rad1)//" && RADIUS <= "//str(rad2)
	#print(comm1//comm2)

        delete(disk_fil//"_"//str(i)//str(j)//"_bg.tab", ver-, >& "dev$null")
	tquery (disk_fil//".tab", disk_fil//"_"//str(i)//str(j)//"_bg"//".tab", comm1//comm2, colum=maghead//" COLOR RADIUS", sort=maghead)
	#tquery (disk_fil//".tab", disk_fil//"_"//str(i)//str(j)//"_bg"//".tab", comm1//comm2, colum="IMAG COLOR RADIUS", sort="")
	tinfo (disk_fil//"_"//str(i)//str(j)//"_bg"//".tab", ttout-)
	ndisk = int(real(tinfo.nrows)/area_fac)
	#printf("%4s %8.3f %8.3f %8.3f %8.3f %4d %4d %8.3f\n", str(i)//str(j), mag_lo, mag_hi, col_lo, col_hi, tinfo.nrows, ndisk, area_fac)

        delete(disk_fil//"_"//str(i)//str(j)//".tab", ver-, >& "dev$null")
	comm2 = " && RADIUS <= "//str(rad0)
	#print(comm1//comm2)
	tquery (disk_fil//".tab", disk_fil//"_"//str(i)//str(j), comm1//comm2, colum=maghead//" COLOR RADIUS "//rahead//" "//dechead, sort=maghead)
	#tquery (disk_fil//".tab", disk_fil//"_"//str(i)//str(j), comm1//comm2, colum="IMAG COLOR RADIUS "//rahead//" "//dechead, sort="")
	tinfo (disk_fil//"_"//str(i)//str(j)//".tab", ttout-)
        nclus = tinfo.nrows #- ndisk
	print(nclus, ndisk, tinfo.nrows)
        if (nclus > 0){
        #print(comm1, " = ", tinfo.nrows, " ", ndisk, " ", nclus)
        tprint (disk_fil//"_"//str(i)//str(j), rows="1-"//str(nclus), showhdr-, showrow-, >> ofildat)
        }
        printf("%4s %8.3f %8.3f %8.3f %8.3f %4d %4d %4d\n", str(i)//str(j), mag_lo, mag_hi, col_lo, col_hi, tinfo.nrows, ndisk, nclus)

        delete(disk_fil//"_"//str(i)//str(j)//"_bg.tab", ver-, >& "dev$null")
        delete(disk_fil//"_"//str(i)//str(j)//".tab", ver-, >& "dev$null")

        }
        }
        #xytoreg(ofildat, regfile=ofilreg, xc=4, yc=5, idc=0, xyunit=coounit, sizunit=radunit, cirrad=0.2)

end
