procedure ingeom(xylist)
#  uses scripts inscale, instat, intrim and inasym
#----------------------------------------------------------------------------------------------------
#Step 1: returns a first-guess automatically determined values of ellipse parameters
#Step 2: reads the ellipse parameters from step#1 (or from its edited version) and reruns ellipse
#        From the new ellipse parameters, constructs a BMODEL and RESIDUAL image
#step 3: Runs asym_par for the entire list and adds an assymmetric index column to the list from step#1
#
# zp = zmag - 2.5*log10(exptime) where exptime is the value of the KEYWORD expkey if (expkey!=null) else inttime
# outtab = DEFAULT creates infil//_geom#.par, .reg and .coo output files, with "#" taking the next available
#       integer. .par, .reg and .coo files are not over-written, and instead a new name is created.
#       outtab value is reset to the last created fileroot name while exiting in step#1.
#----------------------------------------------------------------------------------------------------

string  xylist  = "instat.lis"  {prompt="Input list file containing XY of object"}
string  infil   = "ocoaddJ_align"    	{prompt="Input image"}
int 	step    = 1   		{prompt="Step number of the run"}
string  outdir  = "data"    	{prompt="Directory for output files"}
string  outtab  = "DEFAULT"    	{prompt="Output file containing geometric parameters"}
int  	xtrim   = 500		{prompt="Trim size in x-axis"}
int 	ytrim   = 500		{prompt="Trim size in y-axis"}
real	zmag 	= 26.97		{prompt="Zeropoint mag"}
real	krmsav	= 10.0 		{prompt="Clipping radius in rms/bin units"}
real	inttime = 1.0  		{prompt="Integration time in seconds"}
string  zpkey   = ""    	{prompt="Image header Keyword containing Zeromag"}
string  expkey  = ""     	{prompt="Image header Keyword containing EXPTIME"}
string  scale   = ""            {prompt="Image scale arc/pix"}
string  gainkey = "GAIN"     	{prompt="Image header Keyword containing GAIN"}
real    gain    = 1.0        	{prompt="Gain value in e-/ADU"}
bool	plotell = no  		{prompt="Plot ellipse parameters?"}
real	smastep = 0.1           {prompt="sma step between successive ellipses"}
bool	linear  = no            {prompt="linear sma step?"}
real 	fflux   = 0.90		{prompt="Fractional flux at which PA and ELL are averaged"}
int  	nflux   = 2    		{prompt="Number of SMA for averaging for PA and ELL (2*nflux+1)"}
real    rmspix = 3.0            {prompt="Clipping radius in rms/pix units"}
int     intunit = 1             {prompt="Intensity units (1=counts; 2 = mag/arcsec2)"}
string  low_int = "RFIT"        {prompt="Lower limit of image  [value, RFIT, par]"}
string  hi_int  = "REFF"        {prompt="Higher limit of image [value, REFF, par]"}
bool    verbose = no            {prompt="Verbose each operation"}


begin
	string	imfil, xyfil, fulfile, trimsec, c1, id, trimfil, comm
	string	mask_regfil, froot
	real	x0, y0, rfit, tflux_e, half_flux, reff, xc, yc, ieff, ifit
	real	a0, mina, maxa, eps0, teta0, astep, zeromag, mu_lim
	real	tmag_e, imscale, bg, rms, exptime, ellip_med, pa_med, ellip_err, pa_err
	int	i1, ix1, ix2, iy1, iy2, dx, dy, rowno, r1, r2, npix_e, i2
	int	nfit, neff, npix1, npix2, istep, iter
	string  s1[999], s2[999], ddir, zpmagkey, exptkey, outfil, ans
	real    xx[999], yy[999], zpmag, eccent, pa_ds9, kkrms, rms_lim, frac_flux, rfrac, frac
	real	low_lim, hi_lim, kkrmspix, gadu, obj_loc
	real	sig_sky_noi, sig_obj_noi, dmag, ltv1, ltv2
	string  clow_lim, chi_lim, cscale, gkey, comment, extn, suffix
	int     ddx[999], ddy[999], j, nmin, nmax, nav, nav1 
	int	nfrac, iunit, minrow, maxrow, len
	bool	plot, smalinear, talk, head, imsec 

        struct  line

	imfil = infil; xyfil = xylist; ddir = outdir; outfil = outtab
	zpmagkey = zpkey; exptkey = expkey; zpmag = zmag; exptime = inttime; gkey = gainkey
	istep = step; plot = plotell; smalinear = linear; kkrms = krmsav; kkrmspix = rmspix
	frac = min(fflux, 0.90); frac = max(frac, 0.20); nav = nflux; gadu = gain
	iunit = intunit; clow_lim = low_int; chi_lim = hi_int; talk = verbose; cscale = scale
	low_lim = 0.1; hi_lim = 10000.0
	if (strupr(clow_lim) != "RFIT" && strupr(clow_lim) != "PAR"){low_lim = real(clow_lim)}
	if (strupr(chi_lim) != "REFF" && strupr(chi_lim) != "PAR"){hi_lim = real(chi_lim)}
	#print(low_lim, hi_lim, iunit)
	a0 = 5.0; mina = 1.; eps0 = 0.05; teta0=5.0; astep = smastep; obj_loc = 1.0 

	imsec = no
	if (strupr(substr(imfil,1,2)) == "ID"){
	  imsec = yes; len = strlen(imfil); suffix = substr(imfil,3,len)
	  print(suffix)
	}
	#print(outfil)
	j = 1
        froot = outfil; if (strupr(froot) == "DEFAULT"){froot = imfil//"_geom"}
        outfil = froot//str(j)
	while (access(outfil//".par")){j = j + 1; outfil = froot//str(j)}
	ingeom.outtab = froot 

	if (istep > 1){goto step2}
	print(outfil)
	comment = "#ID                         X0      Y0      rfit nfit    reff neff   tmag_e  merr      bg       rms    PA    PAerr    ELLIP   ELLerr   krms  mu_lim  nf rfrac fflux nav"

        #trimfil = "ocoaddJ_align_obj1"
        delete("ingeom.coo", ver-, >& "dev$null")

	if (!access(xyfil)){
		print("Error*** XY list file named ", xyfil, " not found.") 
		print("Enter correct filename: ")
		scan(xyfil)
		}
	list = xyfil
	id = "#"
        i1 = 0
        while( fscan(list,line) != EOF ) {
        c1 = substr(line,1,1)
        if (c1!="#") {
          i1 = i1 + 1
	#print (line) | scan (id, x0, y0, dx, dy, fulfile)
	#s1[i1] = id; xx[i1] = x0; yy[i1] = y0; ddx[i1] = dx; ddy[i1] = dy; s2[i1] = fulfile
	print (line) | scan (id, x0, y0)
	s1[i1] = id; xx[i1] = x0; yy[i1] = y0; ddx[i1] = xtrim; ddy[i1] = ytrim; s2[i1] = imfil   #"ocoaddJ_align"
	} }

        for (j=1; j <= i1; j +=1) {
	id = s1[j]; x0 = xx[j]; y0 = yy[j]; dx = ddx[j]; dy = ddy[j]; fulfile = s2[j]

	if (imsec) {
	  fulfile = id//suffix
	  trimfil = ddir//"/"//fulfile
          imdelete(trimfil, ver-, >& "dev$null")
	  imcopy (fulfile, ddir, verbo-)
	  hselect (trimfil, "NAXIS1 NAXIS2", yes) | scan(dx, dy)
	  #print(fulfile, " ", dx, dy)
	  if (access(id//"_cmd0.dat")){
	    wcstoxy (id//"_cmd0.dat", refim=trimfil, outxy=id//"_mask.dat", idc=3, xc=1, yc=2, rc=0)
	    xytoreg(id//"_mask.dat", regf=id//"_mask.reg", xc=1, yc=2, idc=0, shap="circle", cirrad=4.0, xyun="pixel", sizun="pixel")
	    #mask_regfil = trimfil//"_mask.reg"
	  }
	  obj_loc = 0.0
	} else {
	  obj_loc = 1.0
	if (fulfile == "" || !imaccess(fulfile)){print ("Input image = ", fulfile, " not found. Exiting...."); goto exit}
	#print("=========================================================")
	print("======Analyzing object ", id, " in image ", fulfile, " ==================")
	intrim (fulfile, x0, y0, dx, dy, name=id, verbose+)
	if (!access(ddir)) {mkdir(ddir)}
        trimfil = ddir//"/"//fulfile//"_"//id
        imdelete(trimfil, ver-, >& "dev$null")
	imrename (fulfile//"_"//id, trimfil, verb-)
	}   #end if imsec
	#date
	instat(trimfil, verbose-)
	if (cscale != ""){imscale = real(cscale)} else {inscale(trimfil, verbose-); imscale = inscale.scale}
	if (exptkey != "") {hselect (trimfil, exptkey, yes) | scan(exptime)}
	if (zpmagkey != "") {hselect (trimfil, zpmagkey, yes) | scan(zpmag)}
	if (gkey != "") {hselect (trimfil, gkey, yes) | scan(gadu)} 
	zeromag = zpmag - 2.5*log10(exptime)

        delete(trimfil//"el.tab", ver-, >& "dev$null")
	maxa = dx/3.0 
	xc = real(dx)/2.0; yc = real(dy)/2.0
		extn = ".fits"

	#creating mask file from reg file using regtomask

	if (imsec){mask_regfil = id//"_mask.reg"} else {mask_regfil = fulfile//"_"//id//"_mask.reg"}
	if (access(mask_regfil)){
	  regtomask (mask_regfil, maskim = "DEFAULT", refim = trimfil, outfits-)
	#}
	#if (imaccess(trimfil//"_mask.pl")){
		print(" ------------------------------------")
		print(" *** Using mask file for ", id, " ***")
		print(" ------------------------------------")
		extn = ".fit"
		imrename(trimfil//".fits", trimfil//".fit", verb-); 
		imrename(trimfil//"_mask.pl", trimfil//".pl", verb-); 
		}
	maxa = 5.0     #find the center as the mean of all ellipses upto sma=5 pix
        delete(trimfil//"el0.tab", ver-, >& "dev$null")

	if (!imsec){
        ellipse (trimfil//extn, trimfil//"el0", sma0=maxa, minsma=mina, maxsma=maxa, inter-, x0=xc, y0=yc, ellip0=eps0, pa0=teta0, hcenter-, hell-, hpa-, step=astep, linear=smalinear, mag0=zeromag, verbose-)
	tstat (trimfil//"el0", column="X0", >& "dev$null"); xc = tstat.mean
	tstat (trimfil//"el0", column="Y0", >& "dev$null"); yc = tstat.mean
	#print (xc, yc, tstat.nrows)
	}

	maxa = dx/3.0 
        ellipse (trimfil//extn, trimfil//"el", sma0=a0, minsma=mina, maxsma=maxa, inter-, x0=xc, y0=yc, ellip0=eps0, pa0=teta0, hcenter+, hell=imsec, hpa=imsec, step=astep, linear=smalinear, mag0=zeromag, verbose-, recen-, olthresh=obj_loc)
	if (imaccess(trimfil//".pl")){imrename(trimfil//".fit", trimfil//".fits", verb-); imrename(trimfil//".pl", trimfil//"_mask.pl", verb-)}

	#-------------Find Rfit and Total Flux  ------------------------------------
        delete(trimfil//"el1.tab", ver-, >& "dev$null")
	bg = instat.bgvalue; rms = instat.rmsvalue

	if (rms > 1.0E30){rms = 1}; if (bg > 100.*rms**2){bg = 0.0}

	iter = 0
	rms_lim = rms*kkrms
do_rfit_again:
	comm = "INTENS >= "//str(bg)//" + "//str(rms_lim)//"/sqrt(NPIX_E)" 
	#print(comm)
	iter = iter + 1
	tquery (trimfil//"el", trimfil//"el1", expr=comm, columns="SMA INTENS ELLIP PA", sort="")
	tinfo (trimfil//"el1", ttout-)
	rowno = tinfo.nrows
	tprint(trimfil//"el", columns="SMA TFLUX_E NPIX_E", row=rowno, showrow-, showhdr-) | scan(rfit, tflux_e, npix_e)
	nfit = rowno
	#print("ROWNO= ", rowno, " RFIT= ",rfit, tflux_e, npix_e)

	#-------------Find Reff ------------------------------------
        delete(trimfil//"el1.tab", ver-, >& "dev$null")
	half_flux = (tflux_e - real(npix_e)*bg)/2.0
	if (half_flux <= 0 && iter < 2){bg = 0.0; print("Recalculating bg",); goto do_rfit_again}
	if (half_flux <= 0)half_flux = 0.5
	tmag_e = -2.5*log10(2.0*half_flux/exptime) + zeromag 
	mu_lim = -2.5*log10(rms_lim/exptime/sqrt(npix_e)) + zeromag + 5.0*log10(imscale)
	
        sig_sky_noi = sqrt(npix_e)*rms/(2.0*half_flux)
        sig_obj_noi = 1.0/sqrt(2.0*half_flux*gadu*exptime)
        dmag = 1.086*sqrt(sig_sky_noi**2 + sig_obj_noi**2)

	comm = "(TFLUX_E - real(NPIX_E)*"//str(bg)//") <= "//str(half_flux)//" && SMA<"//str(nfit) 
	tquery (trimfil//"el", trimfil//"el1", expr=comm, columns="SMA INTENS ELLIP PA TFLUX_E NPIX_E", sort="")
	tinfo (trimfil//"el1", ttout-)
	rowno = tinfo.nrows
	tprint(trimfil//"el", columns="SMA", row=rowno, showrow-, showhdr-) | scan(reff)
	neff = rowno

	#-------------Find Rfrac -----------------------------------
        delete(trimfil//"el1.tab", ver-, >& "dev$null")
        frac_flux = frac*(tflux_e - real(npix_e)*bg)
        comm = "(TFLUX_E - real(NPIX_E)*"//str(bg)//") <= "//str(frac_flux)
        tquery (trimfil//"el", trimfil//"el1", expr=comm, columns="SMA INTENS ELLIP PA TFLUX_E NPIX_E", sort="")
        tinfo (trimfil//"el1", ttout-)
        rowno = tinfo.nrows
        tprint(trimfil//"el", columns="SMA", row=rowno, showrow-, showhdr-) | scan(rfrac)
	#nfrac = rowno
        nfrac = min(nfit - nav - 1, rowno)
	#---------------------------------------------------------------------------
	nmin = 2; nmax = nfit - 1
	r1 = max(nmin, nfrac - nav); r2 = min(nmax, nfrac + nav)
	tprint(trimfil//"el", columns="PA", rows=str(r1)//"-"//str(r2), showrow-, showhdr-) | tstat (, >& "dev$null")
	pa_med = tstat.median; pa_err = tstat.stddev; pa_ds9 = pa_med  + 90.0; nav1 = tstat.nrows
	tprint(trimfil//"el", columns="ELLIP", rows=str(r1)//"-"//str(r2), showrow-, showhdr-) | tstat (, >& "dev$null")
	eccent = tstat.median; ellip_med = 1.0 - tstat.median; ellip_err = tstat.stddev
	printf("%s %s %8.3f %3s %8.3f %s %d %7.1f %s %d %7.1f %s %d %7.1f %5.2f %d %s %8.3f %6.3f %s\n", id, "BG=", bg, "+/-", rms, " RFIT=[", nfit, rfit, "] Reff =[", neff, reff, "] Rfrac =[", nfrac, rfrac, frac, nav1, "] MAG=[", tmag_e, dmag, "]")
	if (j==1)print(comment, >> outfil//".par")

	printf("%24s %7.1f %7.1f %7.1f %4d %7.1f %4d %8.3f %6.3f %8.3f %8.3f %6.1f %6.1f %8.3f %8.3f %6.1f %7.2f %3d %5.1f %5.2f %3d\n", id, xc, yc, rfit, nfit, reff, neff, tmag_e, dmag, bg, rms, pa_med, pa_err, eccent, ellip_err, kkrms, mu_lim, nfrac, rfrac, frac, nav1, >> outfil//".par")

	x0 = x0 - real(dx)/2.0 + xc; y0 = y0 - real(dy)/2.0 + yc
	print(id, " ",x0, y0, rfit, rfit*ellip_med, pa_ds9, " 3", >> outfil//".coo")
	xytoreg(outfil//".coo", regfile=outfil//".reg", xc=2, yc=3, idc=1, color="cyan", xbox=-4.0, ybox=-5.0, theta=-6, shape=-7, xyunit = "physical")
	}
test:
	#print(id)
	#}
	if (istep == 1){goto exit}

step2:
	#j = 99
	j = 1
	#print(outfil)
	#froot = outfil
	if (strupr(froot) == "DEFAULT"){froot = imfil//"_geom"}
	outfil = froot//str(j)
        while (!access(outfil//".par")){print ("par file= ", outfil, " from step1 should exist before running step2"); goto exit}
	while (access(outfil//".par")){j = j + 1; outfil = froot//str(j)}
	outfil = froot//str(j-1)
	print(outfil)
	print(" ===> Step2 ")
	#while (!access(outfil//".par") && j>0){j = j - 1; outfil = froot//str(j)}
	print("Re-running ellipse using parameters in ", outfil)
	ingeom.outtab = froot  

        delete(outfil//".coo", ver-, >& "dev$null")
        delete(outfil//".reg", ver-, >& "dev$null")

	xyfil = outfil//".par"
        if (!access(xyfil)){
                print("Error*** Ellipse parameter list ", xyfil, " not found.")
                print("Enter correct filename: ")
                scan(xyfil)
                }
        list = xyfil
        id = "#"
        i1 = 0
        while( fscan(list, line) != EOF ) {
	#print(line)
        c1 = substr(line,1,1)
        if (c1!="#") {
	#print(c1)
          i1 = i1 + 1
	#print(i1, " ", xyfil)
	print(line) | fields (fields="1-7,10-12,14", line=1) | scan (id, xc, yc, rfit, nfit, reff, neff, bg, rms, teta0, eps0)
	eps0 = max(0.05, eps0)
	if (talk) print("=========================================================")
	if (talk) {print(id, " ", xc, yc, rfit, nfit, reff, neff, bg, rms, teta0, eps0)}

        trimfil = ddir//"/"//imfil//"_"//id
	if (cscale != ""){imscale = real(cscale)} else {inscale(trimfil, verbose-); imscale = inscale.scale}
        if (exptkey != "") {hselect (trimfil, exptkey, yes) | scan(exptime)}
        if (zpmagkey != "") {hselect (trimfil, zpmagkey, yes) | scan(zpmag)}
        zeromag = zpmag - 2.5*log10(exptime)

        delete(trimfil//"el1.tab", ver-, >& "dev$null")
	if (smalinear){rfit = rfit+astep} else {rfit = rfit*(1.0+astep)}

	extn = ".fits"
        if (imaccess(trimfil//"_mask.pl")){
                #print(" ------------------------------------")
                print(" *** Using mask file for ", id, " ***")
                #print(" ------------------------------------")
                extn = ".fit"
                imrename(trimfil//".fits", trimfil//".fit", verb-);
                imrename(trimfil//"_mask.pl", trimfil//".pl", verb-);
                }

        ellipse (trimfil//extn, trimfil//"el1", sma0=a0, minsma=mina, maxsma=rfit, inter-, x0=xc, y0=yc, ellip0=eps0, pa0=teta0, hcenter+, hell+, hpa+, step=astep, linear=smalinear, mag0=zeromag, verbose-, recen-)
	if (plot) {isopall (trimfil//"el1", "SMA"); print("Plotting ", id, " Hit Enter to continue"); scan(ans)}
	if (imaccess(trimfil//".pl")){imrename(trimfil//".fit", trimfil//".fits", verb-); imrename(trimfil//".pl", trimfil//"_mask.pl", verb-)}
	#if (istep == 2){goto exit}

	hselect (trimfil, "LTV1 LTV2", yes) | scan(ltv1, ltv2)
	x0 = xc - ltv1; y0 = yc - ltv2; ellip_med = 1.0 - eps0; pa_ds9 = teta0 + 90.0 
	print(id, " ",x0, y0, rfit, rfit*ellip_med, pa_ds9, " 3", >> outfil//".coo")
	#xytoreg(outfil//".coo", regfile=outfil//".reg", xc=2, yc=3, idc=1, color="cyan", xbox=-4.0, ybox=-5.0, theta=-6, shape=-7, xyunit = "physical")

step3:
	if (istep == 3){
        imdelete(trimfil//"_mod", ver-, >& "dev$null")
        imdelete(trimfil//"_res", ver-, >& "dev$null")
	bmodel (trimfil//"el1", trimfil//"_mod", fulltab+, minsma=1.0, maxsma=1.0, backgr=bg, interp="spline", highar-, verbose-)
	imexpr("a-b", trimfil//"_res", trimfil, trimfil//"_mod", verb-)

	if (talk) {tprint(trimfil//"el1", columns="SMA INTENS", rows=str(neff)//","//str(nfit), showrow+, showhdr-) }
	tprint(trimfil//"el1", columns="INTENS", rows=neff, showrow-, showhdr-) | scan(ieff)
	tprint(trimfil//"el1", columns="INTENS", rows=nfit, showrow-, showhdr-) | scan(ifit)

	if (strupr(clow_lim) == "RFIT"){low_lim = ifit; iunit=1}
	if (strupr(chi_lim ) == "REFF"){hi_lim = ieff; iunit=1}
	#print(low_lim, hi_lim, iunit)

	if (i1 == 1){head = yes} else {head = no}
	inasym (imfil, objnam="_"//id, datadir=ddir, intunit=iunit, low_int=low_lim, hi_int=hi_lim, rmspix=kkrmspix, verb=talk, head=head)
	}

        } 
	}
	xytoreg(outfil//".coo", regfile=outfil//".reg", xc=2, yc=3, idc=1, color="cyan", xbox=-4.0, ybox=-5.0, theta=-6, shape=-7, xyunit = "physical")

exit:
	print("Done")

end
