procedure lfosc_wcs(image, catroot)

#--------------------------------------------------------------------------------------------------------
#procedure to put wcs keywords in the lfosc image headers using the known LFOSC settings,
#          and to do differential photometry using the coordinates of stars in a list
#                              CROTA1=CROTA2=270 scale=1.065 arcsec/pix = 2.9576E-4 deg
# Steps: 1. Identify a star in the REF catalog (interactively or through a list)
#        2. Update the headers using the coordinates of this star 
#        3. Overlay the catalog stars using tvmark, and ask the user if the alignment is good, 
#           if not prompt the user to aupply new IMROT angle, and iterate until user is satisfied.
#        4. Proceed with photometry
#--------------------------------------------------------------------------------------------------------
#	Y.D. Mayya  ydm@inaoep.mx    date: 4-Mar-2011
#--------------------------------------------------------------------------------------------------------

string  image    = "mrk421"             {prompt="Image Name"}
string  catroot  = "mrk421_sdss"        {prompt="Catalog root name [default extn is .cat]"}
bool    intera   = "no"                {prompt="Interactively identify [inter] or get [ra dec x y] of a star from [catroot].id"}
int     starno   = "1"                  {prompt="Ref star number in the catalog"}
int     nmark    = "50"                 {prompt="Number of stars to mark"}
real    imscale  = "1.065"               {prompt="Image scale (arcsec/pix) [0.32/1.065]"}
real    imrota   = "90.0"               {prompt="Image rotation wrt north (degrees) [0.0/270.0]"}
bool	parity   = "yes"		 {prompt="Image parity normal (E to the left of N) [no/yes]"}
bool	checkwcs = "no"		 {prompt="Display and Mark the catalog stars for WCS Check? [no/yes]"}

begin

        string  imfil, mosfil, catfil, c1, regfil, creg, cid[999], cdata[999], starid, catal  #imexfil, filt, c2, c3, c4
	string  photcoo, jd, filt, obsdate, c2, c3, c4, radecfil1, photcoo1, imexfil
	real	rapix, decpix, cdelt1, rotang, ra, dec, a_ra[999], a_dec[999]
	real	raref, decref, xx, yy, rad, exptime, ddd, amag 
	bool	imparity, staridint, check_wcs, wcsok
        int     i1, i2, i3, imark,nx,ny,len, ix, iy, sid  #, twomid1,twomid2,nx,ny,ns,index[999], nadd, agnid,len,nrows

	imfil = image; catal = catroot; check_wcs = checkwcs
        starid = catal//".id"; catfil = catal//".cat"; imark = nmark
        staridint = intera; imparity = parity; rotang = imrota
        decpix  = imscale/3600.; rapix = -1.0*decpix; if (!imparity){rapix = decpix}
	rad = 3.0; sid = starno

	delete("cmds.tempjd", ver-, >& "dev$null")
        photcoo = imfil//"_phot.coo"; delete(photcoo, ver-, >& "dev$null")
        photcoo1 = imfil//"_phot1.coo"; delete(photcoo1, ver-, >& "dev$null")
        hselect (imfil, "i_naxis1, i_naxis2, EXPTIME, FILTER, OBS-DATE", yes) | scan (nx, ny, exptime, filthd, obsdate)

        len = strlen(obsdate); c3 = substr(obsdate,len,len); c4 = substr(obsdate,len-1,len)
        c2 = obsdate; delete("cmds.tempjd", ver-, >& "dev$null")
        i1 = strlstr("-",obsdate); len = strlen(obsdate); i2 = strlstr("T",obsdate)
        if (i1 == 7) {c2 = substr(obsdate,1,5); c1=substr(obsdate,6,len); c2=c2//"0"//c1}
        if (i2-i1 == 2) {c1=substr(c2,9,len); c2=substr(c2,1,8); c2=c2//"0"//c1}
        obsdate = c2
        len = strlen(obsdate); if (len==18) {obsdate=obsdate//c3}; if (len==17) {obsdate=obsdate//c4}

        c2 = 'jd = julday ("'//obsdate//'")'
        print(c2, > "cmds.tempjd")
        asthedit (imfil, "cmds.tempjd", table="", colname="", verbose-)
        hselect (imfil, "jd", yes) | scan (jd)
        print("Julian Date=",jd)

        regfil = imfil//"_canwcshead.reg"; delete(regfil, ver-, >& "dev$null")
        #radecfil = imfil//"_radec.cat"; delete(radecfil, ver-, >& "dev$null")
        radecfil1 = imfil//"_radec1.cat"; delete(radecfil1, ver-, >& "dev$null")
        imexfil = "canwcshead.imexam"; delete(imexfil, ver-, >& "dev$null")

        print("# Filename: ", imfil, >> regfil)
        print("global color=green font=\"helvetica 10 normal\" select=1 highlite=1 edit=1 move=1 delete=1 include=1 fixed=0 source \nfk5", >> regfil)
        i2 = 0
        i3 = 0
        list = catfil
        while( fscan(list,line) != EOF ) {
        c1 = substr(line,1,3)
        #print(line, >> radecfil)
        if (c1=="---") {
        while( fscan(list,line) != EOF ) {
        i2 = fscan(line,ra,dec)
          i3 = i3 + 1
          cdata[i3] = line
          cid[i3] = str(i3)
          a_ra[i3] = ra; a_dec[i3] = dec
        if (i3 <= imark) {
        creg = "# color=red text={"//cid[i3]//"}"
        printf("circle(%10f,%10f,%4f\") %s\n",a_ra[i3],a_dec[i3],rad, creg, >> regfil)
        printf("%15.7f %15.7f \n",a_ra[i3],a_dec[i3], >> radecfil1)
        #printf("%15.7f %15.7f %8.3f %8.3f %8.3f %s\n",a_ra[i3],a_dec[i3],a_jmag[i3],a_hmag[i3],a_kmag[i3],cid[i3], >> radecfil1)
        }
        }
        }
        }

        if (staridint) {
        #print ("Display 2MASS image on DS9 frame 1 using file option and overlay the region file ", regfil, " and note the ID of the calibration star")
        #print ("When ready press ENTER")
        #scan (c1)

	print(" -----------------  ")
        print("RaDec of Ref Star [#",sid, "] is ",a_ra[1],a_dec[1])
	print(" -----------------  ")
        print ("Press letter a on the Reference star and Quit with q")
        display (imfil, 1, fill+, zs+)
        imexamin (frame=1, keeplog+, logfile=imexfil)

        list = imexfil
        while( fscan(list,line) != EOF ) {
        c1 = substr(line,1,1)
        if (c1!="#") {
          i2 = fscan(line,xx,yy,ddd,ddd,ddd,amag)
        }
        }
        raref = a_ra[sid]; decref = a_dec[sid]
        print("RA=",raref," Dec=",decref," ID=",sid," X=",xx," Y=",yy," Mag_instr= ",amag)
        } else {
#-----------------------------------------------
        list = starid
        while( fscan(list,line) != EOF ) {
        c1 = substr(line,1,1)
        if (c1!="#") {
          i2 = fscan(line,raref, decref, xx,yy)
          print (raref, " ", decref, " ", xx," ", yy)
                goto endfile
                     }
        }
        endfile:
	}


        mosfil = imfil
    hedit (mosfil, fields="CTYPE1", value="RA---SIN", add+, delete-, verify-, show-, update+)
    hedit (mosfil, fields="CTYPE2", value="DEC--SIN", add+, delete-, verify-, show-, update+)
    hedit (mosfil, fields="CRVAL1", value=raref, add+, delete-, verify-, show-, update+)
    hedit (mosfil, fields="CRVAL2", value=decref, add+, delete-, verify-, show-, update+)
    hedit (mosfil, fields="CRPIX1", value=xx, add+, delete-, verify-, show-, update+)
    hedit (mosfil, fields="CRPIX2", value=yy, add+, delete-, verify-, show-, update+)
    hedit (mosfil, fields="CDELT1", value=rapix, add+, delete-, verify-, show-, update+)
    hedit (mosfil, fields="CDELT2", value=decpix, add+, delete-, verify-, show-, update+)
    hedit (mosfil, fields="CROTA1", value=rotang, add+, delete-, verify-, show-, update+)
    hedit (mosfil, fields="CROTA2", value=rotang, add+, delete-, verify-, show-, update+)
    hedit (mosfil, fields="EQUINOX", value="2000", add+, delete-, verify-, show-, update+)
    #hedit (mosfil, fields="ZEROMAG", value=zpmag, add+, delete-, verify-, show-, update+)

	if (check_wcs) {
start_check:

	wcsctran (radecfil1, photcoo1, mosfil, "world", "logical", verbose-)

        list = photcoo1
        while( fscan(list,line) != EOF ) {
        c1 = substr(line,1,1)
        if (c1!="#") {
          i2 = fscan(line, ix,iy)
	  if (ix > 0 && ix < nx && iy > 0 && iy < ny) print(ix, iy , >> photcoo)
                     }
        }
	delete(photcoo1, ver-, >& "dev$null")

	wcsok = yes
        display (mosfil, 1, zs+, fill+, >& "dev$null")
        tvmark (1, photcoo, mark="circle", radii=10, number-, nxoff=3, nyoff=3, txsize=2)
	print("Is the WCS ok? [yes] ")
	scan(wcsok)
	if (!wcsok) {
		print("Enter new Rotation angle [", rotang, "] : " ) 
		scan(rotang)
    hedit (mosfil, fields="CROTA1", value=rotang, add+, delete-, verify-, show-, update+)
    hedit (mosfil, fields="CROTA2", value=rotang, add+, delete-, verify-, show-, update+)
		delete(photcoo, ver-, >& "dev$null")
		goto start_check
		}
	print("Final rotation angle is = ",rotang)
	imrota = rotang
	}

#----------------------------------------- photometry part ---------


end

