procedure m81_analyze(catal, object)

#----------------------------------------------------------------------------------------------------------------------
#History: Created on 16th April 2018 from the original version "m81_stars_cmd"    
#
#task to do (1) FIND the RADEC in CATAL of the OBJECT
#           (2) IF ALIGN+, orient the images to the normal astronomical convention
#           (3) TRIM the images such that each side = DISPBOX 
#           (4) iraf.DISPLAY the aligned, trimmed images in IMGLIS in ds9 (max of 9)
#           (5) PLOT a CMD of stars in STARCAT within a RADIUS around the OBJECT
#           (6) MAKE RGB images of aligned, trimmed images using ds9 commands
#           (7) Prepares Halpha image from F606W by subtracting a cont image created using B & I
#   catal = if "", object should contain ID RA DEC [specfil] [radcol]
#   object = name of the object in the catalog  or
#          = ID RA DEC [specfil] [radcol] if catal= "" or   (see example #3)
#          = "all" if all objects in the catalog are to be analyzed
#   catcol should have ID, RA, DEC 
#   specol = column number of spectrum; "" ==> no spectrum 
#   radcol = if a radius column exists a circle of this size is put in reg file; If empty, puts 0.5" radius
#            if -ve, rad=abs(real(radcol)), radcol is also used by cmdplot to select all stars within this radius.
#   imglis = list of multi-band images [order of images in this list is fixed at present]
#
#examples
#1. Analyse the object=HIIB027 in catalog ../cmd/m81bubbles_slits_radec2m.log where ID, RA and DEC are in cloumns 1,4,5, respectively
#  and column=9 contains name of spectral acquisition image
# Note uses the parameters set in plotcmd: plotcmd (148.6312443, 69.1469941, starcat="../cmd/SE_BVI_m81acs05_bvicol_2m.dat", radius=5.,
#  cmddat="plotcmd", ira=1, idec=2, imag=5, icol=7, id=0, magnam="I", colnam="B-I", cmdplot=no)
#
# m81_analyze ../cmd/m81bubbles_slits_radec2m.log HIIB027 catcol="1,4,5" specol="9"
#
#2. same as #1, but without the acquisition image
# m81_analyze ../cmd/m81bubbles_slits_radec2m.log HIIB027 catcol="1,4,5" specol=""
#3. m81_analyze "" "M81HIIB027 148.66482 69.083751 gtc10aob001_object_887ref2m" catcol="1-3" specol="4"
#----------------------------------------------------------------------------------------------------------------------
string  catal  = "../m81csc_bright.cat"      	{prompt="CSC Catalog name"}
string  object = "R06B06945"      	{prompt="Object name"}
string  catcol = "1-3"      		{prompt="Catalog columns ID, RA, DEC, spefil for fields task"}
string  specol = "11"      		{prompt="Spectrum acquisition image name column in catal"}
string  radcol = ""      		{prompt="Bubble radius column no in Catalog"}
string  imglis = "images.lis"		{prompt="File containing list of images to display"}
bool    align  = no        		{prompt="Align the images to NE before display?"}
real    dispbox= 25.0      		{prompt="Box size for display in arcsec"}
bool    imcreat= no        		{prompt="Create PNG images?"}
bool    cmdplot= yes       		{prompt="Plot the CMD of stars?"}
bool    photom  = no        		{prompt="Do photometry of stars inside radcol using IRAF?"}
bool    addcmd  = no        		{prompt="Add COL, MAG columns to catal? [m81_analyze_cmd.cat]"}
string  starcat  = "DEFAULT"      	{prompt="Star Catalog name [plotcmd.starcat] "}
string  rgblis = ""    {prompt="LIST of RGB images [empty or HST_I HST_V HST_B or OSIRIS_r HST_V SDSS_g]"}
string  rgbreg = "-scale log -scale mode zmax"    {prompt="cancat to ds9 command [accepts from @list]"}
string  rgblog = "m81_stars_cmd_rgb.com"	{prompt="File containing ds9 commands to create rgb images"}
string  slitreg= "cmd"				{prompt="Reg file containing Slit positions [cmd/m81_slits.reg]"}  
#string  slitreg= "m81_slits.reg"		{prompt="Reg file containing Slit positions"}
string  apertur    = "0.1,0.2,0.3,0.4,0.5"      {prompt="List of aperture radii in arcsec"}
real    skyannu    = 1.0               		{prompt="Inner radius of sky annulus in arcsec"}
real    skydannu   = 0.5         		{prompt="Width of sky annulus in arcsec"}
int	napert     = 3           		{prompt="Aperture number [3] to be extracted "}
string  zeropoint  = "25.779 26.398 25.501"	{prompt="HST BVI Vega zeropoints"}

begin
	string	cat_name, obj_name, fname, ttt_radec, ttt_xy, c1, name[999], spefil[999]
	string  csc_name, sec, star_cat, zone_filt, tlis, tfil , tfil2, csc_spe
	string	imlis, dum1, dum2, dum3, dum4, mission[99], filnam[99], band[99]
	string	rgbcom, rgbfil, dum5, rgbim[6], dssca, rgbout, dsreg, extn, cm[99]
	string	regslit, catcolform, apcol, spcol, colcmd, comfil, txdump, phot_ap #, szpt
	real	csc_ra, csc_dec, csc_biso, csc_bi, csc_br, csc_b, csc_i, fw_i, ell_i
	real	rad_deg, ra1, ra2, dec1, dec2, xym, disp_size, brad[999] 
	real    tv_rad, im_sca, box_siz, obj_x, obj_y, disp_box, bright_col, bright_mag
	real	ra[999], dec[999], aprad, rad_arcsec, zpvega[4] 
	real	phot_skyannu, phot_skydannu, err_mag, err_col
	int     i, i1, i2, x1, x2, y1, y2, i3, nimages, iobj, idobj, nobj, iobs, iosiris, nstars, iap
	int	nx, ny, ibright
	bool	wcsmos, plotcmd, dis[999], png, wrcmd, phot, wr_rgb2 
	bool	ellipse
        struct  line, comm1, comm2


	cat_name = catal; obj_name = object; star_cat = starcat; rad_arcsec = plotcmd.radfov
	box_siz = dispbox; disp_size = box_siz/2.0; rad_deg = rad_arcsec/3600.   #; disp_res = dispres
	plotcmd = cmdplot; wcsmos = align; wrcmd = addcmd
	rgbcom = rgbreg; rgbfil = rgblis; comfil = rgblog; regslit = slitreg; png = imcreat
	if (strupr(star_cat) == "DEFAULT"){star_cat = plotcmd.starcat}
	#print(star_cat)
	phot = photom
	phot_ap = apertur; phot_skyannu = skyannu; phot_skydannu = skydannu; iap = napert
	#szpt = zeropoint
	print(zeropoint) | fields (fields="1-3") | scan(zpvega[1], zpvega[2], zpvega[3])
	#print(zpvega[1], zpvega[2], zpvega[3])

	imlis = imglis; catcolform = catcol; apcol = radcol; spcol = specol
	#if (spcol != ""){catcolform = catcolform//","//spcol}
	colcmd = str(plotcmd.colcol)//","//str(plotcmd.magcol)

	tlis = "ttt_list.lis"
	ttt_radec = "ttt_radec.cat"
	ttt_xy = "ttt_xy.cat"
        delete(ttt_radec, ver-, >& "dev$null")
        delete(ttt_xy, ver-, >& "dev$null")

	#XY positions for IRAF display task 
	#for (i=0;  i <= 8;  i += 1) {xxc[i+1] = 0.2 + 0.3*mod(i,3); yyc[i+1] = 0.8 - 0.3*(i/3)} 
	#for (i=0;  i <= 8;  i += 1) {print(xxc[i+1], yyc[i+1]}

	if (apcol == ""){aprad = 0.5}
	if (real(apcol) <= 0.0){aprad = abs(real(apcol))}
	if (cat_name != ""){
#-------Read RA-DEC of all Objects from the Catalog ------------------------------
        iobj = 0
        list = cat_name
        while( fscan(list,line) != EOF ) {
        c1 = substr(line,1,1)
        if (c1!="#") {
	  #print (line) | fields(fields=catcolform) | scan (csc_name, csc_ra, csc_dec, csc_spe)
	  print (line) | fields(fields=catcolform) | scan (csc_name, csc_ra, csc_dec)
          iobj = iobj + 1; name[iobj]=csc_name; ra[iobj]=csc_ra; dec[iobj]=csc_dec; spefil[iobj]="null"  #; brad[iobj]=aprad
	  if (spcol != ""){print (line) | fields(fields=spcol) | scan (csc_spe); spefil[iobj]=csc_spe}
	  if (apcol != ""){print (line) | fields(fields=apcol) | scan (aprad)} 
	  brad[iobj]=aprad
          if (csc_name == obj_name)  {idobj = iobj; print (csc_ra, csc_dec, " ", csc_name, >> ttt_radec) }
	  #if (csc_name == obj_name && apcol != ""){print (line) | fields(fields=apcol) | scan (aprad)}
        }
        }
        } else {
	  print (obj_name) | fields(fields=catcolform) | scan (csc_name, csc_ra, csc_dec)
          iobj = 1; name[iobj]=csc_name; ra[iobj]=csc_ra; dec[iobj]=csc_dec; spefil[iobj]="null"; brad[iobj]=aprad
          if (spcol != ""){print (obj_name) | fields(fields=spcol) | scan (csc_spe); spefil[iobj]=csc_spe}
	  if (apcol != ""){print (obj_name) | fields(fields=apcol) | scan (aprad); brad[iobj]=aprad}
	  #obj_name = csc_name; if (csc_name == obj_name)  {}
	  idobj = iobj; print (csc_ra, csc_dec, " ", csc_name, >> ttt_radec) 
	}
	#print(idobj, " ", spefil[idobj])

        if (obj_name == "all"){nobj = iobj} else {nobj = 1}
        for (iobj=1; iobj <= nobj; iobj +=1) {
        if (obj_name == "all"){
                csc_name=name[iobj]; csc_ra=ra[iobj]; csc_dec=dec[iobj]; csc_spe=spefil[iobj]; aprad = brad[iobj]
                delete(ttt_radec, ver-, >& "dev$null")
                print (csc_ra, csc_dec, " ", csc_name, >> ttt_radec)
        } else {
		csc_name=name[idobj]; csc_ra=ra[idobj]; csc_dec=dec[idobj]; csc_spe=spefil[idobj]; aprad=brad[idobj]
	}
	
#-------Read imagenames and scale parameter from their headers-------------------------
        i3 = 0
        list = imlis
        while( fscan(list,line) != EOF ) {c1 = substr(line,1,1)
        if (c1!="#") { i2 = fscan(line, dum1, dum2, dum3, dum4, dum5) 
	              i3 = i3 + 1; mission[i3] = dum1; band[i3] = dum2; filnam[i3] = dum3//dum4
	              if (strlwr(dum4) =="auto") {filnam[i3] = dum3//substr(csc_name,1,3)//band[i3]} 
		      if (int(dum5) == 1){dis[i3] = yes} else {dis[i3] = no}
			iosiris = 0
		      if (strupr(mission[i3]) == "OSIRIS"){filnam[i3] = dum3//csc_spe; iosiris = i3}
	            }}
        nimages = i3
        print(" ======> Number of images ",nimages, " Object: ", csc_name, " RA=", csc_ra, " DEC=",csc_dec)
        #print("Number of images ",nimages, " ", filnam[i3], " ", csc_name)

	i1 = 0
	for (i3=1;  i3 <= nimages;  i3 += 1) {
	if (imaccess(filnam[i3])){
	dum1 = ""; dum2 = ""
	hselect (filnam[i3], "CDELT1", yes) | scan (dum1)
	if (dum1 == ""){
	hselect (filnam[i3], "CD1_1", yes) | scan (dum1)
	hselect (filnam[i3], "CD1_2", yes) | scan (dum2)
	}
	if (dum2 != ""){im_sca = 3600.*sqrt(real(dum1)**2 + real(dum2)**2)} else { im_sca = 3600.*abs(real(dum1)) }
	disp_box = int(disp_size/im_sca)
	#print(im_sca, " ", disp_box, " ", filnam[i3], " ", dum1, " ", dum2)
        #hselect (tfil, "NAXIS1 NAXIS2", yes) | scan (nx, ny)

        #hselect (infil, "NAXIS1 NAXIS2", yes) | scan(x2, y2)
        #x1 = 50.0; y1 = 50.0; x2 = x2 - 50.0; y2 = y2 - 50.0



	tfil2 = csc_name//"_"//mission[i3]//"_"//band[i3]
	imdelete(tfil2, ver-, >& "dev$null")

#-------Align the images if asked to do so & obtain the XY pixels corresponding ----------
#-------to the RADEC of the object in each image------------------------------------------

        if (wcsmos) {
	delete(tlis, ver-, >& "dev$null");  print(filnam[i3], > tlis); tfil = "ttt_align"
	wcsmosaic (imlist=tlis, ofile=tfil, rasize=box_siz, decs=box_siz, raval=csc_ra, decval=csc_dec, rapix=0.0, decpix=0.0, scale=im_sca, northpa=0.0, constan=0.0, mosaic-, verbo-) 
	imrename (tfil//"_wcsmosaic_aligned_1_a", tfil2, verbose-)
	           } else {
        delete(ttt_xy, ver-, >& "dev$null")
	wcsxymatch (filnam[i3], filnam[i3], ttt_xy, coords=ttt_radec, verbose-)
        fields (ttt_xy, "1,2", lines=11) | scan (obj_x, obj_y)
        x1 = int(obj_x - disp_box + 1); x2 = int(obj_x + disp_box + 1)
        y1 = int(obj_y - disp_box + 1); y2 = int(obj_y + disp_box + 1)
        hselect (filnam[i3], "NAXIS1 NAXIS2", yes) | scan (nx, ny)
	#x1 = max(x1, 1); y1 = max(y1, 1); x2 = min(x2, nx); y2 = min(y2, ny)
        sec = "["//str(x1)//":"//str(x2)//","//str(y1)//":"//str(y2)//"]"
	if (x1 > 1 && x2 < nx && y1 > 1 && y2 < ny){
	imcopy (filnam[i3]//sec, tfil2, verbose-)
	} else { 
	bright_col = 99.999; err_col = 99.999; bright_mag = 99.999; err_mag = 99.999; nstars = 0; ibright = 0
	#if (phot && wrcmd){print(csc_name, "  ", csc_ra, csc_dec, aprad, nstars, bright_col, err_col, bright_mag, err_mag, >> "m81_analyze_cmd0.cat")}
	if (phot && wrcmd){printf("%s %12.6f %12.6f %8.3f %4d %8.3f %8.3f %8.3f %8.3f %4d\n", csc_name, csc_ra, csc_dec, aprad, nstars, bright_col, err_col, bright_mag, err_mag, ibright, >> "m81_analyze_cmd0.cat")}
	print("Object section ", sec, " outside the frame size ", nx, ny)
	goto next_obj
	}
		}

	tfil = tfil2
	#Stores the X-Y pixel coordinates corresponding to the given RA-DEC in the image [i3] 
        delete(ttt_xy, ver-, >& "dev$null")
	wcsxymatch (tfil, tfil, ttt_xy, coords=ttt_radec, verbose-)
        fields (ttt_xy, "1,2", lines=11) | scan (obj_x, obj_y)
        delete(ttt_xy, ver-, >& "dev$null"); print(obj_x, obj_y, " ", band[i3], > ttt_xy) 

	}    #imaccess loop
	}    #i3 loop

#----------------------------------------------
#.......Prepare a reg file with a circle of radius=aprad at the position of the given RA-DEC
	xytoreg (ttt_radec, regf=csc_name//"_bubble.reg", xc=1, yc=2, idc=3, shap="circ", cirr=aprad, color="blue", xyun="degree", sizu="arcsec")
	#ttt_radec = csc_name//"_bubble.reg"

	#plotcmd_new = yes
#-------Plots CMD if asked to do so---------------------------------------------
	if (plotcmd){
	#cmddat = csc_name//"_cmd.dat"
	plotcmd(csc_ra, csc_dec, starcat=star_cat, cmddat=csc_name//"_cmd", cmdplot+, objrad=aprad, sortmag-) 
	cat (csc_name//"_bubble.reg", >> csc_name//"_cmd.reg")

	#tinfo (csc_name//"_cmd0.dat", ttout-)
	#nstars = tinfo.nrows
	#if (plotcmd.ncmd > 0){
	#fields ("plotcmd_bright_stars.log", fields="1", lines="1") | scan (nstars)
	#fields ("plotcmd_bright_stars.log", fields="1-2", lines="2") | scan (bright_col, bright_mag)
	#}
	#if (wrcmd){print(csc_name, "  ", csc_ra, csc_dec, aprad, nstars, bright_col, bright_mag, >> "m81_analyze_cmd.cat")}

	}

#-------	Prapare Ha image
        for (i3=1;  i3 <= nimages;  i3 += 1) { cm[i3] = csc_name//"_"//mission[i3]//"_"//band[i3]//".fits" }
	if (!imaccess(cm[21])){cm[21] = cm[8]}
	cm[nimages+1] = csc_name//"_"//mission[1]//"_Ha.fits"

        imdelete(cm[nimages+1], ver-, >& "dev$null")
	imexpr ("(a>0.0 && b>0.0 && c>0.0) ? a - 2.0*b**0.55*c**0.45 : a", cm[nimages+1], cm[2], cm[3], cm[1], verbose-)

#-------- Do photometry on the HST images for the stars in the file csc_name//"_cmd0.dat"
	nstars = plotcmd.ncmd
        if (phot){
        if (nstars > 0){
	txdump = "MAG["//str(iap)//"] MERR["//str(iap)//"]"
	delete("m81_analyze_photfiles.lis", ver-, >& "dev$null")
	#zpvega[1] = 25.779; zpvega[2] = 26.398; zpvega[3] = 25.501 
	for (i=1;  i <= 4;  i += 1) {
	if (i < 4) {tfil = csc_name//"_"//mission[i]//"_"//band[i]} else {tfil = csc_name//"_"//mission[2]//"_Ha"; zpvega[i] = zpvega[2]}
	delete(tfil//".mag.*", ver-, >& "dev$null")
        #phot_ap = apertur; phot_skyannu = skyannu; phot_skydannu = skydannu; iap = napert
	phot (tfil, zmag=zpvega[i], coords=csc_name//"_cmd0.dat", interac-, wcsin="world", scale=0.05, verify-, verbose-, itime=1.0, apertur=phot_ap, annulus=phot_skyannu, dannulus=phot_skydannu, salgori="median", calgori="none")
	#phot (tfil, zmag=zpvega[i], coords=csc_name//"_cmd0.dat", interac-, wcsin="world", scale=0.05, verify-, verbose-, itime=1.0, apertur="0.1,0.2,0.3,0.4,0.5", annulus=1.0, dannulus=0.2, salgori="median", calgori="none")

	#txdump (tfil//".mag.1", "MAG[1-5] ID IMAGE", yes, headers-, parameters-)
	delete(tfil//".mag", ver-, >& "dev$null")
	if (i < 4){txdump (tfil//".mag.1", txdump, yes, headers-, parameters-, > tfil//".mag")} else {
	txdump (tfil//".mag.1", txdump//" ID", yes, headers-, parameters-, > tfil//".mag")}
	print(tfil//".mag", >> "m81_analyze_photfiles.lis")
	}
	delete(csc_name//"_all.mag", ver-, >& "dev$null")
        joinlin ("@m81_analyze_photfiles.lis", output = csc_name//"_all.mag", delim="   ", missing = "Missing", maxchar=161, shortest+, verbo-)
	tcalc (csc_name//"_all.mag", "c10", "c1-c3", datatype = "real", colunits="", colfmt="%8.3g")
	tcalc (csc_name//"_all.mag", "c11", "sqrt(c2**2+c4**2)", datatype = "real", colunits="", colfmt="%8.3g")
	tcalc (csc_name//"_all.mag", "c12", "c1-c5", datatype = "real", colunits="", colfmt="%8.3g")
	tcalc (csc_name//"_all.mag", "c13", "sqrt(c2**2+c6**2)", datatype = "real", colunits="", colfmt="%8.3g")
	tcalc (csc_name//"_all.mag", "c14", "c3-c7", datatype = "real", colunits="", colfmt="%8.3g")
	tcalc (csc_name//"_all.mag", "c15", "sqrt(c4**2+c8**2)", datatype = "real", colunits="", colfmt="%8.3g")
	tsort(csc_name//"_all.mag", columns = "c1", ascend+, casesens+)
	fields (csc_name//"_all.mag", fields="12,13,1,2,9", line="1-1") | scan(bright_col, err_col, bright_mag, err_mag, ibright)
	if (wrcmd){printf("%s %12.6f %12.6f %8.3f %4d %8.3f %8.3f %8.3f %8.3f %4d\n", csc_name, csc_ra, csc_dec, aprad, nstars, bright_col, err_col, bright_mag, err_mag, ibright, >> "m81_analyze_cmd.cat")}
	} else {
	bright_col = 99.999; err_col = 99.999; bright_mag = 99.999; err_mag = 99.999
	if (wrcmd){printf("%s %12.6f %12.6f %8.3f %4d %8.3f %8.3f %8.3f %8.3f %4d\n", csc_name, csc_ra, csc_dec, aprad, nstars, bright_col, err_col, bright_mag, err_mag, ibright, >> "m81_analyze_cmd0.cat")}
	print("No stars present")
	}
	}

#-------Prepares a Figure of Multi-band view using ds9 and save it as png---------------------------------------------

	#ttt_radec = csc_name//"_slits.reg"
	ttt_radec = csc_name//"_bubble.reg"
	comm1 = "-view colorbar no -scale mode zscale -height 400 -width 400 -tile yes "//cm[1]//" -frame new "//cm[22]//" -frame new "//cm[6]//" -frame new "//cm[10]//" -frame new "//cm[13]
	comm2 = " -frame new "//cm[14]//" -frame new "//cm[15]//" -frame new "//cm[17]//" -frame new "//cm[21]//" -zoom to fit -match frame wcs -regions load all "//ttt_radec//" -saveimage "//csc_name//"_all.png "
	#print("ds9 ", comm1//comm2, >> "m81_stars_cmd_rgb.com")
	print("ds9 ", comm1//comm2, >> comfil)
        delete("temp.com", ver-, >& "dev$null")
	print("ds9 ", comm1//comm2, " -exit", >> "temp.com")
	if (!deftask("source")){task $source = "$foreign"}
	print(" Multi-band view : ", csc_name//"_all.png" )
	if (png && !deftask("ds9")){task $ds9 = "$foreign"}
	if (png) {source "temp.com"}

#	Create Stampsize HST RGB images (5"=100 pixels)  --------------------------------

        for (i3=1;  i3 <= 3;  i3 += 1) {
        delete(tlis, ver-, >& "dev$null")
        print(filnam[i3], > tlis)
	tfil2 = csc_name//"_"//mission[i3]//"_"//band[i3]//"2"
	#tfil2 = csc_name//"_"//mission[i3]//"_"//band[i3]  #//"2"
        imdelete(tfil2, ver-, >& "dev$null")
        tfil = "ttt_align"
        delete(tfil, ver-, >& "dev$null")
	imanal.imscale(filnam[i3], verbose-)
        wcsmosaic (imlist=tlis, ofile=tfil, rasize=10.0, decs=10.0, raval=csc_ra, decval=csc_dec, rapix=0.0, decpix=0.0, scale=imanal.imscale.scale, northpa=0.0, constan=0.0, mosaic-, verbo-)
        imrename (tfil//"_wcsmosaic_aligned_1_a", tfil2, verbose-)
	rgbim[i3] = tfil2
        }

	wr_rgb2 = no
	if (wr_rgb2) {
	rgbout = csc_name//"_rgb2"
        dssca = " -view colorbar no -scale log -scale mode zmax -height 100 -width 100 "
        #dsreg = " -regions load m81_sscs_gtc_all.reg "
        #dsreg = " -regions load m81_acs_clusters_2m.reg "
        dsreg = " -regions load "//ttt_radec//" "
        extn = ".fits "
        comm1 = dssca//" -rgb -red "//rgbim[3]//extn//"-green "//rgbim[2]//extn//"-blue "//rgbim[1]//extn//dsreg//" -saveimage "//rgbout//".png -exit"
	#print(comm1)
	#print("ds9 ", comm1, >> "m81_stars_cmd_rgb.com")
	print("ds9 ", comm1, >> comfil)
	#if (!deftask("ds9")){task $ds9 = "$foreign"}
	print(" RGB image ["//rgbim[3]//" "//rgbim[2]//" "//rgbim[1]//"] with log scale: ", rgbout//".png" )
	if (png) {ds9 " "//comm1 }
	}

#-------Prepares RGB image using ds9 and save it as png---------------------------------------------
	if (rgbfil != "") {
	rgbout = csc_name
	for (i=1;  i <= 3;  i += 1) {
        print(rgbfil) | fields (fields=i, lines=1) | scan (tfil)
	rgbim[i] = csc_name//"_"//tfil
	#imhead (rgbim[i])
	rgbout = rgbout//"_"//tfil
	}
	}
	#rgbout = csc_name//"_rgb"
	rgbout = csc_name//"_cmd"

	#ttt_radec = csc_name//"_slits.reg"
	#cat ("m81_slits.reg", >> "ttt_radec.reg")
	#if (access(regslit)){cat (regslit, >> "ttt_radec.reg")}
	if (access(regslit)){ttt_radec = csc_name//"_slits.reg"; cat (regslit, >> ttt_radec)}
	#if (regslit == "cmd" && access(csc_name//"_cmd.reg")){cat (csc_name//"_cmd.reg", >> ttt_radec)}
	if (regslit == "cmd"){ttt_radec = csc_name//"_cmd.reg"}
        tlis = "ds9.com"
        delete(tlis, ver-, >& "dev$null")
        #dssca = " -view colorbar no -scale mode zscale -zoom 4 -height 400 -width 400 "
        dssca = " -view colorbar no -scale log -scale mode zmax -zoom 4 -height 400 -width 400 "
        #dsreg = " -regions load m81_slits.reg -regions load ttt_radec.reg "
        #dsreg = " -regions load ttt_radec.reg "
        dsreg = " -regions load "//ttt_radec//" "
        extn = ".fits "
        if (imaccess(rgbim[1]) && imaccess(rgbim[2]) && imaccess(rgbim[3])) {
        comm1 = dssca//" -rgb -red "//rgbim[3]//extn//"-green "//rgbim[2]//extn//"-blue "//rgbim[1]//extn//dsreg//" -saveimage "//rgbout//".png -exit"
	#print("ds9 ", comm1, >> "m81_stars_cmd_rgb.com")
	print("ds9 ", comm1, >> comfil)
	if (!deftask("ds9")){task $ds9 = "$foreign"}
	print(" RGB image ["//rgbim[3]//" "//rgbim[2]//" "//rgbim[1]//"] with linear scale and slits: ", rgbout//".png" )
	if (png) {ds9 " "//comm1 }
	#print(" RGB image with slits: ", rgbout//".png" )
	}

	if (access(cm[iosiris])){
	ttt_radec = csc_name//"_slits.reg"
	if (regslit == "cmd"){ttt_radec = csc_name//"_bubble.reg"}
        #dsreg = " -regions load ttt_radec.reg "
        dsreg = " -regions load "//ttt_radec//" "
	#comm1 = "-view colorbar no -scale mode zscale -height 400 -width 400 "//cm[21]//" -zoom to fit "//dsreg//" -saveimage "//csc_name//"_slits.png -exit"
	comm1 = "-view colorbar no -scale mode zscale -height 400 -width 400 "//cm[iosiris]//" -zoom to fit "//dsreg//" -saveimage "//csc_name//"_slits.png -exit"
	#print("ds9 ", comm1, >> "m81_stars_cmd_rgb.com")
	print("ds9 ", comm1, >> comfil)
	print(" OSIRIS Acquisition image with slits: ", csc_name//"_slits.png" )
	if (png) {ds9 " "//comm1}
        }
next_obj:
        }

#ellipse_analysis:
#	ellipse = yes
#	ttt_xy = "ttt_xy.cat"
#	ttt_xy1 = "ttt_xy1.cat"
#	ttt_xy2 = "ttt_xy2.cat"
#        delete(ttt_xy, ver-, >& "dev$null")
#        wcsxymatch (filnam[1], filnam[1], ttt_xy, coords=cat_name, xc=2, yc=3, verbose-)
#        fields (cat_name, "1", lines="1-", >> ttt_xy1) 
#        fields (ttt_xy, "1,2", lines="11-", >> ttt_xy2) 
#        delete(ttt_xy, ver-, >& "dev$null")
#	joinlin (ttt_xy1//","//ttt_xy2, output=ttt_xy)


end

