procedure m81_stars_cmd(catal, object)

#----------------------------------------------------------------------------------------------------------------------
#task to do (1) FIND the RADEC in CATAL of the OBJECT
#           (2) IF ALIGN+, orient the images to the normal astronomical convention
#           (3) TRIM the images such that each side = DISPBOX 
#           (4) iraf.DISPLAY the aligned, trimmed images in IMGLIS in ds9 (max of 9)
#           (5) PLOT a CMD of stars in STARCAT within a RADIUS around the OBJECT
#           (6) MAKE RGB images of aligned, trimmed images using ds9 commands
#   catal = if "", object should contain ID RA DEC [specfil]
#   catcol should have minimum ID, RA, DEC
#   specol = column number of spectrum; 0 ==> mo spectrum 
#   radcol = if a radius column exists a circle of this size is put in reg file; otherwise puts 0.5" radius
#		radcol is also used by cmdplot to select all stars within this radius.
#   imglis = list of multi-band images [order of images in this list is fixed at present]
#
#examples
#1. Analyse the object=HIIB027 in catalog ../cmd/m81bubbles_slits_radec2m.log where ID, RA and DEC are in cloumns 1,4,5, respectively
#  and column=9 contains name of spectral acquisition image
# Note uses the parameters set in plotcmd: plotcmd (148.6312443, 69.1469941, starcat="../cmd/SE_BVI_m81acs05_bvicol_2m.dat", radius=5.,
#  cmddat="plotcmd", ira=1, idec=2, imag=5, icol=7, id=0, magnam="I", colnam="B-I", cmdplot=no)
#
# m81_stars_cmd ../cmd/m81bubbles_slits_radec2m.log HIIB027 catcol="1,4,5" specol="9"
#
#2. same as #1, but without the acquisition image
# m81_stars_cmd ../cmd/m81bubbles_slits_radec2m.log HIIB027 catcol="1,4,5" specol=""
#3. m81_stars_cmd "" "M81HIIB027 148.66482 69.083751 gtc10aob001_object_887ref2m" catcol="1-3" specol="4"
#----------------------------------------------------------------------------------------------------------------------
string  catal  = "../m81csc_bright.cat"      	{prompt="CSC Catalog name"}
string  object = "R06B06945"      	{prompt="Object name"}
string  catcol = "1-3"      		{prompt="Catalog columns ID, RA, DEC, spefil for fields task"}
string  specol = "11"      		{prompt="Spectrum acquisition image name column in catal"}
string  radcol = ""      		{prompt="Bubble radius column no in Catalog"}
string  imglis = "images.lis"		{prompt="File containing list of images to display"}
bool    align  = no        		{prompt="Align the images to NE before display?"}
#bool    display= yes       		{prompt="Display in ds9 using IRAF display?"}
real    dispbox= 25.0      		{prompt="Box size for display in arcsec"}
#real    dispres= 70.0      		{prompt="Parameter controlling display resolution"}
bool    cmdplot= yes       		{prompt="Plot the CMD of stars?"}
string  starcat  = "DEFAULT"      	{prompt="Star Catalog name [plotcmd.starcat] "}
string  rgblis = ""    {prompt="LIST of RGB images [empty or HST_I HST_V HST_B or OSIRIS_r HST_V SDSS_g]"}
string  rgbreg = "-scale log -scale mode zmax"    {prompt="cancat to ds9 command [accepts from @list]"}
string  slitreg= "m81_slits.reg"		{prompt="Reg file containing Slit positions"}

begin
	string	cat_name, obj_name, fname, ttt_radec, ttt_xy, c1, name[999], spefil[999]
	string  csc_name, sec, star_cat, zone_filt, tlis, tfil , tfil2, csc_spe
	string	imlis, dum1, dum2, dum3, dum4, mission[99], filnam[99], band[99]
	string	rgbcom, rgbfil, dum5, rgbim[6], dssca, rgbout, dsreg, extn, cm[99]
	string	regslit, catcolform, apcol, spcol, cmddat, colcmd
	real	csc_ra, csc_dec, csc_biso, csc_bi, csc_br, csc_b, csc_i, fw_i, ell_i
	real	rad_deg, ra1, ra2, dec1, dec2, xym, disp_size  #, disp_res
	#real    tv_rad, im_sca, xxc[99], yyc[99], box_siz, obj_x, obj_y, disp_box
	real    tv_rad, im_sca, box_siz, obj_x, obj_y, disp_box
	real	ra[999], dec[999], aprad, rad_arcsec, bicol, imag
	real	bmag_1, bmag_2, aprad_deg, dist_1, dist_2, dist_i 
	int     i1, i2, x1, x2, y1, y2, i3, nimages, iobj, idobj, nobj, iobs, iosiris
	int	i, inext, ilast
	bool	wcsmos, plotcmd , dis[999]    #, disp  , plotcmd_new
        struct  line, comm1, comm2


	cat_name = catal; obj_name = object; star_cat = starcat; rad_arcsec = plotcmd.objrad  #rad_arcsec = plotcmd.radius
	box_siz = dispbox; disp_size = box_siz/2.0; rad_deg = rad_arcsec/3600.   #; disp_res = dispres
	plotcmd = cmdplot; wcsmos = align  #; disp = display
	rgbcom = rgbreg; rgbfil = rgblis; regslit = slitreg
	if (strupr(star_cat) == "DEFAULT"){star_cat = plotcmd.starcat}
	#print(star_cat)

	imlis = imglis; catcolform = catcol; apcol = radcol; spcol = specol
	if (spcol != ""){catcolform = catcolform//","//spcol}
	colcmd = str(plotcmd.colcol)//","//str(plotcmd.magcol)

	tlis = "ttt_list.lis"
	ttt_radec = "ttt_radec.cat"
	ttt_xy = "ttt_xy.cat"
        delete(ttt_radec, ver-, >& "dev$null")
        delete(ttt_xy, ver-, >& "dev$null")

	#XY positions for IRAF display task 
	#for (i=0;  i <= 8;  i += 1) {xxc[i+1] = 0.2 + 0.3*mod(i,3); yyc[i+1] = 0.8 - 0.3*(i/3)} 
	#for (i=0;  i <= 8;  i += 1) {print(xxc[i+1], yyc[i+1]}

	if (apcol == ""){aprad = 0.5}
	if (cat_name != ""){
#-------Read RA-DEC of all Objects from the Catalog ------------------------------
        iobj = 0
        list = cat_name
        while( fscan(list,line) != EOF ) {
        c1 = substr(line,1,1)
        if (c1!="#") {
	  #print (line) | fields(fields=catcolform) | scan (csc_name, csc_ra, csc_dec, csc_spe)
	  print (line) | fields(fields=catcolform) | scan (csc_name, csc_ra, csc_dec)
          iobj = iobj + 1; name[iobj]=csc_name; ra[iobj]=csc_ra; dec[iobj]=csc_dec; spefil[iobj]="null"  #csc_spe
	  if (spcol != ""){print (line) | fields(fields=spcol) | scan (csc_spe); spefil[iobj]=csc_spe}
          if (csc_name == obj_name)  {idobj = iobj; print (csc_ra, csc_dec, " ", csc_name, >> ttt_radec) }
	  if (csc_name == obj_name && apcol != ""){print (line) | fields(fields=apcol) | scan (aprad)}
        }
        }
        } else {
	  print (obj_name) | fields(fields=catcolform) | scan (csc_name, csc_ra, csc_dec)
          iobj = 1; name[iobj]=csc_name; ra[iobj]=csc_ra; dec[iobj]=csc_dec; spefil[iobj]="null" 
          if (spcol != ""){print (obj_name) | fields(fields=spcol) | scan (csc_spe); spefil[iobj]=csc_spe}
	  #obj_name = csc_name; if (csc_name == obj_name)  {}
	  idobj = iobj; print (csc_ra, csc_dec, " ", csc_name, >> ttt_radec) 
	}
	#print(idobj, " ", spefil[idobj])

        if (obj_name == "all"){nobj = iobj} else {nobj = 1}
        for (iobj=1; iobj <= nobj; iobj +=1) {
        if (obj_name == "all"){
                csc_name=name[iobj]; csc_ra=ra[iobj]; csc_dec=dec[iobj]; csc_spe=spefil[iobj]
                delete(ttt_radec, ver-, >& "dev$null")
                print (csc_ra, csc_dec, " ", csc_name, >> ttt_radec)
        } else {
		csc_name=name[idobj]; csc_ra=ra[idobj]; csc_dec=dec[idobj]; csc_spe=spefil[idobj]
	}
	
#-------Read imagenames and scale parameter from their headers-------------------------
        i3 = 0
        list = imlis
        while( fscan(list,line) != EOF ) {c1 = substr(line,1,1)
        if (c1!="#") { i2 = fscan(line, dum1, dum2, dum3, dum4, dum5) 
	              i3 = i3 + 1; mission[i3] = dum1; band[i3] = dum2; filnam[i3] = dum3//dum4
	              if (strlwr(dum4) =="auto") {filnam[i3] = dum3//substr(csc_name,1,3)//band[i3]} 
		      if (int(dum5) == 1){dis[i3] = yes} else {dis[i3] = no}
			iosiris = 0
		      if (strupr(mission[i3]) == "OSIRIS"){filnam[i3] = dum3//csc_spe; iosiris = i3}
	            }}
        nimages = i3
        print("Number of images ",nimages, " Object: ", csc_name, " RA=", csc_ra, " DEC=",csc_dec)
        #print("Number of images ",nimages, " ", filnam[i3], " ", csc_name)

	i1 = 0
	for (i3=1;  i3 <= nimages;  i3 += 1) {
	if (imaccess(filnam[i3])){
	dum1 = ""; dum2 = ""
	hselect (filnam[i3], "CDELT1", yes) | scan (dum1)
	if (dum1 == ""){
	hselect (filnam[i3], "CD1_1", yes) | scan (dum1)
	hselect (filnam[i3], "CD1_2", yes) | scan (dum2)
	}
	if (dum2 != ""){im_sca = 3600.*sqrt(real(dum1)**2 + real(dum2)**2)} else { im_sca = 3600.*abs(real(dum1)) }
	disp_box = int(disp_size/im_sca)
	#print(im_sca, " ", disp_box, " ", filnam[i3], " ", dum1, " ", dum2)
        #hselect (tfil, "NAXIS1 NAXIS2", yes) | scan (nx, ny)


	tfil2 = csc_name//"_"//mission[i3]//"_"//band[i3]
	imdelete(tfil2, ver-, >& "dev$null")

#-------Align the images if asked to do so & obtain the XY pixels corresponding ----------
#-------to the RADEC of the object in each image------------------------------------------

        if (wcsmos) {
	delete(tlis, ver-, >& "dev$null");  print(filnam[i3], > tlis); tfil = "ttt_align"
	wcsmosaic (imlist=tlis, ofile=tfil, rasize=box_siz, decs=box_siz, raval=csc_ra, decval=csc_dec, rapix=0.0, decpix=0.0, scale=im_sca, northpa=0.0, constan=0.0, mosaic-, verbo-) 
	imrename (tfil//"_wcsmosaic_aligned_1_a", tfil2, verbose-)
	           } else {
        delete(ttt_xy, ver-, >& "dev$null")
	wcsxymatch (filnam[i3], filnam[i3], ttt_xy, coords=ttt_radec, verbose-)
        fields (ttt_xy, "1,2", lines=11) | scan (obj_x, obj_y)
        x1 = int(obj_x - disp_box + 1); x2 = int(obj_x + disp_box + 1)
        y1 = int(obj_y - disp_box + 1); y2 = int(obj_y + disp_box + 1)
        sec = "["//str(x1)//":"//str(x2)//","//str(y1)//":"//str(y2)//"]"
	imcopy (filnam[i3]//sec, tfil2, verbose-)
		}

	tfil = tfil2
	#Stores the X-Y pixel coordinates corresponding to the given RA-DEC in the image [i3] 
        delete(ttt_xy, ver-, >& "dev$null")
	wcsxymatch (tfil, tfil, ttt_xy, coords=ttt_radec, verbose-)
        fields (ttt_xy, "1,2", lines=11) | scan (obj_x, obj_y)
        delete(ttt_xy, ver-, >& "dev$null"); print(obj_x, obj_y, " ", band[i3], > ttt_xy) 

	}    #imaccess loop
	}    #i3 loop

	#plotcmd_new = yes
#-------Plots CMD if asked to do so---------------------------------------------
	if (plotcmd){
	cmddat = csc_name//"_cmd.dat"
	#plotcmd(csc_ra, csc_dec, star_cat, radfov=rad_arcsec, cmddat=cmddat, racol=1, deccol=2, magcol=5, colcol=7, idcol=0, magnam="I", colnam = "B-I", cmdplot-) 
	plotcmd(csc_ra, csc_dec, starcat=star_cat, cmddat=csc_name//"_cmd", cmdplot-)   

#.......Read B mag of the nearest star (1st in the list) and find the next nearest that is brighter than the nearest
	#fields (cmddat, fields="3", lines="1") | scan(bmag_1)
	fields (cmddat, fields=colcmd, lines="1") | scan(bicol, imag)
	bmag_1 = imag + bicol
	tinfo (cmddat, ttout-); bmag_2 = 99.0; i = 0 
	fields (cmddat, fields=str(tinfo.ncols), lines=str(1)) | scan(dist_1)
	print("The nearest star#1 with [B,B-I]= ", bmag_1, bicol, " at a distance of ", dist_1*3600., " arcsec.")
        list = cmddat
        #while(bmag_2 >= bmag_1 && fscan(list,line) != EOF ) {i= i+1; print (line) | fields (fields="3", lines="1") | scan(bmag_2)}
        while(bmag_2 >= bmag_1 && fscan(list,line) != EOF ) {
		i= i+1; print (line) | fields (fields=colcmd, lines="1") | scan(bicol, imag)
		bmag_2 = imag + bicol
	}
	inext = i
	if (inext == tinfo.nrows){print("There are no star brighter than star#1 within the search radius.")} else {
	fields (cmddat, fields=str(tinfo.ncols), lines=str(inext)) | scan(dist_2)
	print("There is a brighter star#",inext, " with [B,B-I]= ", bmag_2, bicol, " at a distance of ", dist_2*3600., " arcsec.")
	}

#       .......Find all stars that are within the radius of the bubble
	aprad_deg = aprad/3600.; i = 0
	#fields (cmddat, fields="8", lines="1") | scan(dist_i)
	fields (cmddat, fields=str(tinfo.ncols), lines="1") | scan(dist_i)
        list = cmddat
        while(dist_i < aprad_deg && fscan(list,line) != EOF ) {i= i+1; print (line) | fields (fields="8", lines="1") | scan(dist_i)}
	ilast = i-1
	print("There are ", ilast, " stars within a radius of ", aprad, " arcsec.")
        delete(csc_name//"_cmd0.dat", ver-, >& "dev$null")
	fields (cmddat, fields="1-", lines="1-"//str(ilast), >> csc_name//"_cmd0.dat") 
	#print(inext, bmag_2, dist_2*3600., ilast, dist_i*3600.)

        fields (cmddat, fields=colcmd, lines="1") | sgraph (marker="circle", pointm+, yflip+, wl=-1.0, wr=5, wb=26, wt=18, xlabel=plotcmd.colnam, ylabel=plotcmd.magnam, crvcol=3., app-)
        if (inext > 1 && inext < tinfo.nrows){fields (cmddat, fields=colcmd, lines=inext) | sgraph (marker="circle", pointm+, yflip+, crvcolo=2, app+)}
        fields (cmddat, fields=colcmd, lines="2-"//str(ilast)) | sgraph (marker="plus", pointm+, yflip+, crvcol=5, szmarke=0.02, app+)
        fields (cmddat, fields=colcmd, lines=str(ilast+1)//"-") | sgraph (marker="point", pointm+, yflip+, crvcol=1, szmarke=0.005, app+)
	}

#----------------------------------------------
#.......Prepare a reg file with a circle of radius=aprad at the position of the given RA-DEC
	xytoreg (ttt_radec, regf=csc_name//"_slits.reg", xc=1, yc=2, idc=3, shap="circ", cirr=aprad, color="blue", xyun="degree", sizu="arcsec")
	ttt_radec = csc_name//"_slits.reg"

        for (i3=1;  i3 <= nimages;  i3 += 1) { cm[i3] = csc_name//"_"//mission[i3]//"_"//band[i3]//".fits" }
	if (!imaccess(cm[21])){cm[21] = cm[8]}
	cm[nimages+1] = csc_name//"_"//mission[1]//"_Ha.fits"

#-------	Prapare Ha image
        imdelete(cm[nimages+1], ver-, >& "dev$null")
	imexpr ("(a>0.0 && b>0.0 && c>0.0) ? a - 2.0*b**0.55*c**0.45 : a", cm[nimages+1], cm[2], cm[3], cm[1], verbose-)

#-------Prepares a Figure of Multi-band view using ds9 and save it as png---------------------------------------------

	comm1 = "-view colorbar no -scale mode zscale -height 400 -width 400 -tile yes "//cm[1]//" -frame new "//cm[22]//" -frame new "//cm[6]//" -frame new "//cm[10]//" -frame new "//cm[13]
	comm2 = " -frame new "//cm[14]//" -frame new "//cm[15]//" -frame new "//cm[17]//" -frame new "//cm[21]//" -zoom to fit -match frame wcs -regions load all "//ttt_radec//" -saveimage "//csc_name//"_all.png "
	print("ds9 ", comm1//comm2, >> "m81_stars_cmd_rgb.com")
        delete("temp.com", ver-, >& "dev$null")
	print("ds9 ", comm1//comm2, " -exit", >> "temp.com")
	if (!deftask("source")){task $source = "$foreign"}
	print(" Multi-band view : ", csc_name//"_all.png" )
	source "temp.com"
	#print("temp.com", >> "m81_stars_cmd_rgb.com")
	#ds9 " "//comm1//comm2

#	Create Stampsize HST RGB images (5"=100 pixels)  --------------------------------

        for (i3=1;  i3 <= 3;  i3 += 1) {
        delete(tlis, ver-, >& "dev$null")
        print(filnam[i3], > tlis)
	#tfil2 = csc_name//"_"//mission[i3]//"_"//band[i3]//"_stamp"
	tfil2 = csc_name//"_"//mission[i3]//"_"//band[i3]//"2"
        imdelete(tfil2, ver-, >& "dev$null")
        tfil = "ttt_align"
        delete(tfil, ver-, >& "dev$null")
	imanal.imscale(filnam[i3], verbose-)
	#box_siz = 5.0; im_sca = imanal.imscale.scale
        wcsmosaic (imlist=tlis, ofile=tfil, rasize=5.0, decs=5.0, raval=csc_ra, decval=csc_dec, rapix=0.0, decpix=0.0, scale=imanal.imscale.scale, northpa=0.0, constan=0.0, mosaic-, verbo-)
        imrename (tfil//"_wcsmosaic_aligned_1_a", tfil2, verbose-)
	rgbim[i3] = tfil2
        }
	#rgbout = csc_name//"_"//mission[i3]//"_stamp"
	rgbout = csc_name//"_rgb2"
        dssca = " -view colorbar no -scale log -scale mode zmax -height 100 -width 100 "
        #dsreg = " -regions load m81_sscs_gtc_all.reg "
        #dsreg = " -regions load m81_acs_clusters_2m.reg "
        dsreg = " -regions load "//ttt_radec//" "
        extn = ".fits "
        comm1 = dssca//" -rgb -red "//rgbim[3]//extn//"-green "//rgbim[2]//extn//"-blue "//rgbim[1]//extn//dsreg//" -saveimage "//rgbout//".png -exit"
	#print(comm1)
	print("ds9 ", comm1, >> "m81_stars_cmd_rgb.com")
	if (!deftask("ds9")){task $ds9 = "$foreign"}
	print(" RGB image ["//rgbim[3]//" "//rgbim[2]//" "//rgbim[1]//"] with log scale: ", rgbout//".png" )
	ds9 " "//comm1

#-------Prepares RGB image using ds9 and save it as png---------------------------------------------
	if (rgbfil != "") {
	rgbout = csc_name
	for (i=1;  i <= 3;  i += 1) {
        print(rgbfil) | fields (fields=i, lines=1) | scan (tfil)
	rgbim[i] = csc_name//"_"//tfil
	#imhead (rgbim[i])
	rgbout = rgbout//"_"//tfil
	}
	}
	rgbout = csc_name//"_rgb"

	#cat ("m81_slits.reg", >> "ttt_radec.reg")
	#if (access(regslit)){cat (regslit, >> "ttt_radec.reg")}
	if (access(regslit)){cat (regslit, >> ttt_radec)}
        tlis = "ds9.com"
        delete(tlis, ver-, >& "dev$null")
        dssca = " -view colorbar no -scale mode zscale -zoom 4 -height 400 -width 400 "
        #dsreg = " -regions load m81_slits.reg -regions load ttt_radec.reg "
        #dsreg = " -regions load ttt_radec.reg "
        dsreg = " -regions load "//ttt_radec//" "
        extn = ".fits "
        if (imaccess(rgbim[1]) && imaccess(rgbim[2]) && imaccess(rgbim[3])) {
        comm1 = dssca//" -rgb -red "//rgbim[3]//extn//"-green "//rgbim[2]//extn//"-blue "//rgbim[1]//extn//dsreg//" -saveimage "//rgbout//".png -exit"
	print("ds9 ", comm1, >> "m81_stars_cmd_rgb.com")
	if (!deftask("ds9")){task $ds9 = "$foreign"}
	print(" RGB image ["//rgbim[3]//" "//rgbim[2]//" "//rgbim[1]//"] with linear scale and slits: ", rgbout//".png" )
	ds9 " "//comm1
	#print(" RGB image with slits: ", rgbout//".png" )
	}

	if (access(cm[iosiris])){
        #dsreg = " -regions load ttt_radec.reg "
        dsreg = " -regions load "//ttt_radec//" "
	#comm1 = "-view colorbar no -scale mode zscale -height 400 -width 400 "//cm[21]//" -zoom to fit "//dsreg//" -saveimage "//csc_name//"_slits.png -exit"
	comm1 = "-view colorbar no -scale mode zscale -height 400 -width 400 "//cm[iosiris]//" -zoom to fit "//dsreg//" -saveimage "//csc_name//"_slits.png -exit"
	print("ds9 ", comm1, >> "m81_stars_cmd_rgb.com")
	print(" OSIRIS Acquisition image with slits: ", csc_name//"_slits.png" )
	ds9 " "//comm1
        }
        }

end

