procedure plotcmd(radeg, decdeg)

#-----------------------------------------------------------------------------------------------------------------
#Plots the CMD that includes all stars within a radius of radfov around [radeg, decdeg] from the catalog = starcat
#Stars within a radius of objrad are identified with + symbols.
#The brightest and the nearest stars are also identified by red and green symbols, respectively
#The color=colnam and magnitude=magnam should be available in columns colcol and magcol, respectively.
#All stars within radfov are sorted by their distance from the given center and stored as cmddat.reg and cmddat.dat files 
#       stars within objrad are sorted by magnitude [magcol] and stored as cmddat0.dat
#   Only "good" stars (as defined by limmag, limcol) are stored
# examples
#1. plotcmd 148.80087 69.183574 objrad=1.0 cmdplot+
#-----------------------------------------------------------------------------------------------------------------
#
real    radeg  = 148.6312443            {prompt="RA in degrees"}
real    decdeg = 69.1469941             {prompt="Dec in degrees"}
#string  starcat= "../R2_R13_F2_F17_BRI_ed.cat"        {prompt="Star Catalog name"}
string  starcat= "../cmd/SE_BVI_m81acs05_bvicol_2m.dat"        {prompt="Star Catalog name"}
real    radfov = 5.0                    {prompt="Radius of the FoV for CMD in arcsec"}
string  cmddat = "plotcmd"        	{prompt="Output Star Catalog name [.dat and .reg]"}
int     racol = 1                  	{prompt="RA column number in starcat"}
int     deccol = 2                 	{prompt="Dec column number in starcat"}
int     magcol = 3                 	{prompt="Mag column number in starcat"}
int     colcol = 7                 	{prompt="Col column number in starcat"}
int     idcol  = 0                  	{prompt="ID  column number in starcat (0==> no id col"}
string  magnam = "B"                 	{prompt="Mag name"}
string  colnam = "B-I"                 	{prompt="Color name"}
bool    cmdplot= yes                    {prompt="Plot the CMD of stars?"}
real    objrad = 0.0                    {prompt="Identify all stars within this radius in CMD [arcsec]"}
real    limmag = 99.0                   {prompt="Limiting good magnitude"}
real    limcol = 7.0                    {prompt="Maximum absolute value of color in column colcol"}
bool    sortmag= yes                    {prompt="Sort the magnitudes in 0.dat?"}
bool	saveeps = no			{prompt="Save eps of CMD when cmdplot=yes?"}
int     ncmd    = 0                  	{prompt="Output number of stars within objrad"}

begin
        string  star_cat, colra, coldec, colcmd, zone_filt, outdat, col, mag 
	string  outdat0, outreg, crad, comm1, cobj2, epsfile 
	real	csc_ra, csc_dec, rad_deg, ra1, ra2, dec1, dec2, badmag, badcol
	real 	bicol, imag, mag_1, mag_2, dist_1, dist_2, dist_i, radobj, rad_arc 
	int	icra, icdec, icmag, iccol, i, inext, ilast, nstars, niter
	bool	plotcmd, savecmd

	csc_ra = radeg; csc_dec = decdeg; rad_arc = radfov; rad_deg = rad_arc/3600.; radobj = objrad   #/cos(csc_dec*PI/180.)
	star_cat = starcat; icra = racol; icdec = deccol; iccol = colcol; icmag = magcol
	colra = "c"//str(icra)
	coldec = "c"//str(icdec)
	colcmd = str(iccol)//","//str(icmag)
	plotcmd = cmdplot
	outdat0 = cmddat; outdat = outdat0//".dat"; outreg = outdat0//".reg"
	id = idcol; col = colnam; mag = magnam
	badmag = limmag; badcol = limcol; savecmd = saveeps
	ilast = 0

        ra1=csc_ra - rad_deg/cos(csc_dec*PI/180.)
        ra2=csc_ra + rad_deg/cos(csc_dec*PI/180.)
        dec1=csc_dec - rad_deg
        dec2=csc_dec + rad_deg
        zone_filt = colra//" > "//ra1//" && "//colra//" < "//ra2//" && "//coldec//" > "//dec1//" && "//coldec//" < "//dec2
	#print(zone_filt)

        delete(outdat, ver-, >& "dev$null")

	print("Searching all stars within a radius of ", rad_arc, " arcsec in the catalog ", star_cat)
	tselect (star_cat, outdat, zone_filt) 
        comm1 = "3600.0*sqrt((("//colra//" - "//csc_ra//")*cos("//csc_dec//"/57.295779513082))**2 + ("//coldec//" - "//csc_dec//")**2)"
	#print(comm1)
	tinfo (outdat, ttout-)

	if (tinfo.nrows > 0) {
	crad = "c"//str(tinfo.ncols + 1)
	#print(crad)
        tcalc (outdat, crad, comm1)
        tsort (outdat, crad, ascend+)
	#comm1 = crad//" <= "//rad_arc
	#comm1 = crad//" <= "//rad_arc
	comm1 = crad//" <= "//rad_arc//" && c"//str(icmag)//" <= "//badmag//" && abs(c"//str(iccol)//") < "//badcol
	#print(comm1)
	crad = "c1"
	for (i = 2; i <= tinfo.ncols+1; i +=1) { crad = crad//" c"//str(i) }
        delete("c"//outdat, ver-, >& "dev$null")
	tquery(outdat, "c"//outdat, comm1, crad, sort="") 
        delete(outdat, ver-, >& "dev$null")
	rename("c"//outdat, outdat, field = "all")
	tinfo (outdat, ttout-)
	print(tinfo.nrows, " stars found in a circle of radius = ", rad_arc, " arcsec centered at RA=", csc_ra, " Dec=", csc_dec)
	xytoreg (outdat, regf=outreg, xc=icra, yc=icdec, idc=id, shap="circ", cirr=0.1, color="blue", xyun="degree", sizu="arcsec")

#.......Read B mag of the nearest star (1st in the list) and find the next nearest that is brighter than the nearest
	
	if (tinfo.nrows > 0){

        #fields (outdat, fields=colcmd, lines="1") | scan(bicol, imag); bmag_1 = imag + bicol
        fields (outdat, fields=colcmd, lines="1") | scan(bicol, mag_1)
        #tinfo (outdat, ttout-); 
	mag_2 = 99.0; i = 0
        fields (outdat, fields=str(tinfo.ncols), lines=str(1)) | scan(dist_1)
        print("The nearest star#1 with [B,B-I]= ", mag_1, bicol, " at a distance of ", dist_1, " arcsec.")
        list = outdat
        while(mag_2 >= mag_1 && fscan(list,line) != EOF ) {i= i+1; print (line) | fields (fields=colcmd, lines="1") | scan(bicol, mag_2)}
        inext = i
        if (inext == tinfo.nrows){print("There are no star brighter than star#1 within the search radius.")} else {
        fields (outdat, fields=str(tinfo.ncols), lines=str(inext)) | scan(dist_2)
        print("There is a brighter star#",inext, " with [B,B-I]= ", mag_2, bicol, " at a distance of ", dist_2, " arcsec.")
        }

	ilast = 1
	if (radobj > 0.0) {
#       .......Find all stars that are within the radius of the bubble
        i = 0
        fields (outdat, fields=str(tinfo.ncols), lines="1") | scan(dist_i)
        list = outdat
        while(dist_i < radobj && fscan(list,line) != EOF ) {i= i+1; print (line) | fields (fields="8", lines="1") | scan(dist_i)}
        ilast = i-1
        print("There are ", ilast, " stars within a radius of ", radobj, " arcsec.")
	if (ilast > 0){
        delete(outdat0//"0.dat", ver-, >& "dev$null")
        fields (outdat, fields="1-", lines="1-"//str(ilast), >> outdat0//"0.dat")
	if (sortmag){tsort (outdat0//"0.dat", "c"//str(icmag), ascend+)}
	}
        }

#-------Plots CMD if asked to do so---------------------------------------------
	if (savecmd){niter = 2} else {niter = 1}
        if (plotcmd){
	for (i = 1; i <= niter; i +=1) { 
	if (i == 1) {dvpar.device = "epshalf"}
	if (i == niter) {dvpar.device = "stdgraph"}
	fields (outdat, fields=colcmd, lines="1") | sgraph (marker="circle", pointm+, yflip+, wl=-1.0, wr=5, wb=26, wt=18, xlabel=col, ylabel=mag, crvcol=3)
	if (ilast > 1) {nstars = ilast} else {nstars = tinfo.nrows} 
        if (inext > 1 && inext <= nstars){fields (outdat, fields=colcmd, lines=inext) | sgraph (marker="circle", pointm+, yflip+, crvcolo=2, app+)}
        if (ilast > 1) {fields (outdat, fields=colcmd, lines="2-"//str(ilast)) | sgraph (marker="plus", pointm+, yflip+, crvcol=5, szmarke=0.02, app+)}
        fields (outdat, fields=colcmd, lines=str(ilast+1)//"-") | sgraph (marker="point", pointm+, yflip+, crvcol=1, szmarke=0.005, app+)
        }
	print("The Nearest star (green), the brightest star (red), the stars with rad < ", radobj, " arcsec (+ symbols) are identified. Others shown as dots.")
	if (savecmd){
	comm1 = " -t sgi*.eps "; ls comm1//" " | fields(fields="1-", lines=1) | scan(epsfile) 
	delete(outdat0//"_cmd.eps", ver-, >& "dev$null")
	rename (epsfile, outdat0//"_cmd.eps", field="all")
	print("CMD stored in ", outdat0//"_cmd.eps")
	}
        }

        #delete("plotcmd_bright_stars.log", ver-, >& "dev$null")
	#print(ilast, >> "plotcmd_bright_stars.log")
	#fields (outdat0//"0.dat", fields=colcmd, lines="1", >> "plotcmd_bright_stars.log")
	} #if (tinfo.nrows > 0)  #at least 1 star present
	} #if outdat is empty
	ncmd = ilast

end
