procedure sedphot(image, coofile)

#--------------------------------------------------------------------
#A script to create seds of clusters using multi-band imaging data.
#original cl file: sedphot.cl
#Major change is reads the image data parameters from an input file
#   instead of from headers
#Outputs are written as ofile.sed, ofile.abmag and ofile.vgmag

#--------------------------------------------------------------------
string image      ="sedphot_input.dat" {prompt="Input fits filename(s)[first file is reference]"}
string  coofile    ="test_radec.coo" {prompt="Input coordinate file name"}
string  outfile    ="sedphot_stars" {prompt="Output file roorname - extns: sed,abmag,vgmag"}
string  catcol     = "1-3"              {prompt="Columns ID, RA, DEC in coofile for fields task"}
string  coosys     = "world"            {prompt="Input coordinate system (logical,tv,physical,world)"}
string  calgori    = "none"             {prompt="Centering algorithm (centroid|gauss|none|ofilter)"}
string  salgori    = "constant"           {prompt="Sky fitting algorithm (median|constant)"}
string  apertur    = "1,2,3,4,5"      {prompt="List of aperture radii in arcsec if coosys=world else in pixels"}
real    skyannu    = 15.0               {prompt="Inner radius of sky annulus in arcsec"}
real    skydannu   = 5.         {prompt="Width of sky annulus in arcsec"}
real    skyvalu    = 0.                 {prompt="Sky value if salgori=constant"}
string  napert     = "[3]"          {prompt="Aperture number [3] to be extracted "}
bool	wreg       = no			{prompt="Write ds9 reg file of coofile?"}
bool	ltvhead    = no			{prompt="Delete LTV1 and LTV2 header parameters before running phot?"}

begin
        string  old_aper, old_calgori, old_salgori, infil, coofil, cootyp, comm
        string  tsections1, tsections2, filnam[20], tfile, cflux, tfile1
        string  tsections3, cmag[199], catcolform, tsedphotcoo
        string  scalekey, fluxtyp, cradius, naper, areatyp, merrtyp, instrum
	string  mission, filter, regfil, cid, id[199], c1, creg, ofile, sunit
        real    old_scale, old_annulus, old_dannulus, old_skyvalu, flux_cal
        real    wx1, wx2, wy1, wy2, imsca, xc, yc, pflux[199]
	real	flux20, zp, zpvega, F_vega, mjysrtocgspix, fnu_jy[199,20], abmag[199,20]
	real	plam[20], rad, ra, dec, fnu_cgs[199], cd21, cd22
	real    skynoise[199,20], skynoise2[199,20], area, merr, gain[20]
        int     nimages, i2, nap, nstars, j, i3
	real	ab, ac, ad, ae, af, ag, ah, ai, aj, ak, pmag[199]
	real	lameff[20], lamunit[20], imscale[20], zpab[20], fnu0[20], fwhm[20] 
	real	skymed[20], skyrms[20], exptime[20], nexp[20], vgmag[199,20]
	string  aa, imfil, ofile_sed, ofile_abm, ofile_vgm, ofile_jy 
        bool    plot, wrreg, del_ltv
        struct  line

	sunit = "pixel"
        infil = image; coofil = coofile; cootyp = coosys; del_ltv = ltvhead   #; scalekey = scahead
	catcolform = catcol
	if (cootyp == "world") {sunit = "arcsec"}
	#plot = yes; naper = napert; fluxtyp = "FLUX"//naper; areatyp = "AREA"//naper
	plot = yes; naper = napert; fluxtyp = "MAG"//naper; areatyp = "AREA"//naper
	merrtyp = "MERR"//naper
	ofile = outfile
	ofile_sed = ofile//".sed"
	ofile_abm = ofile//".abmag"
	ofile_vgm = ofile//".vgmag"
	ofile_jy  = ofile//".jy"
	wrreg = wreg
	tsedphotcoo = "tsedphotcoo.coo"
        delete(tsedphotcoo, ver-, >& "dev$null")

        tsections2 = mktemp ("tmps2")

	#fields (coofil, fields='1-2', line=1-4) 
	fields (coofil, fields='1') | wc | scan(nstars)
        #print(" Nobjects=", nstars)

	if (wrreg) {
	rad = 4.0
        regfil = coofil//".reg"
        delete(regfil, ver-, >& "dev$null")
        print("# Region file format: DS9 version 4.0", >> regfil)
        print("# Filename: ", coofil, >> regfil)
        print("global color=green font=\"helvetica 10 normal\" select=1 highlite=1 edit=1 move=1 delete=1 include=1 fixed=0 source \nfk5", >> regfil)
        }

        i2 = 0
        i3 = 0
        list = coofil
        while( fscan(list,line) != EOF ) {
        c1 = substr(line,1,1)
        if (c1 !="#") {
        print (line) | fields(fields=catcolform) | scan (cid, ra, dec)
        #i2 = fscan(line,cid,ra,dec)
        #i2 = fscan(line,ra,dec,cid)
        #i2 = fscan(line,ra,dec)
          i3 = i3 + 1
	#cid = str(i3)
	id[i3] = cid
        if (wrreg) { creg = "# color=red text={"//cid//"}"
        printf("circle(%10f,%10f,%4f\") %s\n",ra,dec,rad, creg, >> regfil)
	#printf("%10f %10f \n",ra,dec, >> tsections2)
		}
        print(ra, dec, cid, >> tsedphotcoo)
	} } 
	#coofil = tsections2
	#nstars = i3

	
        #save the existing phot pset parameters and replace them with the new ones
        old_scale    = datapars.scale     #datapars.scale    = scale
        old_aper     = photpars.apertur;   photpars.apertur  = apertur
        old_annulus  = fitskypars.annulus;   fitskypars.annulus  = skyannu
        old_dannulus = fitskypars.dannulus;  fitskypars.dannulus = skydannu
        old_skyvalu  = fitskypars.skyvalu;  fitskypars.skyvalu  = skyvalu 
        old_calgori  = centerpars.calgori; centerpars.calgori= calgori 
        old_salgori  = fitskypars.salgori; fitskypars.salgori= salgori

        tsections1 = mktemp ("tmps1")
        tsections3 = mktemp ("tmps3")

	fields (infil, fields='1', > tsections3 ) 
        sections ("@"//tsections3,  option="fullname", > tsections1)
        #sections (infil,  option="fullname", > tsections1)
        nimages = sections.nimages
        print("Number of images ",nimages, " Nobjects=", nstars)

	i3 = 0
        list = infil
	while( fscan(list,line) != EOF ) {
        c1 = substr(line,1,1)
        if (c1!="#") {
	i2 = fscan(line, aa, ab, ac,ad,ae  , af  , ag  , ah    , ai    , aj, ak     )
	#i2 =fscan(line, imfil, lameff, lamunit,imscale,zpab, fnuo, fwhm, skymed, skyrms, exptime, nexp)
          i3 = i3 + 1
	filnam[i3] = aa; lameff[i3] = ab; lamunit[i3] = ac; imscale[i3] = ad; zpab[i3] = ae
	fnu0[i3] = af; fwhm[i3] = ag; skymed[i3] = ah; skyrms[i3] = ai; exptime[i3] = aj; nexp[i3] = ak
	exptime[i3] = 1.0
	plam[i3] = ab*ac/1.0E-10
	#printf("%s %g %8.1f %8.3f %8.3f \n", i3, " ",filnam[i3], " ",lameff[i3], " ",imscale[i3], " ", nexp[i3])
	printf("%3s %15g %8.1f %6.2f %8.1f \n", i3, filnam[i3], plam[i3], imscale[i3], nexp[i3])
	             }
        }
	nimages = i3 
        #print("Number of images ",nimages)

        #delete("sedphot_star1.mag", ver-, >& "dev$null")
        #delete("sedphot_stars.mag", ver-, >& "dev$null")
        delete(ofile_sed, ver-, >& "dev$null")
        delete(ofile_abm, ver-, >& "dev$null")
        delete(ofile_vgm, ver-, >& "dev$null")
        delete(ofile_jy, ver-, >& "dev$null")

        for (i=1; i <= nimages; i +=1) {
        delete(filnam[i]//".mag.1", ver-, >& "dev$null")
	if (del_ltv) {ltvhead(filnam[i], verbose-)}
        #phot(filnam[i], coords=coofil, interac-, wcsin=cootyp, scale=imscale[i], verify-, verbose+, zmag=zpab[i], itime=exptime[i], skyvalu=skymed[i])
        phot(filnam[i], coords=tsedphotcoo, interac-, wcsin=cootyp, scale=imscale[i], verify-, verbose+, zmag=zpab[i], itime=exptime[i], skyvalu=skymed[i])

        delete(filnam[i]//".mag", ver-, >& "dev$null")
        txdump (filnam[i]//".mag.1", "ID "//fluxtyp, "yes", headers-, parameters=yes, >> filnam[i]//".mag")

	for (j=1; j <= nstars; j +=1) {
	id[j] = str(j)
	txdump (filnam[i]//".mag.1", fluxtyp, "ID="//str(j), headers-, parameters=yes) | scan(cmag[j])
	txdump (filnam[i]//".mag.1", areatyp, "ID="//str(j), headers-, parameters=yes) | scan (area)
	txdump (filnam[i]//".mag.1", merrtyp, "ID="//str(j), headers-, parameters=yes) | scan (merr)

	abmag[j,i] = -48.60
	gain[i] = 1.0
	#gain[16] = (1.0/55.0)     #Jy/beam to Jy/pix conversion for 20cm image
	skynoise[j,i] = 1.0
	skynoise2[j,i] = 1.0
	if (cmag[j] != "INDEF") {pmag[j] = real(cmag[j])}
	if (cmag[j] == "INDEF") {pmag[j] = 99.99}
	abmag[j,i] = pmag[j]
	pflux[j] = gain[i]*10**(-0.4*(pmag[j]-zpab[i]))*exptime[i]*nexp[i]
        skynoise[j,i] = sqrt(pflux[j] + area*skyrms[i]**2*exptime[i]*nexp[i])/(pflux[j])
        skynoise2[j,i] = sqrt(area*skyrms[i]**2*exptime[i]*nexp[i])/(pflux[j])
	fnu_cgs[j] = 10**(-0.4*(abmag[j,i]+48.6))
	fnu_jy[j,i] = 1.0E+23*fnu_cgs[j]
	vgmag[j,i] = -2.5*log10(fnu_jy[j,i]/fnu0[i])
	}
	}

	printf("#ID  %8.1f %8.1f %8.1f %8.1f %8.1f %8.1f %8.1f %8.1f %8.1f %8.1f %8.1f %8.1f %8.1f %8.1f %8.1f %8.1f\n", plam[1], plam[2], plam[3], plam[4], plam[5], plam[6], plam[7], plam[8], plam[9], plam[10], plam[11], plam[12], plam[13], plam[14], plam[15], plam[16] , >> ofile_abm)
	printf("#ID  %8.1f %8.1f %8.1f %8.1f %8.1f %8.1f %8.1f %8.1f %8.1f %8.1f %8.1f %8.1f %8.1f %8.1f %8.1f %8.1f\n", plam[1], plam[2], plam[3], plam[4], plam[5], plam[6], plam[7], plam[8], plam[9], plam[10], plam[11], plam[12], plam[13], plam[14], plam[15], plam[16] , >> ofile_vgm)
	printf("#ID  %8.1f %8.1f %8.1f %8.1f %8.1f %8.1f %8.1f %8.1f %8.1f %8.1f %8.1f %8.1f %8.1f %8.1f %8.1f %8.1f\n", plam[1], plam[2], plam[3], plam[4], plam[5], plam[6], plam[7], plam[8], plam[9], plam[10], plam[11], plam[12], plam[13], plam[14], plam[15], plam[16] , >> ofile_jy)

	for (j=1; j <= nstars; j +=1) {
	printf("%4s %8.3f %8.3f %8.3f %8.3f %8.3f %8.3f %8.3f %8.3f %8.3f %8.3f %8.3f %8.3f %8.3f %8.3f %8.3f %8.3f\n", id[j], abmag[j,1], abmag[j,2], abmag[j,3], abmag[j,4], abmag[j,5], abmag[j,6], abmag[j,7], abmag[j,8], abmag[j,9], abmag[j,10], abmag[j,11], abmag[j,12], abmag[j,13], abmag[j,14], abmag[j,15], abmag[j,16], >> ofile_abm )
	printf("%4s %8.3f %8.3f %8.3f %8.3f %8.3f %8.3f %8.3f %8.3f %8.3f %8.3f %8.3f %8.3f %8.3f %8.3f %8.3f %8.3f\n", id[j], vgmag[j,1], vgmag[j,2], vgmag[j,3], vgmag[j,4], vgmag[j,5], vgmag[j,6], vgmag[j,7], vgmag[j,8], vgmag[j,9], vgmag[j,10], vgmag[j,11], vgmag[j,12], vgmag[j,13], vgmag[j,14], vgmag[j,15], vgmag[j,16], >> ofile_vgm )
	printf("%4s %8.3e %8.3e %8.3e %8.3e %8.3e %8.3e %8.3e %8.3e %8.3e %8.3e %8.3e %8.3e %8.3e %8.3e %8.3e %8.3e\n", id[j], fnu_jy[j,1], fnu_jy[j,2], fnu_jy[j,3], fnu_jy[j,4], fnu_jy[j,5], fnu_jy[j,6], fnu_jy[j,7], fnu_jy[j,8], fnu_jy[j,9], fnu_jy[j,10], fnu_jy[j,11], fnu_jy[j,12], fnu_jy[j,13], fnu_jy[j,14], fnu_jy[j,15], fnu_jy[j,16], >> ofile_jy )

	printf("#ID= %g APER%g chosen out of %g in %g SKYrad=%6.2f dSKY=%6.2f \n", id[j], naper, photpars.apertur, sunit, fitskypars.annulus, fitskypars.dannulus, >> ofile_sed)
	printf("#lam[Ang]  Fnu[Jy]    ABMAG   Err_F/F  \n", >> ofile_sed)
        for (i=1; i <= nimages; i +=1) {
	#fnu_jy[j] = 1.0E+23*10**(-0.4*(abmag[j,i]+48.6))
        printf("%6.1f %12.5e %8.3f %8.4f %8.4f \n", plam[i], fnu_jy[j,i], abmag[j,i], skynoise[j,i], skynoise2[j,i], >> ofile_sed)
	}
	}

exit:

        #Return the old phot pset parameters
        datapars.scale    = old_scale;   photpars.apertur  = old_aper
        fitskypars.annulus  = old_annulus; fitskypars.dannulus = old_dannulus
        fitskypars.skyvalu = old_skyvalu
        centerpars.calgori= old_calgori; fitskypars.salgori= old_salgori

        #print(tsections1, tsections2)
        delete(tsections1, ver-, >& "dev$null")
        delete(tsections2, ver-, >& "dev$null")
        delete(tsections3, ver-, >& "dev$null")
        delete("tmps*", ver-, >& "dev$null")

end
