procedure sedphot_input(image)

#--------------------------------------------------------------------
#A script to create seds of clusters using multi-band imaging data.
#KEYWORDS to be added:
#
#hedit m81sdss*.fits fields="MISSION" value="SDSS" add+
#hedit m81irac*.fits fields="MISSION" value="SPITZER" add+
#hedit h_m81-*.fits fields="MISSION" value="GALEX" add+
#hedit 2MASS_MESSIER_*.fits fields="MISSION" value="2MASS" add+
#hedit 2MASS_MESSIER_081_K.fits fields="FILTER" value="K" add+
#hedit 2MASS_MESSIER_081_H.fits fields="FILTER" value="H" add+
#hedit 2MASS_MESSIER_081_J.fits fields="FILTER" value="J" add+
#hedit @m81sed_images.lis fields="SKYRMS" value="" add+
#when coosys != world, coordinates in coofile as well as aperture should be pixels.
#sources of Vega AB
#http://mips.as.arizona.edu/~cnaw/sun.html
#--------------------------------------------------------------------
string image      ="@m81sed_images.lis" {prompt="Input fits filename(s)[first file is reference]"}
string vegafil    ="vegakur_fnu.dat"    {prompt="Vega fnu data file [Ang vs fnu (erg/cm2/s/Hz)]"}
string outfil     ="sedphot_input.dat"    {prompt="Output file"}

begin
	string	infil, vegfil, filnam[99], mission, filter, instru
        string  tsections1, imfil, ofil
	real    exptime, imsca, skyrms, zp, mjysrtocgspix, wave, flux20
	real	F_vega, zpvega, w1, w2, fnu0, fnu0_jy, vab, vab_jy
	real	lunit, fwhm, skymed
        int     nimages, i, i2, nexp

        infil = image; vegfil = vegafil; ofil =  outfil
        tsections1 = mktemp ("tmps1")
        delete(ofil, ver-, >& "dev$null")

        sections (infil,  option="fullname", > tsections1)
        nimages = sections.nimages
        print("Number of images ",nimages)

        list = tsections1
        for (i = 1; i <= nimages; i = i + 1) {
        i2 = fscan(list, imfil)
        filnam[i] = imfil
        #print("Image name ",i, " ",filnam[i])
        }

	tinfo(vegfil, ttout-)
	fields (vegfil, fields="1", line=1) | scan(w1)
	fields (vegfil, fields="1", line=tinfo.nrows) | scan(w2)
	#print(w1, w2)

        for (i=1; i <= nimages; i +=1) {
	inscale(filnam[i], verbose-)
	imsca = inscale.scale                 #datapars.scale = imsca
        hselect (filnam[i], "SKYRMS", yes) | scan (skyrms)
        hselect (filnam[i], "EXPTIME", yes) | scan (exptime)
	imstat (filnam[i], fields="midpt", nclip=3, format-) | scan(skymed)
	lunit = 1.0E-10; nexp = 1; fwhm = 1.0

	mission = ""
        hselect (filnam[i], "MISSION", yes) | scan (mission)
	if (mission == "") {hselect (filnam[i], "TELESCOP", yes) | scan (mission)} 

	if (mission == "GALEX") { 
		hselect (filnam[i], "BAND ZP", yes) | scan (filter, zp) 
		if (filter=="2") {wave=1528; vab = 2.085}
		if (filter=="1") {wave=2271; vab = 1.629}
		}

	if (mission == "SDSS") { 
		hselect (filnam[i], "FILTER FLUX20", yes) | scan (filter, flux20) 
		zp = 20.0 + 2.5*log10(flux20) 
		if (filter=="u") {wave=3551; vab = 0.900}
		if (filter=="g") {wave=4686; vab = -0.125}
		if (filter=="r") {wave=6165; vab = 0.119}
		if (filter=="i") {wave=7481; vab = 0.332} 
		if (filter=="z") {wave=8931; vab = 0.494}
		}

	if (mission == "2MASS") { 
		#Vega flux in K = 6.4E-21 erg/s/cm2/Hz
		#Fk_vega = 6.4E-21; Fh_vega = 10.2E-21; Fj_vega = 15.9E-21
		hselect (filnam[i], "FILTER", yes) | scan (filter) 
		hselect (filnam[i], filter//"MAGZP", yes) | scan (zpvega)
		#if (filter=="K") {hselect (filnam[i], "KMAGZP", yes) | scan (zpvega) } 
		if (filter=="J"){F_vega = 15.94E-21; wave=12350; vab = 0.870}
		if (filter=="H"){F_vega = 10.24E-21; wave=16620; vab = 1.344}
		if (filter=="K"){F_vega = 6.667E-21; wave=21590; vab = 1.814}
		zp = -2.5*log10(F_vega) - 48.6 + zpvega
		}

	if (mission == "SPITZER") { 
		hselect (filnam[i], "INSTRUME", yes) | scan (instru) 
		hselect (filnam[i], "CHNLNUM", yes) | scan (filter) 
		#IRAC mjy/sr -> cgs/pix = 1.0E+06*1.0E-23*(imsca*3600.)**2/4.2545E+10
		#mjysrtocgspix = 2.35E-28*(imsca*3600.)**2
		mjysrtocgspix = 2.35E-28*(imsca)**2
		zp = -2.5*log10(mjysrtocgspix) - 48.6
		if (filter=="1" && instru == "IRAC") {wave=35500; vab = 2.758}
		if (filter=="2" && instru == "IRAC") {wave=44900; vab = 3.232}
		if (filter=="3" && instru == "IRAC") {wave=57300; vab = 3.720}
		if (filter=="4" && instru == "IRAC") {wave=78700; vab = 4.360}
		if (filter=="1" && instru == "MIPS") {wave=236800; vab = 6.731}
		}
	if (mission == "HST") { 
	  hselect (filnam[i], "FILTER1", yes) | scan (filter)
	  if (substr(filter, 1, 2) == "CL"){hselect (filnam[i], "FILTER2", yes) | scan (filter)}
	  if (filter=="F435W") {wave=4317.398; zp = 25.673; vab = -0.129}
	  if (filter=="F606W") {wave=5917.678; zp = 26.486; vab =  0.063}
	  if (filter=="F814W") {wave=8059.761; zp = 25.937; vab =  0.400}
	}

	vab_jy = 1.0E23*10**(-0.4*(vab + 48.6 ))

        delete("sedphot_input.lambda", ver-, >& "dev$null")
	print(wave, > "sedphot_input.lambda")
	fnu0_jy = 0.0
	if (wave >= w1 && wave <= w2){interp (vegfil, "sedphot_input.lambda", int_mode="linear", curve_gen-) | scan (dum1, fnu0); fnu0_jy = fnu0*1.0E+23} 
	printf("%2d %20s %8s %6s %8.1f %8.3f %9.2f %9.2f %12.3e %8.3f\n", i, filnam[i], mission, filter, wave, zp, fnu0_jy, vab_jy, skyrms, imsca)
	printf("%20s %8.1f %10.2e %8.3f %8.3f %9.2f %8.3f %10.2e %12.3e %6d %2d\n", filnam[i], wave, lunit, imsca, zp, vab_jy, fwhm, skymed, skyrms, exptime, nexp, >> ofil)
        }

        delete("temmm.mag", ver-, >& "dev$null")
	
	#printf("ID      %8.1f %8.1f %8.1f %8.1f %8.1f %8.1f %8.1f %8.1f %8.1f %8.1f %8.1f %8.1f %8.1f %8.1f \n", wave[1], wave[2], wave[3], wave[4], wave[5], wave[6], wave[7], wave[8], wave[9], wave[10], wave[11], wave[12], wave[13], wave[14] , >> "temmm.mag")


exit:

        delete(tsections1, ver-, >& "dev$null")
        #delete(tsections2, ver-, >& "dev$null")

end

