procedure wcsmosaic(inlist)

#Script to stitch 2MASS images into one single MOSAIC image based on the WCS information
#in the header. The filenames are given a list, with the first file placed at the center
#of the output MOSAIC image of size nx_tot x ny_tot. Takes the mean of the overlapping
#pixels on adjacent images using the conditions in two input files called "comm.1" and "comm.2".
#cat comm.1
#((a > (a.SKYVAL - 10*a.SKYSIG)) && (b > (b.SKYVAL - 10*b.SKYSIG))) ? ((a-a.SKYVAL)+(b-b.SKYVAL))/2.0 : max(a-a.SKYVAL,b-b.SKYVAL)
#cat comm.2
#((a >  0) && (b > (b.SKYVAL - 10*b.SKYSIG))) ? (a+(b-b.SKYVAL))/2.0 : max(a,b-b.SKYVAL)

string  inlist     ="nube76_2mass_k.lis"  	{prompt="List file containing input fits filenames"}
string  output     ="nube76_2mass_K" {prompt="Output mosaicked image name"}
int	nx_tot     =1536	{prompt="Nx pixels in the desired in the output mosaic"}
int	ny_tot     =3072	{prompt="Ny pixels in the desired in the output mosaic"}
string  filter     = "k"   	{prompt="Use only this filter in the image list"}

begin
	string comm1, size, file_out, tmp, file_in[99], s1, file_in_ref 
	string f_out[99], fsum_out[99], reffilter, cfilter1
        struct  line
	int    nx, ny, i1, i2, nfil
	real	x0, y0

	nx = nx_tot; ny = ny_tot; x0 = real(nx)/2.0; y0 = real(ny)/2.0
	file_out = output; reffilter = filter
        imdelete(file_out, ver-, >& "dev$null")

        i1 = 0
        list = inlist
	#hselect ("@"//list, "$I NAXIS1 NAXIS2 SKYVAL SKYSIG FILTER", expr=yes)
        while( fscan(list, line) != EOF ) {
        tmp = substr (line, 1, 1)
        if (tmp != "#") {
	  i1=i1+1
	  i2 = fscanf(line, "%s ", s1)
	  imgets (s1, "FILTER")
          cfilter1  =(imgets.value)
	  if (reffilter == cfilter1) {
	  file_in[i1] = s1
	hselect (file_in[i1], "$I NAXIS1 NAXIS2 SKYVAL SKYSIG FILTER", expr=yes)
	  #print (i1, " ", file_in[i1])
	  }
	}
	}
	nfil = i1
	file_in_ref  = file_in[1]
	print ("No of files = ", nfil, "Ref = ", file_in_ref)

        size=str(nx)//","//str(ny)
	comm1 = "repl(0.0,"//str(nx)//")"
	#print(comm1)
	imexpr (comm1, file_out, dims=size, verb=yes)
	wcscopy (file_out, file_in_ref, verb=yes)
	hedit(file_out, "CRPIX1", x0, add+, addonly-, delete-, verify-, show-)
	hedit(file_out, "CRPIX2", y0, add+, addonly-, delete-, verify-, show-)

	for (k=1; k <= nfil;  k += 1) {
	f_out[k] = file_in[k]//"_ref"
        imdelete(f_out[k], ver-, >& "dev$null")
	print ("Transforming image ",file_in[k], " using ",  file_in[1], " as Reference")
	wregister (file_in[k], file_out, f_out[k],  wcs="world", transpose=no,
fitgeometry="general", function="polynomial", xxorder=2, xyorder=2,
xxterms="half", yxorder=2, yyorder=2, yxterms="half", reject=INDEF,
calctype="real", geometry="geometric", xsample=1., ysample=1.,
interpolant="linear", boundary="constant", constant=0., fluxconserve=yes,
nxblock=512, nyblock=512, wcsinherit=yes, verbose=no, interactive=no,
graphics="stdgraph", gcommands="")
	}
	fsum_out[2] = file_in[2]//"_sum"
        imdelete(fsum_out[2], ver-, >& "dev$null")
	#imexpr.a = f_out[1]; imexpr.b = f_out[2]; 
	imexpr ("@comm.1", fsum_out[2], f_out[1], f_out[2])
	for (k=3; k <= nfil;  k += 1) {
	fsum_out[k] = file_in[k]//"_sum"
	imdelete(fsum_out[k], ver-, >& "dev$null")
        #imexpr.a = fsum_out[k-1]; imexpr.b = f_out[k]; 
	imexpr ("@comm.2", fsum_out[k], fsum_out[k-1], f_out[k])
        imdelete(f_out[k], ver-, >& "dev$null")
	}
        imdelete(f_out[1], ver-, >& "dev$null")
        imdelete(f_out[2], ver-, >& "dev$null")
        imdelete(file_out, ver-, >& "dev$null")
	imrename (fsum_out[nfil], file_out)
	for (k=2; k <= nfil-1;  k += 1) {
	imdelete(fsum_out[k], ver-, >& "dev$null")
	}

end
