procedure       gcpar(mgc, rh)

real    mgc     = 1.0E5               {prompt="Mass of GC [Mo]"}
real    rh      = 4.0                 {prompt="Half-mass radius [pc]"}
real    cenden  = 100.0               {prompt=" Central mass density [Mo/pc**2]"}
real    mstar   = 0.5                 {prompt="Mean mass of the star [Mo]"}
real    mtolb   = 3.0                 {prompt="Mass-to-light ratio in B-band [Mo/Lo]"}
real    vhrms   = 5.0                 {prompt="Orbital velocity at half-mass radius [km/s]"}
real    cking   = 30.0                {prompt="Concentration parameter rt/rc"}
#--------------- host galaxy parameters
real    mgal    = 1.0E10              {prompt="Mass of the galaxy inside rgal [Mo]"}
real    rgal    = 5.0                 {prompt="Galactocentric/perigagalactic distance [kpc]"}
real    pdisk   = 50.0                {prompt="Disk crossing period [Myr]"}
real    gdisk   = 5.0E-9              {prompt="Vertical acceleration [cm/s**2]"}
real    vz      = 100.0               {prompt="Vertical velocity [km/s]"}
real    eccent  = 0.0                 {prompt="Eccentricity of the GC orbit"}
#real    rcore   = 1.0                   {prompt="Core radius"}
#real    sb0     = 100.0                 {prompt="Central surface density [in Mo/pc**2]"}

begin
	real	p_disk, g_disk, V_z, w_hm, t_sh, r_h, t_rh, m_gc, m_star, n_gc
	real	beta, r_gal, m_gal, eps, r_t0, r_t, r_tp
	real	parsec, msun, year, kms
	real	sigb_0, bzp, const, mu_b, MBmag, mbsun, mbylb
	real	a1, a2, a3, a4, a5, c_k, tccbytrh0, tcc, factor
	real	b1, b2, b3, b4, trh0bytcc_beta, tccbytrh0_beta, tcc_beta

	parsec = 3.086E18; msun = 2.0E+33; year = 3.1558E+07; kms = 1.0E5
	p_disk = pdisk/1000.; g_disk = gdisk; V_z = vz*kms; r_h = rh*parsec
	w_hm   = vhrms*kms/r_h 
	m_gc   = mgc*msun; m_star = mstar*msun; n_gc = m_gc/m_star
	r_gal  = rgal*parsec*1000.; m_gal = mgal*msun; eps = eccent
	
	t_sh = (3.0/4.0)*p_disk*V_z**2 * w_hm**2/g_disk**2
	t_rh = 0.138*sqrt(m_gc)*r_h**1.5 / (((6.672E-8)**0.5)*m_star*log(0.4*n_gc))/(3.1558E+16)
	beta = t_rh/t_sh

	#Tidal shocks
	c_k = log10(cking)
	a1 = 1.3162; a2=-1.9809; a3=3.2910; a4=-1.8401; a5=0.2987
	tccbytrh0 = 10**(a1+a2*c_k+a3*c_k**2+a4*c_k**3+a5*c_k**4)
	tcc = tccbytrh0*t_rh
	#b1 = 0.6413; b2=0.5716; b3=-0.9622; b4=0.2167  #c_k=2.4
	b1 = 1.4912; b2=0.2996; b3=-0.7153; b4=0.1606   #c_k=2.0
	#trh0bytcc_beta = (1.0/tccbytrh0)*(1.0 + b1*beta**b2 + b3*beta**b4)
	#tccbytrh0_beta = 1./trh0bytcc_beta; tcc_beta = tccbytrh0_beta*t_rh
	factor = 1.0 + b1*beta**b2 + b3*beta**b4
	tccbytrh0_beta = tccbytrh0/factor; tcc_beta = tccbytrh0_beta*t_rh
	print(c_k, tccbytrh0, tcc)
	print(factor, tccbytrh0_beta, tcc_beta)
	#print(trh0bytcc_beta, tccbytrh0_beta, tcc_beta)
	
	#Tidal radis 
	r_t0 = r_gal*(0.5*m_gc/m_gal)**(1.0/3.0)/parsec
	r_t  = (2.0/3.0)**(4.0/3.0)*r_t0
	r_tp = (2.0/3.0)*r_gal*(m_gc/m_gal/(3.0 + eps))**(1.0/3.0)/parsec

	#Calculation of observed parameters
	mbsun = 5.48; mbylb = mtolb; sigb_0 = cenden; bzp = 27.05  #; izp = 25.65
	#M_king = (8./9.)*(PI*r_c**2)*sigb_0*c_king
	#L_king = M_king/mbylb
	const = m_gc/msun/sigb_0    # = 8.*PI*r_c**2*c_king/9.
	mu_b = bzp -2.5*log10(sigb_0/mbylb)
	MBmag = mu_b + mbsun - bzp -2.5*log10(const)
	#print(mu_b, MBmag, const)

	print (" ############################################")
	printf (" --------- Input GC and Host galaxy Parameters  ---------------- \n \n")
	#print ("   ")
	printf ("GC_mass=%7.2e Mo, r_h=%4.1f pc, cenden=%7.1f Mo/pc**2, M/Lb=%5.2f\n",m_gc/msun, r_h/parsec, sigb_0, mbylb)
	printf ("Mean stellar mass=%5.2f Mo, w_hm=%7.3e /s\n \n", m_star/msun, w_hm)
	printf ("mgal=%8.3e Mo, rgal=%6.2f kpc, pdisk=%6.2f Myr\n", m_gal/msun, r_gal/parsec/1000., p_disk*1000.)
	printf ("gdisk=%8.3e cm/s**2, V_z=%5.1f km/s, eccent=%4.2f \n \n", g_disk, V_z/kms, eps)
	printf (" --------- Calculated Observational Parameters  ----------------\n \n")
	printf ("GC_MAG=%7.2f, muB_0=%7.2f mag/arcsec**2\n \n", MBmag, mu_b)
	#print (" --------------------------------------------")
	printf (" --------- Calculated Theoretical Parameters  ----------------\n \n")
	#print (" ############################################")
	print (" t_sh_Gyr  t_rh_Gyr  beta    r_t0   r_t    r_tp     N")
	printf ("%8.3f %8.3f %8.3e %6.1f %6.1f %6.1f %8.3e \n", t_sh, t_rh, beta, r_t0, r_t, r_tp, n_gc)

end
