procedure manalyze(filnam)

#A script to analyze megara LCB spectra
#1. Read FIBER positions from the header and write it into a text file (fiber# x y)
#2. Obtain an image integrating each spectrum in a band
#3. Extract the spectra centered at given position(s)

string  filnam    = "haro44_lrb_rss"   {prompt="File Name"}
string  outnam    = "DEFAULT"   {prompt="File Name"}
bool    verbose   = yes         {prompt="Print the data for each fiber"}
string  fibcen    = 300          {prompt="Fiber no or position for spectral extraction"}
int     unicen    = 1            {prompt="Units of fibcen [1=fiber#, 2=fiber xy, 3=imagepix, 4=xy file]"}
real    srad      = 1.5          {prompt="Radius in fiber units for spectral extraction"}
real    funit     = 0.465        {prompt="Fiber units [mm]"}
real    imscale   = 0.62         {prompt="Image scale [arcsec/fiber]"}
real    nsample   = 3.00         {prompt="Oversample factor [pix/fiber]"}
int     fiberr    = 0            {prompt="Fiber asignment error [integer number -5 to 5]"}
string  band      = "none"   	{prompt="File Name for sbands task"}
string  senscurv  = ""   	{prompt="Sensitivity function"}
bool    newimage  = no          {prompt="Create New image?"}
int     nfibers   = ""          {prompt="Number of fibers"}
string  out       = ""          {prompt="Output Text File containing fiber positions"}
string  outreg    = ""          {prompt="Output ds9 Reg File containing fiber positions"}
string  head      = ""          {prompt="Short header for the outfile"}
string  obsmod    = ""          {prompt="Observing mode [MOS/LCB]"}
string	cursor 	  = ""   	{prompt="Graphics cursor input"}

begin
        string 	filin, chead, cfib, obmode, datahead, filout, tmp, obsdate
	string	sband, fibcoo, sensfil, filim, filt, fibstr, sx, sy, suffix, xsec
	int	i, nfib, ix, iy, nx, ny, ifib, nnn, i1, id, unifib, ixx, iyy, j, nlin, iunifib, k
	int	inx, iny, ierr, ix1, ix2, nlam
	real	xfib, yfib, scale, xin, yin, dist, radius, fibscale, flux, nearest_dist
	real	texp, ra, dec, ipa, amass, imsca, nsamp, lam1, dlam, lamwid, lamcen
	real	xa[99], ya[99], xs, xe, ys, ye, scale_ori, lam_cen, filwid
	bool print, skip_readfib, newim
        struct  line

        filin = filnam; filout=outnam; print = verbose; skip_readfib = no
	newim = newimage; sband = band; nsamp = nsample; imsca = imscale
	fibscale = funit; sensfil = senscurv
	scale_ori = fibscale; scale = scale_ori/nsamp
	unifib = unicen; fibcoo = fibcen
	ierr = fiberr
	#ifib = ifiber
	radius = fibscale*srad
	#if (strupr(filout) == "DEFAULT"){filout = filin//"_"//str(ifib)}
        datahead = "##i   xfib   yfib   IX   IY"

	#if (strlwr(sband) == ""){
	#slist (filin//"[0]", ap="1", long-) | fields (field="6-8") | scan (lam1, dlam, nlam)
	#lamcen = lam1+dlam*nlam/2.0; lamwid = 0.9*dlam*nlam/2.0
        #delete ("none", ver-, >& "dev$null")
	#print ("cont ", lamcen, "   ", lamwid, "  none", >> "none")
	#filim = filin//"_cont"
	#} else {
	#  tail (sband, nlin=1) | fields (field=1, lines=1) | scan(filt)
	#  filim = filin//"_"//filt
	#}

	slist (filin//"[0]", ap="1", long-) | fields (field="6-8") | scan (lam1, dlam, nlam)
	xsec = str(1)//":"//str(nlam)
	filim = filin//"_cont"
	if (strlwr(sband) != ""){
          tail (sband, nlin=1) | fields (field="1-", lines=1) | scan(filt, lam_cen, filwid)
	  ix1 = max(1, (int((lam_cen - filwid/2.0 - lam1)/dlam) + 1))
	  ix2 = min(nlam, (int((lam_cen + filwid/2.0 - lam1)/dlam) + 1))
	  xsec = str(ix1)//":"//str(ix2)
          filim = filin//"_"//filt
	  print(filt, " ",lam_cen, filwid, " XSECTION ", xsec)
	}
	#print (filin//"[0]["//xsec//","//str(1)//"]")
	  #print(" XSEC ", xsec)

	hselect (filin//"[0]", "$I EXPTIME RADEG DECDEG IPA INSMODE AIRMASS DATE-OBS", expr+)
	hselect (filin//"[1]", "INSMODE NFIBERS", expr+) | scan (obmode, nfib)
	nx = sqrt(nfib)/scale_ori + 10; ny =nx
	print(nx, ny, scale)
        date

	if (access(filin//"_fibers.dat")){
	  wc filin//"_fibers.dat" | scan(nnn)
	  if (nfib == nnn - 2){skip_readfib = yes}
	}
	if (skip_readfib){print("Using the existing fiber list ", filin//"_fibers.dat")}
	else {
        delete (filin//"_fibers.dat", ver-, >& "dev$null")
	hselect (filin//"[0]", "$I EXPTIME RADEG DECDEG IPA INSMODE AIRMASS DATE-OBS", expr+, >> filin//"_fibers.dat") 
	print(datahead, >> filin//"_fibers.dat")

        for (i = 1; i <= nfib; i = i + 1) {
	if (i < 10){cfib = "FIB00"} else if (i < 100){cfib = "FIB0"} else {cfib = "FIB"}
	chead = cfib//str(i)//"_X "//cfib//str(i)//"_Y"
	hselect (filin//"[1]", chead, expr+) | scan(xfib, yfib)
	ix = xfib/scale + int(nx/2) + 1; iy = yfib/scale + int(ny/2)
	printf("%5d  %10.5f  %10.5f %5d %5d \n", i, xfib, yfib, ix, iy, >> filin//"_fibers.dat")
	}    #end of reading header
	}

        manalyze.nfibers = nfib
        manalyze.out = filin//"_fibers.dat"
        manalyze.outreg = filin//"_fibers.reg"
        manalyze.head = datahead
        manalyze.obsmod = mode

	#Reading the header done

        delete ("imexpr.log", ver-, >& "dev$null")
	#Creating an image at the required lambda
	#
        #delete ("test_sbands", ver-, >& "dev$null")
	if (newim) {
        imdelete (filim, ver-, >& "dev$null")
	imexpr ("0.0", filim, dim=str(nx)//","//str(ny), verb-)
        #fields (filin//"_fibers.dat", "2-5", lines=1) | scan (texp, ra, dec, ipa)
	hselect (filin//"[0]", "EXPTIME RADEG DECDEG IPA AIRMASS DATE-OBS", expr+) | scan (texp, ra, dec, ipa, amass, obsdate)
	print(texp, ra, dec, ipa, amass, obsdate)
	hedit(filim, "CRPIX1", int(nx/2) + 1, add+, verif-, show-, update+) 
	hedit(filim, "CRPIX2", int(ny/2) + 1, add+, verif-, show-, update+) 
	hedit(filim, "CRVAL1", ra,  add+, verif-, show-, update+) 
	hedit(filim, "CRVAL2", dec, add+, verif-, show-, update+) 
	hedit(filim, "CDELT1", -imsca/nsamp/3600., add+, verif-, show-, update+) 
	hedit(filim, "CDELT2", imsca/nsamp/3600., add+, verif-, show-, update+) 
	hedit(filim, "CTYPE1", "RA---SIN", add+, verif-, show-, update+) 
	hedit(filim, "CTYPE2", "DEC--SIN", add+, verif-, show-, update+) 
	hedit(filim, "EPOCH", 2000., add+, verif-, show-, update+) 
	hedit(filim, "EXPTIME", texp, add+, verif-, show-, update+) 
	hedit(filim, "AIRMASS", amass, add+, verif-, show-, update+) 
	hedit(filim, "DATE-OBS", obsdate, add+, verif-, show-, update+) 
	hedit(filim, "TELESCOP", "GTC", add+, verif-, show-, update+) 
	hedit(filim, "INSTRUME", "MEGARA/LCB", add+, verif-, show-, update+) 
	hedit(filim, "FILENAME", filin, add+, verif-, show-, update+) 
	#imhead(filin//"_image")
        i1 = 0
        list = filin//"_fibers.dat"
        while(fscan(list, line) != EOF ) {
        tmp = substr (line, 1, 1)
        if (tmp != "#" && tmp != substr(filin, 1, 1)) {
          i1 = fscanf(line, "%d  %f  %f ", id, xfib, yfib)
	#print(id, xfib, yfib)
	ix = xfib/scale + int(nx/2) + 1; iy = yfib/scale + int(ny/2)
	if (abs(ix) <= nx && abs(iy) <= ny) {
	#sbands (filin//"[0][*,"//str(id)//"]", "test_sbands", bands=sband, verb-)
	#tail ("test_sbands", nlin=1) | fields (field=3) | scan(flux)
        imdelete ("temp_imexp", ver-, >& "dev$null")
        #imexpr ("sum(a)", "temp_imexp", filin//"[0][*,"//str(id)//"]",  verb-)
        imexpr ("sum(a)", "temp_imexp", filin//"[0]["//xsec//","//str(id)//"]", outtype="real", verb-)
        imstat ("temp_imexp", fields="mean", format-) | scan(flux)
	flux = max(0.0, flux)
	print(id, " ", ix, " ", iy, " ", flux, >> "imexpr.log")
	imreplace (filim//"["//str(ix)//":"//str(ix)//","//str(iy)//":"//str(iy)//"]", flux)
	}
	}
	}
        imdelete (filim//"_gau", ver-, >& "dev$null")
	gauss (filim, filim//"_gau", sigma=2, nsigma=2)
	}
        date

	if (unifib <=0) {goto exit}
	##############################################
	#Finding the fibers physically neighbouring a given fiber number or position
	#

	# unifib: [1=fiber#, 2=fiber xy, 3=imagepix, 4=xy file]"

	iunifib = unifib
        ifib = 0
	nlin = 1
	print(unifib)
	switch (unifib) {
	case 1: 
		{
		print(fibcoo) | scan(ifib)
       		if (ifib < 10){cfib = "FIB00"} else if (ifib < 100){cfib = "FIB0"} else {cfib = "FIB"}
        	chead = cfib//str(ifib+ierr)//"_X "//cfib//str(ifib+ierr)//"_Y"
        	hselect (filin//"[1]", chead, expr+) | scan(xa[1], ya[1])
		xin = xa[1]; yin = ya[1]
		}
	case 2: 
		{
		#print(fibcoo) | scan(xa[1], ya[1])
		print(fibcoo) | scan(xa[1], ya[1]); xin = xa[1]; yin = ya[1]; print(xin, yin)
		}
	case 3: 
		{
		print(fibcoo) | scan(xin, yin)
		xa[1] = scale*(xin - 1.0 - real(int(nx/2))) 
		ya[1] = scale*(yin - 1.0 - real(int(ny/2)))
		}
	case 4: 
		{
		k = 0
		iunifib = 4
        	fields (fibcoo, "1-3", lines=1) | scan (iunifib, srad, nlin)
		#iunifib is assumed to be 2 i.e. the values are assumed to be xy units not fiber and pixels
		#16/09/2017: iunifib=3 added (pixel units for spectral extraction)
        	radius = fibscale*srad
		if (nlin == 0){
			fields (fibcoo, "4-7", lines=1) | scan (xs, xe, ys, ye)
			#nlin = nint((xe - xs)/radius)*nint((ye - ys)/radius)
			inx = nint(0.5*(xe - xs)/radius) + 1; iny = nint(0.5*(ye - ys)/radius) + 1
        		for (i = 1; i <= inx; i = i + 1) { 
			for (j = 1; j <= iny; j = j + 1) {
			k = k + 1
			xa[k] = xs + real(i-1)*radius
			ya[k] = ys + real(j-1)*radius 
			print(k, xa[k], ya[k])
			} }
			xin = xa[1]; yin = ya[1]
			nlin = k
		} else {
		print("Reading ", nlin, " positions for spectral extraction from file ", fibcoo)
        		for (j = 1; j <= nlin; j = j + 1) {
        		#fields (fibcoo, "1-2", lines=j+1) | scan (xa[j], ya[j])
			#xin = xa[1]; yin = ya[1]
        		fields (fibcoo, "1-2", lines=j+1) | scan (xin, yin)
			xa[j] = scale*(xin - 1.0 - real(int(nx/2))) 
			ya[j] = scale*(yin - 1.0 - real(int(ny/2)))
        		print(j, xin, yin, xa[j], ya[j])
		}
		}
		}
	}
	printf("%5d  %10.5f  %10.5f  %10.5f \n",ifib, xin, yin, radius)

        for (j = 1; j <= nlin; j = j + 1) {
	ifib = 0
	xin = xa[j]; yin = ya[j]
	if (strupr(filout) == "DEFAULT"){filout = filin//"_0"}

        imdelete (filout, ver-, >& "dev$null")
        delete (filout//"_pixel.dat", ver-, >& "dev$null")
	sarith (filin//"[0][*,1]", "*", "0.0", filout)
	nearest_dist = radius
	fibstr = ""; k = 0
	for (i = 1; i <= nfib; i = i + 1) {
        fields (filin//"_fibers.dat", "2-3", lines=i+2) | scan (xfib, yfib)

	#print(xfib, yfib)
	dist = sqrt((xfib - xin)**2 + (yfib - yin)**2)
		if (xin <= 0.0){sx = "x0"} else {sx = "x1"}
		if (yin <= 0.0){sy = "y0"} else {sy = "y1"}
		suffix = "_"//sx//str(nint(abs(xin)*10))//sy//str(nint(abs(yin)*10))
	if (dist < radius) {
		k = k + 1
		nearest_dist = min(dist, nearest_dist)
		if (dist <= nearest_dist){ifib = i}
		printf ("%5d  %10.5f  %10.5f  %10.5f %10.5f %5d\n", i, xfib, yfib, dist, nearest_dist, ifib)
        	imdelete ("tempspe1", ver-, >& "dev$null")
        	imdelete ("tempspe2", ver-, >& "dev$null")
		scopy (filin//"[0][*,"//str(i)//"]", "tempspe1")
		sarith (filout, "+", "tempspe1", "tempspe2")
        	imdelete ("tempspe1", ver-, >& "dev$null")
        	imdelete (filout, ver-, >& "dev$null")
        	imrename ("tempspe2", filout, verb-)
		fibstr = fibstr//" "//str(i)
		#imhead(filout)
		#splot(filout)
	}
	}
	print(xin, yin, ifib, "#", fibstr, " ",k, suffix) 
	hedit(filout, "FIBCEN", ifib, add+, verif-, show-, update+) 
	hedit(filout, "FIBINT", k, add+, verif-, show-, update+) 
	hedit(filout, "FIBSUM", fibstr, add+, verif-, show-, update+) 
	hedit(filout, "FIBXY ", str(xin)//" "//yin, add+, verif-, show-, update+) 
        imdelete (filin//"_"//str(ifib)//suffix, ver-, >& "dev$null")
	imrename (filout, filin//"_"//str(ifib)//suffix, verb-)
	filout = filin//"_"//str(ifib)//suffix
        imdelete (filout//"_cal", ver-, >& "dev$null")
	if (strlwr(sensfil) != ""){
	calibrate (filout, filout//"_cal", extinct+, extinction="gtcinputs$tn_ext_curve.dat", ignorea+, sensiti=sensfil, observa="lapalma")
	splot(filout//"_cal", cursor = cursor)
	} else { splot(filout, cursor = cursor) }
	print(filout)
	}  #end reading the xyfib from file
exit:
	print("==========================")

end
	#if (unifib < 4){
	#print(fibcoo) | scan(ifib)
	#print(ifib)
	#if (ifib != 0) {
	#if (unifib == 1){print(fibcoo) | scan(ifib)
	#if (ifib < 10){cfib = "FIB00"} else if (ifib < 100){cfib = "FIB0"} else {cfib = "FIB"}
	#chead = cfib//str(ifib)//"_X "//cfib//str(ifib)//"_Y"
	#hselect (filin//"[1]", chead, expr+) | scan(xin, yin)
	#} else {print(fibcoo) | scan(xin, yin); ifib = 0}
	#if (unifib == 3){xin = scale*(xin - 1.0 - real(int(nx/2))); yin = scale*(yin - 1.0 - real(int(ny/2))) }
	#printf("%5d  %10.5f  %10.5f  %10.5f \n",ifib, xin, yin, radius)
	#print("   ")
	#}
	#}
	#if (unifib == 4){iunifib = 4} else {iunifib = unifib}
	#if (iunifib == 4){
        #fields (fibcoo, "1-3", lines=1) | scan (unifib, srad, nlin)
	#radius = fibscale*srad
	#for (j = 1; j <= nlin; j = j + 1) {
        #fields (fibcoo, "1-2", lines=j+1) | scan (xin, yin)
	##print(j, xin, yin)
