# RINGAVG (Nov02)                   proto                  RINGAVG (Nov02)
# 
# 
# NAME
#     ringavg -- compute pixel averages in concentric  rings  about  given
#     center
#     
procedure ringavg (image, xcr, ycr, xcl, ycl)

file	image			{prompt="Input image"}
real	xcr			{prompt="X center of right CCD"}
real	ycr			{prompt="Y center of right CCD"}
real	xcl			{prompt="X center of left CCD"}
real	ycl			{prompt="Y center of left CCD"}

real	r1 = 0			{prompt="Inner radius of first bin"}
real	r2 = 10			{prompt="Outer radius of last bin"}
real	dr = 1			{prompt="Radial bin width"}

bool	labels = yes		{prompt="Print column labels?"}
bool	vebars = no		{prompt="Format for error bars in GRAPH?"}

struct	*fd

begin
	file	temp, temp1, profile
	string	com_rad, comm1, size, infil, datfile
	real	n, r, val, ra, rb, ravg, rstddev, rmean, upplim, lowlim, median1, permed, stddevlim
	real	skymin, skymax, zmode, zmedian, zrms, histlo, histhi, dhist, drad
	int	 m1, m2, ixc, iyc, nx, ny, j, k, peak, nbin, ishi
	bool	del_star

	#del_star, lowlim, skymax, dhist, lowlim
	#xc = 461; yc=523
	infil = image
	lowlim=0.001; skymin = lowlim; skymax = 500.0
	dhist = 25.0; del_star = no
	#ixcr = int(xcr); iycr=int(ycr); 
	#ixcl = int(xcl); iycl=int(ycl)
	drad = dr

	imdelete("t2,t4,t6", ver-, >& "dev$null")  #; imdelete("t4", ver-, >& "dev$null"); imdelete("t6", ver-, >& "dev$null")

	imexpr ("a*0.0", "t2", infil, verbose-)
	imcopy ("t2", "t4", verbose-)
	imcopy ("t2", "t6", verbose-)

	imdelete(infil//"_mask", ver-, >& "dev$null")
	if (del_star) {
		imdelete("tt2,tt3,tt4", ver-, >& "dev$null")
		#imdelete("tt3", ver-, >& "dev$null"); imdelete("tt4", ver-, >& "dev$null")
	imexpr ("(a > b && a < c) ? a : -99.0", "tt2", infil, skymin, skymax, verbose-)
	imexpr ("(a <= b && shift(a,50) > b) ? shift(a,50) : a", "tt3", "tt2", skymin, verbose-)
	imexpr ("(a <= b && shift(a,-50) > b) ? shift(a,-50) : a", "tt4", "tt3", skymin, verbose-)
	imrename ("tt4", infil//"_mask", verbose-)
		imdelete("tt2,tt3", ver-, >& "dev$null")
		#imdelete("tt3", ver-, >& "dev$null")
	} else { 
			imcopy (infil, infil//"_mask", verbose-) 
		}

        hselect (infil, "i_naxis1, i_naxis2", expr+) | scan (nx, ny)
        size=str(nx)//","//str(ny)

	 imstat (infil, fields = "midpt, stddev", lower=INDEF, upper=INDEF,  format-) | scan (zmedian, zrms)
	  histhi = zmedian + 1.0*zrms
	  histlo = max(zmedian - 1.0*zrms, -50.0)
	  nbin = (histhi - histlo)/dhist
	print (zmedian, zrms, histhi, histlo, nbin)

	delete (temp, verify-, >& "dev$null")
	# Extract the pixel values sorted by radius, first for the right CCD (90-270 deg) and then for the left
	temp = mktemp ("temp")
	temp1 = mktemp ("temp1")
	pradprof (infil//"_mask", xcr, ycr, radius=r2, az1=95., az2=265., center=no, list=yes) |
	    sort ("STDIN", column=1, ignore_white+, numeric+, reverse-, > temp)
	pradprof (infil//"_mask", xcl, ycl, radius=r2, az1=275., az2=445., center=no, list=yes) |
	    sort ("STDIN", column=1, ignore_white+, numeric+, reverse-, > temp1)

	datfile = infil//"_stat.dat"; delete (datfile, verify-, >& "dev$null")
	if (label)
	    printf ("# %6s %8s %8s %8s %8s %8s %10s %10s %10s %10s %9s %9s %5s %6s %10s %8s\n", "R min", "R max", "Npix",
	        "Npix_fin", "Row_beg", "Row_end", "Median", "Average", "Std Dev", "Std Err", "Min", "Max", "Niter", "%Chnge", "Hist_peak", "N_peak")
	    printf ("# %6s %8s %8s %8s %8s %8s %10s %10s %10s %10s %9s %9s %5s %6s %10s %8s\n", "R min", "R max", "Npix",
	        "Npix_fin", "Row_beg", "Row_end", "Median", "Average", "Std Dev", "Std Err", "Min", "Max", "Niter", "%Chnge", "Hist_peak", "N_peak", > datfile)
	    #printf ("# %6s %8s %8s %8s %8s %8s %10s %10s %10s %10s %10s %10s %3s\n", "R min", "R max", "Npix",
	    #    "Npix_fin", "Row_beg", "Row_end", "Median", "Average", "Std Dev", "Std Err", "Min", "Max", "Niter", > datfile)

	# Read through the file.  Skip the first two comment lines.
	fd = temp
	profile = temp
	for (k=1; k <= 2; k +=1) {
	i = fscan (fd)
	i = fscan (fd)
	n = 0
	m1 = 0
	m2 = 0
	rb = -1
	while (fscan (fd, r, val) != EOF) {
	    if (r < r1)
		next
	    if (r > r2)
		break
	    if (r > rb) {
		if (n > 0) {
		    ravg = ravg / n
		    rstddev = sqrt (rstddev / n - ravg ** 2)
		    if (vebar)
			rstddev = -rstddev
		    if (n > 1)
			rstddev = rstddev * sqrt (n / (n - 1.))
		    rmean = rstddev / sqrt (n)
		    m1 = m2 - int(n) + 1
	  thistogram (profile, col="c2", rows=str(m1)//"-"//str(m2), low=histlo, nbin=nbin, dx=dhist) | sort (col=2, ignore_white+, numeric+, reverse+) | fields (fields="1,2", lines="2") | scan (zmode, peak) 
	#print (zmedian, zrms, histhi, histlo, nbin, zmode, peak)

            tstat (profile, "c2", rows=str(m1)//"-"//str(m2), >& "dev$null")
	  #lowlim = zmode - 10*dhist; upplim = zmode + 10*dhist
          #tstat (profile, "c2", rows=str(m1)//"-"//str(m2), lowlim=lowlim, highlim=upplim, >& "dev$null")
	for (j=1; j <= 11; j +=1) {
	  upplim = tstat.median + tstat.stddev
	  median1 = tstat.median
          tstat (profile, "c2", rows=str(m1)//"-"//str(m2), lowlim=lowlim, highlim=upplim, >& "dev$null")
	  permed = (tstat.median - median1)/tstat.median
	  if (abs(permed) <= 0.03 || tstat.nrows/n < 0.5) break
	}
	#j = 1; permed = (tstat.median - tstat.mean)/tstat.median
	    rmean = tstat.stddev / sqrt (tstat.nrows)
	    printf ("%8.2f %8.2f %8d %8d %8d %8d %10.4g %10.4g %10.4g %10.4g %10.4g %10.4g %3d %6.2g %10.4g %8d\n", 
		ra, rb, n, tstat.nrows, m1, m2, tstat.median, tstat.mean, tstat.stddev, rmean, tstat.vmin, tstat.vmax, j, permed, zmode, peak)
	    printf ("%8.2f %8.2f %8d %8d %8d %8d %10.4g %10.4g %10.4g %10.4g %10.4g %10.4g %3d %6.2g %10.4g %8d \n", ra, rb, n, tstat.nrows, m1, m2, tstat.median, tstat.mean, tstat.stddev, rmean, tstat.vmin, tstat.vmax, j, permed, zmode, peak, >> datfile)

	if (k == 1) {ixc = int(xcr); iyc = int(ycr)}
	if (k == 2) {ixc = int(xcl); iyc = int(ycl)}
        com_rad = "sqrt((I-"//str(ixc)//")**2 + (J-"//str(iyc)//")**2)"
        comm1 = "("//com_rad//" >= "//str(ra)//" && "//com_rad//" <= "//str(rb)//" ? a : b)"
	if (m1 == 1 && r1 > 0) comm1 = "("//com_rad//" <= "//str(rb)//" ? a : b)"
	#print(comm1)
		imexpr (comm1, "t1", tstat.median, "t2", dims=size, verbose-)
		imexpr (comm1, "t3", tstat.mean, "t4", dims=size, verbose-)
		imexpr (comm1, "t5", zmode, "t6", dims=size, verbose-)
		imdelete("t2", ver-, >& "dev$null"); imrename ("t1", "t2", verbose-)
		imdelete("t4", ver-, >& "dev$null"); imrename ("t3", "t4", verbose-)
		imdelete("t6", ver-, >& "dev$null"); imrename ("t5", "t6", verbose-)
		}
		ravg = 0.
		rstddev = 0.
		n = 0
		ra = int (r / dr) * dr
		rb = ra + dr
	    }

	    ravg = ravg + val
	    rstddev = rstddev + val * val
	    n = n + 1
	    m2 = m2 + 1
	}

	  thistogram (profile, col="c2", rows=str(m1)//"-"//str(m2), low=histlo, nbin=nbin, dx=dhist) | sort (col=2, ignore_white+, numeric+, reverse+) | fields (fields="1,2", lines="2") | scan (zmode, peak) 

	if (n > 0) {
	    ravg = ravg / n
	    rstddev = sqrt (rstddev / n - ravg ** 2)
	    if (vebar)
		rstddev = -rstddev
	    if (n > 1)
		rstddev = rstddev * sqrt (n / (n - 1.))
	    #rmean = rstddev / sqrt (n)
		    m1 = m2 - int(n) + 1
            tstat (profile, "c2", rows=str(m1)//"-"//str(m2), >& "dev$null")
	for (j=1; j <= 11; j +=1) {
	  upplim = tstat.median + tstat.stddev
	  median1 = tstat.median
          tstat (profile, "c2", rows=str(m1)//"-"//str(m2), lowlim=lowlim, highlim=upplim, >& "dev$null")
	  permed = (tstat.median - median1)/tstat.median
	  if (abs(permed) <= 0.03 || tstat.nrows/n < 0.5) break
	}
	#j = 1; permed = (tstat.median - tstat.mean)/tstat.median
	    rmean = tstat.stddev / sqrt (tstat.nrows)

	    printf ("%8.2f %8.2f %8d %8d %8d %8d %10.4g %10.4g %10.4g %10.4g %10.4g %10.4g %3d %6.2g %10.4g %8d\n", ra, rb, n, tstat.nrows, m1, m2, tstat.median, tstat.mean, tstat.stddev, rmean, tstat.vmin, tstat.vmax, j, permed, zmode, peak)

	    printf ("%8.2f %8.2f %8d %8d %8d %8d %10.4g %10.4g %10.4g %10.4g %10.4g %10.4g %3d %6.2g %10.4g %8d \n", ra, rb, n, tstat.nrows, m1, m2, tstat.median, tstat.mean, tstat.stddev, rmean, tstat.vmin, tstat.vmax, j, permed, zmode, peak, >> datfile)
	}

	if (k == 1) {
	imdelete("t2r", ver-, >& "dev$null"); imcopy ("t2", "t2r", verbose-)
	imdelete("t4r", ver-, >& "dev$null"); imcopy ("t4", "t4r", verbose-)
	imdelete("t6r", ver-, >& "dev$null"); imcopy ("t6", "t6r", verbose-)
	}

	fd = ""
	fd = temp1
	profile = temp1
	}

        com_rad = "sqrt((I-"//str(ixc)//")**2 + (J-"//str(iyc)//")**2)"
	ishi = 50+int(drad)
        comm1 = "("//com_rad//" <= "//str(r1+drad)//") && a > "//str(lowlim)//" ? shift(a,"//str(ishi)//") : a"
	#print(comm1)

	imdelete(infil//"_msky", ver-, >& "dev$null")
	imdelete(infil//"_msub", ver-, >& "dev$null")
	imdelete(infil//"_msub0", ver-, >& "dev$null")
	imexpr ("I < c ? a : b", infil//"_msky", "t2", "t2r", ixc, dims=size, verbose-)
	imexpr ("I < c ? d - a : d - b", infil//"_msub", "t2", "t2r", ixc, infil, dims=size, verbose-)
	#imexpr (comm1, infil//"_msub", infil//"_msub0", dims=size, verbose-)
	#imdelete(infil//"_msub0", ver-, >& "dev$null")

	imdelete(infil//"_asky", ver-, >& "dev$null")
	imdelete(infil//"_asub", ver-, >& "dev$null")
	imdelete(infil//"_asub0", ver-, >& "dev$null")
	imexpr ("I < c ? a : b", infil//"_asky", "t4", "t4r", ixc, dims=size, verbose-)
	imexpr ("I < c ? d - a : d - b", infil//"_asub", "t4", "t4r", ixc, infil, dims=size, verbose-)
	#imexpr (comm1, infil//"_asub", infil//"_asub0", dims=size, verbose-)
	#imdelete(infil//"_asub0", ver-, >& "dev$null")

	imdelete(infil//"_hsky", ver-, >& "dev$null")
	imdelete(infil//"_hsub", ver-, >& "dev$null")
	imdelete(infil//"_hsub0", ver-, >& "dev$null")
	imexpr ("I < c ? a : b", infil//"_hsky", "t6", "t6r", ixc, dims=size, verbose-)
	imexpr ("I < c ? d - a : d - b", infil//"_hsub", "t6", "t6r", ixc, infil, dims=size, verbose-)
	#imexpr (comm1, infil//"_hsub", infil//"_hsub0", dims=size, verbose-)
	#imdelete(infil//"_hsub0", ver-, >& "dev$null")

	fd = ""
	#delete (temp, verify-)
	#delete (temp1, verify-)
end
