procedure  gtc_osiris_tf (inlist)

#==============================================================================================
#This script produces monochromatic emission line images from a sequence of OSIRIS images
#The input images can be the entire image or of the individual LEFT or Right CCDs 
#-------------------------------------------------------------------------------------------------
#Steps still in experimental stage: the values of the following controlling parameters can be changed only by editting the script, and not by epar.
#1. Ha contamination in NII filters: unc_fac_6548 = 0.9; unc_fac_6583 = 0.9 
#2. sky subtraction procedure: cont_sky_sub=no; if yes subtracts "median" sky from the continuum
#      sky_annular=no  ===> only subtracts a mean sky from the chosen sections independently for the two CCDs.
#      sky_annular=yes ===> In additions subtracts "mode" in annular zones (width=dellam in pixels)
#3. imalign=yes, runs imalign with the continuum image as reference. There should be continuum_root_name.coo file with at least
#                         one stellar coordinate. 
#-------------------------------------------------------------------------------------------------
#load the following packages:
#uses phot: digiphot, apphot
#==============================================================================================

string  inlist     ="tf_images.lis" {prompt="List file containing input fits filenames"}
string  contin     =" "          {prompt="Input continuum Image file"}
string  output     ="tf"         {prompt="Prefix for the output monochromatic image"}
#real    dellam     = 18.         {prompt="Width of TF in Ang "}
real    xcen       = -52.        {prompt="X center of the TF wrt the image -52 or 2068 "}
real    ycen       = 1966.       {prompt="Y center of the TF wrt the image 1966 "}
real    scale      = 0.125       {prompt="Image scale in arcsec/pixel"}
string  skysec1    ="[*,*]"       {prompt="Sky section in the LEFT CCD "}
string  skysec2    ="[*,*]"       {prompt="Sky section in the RIGHT CCD "}
real    lammin     = 3500.0      {prompt="Min lambda for image reconstruction "}
real    lammax     = 9500.0      {prompt="Max lambda for image reconstruction "}
real    ksigma     = 5.00        {prompt="Apply response correction if emission flux > ksigma*rms "}
real    resp_cut   = 0.40        {prompt="Response function value above which image is monochromatic"}
real    cont_cut   = 0.10        {prompt="Response function value below which image is line-free"}
bool    skipbasi   = no          {prompt="Skip the basic reconstruction process and start postprocessing?"}
bool    proccont   = no          {prompt="Obtain a line-free continuum from the data-set?"}
string  proctype   ="none"       {prompt="Post-processing type (none/Ha/S2/S3) "}
bool    respcor    = no          {prompt="Correct the monochromatic image sections for the response function?"}
real    mingood    = 0.000001    {prompt="Minimum good data value"}
real    maxgood    = 999999      {prompt="Maximum good data value"}
bool    inform     = no          {prompt="Verbose the results?"}

begin
        string  infile, ofile[9], size, comm1, ofile_root, contfile
        string  tfile1, tfile2, tfile3, tfile4, tmp, s1, infiles[9], logfil
        string  tmpsky, tfile5, combmono, combwmono, skyctr_file, postproc
        string  lfile[9], rfile[9], combresp, cfile[9], nfile[9], stat_sec1, stat_sec2
        string  ofile_Ha[9], ofile_N2b[9], ofile_N2a[9], snr_N2a[9], snr_N2b[9]
        string  rfile_Ha[9], rfile_N2b[9], rfile_N2a[9], combHa, combfin
        string  ofile_S2a[9], ofile_S2b[9], rfile_S2a[9], rfile_S2b[9], cont_comb 
	string  contlist_file, contwlist_file, lineslist_file, lineolist_file 
        string  combnoi, combsnr, combrat, line2olist_file, line2slist_file
	string  linelist_file, linewlist_file, line2list_file, line2wlist_file
        string  rfile_S2ab[9], rfile_S2ba[9], linebwlist_file, line2bwlist_file
        string  combS2a, combS2b, combmono2, combrespb, combresp2, combresp2b
        string  rfile_S3[9], combS3, input_list, align_list
        real    xc, yc, lam0, dlam, minlam, maxlam, r_cut, x_cut, exptime
        real    vel, rl1, rl2, rl3, rl4, rl5, rl6, rl7, rl8, rl9, dopfac
        real    lam[9], zmode, r1, wav0[9], r2, dwav[9], zstddev, good_min, lamline[6]
        real    pixsec, pixmin, pixmin2, skyrad, skydr, ksig, goodmin, goodmax
	real	goodmin1, goodmax1, zrms, zmedian, unc_fac_6548, unc_fac_6583
	real	zmode1, zstddev1, zmode2, zstddev2, zmedian1, zmedian2
        int     nx, ny, nlam, i1, i2, nlines, i2l, i2v, i2c
        #int    ig, filt_no[9,9], nfilt[9]
        bool    talk, emline, resp_cutoff, rescor, proc_cont, skipbasic, filedel
        bool    cont_sky_sub, sky_annular, imalign
        struct  line

        #--------------------------------------------------------------------
        #redefine parameters controlling the running of the script
        #--------------------------------------------------------------------

	input_list = inlist
        ofile_root=output
        #contfile = ofile_root//"cont"
        contfile = contin
        rescor = respcor
        talk = inform
        proc_cont = proccont
        postproc = proctype
	#if (postproc != "Ha" || postproc != "S2" || postproc != "S3"){
	#	print ("***Error*** parameter [proctype= ",postproc, "] invalid.")
	#	goto exit
	#}
	skipbasic = skipbasi
        xc = xcen; yc = ycen; pixsec = scale #dlam=dellam; 
	minlam = lammin; maxlam = lammax
        pixmin =(pixsec/60.0); pixmin2 =pixmin**2
        r_cut = resp_cut; x_cut = cont_cut; ksig = ksigma
        resp_cutoff = yes
	goodmin = mingood; goodmax = maxgood
	filedel = yes
	stat_sec1 = skysec1; stat_sec2 = skysec2
	cont_comb = "sum"  #"median"    #sum, median, average
	cont_sky_sub = no; sky_annular = no; imalign = no    #yes
        #if (r_cut > 0.49){resp_cutoff = no}
        if (r_cut == 0.50){resp_cutoff = no}

        skyctr_file = "tf_monimage_skyctr.coo"
        delete(skyctr_file, ver-, >& "dev$null")
        print(xc, yc, > skyctr_file)

        #--------------------------------------------------------------------
        #Read the list containing fits images to be processed
        #gets filename, lam0
        #--------------------------------------------------------------------
        tmpsky = mktemp ("tmp$ia_tmp.")
        i1 = 0
        list = input_list    #inlist
        while( fscan(list, line) != EOF ) {
        tmp = substr (line, 1, 1)
        if (tmp != "#") {
          i1=i1+1
          if (i1 == 1) {i2l = fscanf(line, "%g %g %g %g %g %g ", rl1, rl2, rl3, rl4, rl5, rl6)}
          if (i1 == 2) {i2v = fscanf(line, "%g ", vel)
        dopfac = (1.0 + vel/2.9997E+05)
        lamline[1] = rl1*dopfac; lamline[2] = rl2*dopfac; lamline[3] = rl3*dopfac
        lamline[4] = rl4*dopfac; lamline[5] = rl5*dopfac; lamline[6] = rl6*dopfac; nlines=i2l
        for (i=2; i <= nlines; i +=1) {
                #dif_next_lam = lamline[i] - lamline[i-1]
                if (lamline[i] < lamline[i-1] ) {
                        print("ERROR: Emission line wavelengths in the input list should be in the increasing order")
                        goto exit
                        }
                }
        }
          if (i1 == 3) {i2c = fscanf(line, "%g %g %g %g %g %g ", rl1, rl2, rl3, rl4, rl5, rl6)
        #--------------------------------------------------------------------
        #nlam is the number of images that go into forming a line image.
        #--------------------------------------------------------------------
        nlam=i2c + nlines
        for (i=1; i <= nlines; i +=1) {lam[i] = lamline[i] }
        lam[nlines+1] = rl1; lam[nlines+2] = rl2; lam[nlines+3] = rl3 
        lam[nlines+4] = rl4; lam[nlines+5] = rl5; lam[nlines+6] = rl6 
        }
          if (i1 > 3) {
          i2 = fscanf(line, "%s %g %g ", s1, r1, r2)
          infiles[i1-3] = s1; wav0[i1-3] = r1; dwav[i1-3] = r2
          #print (infiles[i1-3])
          print (infiles[i1-3], >> tmpsky)
          }
        } }

        i1 = i1 - 3

        logfil = ofile_root//"_mon.log"
	print("--------------------------------------- gtc_osiris_tf ----------------------", >> logfil)
	date (, >> logfil)
	print("--------------------------------------- ", input_list,  " -------------------------", >> logfil)

        hselect (infiles[1], "i_naxis1, i_naxis2", expr+) | scan (nx, ny)
        #hselect (infiles[1], "i_naxis1, i_naxis2, EXPTIME", expr+) | scan (nx, ny, exptime)
        hselect ("@"//tmpsky, "i_naxis1, i_naxis2, EXPTIME GAIN FILTER1 FILTER2 FILTER3 OBJECT $I", expr+, >> logfil)
        hselect ("@"//tmpsky, "i_naxis1, i_naxis2, EXPTIME GAIN FILTER1 FILTER2 FILTER3 OBJECT $I", expr+)
        size=str(nx)//","//str(ny)
        print (" Nlines = ",nlines," Ncont = ", i2c, " Nfilt = ",i1)
        print (" Nlines = ",nlines," Ncont = ", i2c, " Nfilt = ",i1, >> logfil)
        #print (" Nlines = ",nlines," Ncont = ", i2c, " Nlam = ",nlam, " Nfilt = ",i1)

        #goto exit   #stitch
	#if (skipbasic) {goto post_proc}

	#for (i=1; i <= nlam ; i +=1) {
	#imdelete(ofile_root//"*"//str(int(lam[i]))//".fits", ver-, >& "dev$null")
	#}

        #--------------------------------------------------------------------
	#Obtaining the typical rms value in the images using the continuum image
        #--------------------------------------------------------------------
        print ("Obtaining the typical rms value in the images using the continuum image ")
        #imstat (contin, fields = "mode, stddev", lower=0.001, upper=99999.0,  format-) | scan (zmode, zstddev)
        imstat (contfile, fields = "mode, midpt, stddev", lower=goodmin, upper=goodmax,  format-) | scan (zmode, zmedian, zstddev)
	goodmin1=zmode-3.0*zstddev; goodmax1=zmode+3.0*zstddev
        #imstat (contfile//stat_sec1, fields = "mode, midpt, stddev", lower=goodmin, upper=goodmax,  format-) | scan (zmode1, zmedian1, zstddev1)
        #imstat (contfile//stat_sec2, fields = "mode, midpt, stddev", lower=goodmin, upper=goodmax,  format-) | scan (zmode2, zmedian2, zstddev2)
        imstat (contfile//stat_sec1, fields = "mode, midpt, stddev", nclip=12,  format-) | scan (zmode1, zmedian1, zstddev1)
        imstat (contfile//stat_sec2, fields = "mode, midpt, stddev", nclip=12,  format-) | scan (zmode2, zmedian2, zstddev2)
	zstddev = (zstddev1 + zstddev2)/2.0; zmode = (zmode1 + zmode2)/2.0; zmedian = (zmedian1 + zmedian2)/2.0

        print("Sky value in the continuum image = ",zmode1, " ", zmode2, " rms = ", zstddev1, " ", zstddev2, " ", zstddev)
        print("Sky value in the continuum image = ",zmode1, " ", zmode2, " ", zmedian, " rms = ", zstddev1, " ", zstddev2, " ", zstddev, >> logfil)
        #print("Sky value in the continuum image = ",zmode, " rms = ", zstddev, >> logfil)

	#comm1 = "(I < "//str(int(xc))//") ? a - "//str(zmode1)//" : a - "//str(zmode2)
	comm1 = "(I < "//str(int(xc))//") ? a - "//str(zmedian1)//" : a - "//str(zmedian2)
	print (contfile, " ", comm1)

        imdelete(contfile//"s", ver-, >& "dev$null")
	imexpr(comm1, contfile//"s", contfile, dims=size, verbose=talk)

	goodmin1=zmode-3.0*zstddev; goodmax1=zmode+3.0*zstddev
	align_list = ofile_root//"align_files.lis"
	delete(align_list, ver-, >& "dev$null")
	print (contfile//"s", >> align_list)	
        for (j=1; j <= i1; j +=1) {
        imdelete(infiles[j]//"s", ver-, >& "dev$null")
        #imstat (infiles[j]//stat_sec1, fields = "mode, stddev", lower=goodmin1, upper=goodmax1,  format-) | scan (zmode1, zstddev1)
        #imstat (infiles[j]//stat_sec2, fields = "mode, stddev", lower=goodmin1, upper=goodmax1,  format-) | scan (zmode2, zstddev2)
        imstat (infiles[j]//stat_sec1, fields = "mode, midpt, stddev", nclip=12,  format-) | scan (zmode1, zmedian1, zstddev1)
        imstat (infiles[j]//stat_sec2, fields = "mode, midpt, stddev", nclip=12,  format-) | scan (zmode2, zmedian2, zstddev2)
        #imstat (infiles[j]//stat_sec1, fields = "mode, midpt, stddev", lower=goodmin, upper=goodmax,  format-) | scan (zmode1, zmedian1, zstddev1)
        #imstat (infiles[j]//stat_sec2, fields = "mode, midpt, stddev", lower=goodmin, upper=goodmax,  format-) | scan (zmode2, zmedian2, zstddev2)
	print(infiles[j], " ", zmode1, " ", zmode2, " ", zstddev1, " ", zstddev2, >> logfil)
	#comm1 = "(I < "//str(int(xc))//") ? a - "//str(zmode1)//" : a - "//str(zmode2)
	comm1 = "(I < "//str(int(xc))//") ? a - "//str(zmedian1)//" : a - "//str(zmedian2)
	print (infiles[j], " ", comm1)
	imexpr(comm1, infiles[j]//"s", infiles[j], dims=size, verbose=talk)
	print (infiles[j]//"s", >> align_list)	
	}

	if (imalign) {
	#imalign Ha6528s Ha6568s output=Ha6528s boundar=constan
imalign ("@"//align_list, contfile//"s", contfile//".coo", "@"//align_list,
 shifts="", boxsize=7, bigbox=11,
negative=no, background=INDEF, lower=INDEF, upper=INDEF, niterate=3,
tolerance=0, maxshift=INDEF, shiftimages=yes, interp_type="linear",
boundary_typ="constant", constant=0., trimimages=no, verbose=yes, list="", >> logfil)
	}

        #goto exit   
	if (skipbasic) {goto post_proc}

        #--------------------------------------------------------------------
        #Start 2 loops. 1 (j loop): for each image. 2 (i loop): for lambdas in steps of "dlam"
        #--------------------------------------------------------------------

        #centerpars.calgori="none"; fitskypars.salgori="mode"; photpars.apertur="500.,800.0,1200.0" #histplot
        centerpars.calgori="none"; fitskypars.salgori="median"; photpars.apertur="500.,800.0,1200.0" #histplot
        for (j=1; j <= i1; j +=1) {
        infile = infiles[j]//"s"; lam0 = wav0[j]; dlam = dwav[j]
        hselect (infile, "i_naxis1, i_naxis2", expr+) | scan (nx, ny)
        size=str(nx)//","//str(ny)

        #--------------------------------------------------------------------
        #step 1: Calculate expected TF lambda in each pixel of the image
        #--------------------------------------------------------------------

        #comm1 = str(lam0)//"*(1.0-7.9520E-04*"//str(pixmin2)//"*((real(I)-"//str(xc)//")**2 + (real(J)-"//str(yc)//")**2))"
        comm1 = "a*0.0+"//str(lam0)//"*(1.0-7.9520E-04*"//str(pixmin2)//"*((real(I)-"//str(xc)//")**2 + (real(J)-"//str(yc)//")**2))"
        lfile[j] = ofile_root//"_"//infile//"_lambda"
        tfile2 = lfile[j]
        imdelete(tfile2, ver-, >& "dev$null")
        #print ("Calculating TF lambda at each pixel using ",comm1)
        imexpr (comm1, tfile2, infile, dims=size, verbose=talk)

        for (i=1; i <= nlam ; i +=1) {
        if ((lam0 - lam[i]) >= 350.0){goto skip_restore}
                if (lam[i] > maxlam || lam[i] < minlam) {goto skip_restore}
                emline = no
                for (k=1; k <= nlines; k +=1) { 
                if ((abs(lam[i] - lamline[k])) < dlam/2.0) {emline = yes}
                }
        #print(" ", lam[i], " line = ", emline)

        tfile1 = "tfile_lamobj"
        tfile3 = "tfile_tfrf"; tfile4 = "tfile4"; tfile5 = "tfile5"
        imdelete(tfile1, ver-, >& "dev$null")

        comm1 = "a*0.0+"//str(lam[i])
        #print ("Calculating the lambda at each pixel that has to be mapped ",comm1)
        imexpr(comm1, tfile1, infile, dims=size, verbose=talk)

        #--------------------------------------------------------------------
        #step 2: Define the image ring/annulus where the TF response is maximum
        #--------------------------------------------------------------------

        ofile[i] = ofile_root//"_"//infile//"_mono_"//str(int(lam[i]))
        rfile[i] = ofile_root//"_"//infile//"_resp_"//str(int(lam[i]))
        tfile3 = rfile[i]
        #print ("Calculating TF response function at each pixel using ",comm1)
        imdelete(tfile3, ver-, >& "dev$null")
        #comm1 = "1.0/(1.0 + 2.0*((a-b)/"//str(dlam)//")**2)"
        #imexpr(comm1, tfile3, tfile1, tfile2, dims=size, verbose=talk)
        comm1 = "1.0/(1.0 + 2.0*(("//str(lam[i])//"-a)/"//str(dlam)//")**2)"
        imexpr(comm1, tfile3, tfile2, dims=size, verbose=talk)

        if (lam[i] > (lam0+dlam/2.0) || lam[i] > maxlam || lam[i] < minlam)
                        {goto skip_restore}

        print ("Restoring the monochromatic image at ", lam[i], " from image ", infile)

        #--------------------------------------------------------------------
        #step 3: Retain only the image ring/annulus where the TF response is above "r_cut"
        #        or lambda is within "dlam" of the desired lambda [lam(i)]
        #     3a: subtract the continuum image from the "line" images
        #     3b: subtract the residual sky using the mode of the pixels having same lambda
        #--------------------------------------------------------------------

        imdelete(ofile[i], ver-, >& "dev$null")
        imdelete(tfile4, ver-, >& "dev$null")
                if (emline) {
        comm1 = "a>"//str(r_cut)//" ? (b-c) : 0.0"
	#print(comm1)
        imexpr(comm1, tfile4, tfile3, infile, contfile//"s", dims=size, verbose=talk)
                        } else {
        comm1 = "a>"//str(r_cut)//" ? b : 0.0"
        imexpr(comm1, tfile4, tfile3, infile, dims=size, verbose=talk)
                        }
	zmode = 0.0
	if (!sky_annular) goto sky_annular_end
sky_annular_begin:
        if (lam[i]+dlam/2.0 > lam0) {
                skyrad = sqrt((1.0 - (lam[i]-dlam/2.0)/lam0)/7.9520E-04)/2.0   #For the 1st ring use half the semicircle
                skydr = skyrad
                } else {
                skyrad = sqrt((1.0 - (lam[i]+dlam/2.0)/lam0)/7.9520E-04)
                skydr  = dlam/(2.0*skyrad*lam0*7.9520E-04)
                }
        fitskypars.annulus=skyrad/pixmin; fitskypars.dannulu=skydr/pixmin
        delete(tfile4//".mag.*", ver-, >& "dev$null")
        phot(tfile4, coords=skyctr_file, interac-, verify-, verbose+) | scanf ("%s %g %g %g %s %s %s %s ", s1, r1, r1, zmode, s1, s1, s1, s1)
        good_min = ksig*zstddev
                #if (!emline) { good_min = 99999.0 }
        #print (infile, " ", lam[i], " Residual sky = ", zmode, " stddev = ", zstddev, " Inner Rad=",fitskypars.annulus, " Width=",fitskypars.dannulu, "goodmin=",good_min, >> logfil)
	#alternative way to find sky
	#zmode1 = zmode
        #imstat (tfile4, fields = "mode, midpt, stddev", lower=goodmin, upper=goodmax,  format-) | scan (zmode, zmedian, zrms)
        #imstat (tfile4, fields = "mode, midpt, stddev", lower=INDEF, upper=INDEF,  format-) | scan (zmode1, zmedian, zrms)
        imstat (tfile4, fields = "mode, midpt, stddev", lower=goodmin, upper=goodmax,  format-) | scan (zmode1, zmedian, zrms)
	goodmin1=zmedian- 3.0*zrms; goodmax1=zmedian + 3.0*zrms   
        #imstat (tfile4, fields = "mode, midpt, stddev", lower=goodmin1, upper=goodmax1,  format-) | scan (zmode1, zmedian, zrms)
        print (infile, " ", lam[i], " Residual sky = ", zmode, zmode1, zmedian, " stddev = ", zstddev, zrms, " Inner Rad=",fitskypars.annulus, " Width=",fitskypars.dannulu, "goodmin=",good_min, goodmin1, >> logfil)
	#print (" Inner_rad Outer_rad Sky rms Infile lam" , >> skyfil)
        #print (infile, " ", lam[i], " Residual sky = ", zmode, zmedian, " stddev = ", zstddev, zrms, " Inner Rad=",fitskypars.annulus, " Width=",fitskypars.dannulu, "goodmin=",good_min, goodmin1, >> skyfil)
	#alternative way to find sky - end
	  #imcopy (tfile4, ofile[i], verbose-)
	#zmode = zmedian
sky_annular_end:

        imdelete(tfile5, ver-, >& "dev$null")
        if (resp_cutoff){
                comm1 = "a>"//str(r_cut)//" ? (b-"//str(zmode)//") : 0.0"
		#print(comm1)
                imexpr(comm1, tfile5, tfile3, tfile4, dims=size, verbose=talk)
                        } else {
        comm1 = "abs("//str(lam[i])//"-a) <= "//str(dlam/2.0)//" ? (b-"//str(zmode)//") : 0.0"
        imexpr(comm1, tfile5, tfile2, tfile4, dims=size, verbose=talk)
                                }

	#--------------------------------------------------------------------
	#step 4: Correct the line-emitting pixels for the TF response
	#     4a: Determine the line-emitting pixels as those k*sig above the background
	#--------------------------------------------------------------------
	if (rescor && emline) {
	  #comm1 = "a > "//str(good_min)//" ? a/b : a"
	  comm1 = "a > "//str(good_min)//" ? a/b : 0"
	  imexpr(comm1, ofile[i], tfile5, tfile3, dims=size, verbose=talk)
	  print("RESPCOR=",rescor, "EMISSION? ",emline)
	} else {
	  #print("copying ",tfile5, " to ",ofile[i])
	  imcopy (tfile5, ofile[i], verbose-)
	}

skip_restore:
	}      
	}    
	#i and j loops end

        delete (tmpsky, ver-, >& "dev$null")
	delete(tfile4//".mag.*", ver-, >& "dev$null")

post_proc:
	if (!proc_cont) {goto stitch}
        contlist_file = "tf_monimage_contfiles.lis"
        delete(contlist_file, ver-, >& "dev$null")
        contwlist_file = "tf_monimage_contwfiles.lis"
        delete(contwlist_file, ver-, >& "dev$null")

	#--------------------------------------------------------------------
	#step 5: Reconstruct the monochromatic image by summing all images in a desired lambda
	#--------------------------------------------------------------------

	#Reconstruct the images, eliminating the contribution from contaminating lines

	#1. Reconstruction of the continuum using only those pixels/sections that are not contaminated by lines

	tfile5 = "tfile5"
	print ("Reconstructing the line-free continuum image from the data sequence ")
        for (j=1; j <= i1; j +=1) {
        infile = infiles[j]//"s"; lam0 = wav0[j]; dlam = dwav[j]
	for (i=1; i <= nlines; i +=1) {
	rfile[i] = ofile_root//"_"//infile//"_resp_"//str(int(lam[i]))
	}
	cfile[j] = ofile_root//"_"//infile//"_cont"
        nfile[j] = ofile_root//"_"//infile//"_cont_weight"      #
	imdelete(cfile[j], ver-, >& "dev$null")
	imdelete(nfile[j], ver-, >& "dev$null")                 #
	if (lam[1] > (lam0+dlam/2.0)) {
		imcopy(infile, cfile[j], ver-)
		imexpr("a*0.0+1.0", nfile[j], cfile[j], dims=size, verbose=talk)  #
        print(cfile[j], >> contlist_file)
        print(nfile[j], >> contwlist_file)
		goto skip_recons
		}
	if (nlines==3) {
	if ((lam[2] > (lam0+dlam/2.0)) || (lam[3] > (lam0+dlam/2.0))) {goto skip_recons}
	#a,b and c are the response functions of Ha and NII lines in each image
	comm1 = "(a>"//str(x_cut)//") || (b>"//str(x_cut)//") || (c>"//str(x_cut)//") ? 0.0 : d"
	imexpr(comm1, cfile[j], rfile[1], rfile[2], rfile[3], infile, dims=size, verbose=talk)
        comm1 = "(a>"//str(x_cut)//") || (b>"//str(x_cut)//") || (c>"//str(x_cut)//") ? 0 : 1"
        imexpr(comm1, nfile[j], rfile[1], rfile[2], rfile[3], dims=size, verbose=talk)
	}
	if (nlines==2) {
	if ((lam[2] > (lam0+dlam/2.0)) ) {goto skip_recons}
	#a and b are the response functions of the two SII lines in each image
	comm1 = "(a>"//str(x_cut)//") || (b>"//str(x_cut)//") ? 0.0 : c"
	imexpr(comm1, cfile[j], rfile[1], rfile[2], infile, dims=size, verbose=talk)
        comm1 = "(a>"//str(x_cut)//") || (b>"//str(x_cut)//") ? 0 : 1"
        imexpr(comm1, nfile[j], rfile[1], rfile[2], dims=size, verbose=talk)
	}
	if (nlines==1) {
	if ((lam[1] > (lam0+dlam/2.0)) ) {goto skip_recons}
	comm1 = "(a>"//str(x_cut)//") ? 0.0 : b"
	imexpr(comm1, cfile[j], rfile[1], infile, dims=size, verbose=talk)
	comm1 = "(a>"//str(x_cut)//") ? 0.0 : 1"
	imexpr(comm1, nfile[j], rfile[1], dims=size, verbose=talk)
	}
	if (cont_sky_sub) {
	imstat (cfile[j], fields = "mode, midpt, stddev", lower=goodmin, upper=goodmax,  format-) | scan (zmode, zmedian, zrms)
        imdelete(tfile5, ver-, >& "dev$null")
	#comm1 = "(a > 0.0 ? a-"//str(zmedian)//" : 0.0)"
	comm1 = "a-"//str(zmedian)
	print(cfile[j], " ", comm1)
	imexpr(comm1, tfile5, cfile[j], dims=size, verbose=talk)
        imdelete(cfile[j], ver-, >& "dev$null")
	imrename (tfile5, cfile[j], verbose-)
	}
        print(cfile[j], >> contlist_file)
        print(nfile[j], >> contwlist_file)

	print(cfile[j], " ",lam0)
	#print(cfile[j], " ",lam0, zmode, zmedian, zstddev, zrms)
skip_recons:
	}

	#print(contlist_file)
	#print ("Reconstructing the monochromatic image at ", lam[i])
	combmono = ofile_root//"_comb_av_cont"   #//str(int(lam[i]))
        combwmono = ofile_root//"_comb_av_cont_weight"   #//str(int(lam[i]))
	imdelete(combmono, ver-, >& "dev$null")
	print ("Type of continuum combine is ", cont_comb)
	if (cont_comb=="sum") {
        imdelete(combwmono, ver-, >& "dev$null")
        imdelete("tfile1", ver-, >& "dev$null")
        imdelete("tfile2", ver-, >& "dev$null")
        imcombine ("@"//contlist_file, "tfile1", combine="sum", reject="none", scale="none", zero="none", logfile=logfil)
        imcombine ("@"//contwlist_file, combwmono, combine="sum", reject="none", scale="none", zero="none", logfile=logfil)
        imarith ("tfile1", "/" , combwmono, combmono)
	} else {
        imcombine ("@"//contlist_file, combmono, combine=cont_comb, lthresh=goodmin, reject="minmax", scale="none", zero="none", logfile=logfil)
	}

	#if (filedel) {imdelete("@"//contlist_file, ver-, >& "dev$null")}

stitch:
	if (postproc=="none") {goto exit}
	unc_fac_6548 = 1.00; unc_fac_6583 = 1.00
	#unc_fac_6548 = 0.70; unc_fac_6583 = 0.70
        #print ("Reconstructing the Halpha, and the NII lines")
	if (postproc=="S2") {goto SII_proc}
	if (postproc=="S3") {goto SIII_proc}
	print ("Reconstructing the Halpha image")
        linelist_file = "tf_monimage_linefiles.lis"
        delete(linelist_file, ver-, >& "dev$null")
        linewlist_file = "tf_monimage_linewfiles.lis"
        delete(linewlist_file, ver-, >& "dev$null")
        imdelete(ofile_root//"*"//"_Ha_resp.fits", ver-, >& "dev$null")
	#Ha-Flux-corrected line images 
	imstat (contfile//"s"//stat_sec1, fields = "mode, stddev", lower=goodmin, upper=goodmax,  format-) | scan (zmode, zstddev)
        good_min = ksig*zstddev

        for (j=1; j <= i1; j +=1) {
        infile = infiles[j]//"s"; lam0 = wav0[j]; dlam = dwav[j]
        #print(infile, " ",lam0)
	#probably remove the loop and define rfile[2] and ofile[2]
        rfile[2] = ofile_root//"_"//infile//"_resp_"//str(int(lam[2]))
	ofile[2] = ofile_root//"_"//infile//"_mono_"//str(int(lam[2]))
	#print ("Reconstructing the monochromatic image at ", lam[2], " from ",infile)
	if (lam[2] <= (lam0+dlam/2.0) && lam[2] < maxlam && lam[2] > minlam) {
	rfile_Ha[j] = ofile_root//"_"//infile//"_Ha_resp"
        imdelete(rfile_Ha[j], ver-, >& "dev$null")
	comm1 = "a>"//str(r_cut)//" ? a : 0.0"
        imexpr(comm1, rfile_Ha[j], rfile[2], dims=size, verbose=talk)
        print(ofile[2], >> linelist_file)
        print(rfile_Ha[j], >> linewlist_file)
	}
skip_last:   
	}

        combmono = ofile_root//"_comb_Ha" 
        combresp = ofile_root//"_comb_Ha_resp" 
        imdelete(combmono, ver-, >& "dev$null")
        imdelete(combresp, ver-, >& "dev$null")
	imcombine ("@"//linelist_file, combmono, combine="sum", reject="none", scale="none", zero="none", logfile=logfil)
	imcombine ("@"//linewlist_file, combresp, combine="sum", reject="none", scale="none", zero="none", logfile=logfil)
	combHa = ofile_root//"_comb_Ha_fin"
        imdelete(combHa, ver-, >& "dev$null")
	imarith (combmono, "/", combresp, combHa, ver-)

	#if (filedel) {imdelete("@"//linelist_file, ver-, >& "dev$null")}
	if (filedel) {imdelete("@"//linewlist_file, ver-, >& "dev$null")}
	#if (filedel) {imdelete(combmono, ver-, >& "dev$null")}

	#Subtraction of contaminating Halpha from NII filters
	#print ("Reconstructing the NII6548 and NII6583 images")
	print ("Reconstructing the NII6548 and NII6583 images using unc_fac_6548=",unc_fac_6548, " and unc_fac_6583=",unc_fac_6583)
        delete(linelist_file, ver-, >& "dev$null")
        delete(linewlist_file, ver-, >& "dev$null")
        lineslist_file = "tf_monimage_linesfiles.lis"
        delete(lineslist_file, ver-, >& "dev$null")
        lineolist_file = "tf_monimage_lineofiles.lis"
        delete(lineolist_file, ver-, >& "dev$null")

        line2list_file = "tf_monimage_line2files.lis"
        delete(line2list_file, ver-, >& "dev$null")
        line2wlist_file = "tf_monimage_line2wfiles.lis"
        delete(line2wlist_file, ver-, >& "dev$null")
        line2slist_file = "tf_monimage_line2sfiles.lis"
        delete(line2slist_file, ver-, >& "dev$null")
        line2olist_file = "tf_monimage_line2ofiles.lis"
        delete(line2olist_file, ver-, >& "dev$null")

        for (j=1; j <= i1; j +=1) {
        infile = infiles[j]//"s"; lam0 = wav0[j]; dlam = dwav[j]
        for (i=1; i <= nlines; i +=1) {
        rfile[i] = ofile_root//"_"//infile//"_resp_"//str(int(lam[i]))
	ofile[i] = ofile_root//"_"//infile//"_mono_"//str(int(lam[i]))
	#print ("Reconstructing the monochromatic image at ", lam[i], " from ",infile, " using ", rfile[2])
        }

	if (lam[1] <= (lam0+dlam/2.0) && lam[1] < maxlam && lam[1] > minlam) {
	#print ("Reconstructing the monochromatic image at ", lam[1], " from ",infile, " using ", rfile[2])
        #comm1 = "a > b*c"//" ? (a-b*c) : 0.0"
        #comm1 = "a > "//str(good_min)//" ? (a-b*c*0.90) : a"
        #comm1 = "a > "//str(good_min)//" ? (a-b*c*0.90) : 0.0"
        comm1 = "a > "//str(good_min)//" ? (a-b*c*"//str(unc_fac_6548)//") : 0.0"
	ofile_N2a[j] = ofile_root//"_"//infile//"_NII6548"
        imdelete(ofile_N2a[j], ver-, >& "dev$null")
        imexpr(comm1, ofile_N2a[j], ofile[1], combHa, rfile[2], dims=size, verbose=talk)

	#construct the weight files for each line filter

	#snr_N2a[j] = ofile_root//"_"//infile//"_NII6548_snr"
	snr_N2a[j] = ofile_root//"_"//infile//"_NII6548_noisq"
        imdelete(snr_N2a[j], ver-, >& "dev$null")
	#imexpr("b*a/(sqrt(b*a+c*d*a))", snr_N2a[j], "9.5", ofile[1], combHa, rfile[2])
	#noise**2 images
	imexpr("b*a+c*d*a", snr_N2a[j], "9.5", ofile[1], combHa, rfile[2], dims=size, verbose=talk)
	rfile_N2a[j] = ofile_root//"_"//infile//"_NII6548_resp"
        imdelete(rfile_N2a[j], ver-, >& "dev$null")
	comm1 = "a>"//str(r_cut)//" ? a : 0.0"
        imexpr(comm1, rfile_N2a[j], rfile[1], dims=size, verbose=talk, verbose=talk)
        print(ofile_N2a[j], >> line2list_file)
        print(rfile_N2a[j], >> line2wlist_file)
        print(snr_N2a[j], >> line2slist_file)
        print(ofile[1], >> line2olist_file)
	}

	#print ("Reconstructing the NII6583 image")
	if (lam[3] <= (lam0+dlam/2.0) && lam[3] < maxlam && lam[3] > minlam) {
	#print ("Reconstructing the monochromatic image at ", lam[3], " from ",infile, " using ", rfile[2])
        #comm1 = "a > "//str(good_min)//" ? (a-b*c*0.70) : a"
        #comm1 = "a > "//str(good_min)//" ? (a-b*c*0.70) : 0"
        #comm1 = "a > "//str(good_min)//" ? (a-b*c*0.70) : 0"
        comm1 = "a > "//str(good_min)//" ? (a-b*c*"//str(unc_fac_6583)//") : 0.0"
	ofile_N2b[j] = ofile_root//"_"//infile//"_NII6583"
        imdelete(ofile_N2b[j], ver-, >& "dev$null")
        imexpr(comm1, ofile_N2b[j], ofile[3], combHa, rfile[2], dims=size, verbose=talk)

	#(b*a/(sqrt(b*a+c*d*a))  )  #a=9.5
	#snr_N2b[j] = ofile_root//"_"//infile//"_NII6583_snr"
	snr_N2b[j] = ofile_root//"_"//infile//"_NII6583_noisq"
        imdelete(snr_N2b[j], ver-, >& "dev$null")
	#imexpr("b*a/(sqrt(b*a+c*d*a))", snr_N2b[j], "9.5", ofile[3], combHa, rfile[2], verbose=talk)
	imexpr("b*a+c*d*a", snr_N2b[j], "9.5", ofile[3], combHa, rfile[2], dims=size, verbose=talk)

	rfile_N2b[j] = ofile_root//"_"//infile//"_NII6583_resp"
        imdelete(rfile_N2b[j], ver-, >& "dev$null")
	comm1 = "a>"//str(r_cut)//" ? a : 0.0"
        imexpr(comm1, rfile_N2b[j], rfile[3], dims=size, verbose=talk)

        print(ofile_N2b[j], >> linelist_file)
        print(rfile_N2b[j], >> linewlist_file)
        print(snr_N2b[j], >> lineslist_file)
        print(ofile[3], >> lineolist_file)
	}

skip_last2:
	}

        combmono = ofile_root//"_comb_NII6583" 
        combresp = ofile_root//"_comb_NII6583_resp" 
        combnoi  = ofile_root//"_comb_NII6583_noi" 
        combsnr  = ofile_root//"_comb_NII6583_snr" 
        combrat  = ofile_root//"_comb_NII6583_rat" 
        imdelete(combrat, ver-, >& "dev$null")
        imdelete(combmono, ver-, >& "dev$null")
        imdelete(combresp, ver-, >& "dev$null")
        imdelete(combnoi, ver-, >& "dev$null")
        imdelete(combsnr, ver-, >& "dev$null")
	#ofile_N2b[j] = ofile_root//"_"//infile//"_NII6583"
	imcombine ("@"//linelist_file, combmono, combine="sum", reject="none", scale="none", zero="none", logfile=logfil)
	imcombine ("@"//linewlist_file, combresp, combine="sum", reject="none", scale="none", zero="none", logfile=logfil)
	combfin = ofile_root//"_comb_NII6583_fin"
        imdelete(combfin, ver-, >& "dev$null")
	imarith (combmono, "/", combresp, combfin, ver-)
	imcombine ("@"//lineslist_file, combnoi, combine="sum", reject="none", scale="none", zero="none", logfile=logfil)
        imdelete(combmono, ver-, >& "dev$null")
	imcombine ("@"//lineolist_file, combmono, combine="sum", reject="none", scale="none", zero="none", logfile=logfil)
	imexpr("b*a/(sqrt(c+0.00001))", combsnr, "9.5", combmono, combnoi, dims=size, verbose=talk)
        imdelete(combmono, ver-, >& "dev$null")
        imdelete(combnoi, ver-, >& "dev$null")
	if (filedel) {imdelete("@"//lineslist_file, ver-, >& "dev$null")}
	if (filedel) {imdelete("@"//linewlist_file, ver-, >& "dev$null")}
        comm1 = "b > "//str(good_min)//" ? a/b : 0.0"
        imexpr(comm1, combrat, combfin, combHa, dims=size, verbose=talk)

        combmono = ofile_root//"_comb_NII6548" 
        combresp = ofile_root//"_comb_NII6548_resp" 
        combnoi  = ofile_root//"_comb_NII6548_noi" 
        combsnr  = ofile_root//"_comb_NII6548_snr" 
        combrat  = ofile_root//"_comb_NII6548_rat" 
        imdelete(combrat, ver-, >& "dev$null")
        imdelete(combmono, ver-, >& "dev$null")
        imdelete(combresp, ver-, >& "dev$null")
        imdelete(combnoi, ver-, >& "dev$null")
        imdelete(combsnr, ver-, >& "dev$null")
	imcombine ("@"//line2list_file, combmono, combine="sum", reject="none", scale="none", zero="none", logfile=logfil)
	imcombine ("@"//line2wlist_file, combresp, combine="sum", reject="none", scale="none", zero="none", logfile=logfil)
	combfin  = ofile_root//"_comb_NII6548_fin"
        imdelete(combfin, ver-, >& "dev$null")
	imarith (combmono, "/", combresp, combfin, ver-)
	imcombine ("@"//line2slist_file, combnoi, combine="sum", reject="none", scale="none", zero="none", logfile=logfil)
        imdelete(combmono, ver-, >& "dev$null")
	imcombine ("@"//line2olist_file, combmono, combine="sum", reject="none", scale="none", zero="none", logfile=logfil)
	imexpr("b*a/(sqrt(c+0.00001))", combsnr, "9.5", combmono, combnoi, dims=size, verbose=talk)
        imdelete(combmono, ver-, >& "dev$null")
        imdelete(combnoi, ver-, >& "dev$null")
	if (filedel) {imdelete("@"//line2slist_file, ver-, >& "dev$null")}
	if (filedel) {imdelete("@"//line2wlist_file, ver-, >& "dev$null")}

        comm1 = "b > "//str(good_min)//" ? a/b : 0.0"
        imexpr(comm1, combrat, combfin, combHa, dims=size, verbose=talk)
	#imarith (combfin, "/", combHa, ofile_root//"_comb_NII6548_rat", ver-)
	if (postproc=="Ha") goto exit

SII_proc:
        print ("Reconstructing the SII images")
        print ("Reconstructing the SII images", >> logfil)
        linelist_file = "tf_monimage_linefiles.lis"
        delete(linelist_file, ver-, >& "dev$null")
        linewlist_file = "tf_monimage_linewfiles.lis"
        delete(linewlist_file, ver-, >& "dev$null")

        lineslist_file = "tf_monimage_linesfiles.lis"
        delete(lineslist_file, ver-, >& "dev$null")
        lineolist_file = "tf_monimage_lineofiles.lis"
        delete(lineolist_file, ver-, >& "dev$null")

        line2list_file = "tf_monimage_line2files.lis"
        delete(line2list_file, ver-, >& "dev$null")
        line2wlist_file = "tf_monimage_line2wfiles.lis"
        delete(line2wlist_file, ver-, >& "dev$null")
        line2slist_file = "tf_monimage_line2sfiles.lis"
        delete(line2slist_file, ver-, >& "dev$null")
        line2olist_file = "tf_monimage_line2ofiles.lis"
        delete(line2olist_file, ver-, >& "dev$null")
        linebwlist_file = "tf_monimage_linebwfiles.lis"
        delete(linebwlist_file, ver-, >& "dev$null")
        line2bwlist_file = "tf_monimage_line2bwfiles.lis"
        delete(line2bwlist_file, ver-, >& "dev$null")

        #SII-Flux-corrected line images
        #imstat (contfile//stat_sec1, fields = "mode, stddev", lower=goodmin, upper=goodmax,  format-) | scan (zmode, zstddev)
        good_min = ksig*zstddev
	print("Minimum good pixel value = ",good_min)
	print("Minimum good pixel value = ",good_min, >> logfil)

        for (j=1; j <= i1; j +=1) {
        infile = infiles[j]//"s"; lam0 = wav0[j]; dlam = dwav[j]
        #print(infile, " ",lam0)
        for (i=1; i <= nlines; i +=1) {
        rfile[i] = ofile_root//"_"//infile//"_resp_"//str(int(lam[i]))
	ofile[i] = ofile_root//"_"//infile//"_mono_"//str(int(lam[i]))
        #rfile[2] = ofile_root//"_"//infile//"_resp_"//str(int(lam[2]))
        #ofile[2] = ofile_root//"_"//infile//"_mono_"//str(int(lam[2]))
        #print ("Reconstructing the monochromatic image at ", lam[2], " from ",infile)
	}

        if (lam[1] <= (lam0+dlam/2.0) && lam[1] < maxlam && lam[1] > minlam) {
        rfile_S2a[j] = ofile_root//"_"//infile//"_S2a_resp"
        rfile_S2b[j] = ofile_root//"_"//infile//"_S2b_resp"
        rfile_S2ab[j] = ofile_root//"_"//infile//"_S2ab_resp"
        rfile_S2ba[j] = ofile_root//"_"//infile//"_S2ba_resp"
        imdelete(rfile_S2a[j], ver-, >& "dev$null")
        imdelete(rfile_S2b[j], ver-, >& "dev$null")
        imdelete(rfile_S2ab[j], ver-, >& "dev$null")
        imdelete(rfile_S2ba[j], ver-, >& "dev$null")
        #comm1 = "a>"//str(r_cut)//" ? (a+b*c) : 0.0"
        #imexpr(comm1, rfile_S2a[j], rfile[1], rfile[2], "0.5", dims=size, verbose=talk)
        #comm1 = "a>"//str(r_cut)//" ? (a+b/c) : 0.0"
        #imexpr(comm1, rfile_S2b[j], rfile[2], rfile[1], "0.5", dims=size, verbose=talk)
        comm1 = "a>"//str(r_cut)//" ? a : 0.0"
        imexpr(comm1, rfile_S2a[j], rfile[1], dims=size, verbose=talk)
        imexpr(comm1, rfile_S2b[j], rfile[2], dims=size, verbose=talk)
        comm1 = "a>"//str(r_cut)//" ? b : 0.0"
        imexpr(comm1, rfile_S2ab[j], rfile[1], rfile[2], dims=size, verbose=talk)
        imexpr(comm1, rfile_S2ba[j], rfile[2], rfile[1], dims=size, verbose=talk)
	#ab ===> response of 6731 line at 6717-bright sections
	#ba ===> response of 6717 line at 6731-bright sections
	#Files to reconstruct 6717-bright sections
        print(ofile[1], >> linelist_file)
        print(rfile_S2a[j], >> linewlist_file)
        print(rfile_S2ab[j], >> linebwlist_file)
	#
	#Files to reconstruct 6731-bright sections
        if (lam[2] <= (lam0+dlam/2.0) && lam[2] < maxlam && lam[2] > minlam) {
        print(ofile[2], >> line2list_file)
        print(rfile_S2b[j], >> line2wlist_file)
        print(rfile_S2ba[j], >> line2bwlist_file)
        }
        }
skip_last3:
        }

        combmono = ofile_root//"_comb_S2a"
        combresp = ofile_root//"_comb_S2a_resp"
        combmono2 = ofile_root//"_comb_S2b"
        combrespb = ofile_root//"_comb_S2ab_resp"
        combresp2 = ofile_root//"_comb_S2b_resp"
        combresp2b = ofile_root//"_comb_S2ba_resp"
        imdelete(combmono, ver-, >& "dev$null")
        imdelete(combresp, ver-, >& "dev$null")
        imdelete(combmono2, ver-, >& "dev$null")
        imdelete(combrespb, ver-, >& "dev$null")
        imdelete(combresp2, ver-, >& "dev$null")
        imdelete(combresp2b, ver-, >& "dev$null")
        imcombine ("@"//linelist_file, combmono, combine="sum", reject="none", scale="none", zero="none", logfile=logfil)
        imcombine ("@"//line2list_file, combmono2, combine="sum", reject="none", scale="none", zero="none", logfile=logfil)
        imcombine ("@"//linewlist_file, combresp, combine="sum", reject="none", scale="none", zero="none", logfile=logfil)
        imcombine ("@"//linebwlist_file, combrespb, combine="sum", reject="none", scale="none", zero="none", logfile=logfil)
        imcombine ("@"//line2wlist_file, combresp2, combine="sum", reject="none", scale="none", zero="none", logfile=logfil)
        imcombine ("@"//line2bwlist_file, combresp2b, combine="sum", reject="none", scale="none", zero="none", logfile=logfil)
        combS2a = ofile_root//"_comb_S2a_fin"
        combS2b = ofile_root//"_comb_S2b_fin"
        combrat = ofile_root//"_comb_S2ab_rat" 
        imdelete(combS2a, ver-, >& "dev$null")
        imdelete(combS2b, ver-, >& "dev$null")
        imdelete(combrat, ver-, >& "dev$null")

	#Assume a constant ratio of S2 lines, and go on improving it until
	#the dispersion in ratio between the two successive "ratio images" is less than 5%.
        imexpr("a*0.0+0.5", combrat, combmono, dims=size, verbose=talk)
improve:
        imdelete(combS2a, ver-, >& "dev$null")
        imdelete(combS2b, ver-, >& "dev$null")
        imdelete(combrat//"_1", ver-, >& "dev$null")
        imdelete(combrat//"_2", ver-, >& "dev$null")
	#Calculate the SII6717 and SII6731 line maps for the assumed ratio of SII6717/SII6731
	comm1 = "a/(b+c*d)"
        imexpr(comm1, combS2a, combmono, combresp, combrespb, combrat, dims=size, verbose=talk)
        imexpr(comm1, combS2b, combmono2, combresp2, combresp2b, combrat, dims=size, verbose=talk)
	imcopy (combrat, combrat//"_1", verbose-)            #Save the input ratio image SII6717/SII6731
        imdelete(combrat, ver-, >& "dev$null")
        comm1 = "b > "//str(good_min)//" ? a/b : 0.0"
        imexpr(comm1, combrat, combS2a, combS2b, dims=size, verbose=talk)    #Calculate the new ratio
        comm1 = "b > 0.0 ? a/b : 0.0"
        imexpr(comm1, combrat//"_2", combrat//"_1", combrat, dims=size, verbose=talk)
        imstat (combrat//"_2", fields = "mode, midpt, stddev", lower=0.2, upper=5,  format-) | scan (zmode, zmedian, zstddev)
	print(zmode, " ",zmedian, " ",zstddev)
	#if (zmode < 0.99 || zmode > 1.01) {
	if (zmedian < 0.99 || zmedian > 1.01) {
        #imdelete(combrat//"_1", ver-, >& "dev$null")
        #imdelete(combrat//"_2", ver-, >& "dev$null")
		goto improve
		}
        imdelete(combrat//"_1", ver-, >& "dev$null")
        imdelete(combrat//"_2", ver-, >& "dev$null")
        #imexpr(comm1, , dims=size, verbose=talk)
	if (postproc=="S2") goto exit

SIII_proc:
	print ("Reconstructing the SIII image")
        linelist_file = "tf_monimage_linefiles.lis"
        delete(linelist_file, ver-, >& "dev$null")
        linewlist_file = "tf_monimage_linewfiles.lis"
        delete(linewlist_file, ver-, >& "dev$null")
        imdelete(ofile_root//"*"//"_S3_resp.fits", ver-, >& "dev$null")
        good_min = ksig*zstddev

        for (j=1; j <= i1; j +=1) {
        infile = infiles[j]//"s"; lam0 = wav0[j]; dlam = dwav[j]
        #print(infile, " ",lam0)
        rfile[1] = ofile_root//"_"//infile//"_resp_"//str(int(lam[1]))
	ofile[1] = ofile_root//"_"//infile//"_mono_"//str(int(lam[1]))
	#print ("Reconstructing the monochromatic image at ", lam[1], " from ",infile)
	if (lam[1] <= (lam0+dlam/2.0) && lam[1] < maxlam && lam[1] > minlam) {
	rfile_S3[j] = ofile_root//"_"//infile//"_S3_resp"
        imdelete(rfile_S3[j], ver-, >& "dev$null")
	comm1 = "a>"//str(r_cut)//" ? a : 0.0"
        imexpr(comm1, rfile_S3[j], rfile[1], dims=size, verbose=talk)
        print(ofile[1], >> linelist_file)
        print(rfile_S3[j], >> linewlist_file)
	}
skip_last_S3:   
	}

        combmono = ofile_root//"_comb_S3" 
        combresp = ofile_root//"_comb_S3_resp" 
        imdelete(combmono, ver-, >& "dev$null")
        imdelete(combresp, ver-, >& "dev$null")
	imcombine ("@"//linelist_file, combmono, combine="sum", reject="none", scale="none", zero="none", logfile=logfil)
	imcombine ("@"//linewlist_file, combresp, combine="sum", reject="none", scale="none", zero="none", logfile=logfil)
	combS3 = ofile_root//"_comb_S3_fin"
        imdelete(combS3, ver-, >& "dev$null")
	imarith (combmono, "/", combresp, combS3, ver-)

	#if (filedel) {imdelete("@"//linelist_file, ver-, >& "dev$null")}
	#if (filedel) {imdelete("@"//linewlist_file, ver-, >& "dev$null")}
	#if (filedel) {imdelete(combmono, ver-, >& "dev$null")}

exit:
	date (, >> logfil)
        print("***DONE***")
end

