procedure  gtc_osiris_tf_calib (inlist, phot_list)

string  inlist     ="tf_Ha.s_blkav_images.lis" {prompt="List file containing input fits filenames"}
string  phot_list  ="M101_GoodStars_xy.coo" {prompt="List file containing coordinates of stars"}
string  output     ="TFrec/tf"         {prompt="Prefix for the output monochromatic image"}
real    xcen       = 461.        {prompt="X center of the TF wrt the image -52 or 2068 "}
real    ycen       = 523.        {prompt="Y center of the TF wrt the image 1966 "}
real    scale      = 0.50       {prompt="Image scale in arcsec/pixel"}
string  ccdstar    = "both"                     {prompt="Use star in [both/right/left] CCD"}
string  plottype   = "fac"                      {prompt="Type of plot [fac/mag/flux]]"}
bool    plotnum    = yes                        {prompt="Plot TF number instead of wavelength?"}
string  dirstar    = "segue_spectra_stars/"     {prompt="Directory name containing SDSS spectral fits"}
string  extfile    = "tn_ext_curve_edit.dat"    {prompt="Exinction curve file name [lam(ang) vs ext]"}
int     colno      = 2                          {prompt="Col number to be used for the stellar spectrum"}
real    fluxunit   = 1.0E-17                    {prompt="Units of flux scale in dirstar files"}
real    magcor     = 0.03                       {prompt="Magnitude growth between apno 3 and 5"}
bool    useosf     = yes                        {prompt="Use order sorter filter?"}
bool    headup     = no                         {prompt="Update the header with the results of calibration process?"}
string  calfile    ="gtc_osiris_Ha_efficiency.dat" {prompt="Cal coeff file if updatehead=yes"}
bool    calchec    = no                         {prompt="Check the calibration process instead of calculating Cal coeff?"}


begin
	string  infile, ofile[9], size, comm1, ofile_root, cal_file
	string  tmp, s1, infiles[9], logfil, tmpsky, tmp2sky, statfil, range
	string  input_list, file_star[99], dir_star, name_star[99], tmp2mag
	string  phot_file, tmpmag, plot_type, ext_file, ord_sort_fil[9], ylabel
	string  format1, format2, format3, format4, use_stars, phot_coo, graph_fil
	real    xc, yc, lam0, dlam, exptime, airmass, ext_coef, max_eff_filt
	real    vel, rl1, rl2, rl3, rl4, rl5, rl6, rl7, rl8, rl9, dopfac
	real    lam[9], zmode, r1, wav0[9], r2, dwav[9], lamline[6]
	real    pixsec, pixmin, pixmin2, lam_xy, xx, yy, mag, flux 
	real    xcent, ycent, mag0, merr, ferr, ferrn, mag_shift, flux_unit, ps
	real    star_flux, xdum, mag_apcor, yflux1, yflux2, wx1, wx2, wy1, wy2
	real    wave_1, wave_2, wave_int, eff_filt, band_flux_rms, wx1_old
	real    band_fac_median, band_fac_rms, mag5, mean_dif, flux_calr, flux_call
	real    cal_rel_mean, cal_rel_right, cal_rel_left, flux_cal_right, flux_cal_mean
	real    cal_coef, cal_coef_sec, cal_coef_secn, cal_coef_sec1, flux_cal_left
	real    mag_mean[99], mag_median[99], mag_rms[99], rad_star[99], flux_mean[99]
        real    flux_median[99], per_err[99], xstar[99], ystar[99]
	int     nx, ny, nlam, i1, i2, nlines, i2l, i2v, i2c,icol
	int     ii, id, nstars, n_wave
	bool    emline, plotno, eff_corr, check_calib, update_head
	struct  line

        #--------------------------------------------------------------------
        #redefine parameters controlling the running of the script
        #--------------------------------------------------------------------

	input_list = inlist; phot_file = phot_list; ofile_root=output
        xc = xcen; yc = ycen; pixsec = scale; pixmin =(pixsec/60.0); pixmin2 =pixmin**2
        plot_type = plottype; dir_star = dirstar; ext_file = extfile 
	mag_apcor = magcor; flux_unit = fluxunit; icol = colno; plotno = plotnum
        cal_file = calfile; check_calib = calchec; update_head = headup; eff_corr = useosf 
	use_stars = ccdstar
        ps = pixsec/0.125
	yflux1 = 7E-19; yflux2 = 1.5E-16; eff_filt=100.0

        logfil = ofile_root//"_calib.log"
        statfil = ofile_root//"_calib_stat.log"
        delete(statfil, ver-, >& "dev$null")
	phot_coo = "temp_calib_phot.coo"
        delete(phot_coo, ver-, >& "dev$null")
	
        #--------------------------------------------------------------------
        #Read the list containing star positions and name to find out the number of stars to be used for calibration
        #--------------------------------------------------------------------
        tmpsky = mktemp ("tmp$ia_tmp.")
        tmp2sky = mktemp ("tmp$ia_tmp.")
        tmpmag = mktemp ("tmp$ia_tmp.")
        tmp2mag = mktemp ("tmp$ia_tmp.")

        ii = 0
        list = phot_file
        while( fscan(list, line) != EOF ) {
        tmp = substr (line, 1, 1)
        if (tmp != "#") {
          i2 = fscanf(line, "%g %g %s", r1, r2, s1)
	  if (use_stars == "right" && r1 < xc) goto skip_star 
	  if (use_stars == "left" && r1 > xc) goto skip_star 
	  ii = ii + 1
	  xstar[ii] = r1; ystar[ii] = r2; file_star[ii]=dir_star//s1; name_star[ii]=s1
	  rad_star[ii]=sqrt(pixmin2*((xstar[ii]-xc)**2 + (ystar[ii]-yc)**2))
	  print(r1, r2, >> phot_coo)
	  #print(ii, xstar[ii], ystar[ii], " ",rad_star[ii], " ", file_star[ii], " ", name_star[ii])
	}
skip_star:
	}
	nstars = ii; i1 = ii
	wy1=-0.3; wy2=real(nstars-1)*0.2 + 0.3
	print("No of stars = ",nstars, wy1, wy2)

        #--------------------------------------------------------------------
        #Read the list containing fits images to be processed #gets filename, lam0
        #--------------------------------------------------------------------

        i1 = 0
        list = input_list    
        while( fscan(list, line) != EOF ) {
        tmp = substr (line, 1, 1)
        if (tmp != "#") {
          i1=i1+1
          if (i1 == 1) {i2l = fscanf(line, "%g %g %g %g %g %g ", rl1, rl2, rl3, rl4, rl5, rl6)}
          if (i1 == 2) {i2v = fscanf(line, "%g ", vel)
        dopfac = (1.0 + vel/2.9997E+05)
        lamline[1] = rl1*dopfac; lamline[2] = rl2*dopfac; lamline[3] = rl3*dopfac
        lamline[4] = rl4*dopfac; lamline[5] = rl5*dopfac; lamline[6] = rl6*dopfac; nlines=i2l
        for (i=2; i <= nlines; i +=1) {
                if (lamline[i] < lamline[i-1] ) {
                        print("ERROR: Emission line wavelengths in the input list should be in the increasing order")
                        goto exit
                        }
                }
        }
          if (i1 == 3) {i2c = fscanf(line, "%g %g %g %g %g %g ", rl1, rl2, rl3, rl4, rl5, rl6)
        #--------------------------------------------------------------------
        #nlam is the number of images that go into forming a line image.
        #--------------------------------------------------------------------
        nlam=i2c + nlines
        for (i=1; i <= nlines; i +=1) {lam[i] = lamline[i] }
        lam[nlines+1] = rl1; lam[nlines+2] = rl2; lam[nlines+3] = rl3 
        lam[nlines+4] = rl4; lam[nlines+5] = rl5; lam[nlines+6] = rl6 
        }
          if (i1 > 3) {
          i2 = fscanf(line, "%s %g %g %s ", s1, r1, r2, s2)
          infiles[i1-3] = s1; wav0[i1-3] = r1; dwav[i1-3] = r2; ord_sort_fil[i1-3] = s2
          #print (infiles[i1-3])
          print (infiles[i1-3], >> tmpsky)
          print (infiles[i1-3]//"c", >> tmp2sky)
          print (infiles[i1-3]//".mag.1", >> tmpmag)
          print (infiles[i1-3]//"c.mag.1", >> tmp2mag)
          }
        } }

        i1 = i1 - 3

        delete("@"//tmpmag, ver-, >& "dev$null")
        delete("@"//tmp2mag, ver-, >& "dev$null")

	print("--------------------------------------- gtc_osiris_tf_calib ----------------------", >> logfil)
	date (, >> logfil)
	print("--------------------------------------- ", input_list,  " ", phot_file, " -------------------------", >> logfil)

        hselect (infiles[1], "i_naxis1 i_naxis2 EXPTIME, AIRMASS", expr+) | scan (nx, ny, exptime, airmass)
        hselect ("@"//tmpsky, "i_naxis1 i_naxis2 EXPTIME GAIN FILTER1 FILTER2 FILTER3 AIRMASS OBJECT $I", expr+, >> logfil)
        hselect ("@"//tmpsky, "i_naxis1 i_naxis2 EXPTIME GAIN FILTER1 FILTER2 FILTER3 AIRMASS OBJECT $I", expr+)
        size=str(nx)//","//str(ny)
        print (" Nlines = ",nlines," Ncont = ", i2c, " Nfilt = ",i1)
        print (" Nlines = ",nlines," Ncont = ", i2c, " Nfilt = ",i1, >> logfil)

	wx1_old = wav0[1]*(1.0-7.9520E-04*pixmin2*((real(nx)/2.0)**2))
	wx1 = wav0[1]/sqrt(1.0+6.5247E-09*((real(nx)/2.0)**2)*(pixsec/0.125)**2)
	wx2 = wav0[i1] 
	print(wx2, wx1, wx1_old)

	if (check_calib) {
	for (j=1; j <= i1; j +=1) {
        imdelete(infiles[j]//"c", ver-, >& "dev$null")
	imexpr ("(I > c) ? (a-b)*a.FLUX_REL : (a-b)*a.FLUX_REL*a.FLUX_LEF/a.FLUX_RIG", infiles[j]//"c", infiles[j], 0.0, xc, verbose- )
        }
	}

	datapars.scale=1.0; datapars.emissio=yes
        centerpars.calgori="centroid"; centerpars.cbox=5.0; fitskypars.salgori="mode"
	photpars.apertur=str(12.0/ps)//","//str(16.0/ps)//","//str(20.0/ps)//","//str(24.0/ps)//","//str(32.0/ps)
        fitskypars.annulus=32.0/ps; fitskypars.dannulu=10.0/ps
	print(photpars.apertur," ", fitskypars.annulus," ", fitskypars.dannulu)
        if (check_calib) { phot("@"//tmp2sky, coords=phot_coo, interac-, verify-, verbose-)
                 } else  { phot("@"//tmpsky, coords=phot_coo, interac-, verify-, verbose-)  }

	for (ii=1; ii <= nstars; ii +=1) { 
	  #print(xstar[ii], ystar[ii], " ",rad_star[ii], " ", file_star[ii], " ", name_star[ii])
        delete("temp_graph.coef", ver-, >& "dev$null")
        delete("temp_graph.mag", ver-, >& "dev$null")
        delete("temp_graph.fac", ver-, >& "dev$null")

        for (j=1; j <= i1; j +=1) {                                   # loop over i1=9 filters
        infile = infiles[j]; lam0 = wav0[j]; dlam = dwav[j]
        if (check_calib) {infile = infiles[j]//"c"}
	comm1 = "ID="//str(ii)
	txdump (infile//".mag.1", "MAG[3] FLUX[3] MERR[3] MAG[5] XCEN YCEN ID", comm1) | scan (mag, flux, merr, mag5, xcent, ycent, id)
	#mag_apcor = mag - mag5; mag_apcor = 0.03
        hselect (infiles[1], "EXPTIME AIRMASS", expr+) | scan (exptime, airmass)

	#  if (j==1) {printf("# %8.2f %8.2f %8.3f %3d %s \n", xstar[ii], ystar[ii], mag0, id, file_star[ii], >> logfil)}
	xx = xstar[ii]; yy = ystar[ii]
	#lam_xy = lam0*(1.0-7.9520E-04*pixmin2*((xx-xc)**2 + (yy-yc)**2))
	lam_xy = lam0/sqrt(1.0+6.5247E-09*((xx-xc)**2 + (yy-yc)**2)*(pixsec/0.125)**2)
        delete("temp_wavelength.tbl", ver-, >& "dev$null")
        delete("temp_wavelength_nm.tbl", ver-, >& "dev$null")
	print(lam_xy, > "temp_wavelength.tbl")
        print(lam_xy/10., > "temp_wavelength_nm.tbl")
        delete("temp_spect.tbl", ver-, >& "dev$null")
	#fields (file_star[ii], "1,2", > "temp_spect.tbl")  
        fields (file_star[ii], "1,"//str(icol), > "temp_spect.tbl")
	interp ("temp_spect.tbl", "temp_wavelength.tbl") | scan (xdum, star_flux)
	star_flux = star_flux*flux_unit
	interp (ext_file, "temp_wavelength.tbl") | scan (xdum, ext_coef)
	if (eff_corr){
		#interp (ord_sort_fil[j], "temp_wavelength_nm.tbl")
		interp (ord_sort_fil[j], "temp_wavelength_nm.tbl") | scan (xdum, eff_filt)
        	tstat (ord_sort_fil[j], "c2", >& "dev$null"); max_eff_filt = tstat.vmax
        	flux = (max_eff_filt/eff_filt)*flux*10**(0.4*(mag_apcor+ext_coef*airmass))
                } else {
			flux = flux*10**(0.4*(mag_apcor+ext_coef*airmass)) 
		}

	#mag = mag - mag_apcor
	mag = -2.5*log10(flux)
	if (j==1) {mag0 = mag}
	mag_shift = real(ii-1)*0.2
	mag = mag - mag0 + real(ii-1)*0.2
	cal_coef = dlam*star_flux/flux
	cal_coef_sec = exptime*dlam*star_flux/flux
	ferr = flux*(1.0 - 10**(-0.4*merr))
	merr = -1.0*merr
	if (plotno){print(j, mag, merr, >> "temp_graph.mag")} else {print(lam_xy, mag, merr, >> "temp_graph.mag")}
	if (plotno){print(j,cal_coef_sec, ferr, >> "temp_graph.coef")} else {print(lam_xy, cal_coef_sec, ferr, >> "temp_graph.coef")}
	mag = mag + mag0 - mag_shift

	if (j ==1) {cal_coef_sec1 = cal_coef_sec}
	cal_coef_secn = cal_coef_sec/cal_coef_sec1
	ferrn = ferr/flux
        if (plotno){print(j, cal_coef_secn, ferr, >> "temp_graph.fac")} else { print(lam_xy, cal_coef_secn, "0.00", >> "temp_graph.fac") }
        #if (plotno){print(j, cal_coef_secn, "0.00", >> "temp_graph.fac")} else { print(lam_xy, cal_coef_secn, "0.00", >> "temp_graph.fac") }
	#print(j, cal_coef_secn, "0.00", >> "temp_graph.fac")
	#print(lam_xy, cal_coef_secn, "0.00", >> "temp_graph.fac")
	format1 = "# %8.2f %8.2f %8.3f %3d %s \n"
	format2 = "%8.1f %8.1f %8.1f %8.1f %8.1f %8.3f %10.1f %8.1f %8.1f %12.5e %12.5e %12.5e %7.3f %3d \n"
	format4 = "%d %d %s %7.1f %7.2f %6.3f %12.1f %8.3f %8.3f %6.3f %7.1f %7.1f \n"
	format3 = "%7.1f %7.1f %7.1f %6.3f %8.1f %8.3f %10.1f %7.1f %7.1f %12.5e %12.5e %12.5e %7.3f %3d \n"
	if (j==1) {printf(format1, xstar[ii], ystar[ii], mag0, id, file_star[ii], >> logfil)}
	printf(format2, lam0, xstar[ii], ystar[ii], rad_star[ii], lam_xy, mag, flux, xcent, ycent, star_flux, cal_coef, cal_coef_sec, cal_coef_secn, id, >> logfil)
	printf(format3, lam0, xstar[ii], ystar[ii], rad_star[ii], lam_xy, mag, flux, xcent, ycent, star_flux, cal_coef, cal_coef_sec, cal_coef_secn, id, >> statfil)
	printf(format4, ii, j, ord_sort_fil[j], lam_xy, eff_filt, ext_coef, flux, mag, mag_apcor, cal_coef_secn, xstar[ii], ystar[ii], >> "gtc_osiris_tf_calib.log")
	}

	tstat ("temp_graph.mag", "c2", >> logfil)
	mag_mean[ii] = tstat.mean;  mag_median[ii]= tstat.median; mag_rms[ii] = tstat.stddev
        delete("temp_tstat.log", ver-, >& "dev$null")
	tstat ("temp_graph.coef", "c2", >& "dev$null") #outtabl="temp_tstat.log")
	   yflux1 = tstat.median - 2.0*tstat.stddev; yflux2 = tstat.median + 2.0*tstat.stddev
	tstat ("temp_graph.coef", "c2", lowlim=yflux1, highlim=yflux2, >> logfil)
	flux_mean[ii] = tstat.mean; flux_median[ii] = tstat.median; per_err[ii] = tstat.stddev/tstat.mean
	format1 = "%8.2f %8.2f %8.3f %8.3f %8.3f %3d %s %9.3e %6.3f %9.3e %4d \n"
	  printf(format1, xstar[ii], ystar[ii], rad_star[ii], mag0, mag_rms[ii], id, name_star[ii], flux_mean[ii], per_err[ii], flux_median[ii], tstat.nrows)
	  printf(format1, xstar[ii], ystar[ii], rad_star[ii], mag0, mag_rms[ii], id, name_star[ii], flux_mean[ii], per_err[ii], flux_median[ii], tstat.nrows, >> "gtc_osiris_tf_calib.log")

	if (plot_type == "mag") {graph_fil = "temp_graph.mag"; ylabel="Delta Mag"}
	if (plot_type == "fac") {graph_fil = "temp_graph.fac"; ylabel="Relative efficiency"; wy1=0.85; wy2=1.22}
	#if (plot_type == "fac") {graph_fil = "temp_graph.fac"; ylabel="Relative efficiency"; wy1=0.77; wy2=1.07}
	if (plot_type == "flux") {graph_fil = "temp_graph.coef"; ylabel="Cal Coef"; wy1 = tstat.median - 5.0*tstat.stddev; wy2 = tstat.median + 5.0*tstat.stddev}
	if (ii==1) {
         if (plotno){wx1 = 1.0 - real(i1)*0.08; wx2 = real(i1)*1.08
                graph ("plot_nothing.dat", pointmo-, wx1=wx1, wx2=wx2, wy1=wy1, wy2=wy2, colors=ii, append-, xlab="TF scan number", ylab=ylabel)
                } else {
         graph ("plot_nothing.dat", pointmo-, wx1=wx1, wx2=wx2, wy1=wy1, wy2=wy2, colors=ii, append-, xlab="Wavelength (Ang)", ylab=ylabel)
                }
        }  #ii==1 end
        graph (graph_fil, pointmo+, szmarke=0, marker="vebar", append+)
        if (xstar[ii] > xc) {
                graph (graph_fil, pointmo-, colors=ii, ltype=1, append+)
                } else {
                graph (graph_fil, pointmo-, colors=ii, ltype=2, append+)
                }
	}    # loop ii over nstars end

        for (j=1; j <= i1; j +=1) {
	range = str(j)
        for (i=j+i1; i <= nstars*i1; i +=i1) { range = range//","//str(i) }
	print(infiles[j], " ", range, >> logfil)
        delete("temp_tstat.log", ver-, >& "dev$null")
	fields (statfil, "13", lines=range) | tstat (, >& "dev$null") #outtabl="temp_tstat.log")
	band_fac_rms = tstat.stddev; band_fac_median = tstat.median
	fields (statfil, "12", lines=range) | tstat (, >& "dev$null") #outtabl="temp_tstat.log")
	band_flux_rms = tstat.stddev/tstat.median
	yflux1 = tstat.median - 2.0*tstat.stddev; yflux2 = tstat.median + 2.0*tstat.stddev
	fields (statfil, "12", lines=range) | tstat (lowlim=yflux1, highlim=yflux2 ,>> logfil)
	format1 = "%s %12.4e %8.3f %12.4e %4d %6.3f %6.3f \n"
	printf(format1, infiles[j], tstat.mean, band_flux_rms, tstat.median, tstat.nrows, band_fac_median, band_fac_rms)
	printf(format1, infiles[j], tstat.mean, band_flux_rms, tstat.median, tstat.nrows, band_fac_median, band_fac_rms, >> "gtc_osiris_tf_calib.log")
	}

	if (update_head) {
        delete("temp_tstat2.log", ver-, >& "dev$null")
        list = cal_file       #"gtc_osiris_Ha_efficiency.dat"
          i2 = fscan(list, flux_cal_right, flux_cal_left, flux_cal_mean, mean_dif)
	#print(flux_cal_right, flux_cal_left, flux_cal_mean)
	flux_calr = flux_cal_right; flux_call = flux_cal_right*(1.0+mean_dif)
	#flux_calr = flux_cal_right; flux_call = flux_cal_right*(flux_cal_left/flux_cal_right)

        for (j=1; j <= i1; j +=1) {
          i2 = fscan(list, cal_rel_mean, cal_rel_right, cal_rel_left)
	  #print(cal_rel_mean, cal_rel_right, cal_rel_left)
	  mean_dif = (cal_rel_left*flux_cal_left - cal_rel_right*flux_cal_right)/flux_cal_right
	print(mean_dif, >> "temp_tstat2.log")
	hedit (infiles[j], fields="FLUX_RIG", value=flux_calr, add+, delete-, verify-, show-, update+)
	hedit (infiles[j], fields="FLUX_LEF", value=flux_call, add+, delete-, verify-, show-, update+)
	hedit (infiles[j], fields="FLUX_REL", value=cal_rel_mean, add+, delete-, verify-, show-, update+)
	}
	fields ("temp_tstat2.log", "1") | tstat  #(, >& "dev$null") #outtabl="temp_tstat.log")
	}

exit:
        print("***DONE***")
end
