procedure  osiris_subsky(infile, xshi)

#-------------------  osiris_subsky   ydm@inaoep.mx   11-Jan-2011  -----------------------------------------
# This script prepares the images for the scripts osiris_recon and osiris_calib
# Bins the images by "binfac x binfac" if binfac > 1
# Prepares a background image b[i,j] by taking the min of b[i,j], b[i-xshi,j], b[i+xshi,j], b[i-xshi/2,j], b[i+xshi/2,j] 
# Subtracts the background image "infile_med" from the "infile" and stores it in "infile_s"
# The critical background subtraction parameters [xshi, skysec1, skysec2, xcen, smobox] 
#          could be wrt the binned image [binpar=yes] or the original image (binpar=no)
#-----------------------------------------------------------------------------------------------------------

string  infile     ="M101Ha6608b.ss"          {prompt="Input Image file"}
int 	binfac     = 4		 {prompt="Rebinning factor for the image using blkav"}
bool	binpar     = yes         {prompt="Subsky parameters are wrt the binned image?"}
int     xshi       = 90          {prompt="shift in x"}
string  skysec1    ="[381:420,501:540]"       {prompt="Sky section in the LEFT CCD "}
string  skysec2    ="[501:540,661:700]"       {prompt="Sky section in the RIGHT CCD "}
real    xcen       = 461.        {prompt="X center of the TF wrt the image 461 "}
int     smobox     = 21          {prompt="Smoothing box size"}
#string  statsec    ="[601:605,601:605]"       {prompt="Object-free section for imstat"}
#int     gapxl      = 450         {prompt="Left x value of the gap"}
#int     gapxr      = 470         {prompt="Right x value of the gap"}

begin
	string	infil, size, comm1, stat_sec1, stat_sec2, infilb 
	string	infilmed, ofil, tsections1, imfiltemp, imfil
	real	 smo_min, smo_max, zmedian, zstddev, zmedian1, zstddev1, zmedian2, zstddev2
	int	nx, ny, ixs, ixm, ix2, ix2m, ixb, ixc  #ixl, ixr
	int	nimages, i1, i2, bin
	bool	binp

        imfiltemp = infile 
	bin = binfac; binp = binpar
	#infil = infile
	if (binp) {
	ixb = smobox; ixc = int(xc)
	stat_sec1 = skysec1; stat_sec2 = skysec2
	ixs = xshi; ixm = -ixs; ix2 = ixs/2; ix2m = -ix2
	} else {
	ixb = smobox/bin; ixc = int(xc/bin)
	stat_sec1 = "[*,*]"; stat_sec2 = "[*,*]"
	ixs = xshi/bin; ixm = -ixs; ix2 = ixs/2; ix2m = -ix2
	#ixl = gapxl; ixr = gapxr; 
	}
        imdelete("tttsky1", ver-, >& "dev$null")
        imdelete("tttsky2", ver-, >& "dev$null")
        imdelete("tttsky3", ver-, >& "dev$null")
        imdelete("tttsky4", ver-, >& "dev$null")
        imdelete("tttsky5", ver-, >& "dev$null")

	#------------------------------------------------
	#analysing the infile list

        tsections1 = mktemp ("tmps1")

        sections (imfiltemp,  option="fullname", > tsections1)
        nimages = sections.nimages

        list = tsections1
        for (i = 1; i <= nimages; i = i + 1) {
        i2 = fscan(list, infil)

        #c1 = fitsext; #len = strlen(infil); #i1 = strlstr(c1,infil); 
	i1 = strlstr(".fits",infil) 
	if (i1 > 0) {infil = substr(infil,1,i1-1)}

	#------------------------------------------------

        imdelete("tttsky0", ver-, >& "dev$null")
        imdelete("tttsky1", ver-, >& "dev$null")
        imdelete("tttsky2", ver-, >& "dev$null")
        imdelete("tttsky3", ver-, >& "dev$null")
        imdelete("tttsky4", ver-, >& "dev$null")
        imdelete("tttsky5", ver-, >& "dev$null")
	infilmed = infil//"_med"; ofil = infil//"_s"
        imdelete(infilmed, ver-, >& "dev$null")
        imdelete(ofil, ver-, >& "dev$null")

	infilb = infil
	if (bin > 1) {
	infilb = infil//"_b"
        imdelete(infilb, ver-, >& "dev$null")
	blkav (infil, infilb, option="sum", b1=bin, b2=bin)
	}
        hselect (infilb, "i_naxis1, i_naxis2", expr+) | scan (nx, ny)
        size=str(nx)//","//str(ny)

        imstat (infilb//stat_sec1, fields = "midpt, stddev", nclip=12,  format-) | scan (zmedian1, zstddev1)
        imstat (infilb//stat_sec2, fields = "midpt, stddev", nclip=12,  format-) | scan (zmedian2, zstddev2)
        imexpr ("(I > b) ? a-c : a-d", "tttsky0", infilb, ixc, zmedian2, zmedian1, dims=size, verbose- )

	imfil = "tttsky0"
	comm1 = "(a < shift(a,"//str(ixs)//") ) ? a : shift(a,"//str(ixs)//")"
	#print (comm1)
        imexpr(comm1, "tttsky1", imfil, dims=size, verbose-)
	comm1 = "(shift(a,"//str(ixs)//") < 0.0) ? min(a,shift(a,"//str(ix2m)//")) : b"
	#print (comm1)
        imexpr(comm1, "tttsky2", imfil, "tttsky1", dims=size, verbose-)

	comm1 = "(a < shift(a,"//str(ixm)//") ) ? a : shift(a,"//str(ixm)//")"
	#print (comm1)
        imexpr(comm1, "tttsky3", "tttsky2", dims=size, verbose-)
	comm1 = "(shift(a,"//str(ixm)//") < 0.0) ? min(a,shift(a,"//str(ix2)//")) : b"
	#print (comm1)
        imexpr(comm1, "tttsky4", imfil, "tttsky3", dims=size, verbose-)
	imstat ("tttsky4"//stat_sec2, fields = "stddev", lower=0.001, upper=INDEF,  format-) | scan (zstddev)
	imstat ("tttsky4", fields = "midpt", lower=0.001, upper=INDEF,  format-) | scan (zmedian)
	print(" ===> Getting a smoothed version of the image ",infil, " using a median box of ", ixb, "x", ixb)
	smo_min = 0.001; smo_max = zmedian + 2.0*zstddev
	median("tttsky4", "tttsky5", ixb, ixb, zloreje=smo_min, zhireje=smo_max, boundar="constant", constant=zmedian, verbose+)
	imcopy ("tttsky5", infilmed, verbose-)
	#comm1 = "I > "//str(ixl)//" && I < "//str(ixr)//" && a < 0.0 ? a : b"
	#print (comm1)
        #imexpr(comm1, infilmed, imfil, "tttsky5", dims=size, verbose-)
	imarith (imfil, "-", infilmed, ofil, title="", divzero=0., hparams="", pixtype="", calctyp="", verbose-, noact-)
	#imarith (imfil, "-", "tttsky5", ofil, title="", divzero=0., hparams="", pixtype="", calctyp="", verbose-, noact-)
}

end

#imexpr "(a < shift(a,90) ) ? a : shift(a,90)" temp M101Ha6608b.ss
#imexpr "(shift(a,90) < 0.0 ) ? min(a,shift(a,-50)) : b" temp1 M101Ha6608b.ss temp
#imexpr "a < shift(a,-90) ? a : shift(a,-90)" temp2 temp1
#imexpr "(shift(a,-90) < 0.0 ) ? min(a,shift(a,70)) : b" temp3 M101Ha6608b.ss temp2
#imcopy temp3 ttemp3
#median ttemp3 ttemp3med 55 55
#imarith M101Ha6608b.ss - ttemp3med ttemp3c
#imexpr "(shift(a,90) < 0.0 || shift(a,-90) < 0.0 ) ? min(a,shift(a,-50)) : b" temp1 M101Ha6608b.ss temp
#imexpr "I > 200 && I < 800 && a < 0.0 ? a : b" temp4 M101Ha6608b.ss tttsky5
