procedure fluxerr(objfil)

#Input file should have the subtracted sky spectrum in band2 of each aperture
#Necessary to give the sensitivity curve
# Method:  Fit a smooth continuum to the object spectrum using omsperms or directly continuum
#         Use the formula   ferr = sqrt((fcont + 2 fsky)*fconv )

string  objfil  = "obj_spectrum"          {prompt="Extracted object spectrum"}
string  skyfil  = ""          		  {prompt="Extracted sky    spectrum"}
string  sensfil = "gtc_sens_curve"        {prompt="Sensitivity function"}  
string  expkey  ="EXPTIME"                {prompt="Header keyword name for exposure time/frame"}
string  ncombkey="NCOMBINE"               {prompt="Header keyword name for Number of images combined"}
string  gainkey ="GAIN"                {prompt="Header keyword name for GAIN parameter"}
int     apno    = 1                     {prompt="Aperture no spectrum to measure"}
bool    inter   = yes                   {prompt="Fit continumm interactively?"}
real    lammin  = 3700.0                {prompt="Minimum good lambda in Ang"}
real    lammax  = 9999.0                {prompt="Maximum good lambda in Ang"}
bool    wrtext  = no                    {prompt="Write the spectra and SNRs into an ASCII table?"}

begin
        string  filein, filobj, filsky, filsens, keyexp, keynim, sample, filcal, keygain, objcal, filesk
	real	exptime, xmin, xmax, lam1, lamdelt, lam2, gain, multfact
	int	ncomb, nap, naxis, npix, iap, ap1
	bool	ask, wtext

	filein = objfil; filesk = skyfil; filsens = sensfil
	keyexp = expkey; keynim = ncombkey; keygain = gainkey 
	nap = apno; xmin = lammin; xmax = lammax; ask = inter; wtext = wrtext

	naxis = 1; lamdelt = 0.0
        hselect (filein, "NAXIS", yes) | scan (naxis)
        hselect (filein, "NAXIS1 CRVAL1 CDELT1", yes) | scan (npix, lam1, lamdelt)
	if (lamdelt == 0.0){hselect (filein, "CD1_1", yes) | scan (lamdelt)}
	lam2 = lam1 + real(npix-1)*lamdelt
	if (xmin < lam1) {xmin = lam1}
	if (xmax > lam2) {xmax = lam2}
        sample = str(xmin)//":"//str(xmax)

	exptime = 1.0; ncomb = 1
        hselect (filein, keyexp, yes) | scan (exptime)
        hselect (filein, keynim, yes) | scan (ncomb)
        hselect (filein, keygain, yes) | scan (gain)
        #hselect (filobj, keygain, yes) | scan (gain)
	multfact = exptime*ncomb*gain

	print(filein, " ", naxis, exptime, ncomb, gain, " ", sample)

	if (nap == 0) {
	imhead (filein, long+) | match ("APNUM") | wc | scan(nap)
	ap1 = 1
	if (nap == 0){print("ERROR: ", nap, " apertures. Spectrum is not the output of apall"); goto exit}
	} else {ap1 = nap}

	for (iap=ap1; iap <= nap; iap = iap + 1) {
	
	filobj = filein//"_"//str(iap); imdelete(filobj, ver-, >& "dev$null")
	if (naxis > 1) {
        #imsec = "[*,"//str(iap)//",1]"
        scopy (filein//"[*,"//str(iap)//",1]", filobj, w1=xmin, w2=xmax, rebin+)
	if (filesk == ""){
	filsky = filein//"_sky"//str(iap); imdelete(filsky, ver-, >& "dev$null")
		scopy (filein//"[*,"//str(iap)//",2]", filsky, w1=xmin, w2=xmax, rebin+)
			} else {filsky = filesk}
	} else {scopy (filein, filobj, w1=xmin, w2=xmax, rebin+)}
	print(filobj, " ", filsky)

        omsperms(filobj, sample=sample, inter=ask)

	filcal = filsens//"_cgs"; imdelete(filcal, ver-, >& "dev$null")
	objcal = filobj//"_invcgs"; imdelete(objcal, ver-, >& "dev$null")
	imdelete(filsens//"_scp", ver-, >& "dev$null")
	scopy (filsens, filsens//"_scp", w1=xmin, w2=xmax, rebin+)
	imexpr ("10**(-0.4*a)", filcal, filsens//"_scp", verb-)
	imexpr ("a/b", objcal, multfact, filcal, verb-)
	imdelete(filobj//"_therms", ver-, >& "dev$null")
        imexpr ("sqrt((a + 1.0*b)*c)/c", filobj//"_therms", filobj//"_cont", filsky, objcal, verb-)
        #imexpr ("sqrt((a + 2.0*b)*c)/c", filobj//"_therms", filobj//"_cont", filsky, objcal, verb-)
	imdelete(filobj//"_ferr", ver-, >& "dev$null")
	imdelete(filobj//"_osnr", ver-, >& "dev$null")
	imdelete(filobj//"_csnr", ver-, >& "dev$null")
        #imexpr ("b>0.0 ? a/b : 1.0", filobj//"_ferr", filobj//"_therms", filobj, verb-)
        imexpr ("b>a ? a/b : 1.0", filobj//"_ferr", filobj//"_therms", filobj, verb-)
        imexpr ("b>0.0 ? b/a : 0.0", filobj//"_osnr", filobj//"_therms", filobj, verb-)
        imexpr ("b>0.0 ? b/a : 0.0", filobj//"_csnr", filobj//"_therms", filobj//"_cont", verb-)

        #imexpr ("sqrt((a + 2.0*b)*c)/c", filobj//"_therms", filobj//"_cont", filsky, 1.0E18, verb-)
        #imexpr ("sqrt(a**2 + b**2)", nebfil//"_totrms", nebfil//"_rms", skyfil//"_rms", verb-)
	#imdelete(filobj//"_plurms", ver-, >& "dev$null")
	#imdelete(filobj//"_minrms", ver-, >& "dev$null")
	#sarith (filobj, "+", filobj//"_therms", filobj//"_plurms", w1=xmin, w2=xmax, rebin+, verb-)
	#sarith (filobj, "-", filobj//"_therms", filobj//"_minrms", w1=xmin, w2=xmax, rebin+, verb-)

	#cleanup
	imdelete(filsens//"_scp", ver-, >& "dev$null")
	imdelete(filsens//"_cgs", ver-, >& "dev$null")
	imdelete(objcal, ver-, >& "dev$null")

	#wtext = yes
        if (wtext){
        delete(filobj//"1.txt", ver-, >& "dev$null")
        delete(filobj//"2.txt", ver-, >& "dev$null")
        delete(filobj//"3.txt", ver-, >& "dev$null")
        delete(filobj//"4.txt", ver-, >& "dev$null")
        delete(filobj//"5.txt", ver-, >& "dev$null")
        delete(filobj//".txt", ver-, >& "dev$null")

        wspectext (filobj, filobj//"1.txt", header-, wformat="%0.2f")

        wspectext (filobj//"_therms", filobj//"2.txt", header-, wformat="%0.2f")
        wspectext (filobj//"_ferr", filobj//"3.txt", header-, wformat="%0.2f")
        wspectext (filobj//"_csnr", filobj//"4.txt", header-, wformat="%0.2f")
        wspectext (filobj//"_osnr", filobj//"5.txt", header-, wformat="%0.2f")
        #fields (filobj//"2.txt", fields=2, > filobj//"2a.txt")
        #fields (filobj//"3.txt", fields=2, > filobj//"3a.txt")
        #fields (filobj//"4.txt", fields=2, > filobj//"4a.txt")
        #fields (filobj//"5.txt", fields=2, > filobj//"5a.txt")

	fields (filobj//"2.txt", fields=2) | join (filobj//"1.txt", "STDIN", output = filobj//".txt", delim="   ")
        delete (filobj//"1.txt", ver-, >& "dev$null"); rename (filobj//".txt", filobj//"1.txt", field = "all")
	fields (filobj//"3.txt", fields=2) | join (filobj//"1.txt", "STDIN", output = filobj//".txt", delim="   ")
        delete (filobj//"1.txt", ver-, >& "dev$null"); rename (filobj//".txt", filobj//"1.txt", field = "all")
	fields (filobj//"4.txt", fields=2) | join (filobj//"1.txt", "STDIN", output = filobj//".txt", delim="   ")
        delete (filobj//"1.txt", ver-, >& "dev$null"); rename (filobj//".txt", filobj//"1.txt", field = "all")
	fields (filobj//"5.txt", fields=2) | join (filobj//"1.txt", "STDIN", output = filobj//".txt", delim="   ")

	printf('%1s', "#", >> filobj//".txt")
	imhead (filobj, long+) | match ("APCEN"//str(iap), >> filobj//".txt") 
	print("#lambda  f_lamda        e_flam          e_flam/f_lamda   SNR_cont       SNR_obj", >> filobj//".txt")
	#fields (filobj//"5.txt", fields=2) | join (filobj//"1.txt", "STDIN", output = filobj//".txt", delim="   ")

        #join (filobj//"1.txt", filobj//"2a.txt", output = filobj//".txt", delim="   ")
        #join (filobj//"1.txt", filobj//"3.txt", output = filobj//".txt", delim="   ")

        print("Output text file [wav flux rms] is = ", filobj//".txt")
        delete(filobj//"1.txt", ver-, >& "dev$null")
        delete(filobj//"2.txt", ver-, >& "dev$null")
        delete(filobj//"3.txt", ver-, >& "dev$null")
        delete(filobj//"4.txt", ver-, >& "dev$null")
        delete(filobj//"5.txt", ver-, >& "dev$null")
	}
	}

exit:
        print(" ")

end
