procedure manalyze(filnam)

#A script to analyze megara LCB spectra
#1. Read FIBER positions from the header and write it into a text file (fiber# x y)
#2. Obtain an image integrating each spectrum in a band
#3. Extract the spectra centered at given position(s)
#help after changes in jun 2019 (06/06/2019)
#The image option (unicen=0) is re-designed. the redundant newimage parameters is deleted.
#image option (unicen=0) should have a valid band parameters e.g. band=manalyze_band.dat
#conventions: if the band name starts with "manal", the task assumes that a muti-band output is sought.
#In such a case, it should contain the band information for each image. see "manalyze_band.dat" for format info
#The output image (*_image_gau.fits) will be 3-d, with the third axis being the image in in requested band
#-------------------------------------------------------------------------------------------

string  filnam    = "haro44_lrb_rss"   {prompt="File Name"}
string  outnam    = "DEFAULT"   {prompt="File Name"}
string  fibpos    = "DEFAULT"   {prompt="File Name containing fiber xy positions [DEFAULT===>read header]"}
bool    verbose   = yes         {prompt="Print the data for each fiber"}
string  fibcen    = 300          {prompt="Fiber no or position for spectral extraction"}
int     unicen    = 1            {prompt="Units of fibcen [1=fiber#, 2=fiber xy, 3=imagepix, 4=xy file, 0=image]"}
real    srad      = 1.5          {prompt="Radius in fiber units for spectral extraction"}
real    funit     = 0.465        {prompt="Fiber units [mm]"}
real    imscale   = 0.62         {prompt="Image scale [arcsec/fiber]"}
real    nsample   = 3.00         {prompt="Oversample factor [pix/fiber]"}
int     fiberr    = 0            {prompt="Fiber asignment error [integer number -5 to 5]"}
string  band      = ""   	{prompt="File with band info for creating image (when unicen=0)"}
string  senscurv  = ""   	{prompt="Sensitivity function"}
#bool    newimage  = no          {prompt="Create New image?"}
int     nfibers   = ""          {prompt="Number of fibers"}
string  out       = ""          {prompt="Output Text File containing fiber positions"}
string  outreg    = ""          {prompt="Output ds9 Reg File containing fiber positions"}
string  head      = ""          {prompt="Short header for the outfile"}
string  obsmod    = ""          {prompt="Observing mode [MOS/LCB]"}
string	cursor 	  = ""   	{prompt="Graphics cursor input"}

begin
        string 	filin, chead, cfib, obmode, datahead, filout, tmp, obsdate, filspe, filein
	string	sband, fibcoo, sensfil, filim, filt, fibstr, sx, sy, suffix, xsec, fil2d
	string  fibposfil, imtyp, filt_typ, quant[99], reco[99], fband[99], parcol, s1, s2 
	int	i, nfib, ix, iy, nx, ny, ifib, nnn, i1, id, unifib, ixx, iyy, j, nlin, iunifib, k
	int	inx, iny, ierr, ix1, ix2, nlam, kk, nz, len, npar, jfib, iq, nq
	int	k1, k2, fibcol[99], paracol[99]
	real	xfib, yfib, scale, xin, yin, dist, radius, fibscale, flux, nearest_dist
	real	texp, ra, dec, ipa, amass, imsca, nsamp, lam1, dlam, lamwid, lamcen
	real	xa[99], ya[99], xs, xe, ys, ye, scale_ori, lam_cen, filwid, paraval[623]
	bool print, skip_readfib, newim, datacube, multi
        struct  line

        filein = filnam; filout=outnam; print = verbose; skip_readfib = no
	newim = newimage; sband = band; nsamp = nsample; imsca = imscale
	fibscale = funit; sensfil = senscurv
	scale_ori = fibscale; scale = scale_ori/nsamp
	unifib = unicen; fibcoo = fibcen
	ierr = fiberr; fibposfil = fibpos
	#ifib = ifiber
	radius = fibscale*srad
	if (strupr(fibposfil) == "DEFAULT"){fibposfil = filein//"_fibers.dat"} #else { skip_readfib = yes}
	#if (strupr(filout) == "DEFAULT"){filout = filin//"_"//str(ifib)}
        datahead = "##i   xfib   yfib   IX   IY"

	imextensions (filein, output="none")
	if (imextensions.nimages < 2){ filin = filein } else {
		filin = filein//"_2d"
          	imdelete (filin, ver-, >& "dev$null")
	  	imexpr ("a", filin, filein//"[0]", verb-)
		}

	hselect (filin, "$I EXPTIME RADEG DECDEG IPA INSMODE AIRMASS DATE-OBS", expr+)
	#hselect (filein//"[1]", "INSMODE NFIBERS", expr+) | scan (obmode, nfib)
	hselect (filin, "INSMODE NAXIS2", expr+) | scan (obmode, nfib)
	nx = sqrt(nfib)/scale_ori + 10; ny =nx
	print(nx, ny, scale, nfib)
        date

	filt_typ = "none"; imtyp = "none"
	datacube = no
	slist (filin, ap="1", long-) | fields (field="6-8") | scan (lam1, dlam, nlam)
	xsec = str(1)//":"//str(nlam)
	filim = filein//"_image"

	if (access(fibposfil)){
	  wc (fibposfil) | scan(nnn)
	  if (nfib == nnn - 2){skip_readfib = yes}
	}

	if (skip_readfib){print("Using the existing fiber list ", fibposfil)} else {
	  if (imextensions.nimages < 2){print("Error: Should provide fiber xy coordinate"); goto exit}
	  fibposfil = filein//"_fibers.dat"
          #delete (filein//"_fibers.dat", ver-, >& "dev$null")
          delete (fibposfil, ver-, >& "dev$null")
	  hselect (filin, "$I EXPTIME RADEG DECDEG IPA INSMODE AIRMASS DATE-OBS", expr+, >> fibposfil) 
	  print(datahead, >> fibposfil)

          for (i = 1; i <= nfib; i = i + 1) {
	    if (i < 10){cfib = "FIB00"} else if (i < 100){cfib = "FIB0"} else {cfib = "FIB"}
	    chead = cfib//str(i)//"_X "//cfib//str(i)//"_Y"
	    hselect (filein//"[1]", chead, expr+) | scan(xfib, yfib)
	    ix = xfib/scale + int(nx/2) + 1; iy = yfib/scale + int(ny/2)
	    printf("%5d  %10.5f  %10.5f %5d %5d \n", i, xfib, yfib, ix, iy, >> fibposfil)
	    #printf("%5d  %10.5f  %10.5f %5d %5d \n", i, xfib, yfib, ix, iy, >> filein//"_fibers.dat")
	  }    #end of reading header
	}

        manalyze.nfibers = nfib
        manalyze.out = fibposfil    #filein//"_fibers.dat"
        manalyze.outreg = filein//"_fibers.reg"
        manalyze.head = datahead
        manalyze.obsmod = mode

	#Reading the header done
	#==========================================================================

        delete ("imexpr.log", ver-, >& "dev$null")
	#Creating an image at the required lambda
	#
        #delete ("test_sbands", ver-, >& "dev$null")
	#if (newim) {
	if (unifib <=0) {
	    nq = 1; multi = no
	  #----------------------------------------------------------
	  if (strlwr(sband) != "" && substr(sband, 1, 5) == "manal"){
	    multi = yes; iq = 0; list = sband
            while(fscan(list, line) != EOF ) {
              if (substr (line, 1, 1) != "#") {iq = iq + 1; reco[iq] = line; i1 = fscanf(line, "%s", s1); quant[iq] = s1}
	    } #EOF
	    #nq = iq; for (j = 1; j <= nq; j = j + 1) {if (quant[j] == "CONT") {fields (line, fields=2, line=1)}; print (quant[j])}
	    nq = iq; for (j = 1; j <= nq; j = j + 1) {
		if (quant[j] == "CONT") {line = reco[j]; i1 = fscanf(line, "%s %s", s1, s1); fband[j] = s1} 
		if (quant[j] == "PARA") {line = reco[j]; i1 = fscanf(line, "%s %s %d %d %s", s1, s1, k1, k2, s2) 
		  quant[j] = s1; fibcol[j] = k1; paracol[j] = k2; fband[j] = s2} 
			print (j, " ", quant[j], " ", fband[j]) 
		}
	  }  #manal

	  #goto exit

          imdelete (filim, ver-, >& "dev$null")
	  #if (datacube) {imexpr ("0.0", filim, dim=str(nx)//","//str(ny)//","//str(nz), verb-)} else {
	  if (datacube) {print(" Datacube option: Not yet available ....."} else {
	    #imexpr ("0.0", filim, dim=str(nx)//","//str(ny)//","//str(nq), verb-) 
	    imexpr ("a*0.0", filim, filin, dim=str(nx)//","//str(ny), verb-) 
	  }
	    hselect (filin, "EXPTIME RADEG DECDEG IPA AIRMASS DATE-OBS", expr+) | scan (texp, ra, dec, ipa, amass, obsdate)
	    print(texp, ra, dec, ipa, amass, obsdate)
	    hedit(filim, "CRPIX1", int(nx/2) + 1, add+, verif-, show-, update+) 
	    hedit(filim, "CRPIX2", int(ny/2) + 1, add+, verif-, show-, update+) 
	    hedit(filim, "CRVAL1", ra,  add+, verif-, show-, update+) 
	    hedit(filim, "CRVAL2", dec, add+, verif-, show-, update+) 
	    hedit(filim, "CDELT1", -imsca/nsamp/3600., add+, verif-, show-, update+) 
	    hedit(filim, "CDELT2", imsca/nsamp/3600., add+, verif-, show-, update+) 
	    hedit(filim, "CTYPE1", "RA---SIN", add+, verif-, show-, update+) 
	    hedit(filim, "CTYPE2", "DEC--SIN", add+, verif-, show-, update+) 
	    hedit(filim, "EPOCH", 2000., add+, verif-, show-, update+) 
	    hedit(filim, "EXPTIME", texp, add+, verif-, show-, update+) 
	    hedit(filim, "AIRMASS", amass, add+, verif-, show-, update+) 
	    hedit(filim, "DATE-OBS", obsdate, add+, verif-, show-, update+) 
	    hedit(filim, "TELESCOP", "GTC", add+, verif-, show-, update+) 
	    hedit(filim, "INSTRUME", "MEGARA/LCB", add+, verif-, show-, update+) 
	    hedit(filim, "FILENAME", filein, add+, verif-, show-, update+) 

	  #----------------------------------------------------------
	for (iq = 1; iq <= nq; iq = iq + 1) {
	if (multi){sband = fband[iq]; filt = quant[iq]}
          filout = filein//"_temp_"//str(iq)
          imdelete (filout, ver-, >& "dev$null")
	  imexpr ("a*0.0", filout, filim, dim=str(nx)//","//str(ny), verb-) 

	if (strlwr(sband) != ""){
		len = strlen(sband); filt_typ = substr(sband, len-8, len)
		if (filt_typ == "_band.dat"){imtyp = "band"}
		if (filt_typ == "_para.dat"){imtyp = "para"}
		if (filt_typ == "_cube.dat"){imtyp = "cube"}
		}

	if (imtyp == "band"){
          tail (sband, nlin=1) | fields (field="1-", lines=1) | scan(filt, lam_cen, filwid)
	  ix1 = max(1, (int((lam_cen - filwid/2.0 - lam1)/dlam) + 1))
	  ix2 = min(nlam, (int((lam_cen + filwid/2.0 - lam1)/dlam) + 1))
	  xsec = str(ix1)//":"//str(ix2)
          #filim = filein//"_"//filt
	  #if (filt == "cube"){datacube = yes; nz = ix2 - ix1 + 1}
	  print(filt, " ",lam_cen, filwid, " XSECTION ", xsec)
	  quant[iq] = filt
	}

	if (imtyp == "cube"){datacube = yes; nz = ix2 - ix1 + 1}

	if (imtyp == "para"){   #paracol = 8; paranam = "HeII_WR"; npar = 295
		npar = 295; parcol = str(fibcol[iq])//","//str(paracol[iq])
		print(parcol)
          for (j = 1; j <= nfib; j = j + 1) {paraval[j] = 0.0}
            for (i = 3; i <= npar; i = i + 1) {
	      #fields (sband, "1,"//str(paracol), lines=i) | scan (jfib, flux)
	      fields (sband, parcol, lines=i) | scan (jfib, flux)
		paraval[jfib] = flux
	    }
	}
	#----------------------------------------------------------

	    #imhead(filein//"_image")
            i1 = 0
            #list = filein//"_fibers.dat"
            list = fibposfil
            while(fscan(list, line) != EOF ) {
            tmp = substr (line, 1, 1)
            if (tmp != "#" && tmp != substr(filein, 1, 1)) {
              i1 = fscanf(line, "%d  %f  %f ", id, xfib, yfib)
	    #print(id, xfib, yfib)
	    ix = xfib/scale + int(nx/2) + 1; iy = yfib/scale + int(ny/2)
	    ##&& ix > 0 && iy > 0  in the following line added on 30/mar/2018
	    if (abs(ix) <= nx && abs(iy) <= ny && ix > 0 && iy > 0) {       
	    #if (datacube) {
	    if (imtyp == "cube"){
	      #imcopy (filin//str(i)//":"//str(i)//","//str(id)//"]", 1G
	      kk = 0
	      for (i = ix1; i <= ix2; i = i + 1) {
	      kk = kk + 1
	      listpixels (filin//"["//str(i)//":"//str(i)//","//str(id)//"]") | fields (fields=2) | scan(flux)
	      imreplace (filim//"["//str(ix)//":"//str(ix)//","//str(iy)//":"//str(iy)//","//str(kk)//"]", flux)
	  	}
	    } #else {
	    if (imtyp == "band"){
	      #sbands (filin//"[*,"//str(id)//"]", "test_sbands", bands=sband, verb-)
	      #tail ("test_sbands", nlin=1) | fields (field=3) | scan(flux)
              imdelete ("temp_imexp", ver-, >& "dev$null")
              #imexpr ("sum(a)", "temp_imexp", filin//"[*,"//str(id)//"]",  verb-)
              imexpr ("sum(a)", "temp_imexp", filin//"["//xsec//","//str(id)//"]", outtype="real", verb-)
              imstat ("temp_imexp", fields="mean", format-) | scan(flux)
	      flux = max(0.0, flux)
	      print(id, " ", ix, " ", iy, " ", flux, >> "imexpr.log")
	      #imreplace (filim//"["//str(ix)//":"//str(ix)//","//str(iy)//":"//str(iy)//","//str(iq)//"]", flux)
	      imreplace (filout//"["//str(ix)//":"//str(ix)//","//str(iy)//":"//str(iy)//"]", flux)
	    }  #datacube   //","//str(nq), 
	    if (imtyp == "para"){
	      #imreplace (filim//"["//str(ix)//":"//str(ix)//","//str(iy)//":"//str(iy)//","//str(iq)//"]", paraval[id])
	      imreplace (filout//"["//str(ix)//":"//str(ix)//","//str(iy)//":"//str(iy)//"]", paraval[id])
	    }

		} #if inside the image boundaries
		} #reading lines
		} #EOF

        	imdelete (filout//"_gau", ver-, >& "dev$null")
		gauss (filout, filout//"_gau", sigma=2, nsigma=2)
		#imcopy (filout, filim//"[*,*,"//str(iq)//"]", verb-)
		#if (iq == 1){imdelete (filim//"_gau", ver-, >& "dev$null"); imexpr ("a*0.0", filim//"_gau", filim, verb-)}
		#imcopy (filout//"_gau", filim//"_gau[*,*,"//str(iq)//"]", verb-)
	        #hedit(filim, "PARA"//str(iq), filt, add+, verif-, show-, update+) 
	        #hedit(filim//"_gau", "PARA"//str(iq), filt, add+, verif-, show-, update+) 
        	#imdelete (filout, ver-, >& "dev$null")
        	#imdelete (filout//"_gau", ver-, >& "dev$null")
		if (iq == 1){delete (filein//"_manalize*.lis", ver-, >& "dev$null")}
		print (filout, >> filein//"_manalize.lis")
		print (filout//"_gau", >> filein//"_manalize_gau.lis")
		#if (nq == 1){quant[iq] = filt}
	}  #unifib  #newim
        #date
	} #nq
        if (unifib <=0) {
          imdelete (filim, ver-, >& "dev$null"); imdelete (filim//"_gau", ver-, >& "dev$null")
	  if (nq == 1){imrename (filout, filim, verb-); imrename (filout//"_gau", filim//"_gau", verb-)} else {
	  imstack ("@"//filein//"_manalize.lis", filim); imstack ("@"//filein//"_manalize_gau.lis", filim//"_gau")}
	  for (iq = 1; iq <= nq; iq = iq + 1) {
	    hedit(filim, "PARA"//str(iq), quant[iq], add+, verif-, show-, update+) 
	    hedit(filim//"_gau", "PARA"//str(iq), quant[iq], add+, verif-, show-, update+) 
	  }
	}


	if (unifib <=0) {goto exit}
	##############################################
	#Finding the fibers physically neighbouring a given fiber number or position
	#

	# unifib: [1=fiber#, 2=fiber xy, 3=imagepix, 4=xy file]"

	iunifib = unifib
        ifib = 0
	nlin = 1
	print(unifib)
	switch (unifib) {
	case 1: 
		{
		print(fibcoo) | scan(ifib)
       		if (ifib < 10){cfib = "FIB00"} else if (ifib < 100){cfib = "FIB0"} else {cfib = "FIB"}
        	chead = cfib//str(ifib+ierr)//"_X "//cfib//str(ifib+ierr)//"_Y"
        	hselect (filein//"[1]", chead, expr+) | scan(xa[1], ya[1])
		xin = xa[1]; yin = ya[1]
		}
	case 2: 
		{
		#print(fibcoo) | scan(xa[1], ya[1])
		print(fibcoo) | scan(xa[1], ya[1]); xin = xa[1]; yin = ya[1]; print(xin, yin)
		}
	case 3: 
		{
		print(fibcoo) | scan(xin, yin)
		xa[1] = scale*(xin - 1.0 - real(int(nx/2))) 
		ya[1] = scale*(yin - 1.0 - real(int(ny/2)))
		}
	case 4: 
		{
		k = 0
		iunifib = 4
        	fields (fibcoo, "1-3", lines=1) | scan (iunifib, srad, nlin)
		#iunifib is assumed to be 2 i.e. the values are assumed to be xy units not fiber and pixels
		#16/09/2017: iunifib=3 added (pixel units for spectral extraction)
        	radius = fibscale*srad
		if (nlin == 0){
			fields (fibcoo, "4-7", lines=1) | scan (xs, xe, ys, ye)
			#nlin = nint((xe - xs)/radius)*nint((ye - ys)/radius)
			inx = nint(0.5*(xe - xs)/radius) + 1; iny = nint(0.5*(ye - ys)/radius) + 1
        		for (i = 1; i <= inx; i = i + 1) { 
			for (j = 1; j <= iny; j = j + 1) {
			k = k + 1
			xa[k] = xs + real(i-1)*radius
			ya[k] = ys + real(j-1)*radius 
			print(k, xa[k], ya[k])
			} }
			xin = xa[1]; yin = ya[1]
			nlin = k
		} else {
		print("Reading ", nlin, " positions for spectral extraction from file ", fibcoo)
        		for (j = 1; j <= nlin; j = j + 1) {
        		#fields (fibcoo, "1-2", lines=j+1) | scan (xa[j], ya[j])
			#xin = xa[1]; yin = ya[1]
        		fields (fibcoo, "1-2", lines=j+1) | scan (xin, yin)
			xa[j] = scale*(xin - 1.0 - real(int(nx/2))) 
			ya[j] = scale*(yin - 1.0 - real(int(ny/2)))
        		print(j, xin, yin, xa[j], ya[j])
		}
		}
		}
	}
	printf("%5d  %10.5f  %10.5f  %10.5f \n",ifib, xin, yin, radius)

	filspe = filein//"_spe"
	fil2d  = filin   #filein//"_2d"
        imdelete (filspe, ver-, >& "dev$null")
	imexpr ("a*0.0", filspe, filin//"[*,1:"//str(nlin)//"]", verb-)

        for (j = 1; j <= nlin; j = j + 1) {
	ifib = 0
	xin = xa[j]; yin = ya[j]
	if (strupr(filout) == "DEFAULT"){filout = filein//"_0"}

        if (j==1) {imdelete (filout, ver-, >& "dev$null"); sarith (fil2d, "*", "0.0", filout, aper=1)}    
        delete (filout//"_pixel.dat", ver-, >& "dev$null")
	nearest_dist = radius
	fibstr = ""; k = 0
	for (i = 1; i <= nfib; i = i + 1) {
        #fields (filein//"_fibers.dat", "2-3", lines=i+2) | scan (xfib, yfib)
        fields (fibposfil, "2-3", lines=i+2) | scan (xfib, yfib)

	#print(xfib, yfib)
	dist = sqrt((xfib - xin)**2 + (yfib - yin)**2)
		if (xin <= 0.0){sx = "x0"} else {sx = "x1"}
		if (yin <= 0.0){sy = "y0"} else {sy = "y1"}
		suffix = "_"//sx//str(nint(abs(xin)*10))//sy//str(nint(abs(yin)*10))
	if (dist < radius) {
		k = k + 1
		nearest_dist = min(dist, nearest_dist)
		if (dist <= nearest_dist){ifib = i}
		printf ("%5d  %10.5f  %10.5f  %10.5f %10.5f %5d\n", i, xfib, yfib, dist, nearest_dist, ifib)
        	imdelete ("tempspe1", ver-, >& "dev$null")
        	imdelete ("tempspe2", ver-, >& "dev$null")
		scopy (fil2d, "tempspe1", aper=i)
		imexpr ("a+b", "tempspe2", filout, "tempspe1", ver-, >& "dev$null")
		#imstat ("tempspe2")
        	imdelete ("tempspe1", ver-, >& "dev$null")
        	imdelete (filout, ver-, >& "dev$null")
        	imrename ("tempspe2", filout, verb-)
		fibstr = fibstr//" "//str(i)
		#imhead(filout)
		#imstat(filout)
		#splot(filout)
	}
	}
	print(xin, yin, ifib, "#", fibstr, " ",k, suffix) 
	hedit(filout, "FIBCEN", ifib, add+, verif-, show-, update+) 
	hedit(filout, "FIBINT", k, add+, verif-, show-, update+) 
	hedit(filout, "FIBSUM", fibstr, add+, verif-, show-, update+) 
	hedit(filout, "FIBXY ", str(xin)//" "//yin, add+, verif-, show-, update+) 
        imdelete (filein//"_"//str(ifib)//suffix, ver-, >& "dev$null")
	imrename (filout, filein//"_"//str(ifib)//suffix, verb-)
	filout = filein//"_"//str(ifib)//suffix
        imdelete (filout//"_cal", ver-, >& "dev$null")
	if (strlwr(sensfil) != ""){
	calibrate (filout, filout//"_cal", extinct+, extinction="gtcinputs$tn_ext_curve.dat", ignorea+, sensiti=sensfil, observa="lapalma")
	splot(filout//"_cal", cursor = cursor)
	} else { splot(filout, cursor = cursor) }
	print(filout)
	imcopy (filout, filspe//"[*,"//str(j)//"]", verb-)
	imhead (filspe)
        hedit(filspe, "SPEROW"//str(j), str(ifib)//":"//fibstr//" "//suffix, add+, verif-, show-, update+)
	}  #end reading the xyfib from file
exit:
	print("==========================")

end
	#if (unifib < 4){
	#print(fibcoo) | scan(ifib)
	#print(ifib)
	#if (ifib != 0) {
	#if (unifib == 1){print(fibcoo) | scan(ifib)
	#if (ifib < 10){cfib = "FIB00"} else if (ifib < 100){cfib = "FIB0"} else {cfib = "FIB"}
	#chead = cfib//str(ifib)//"_X "//cfib//str(ifib)//"_Y"
	#hselect (filein//"[1]", chead, expr+) | scan(xin, yin)
	#} else {print(fibcoo) | scan(xin, yin); ifib = 0}
	#if (unifib == 3){xin = scale*(xin - 1.0 - real(int(nx/2))); yin = scale*(yin - 1.0 - real(int(ny/2))) }
	#printf("%5d  %10.5f  %10.5f  %10.5f \n",ifib, xin, yin, radius)
	#print("   ")
	#}
	#}
	#if (unifib == 4){iunifib = 4} else {iunifib = unifib}
	#if (iunifib == 4){
        #fields (fibcoo, "1-3", lines=1) | scan (unifib, srad, nlin)
	#radius = fibscale*srad
	#for (j = 1; j <= nlin; j = j + 1) {
        #fields (fibcoo, "1-2", lines=j+1) | scan (xin, yin)
	##print(j, xin, yin)
