procedure mcontmap(musecube)
#=====================================================================================================
#This task takes the science stack of the muse datacube and returns a continuum map
#at the given wavelength, which could be "average|median|sum" of width=contwid.
#Optionally an image section can be analyzed.
#Output fluxes are in erg/cm2/s/Ang units.
#=====================================================================================================
string  musecube  = "am644_741_muse_sci"     	{prompt="Input muse datacube to measure flux"}
string  ofile     = "am644_741_"     		{prompt="Input muse datacube to measure flux"}
real    contlam   = 5550.0                   	{prompt="Rest wavelength of the line in Angstroms"}
real    contwid   = 500.0                    	{prompt="Window width in Angstroms for evaluating continuum [<=500]"}
string  combine   = "average"     		{prompt="Type of combine operation [average|median|sum]"}
int     x1        = 0                        	{prompt="xmin of the subimage"}
int     x2        = 0                        	{prompt="xmax of the subimage"}
int     y1        = 0                        	{prompt="ymin of the subimage"}
int     y2        = 0                        	{prompt="ymax of the subimage"}

begin
	string 	fmusecube, froot, file_cont, comb_type, ctype, imsec
	real	cpix, clam, dlam, band_cen, band_wid 
	int	naxis, nx, ny, nlam, ix1, ix2, iy1, iy2, il1, il2
	#int	nx0, ny0, npix_line, npix_cont, il1_blu, il2_blu, il1_red, il2_red

	date

	fmusecube = musecube   #"am644_741_muse_sci"
	froot = ofile     #"am644_"
	band_cen = contlam   #5500
	band_wid = contwid  #1000/2 
	comb_type = combine

	file_cont = froot//str(nint(band_cen))//"_"//str(nint(band_wid))//"_"//comb_type

        hselect (fmusecube, "NAXIS NAXIS1 NAXIS2 NAXIS3", yes) | scan (naxis, nx, ny, nlam)
        hselect (fmusecube, "CTYPE3 CRPIX3 CRVAL3 CD3_3", yes) | scan (ctype, cpix, clam, dlam)
        print (naxis, nx, ny, nlam)
        print (ctype, " ", cpix, clam, dlam)
        #ix1 = 1; ix2 = nx; iy1 = 1; iy2 = ny
        #ix1 = 280; ix2 = 306; iy1 = 130; iy2 = 160

        ix1 = x1; ix2 = x2; iy1 = y1; iy2 = y2
        if (ix1 <= 0 || ix1 > ix2){ix1 = 1}; if (iy1 <= 0 || iy1 > iy2){iy1 = 1}
        if (ix2 <= ix1 || ix2 > nx){ix2 = nx}; if (iy2 <= iy1 || iy2 > ny){iy2 = ny}

	il1 = (band_cen - band_wid/2.0 - clam)/dlam + cpix
	il2 = (band_cen + band_wid/2.0 - clam)/dlam + cpix
	if (il1 < 1){il1 = 1}
	if (il2 > nlam){il1 = nlam}
        imsec = "["//str(ix1)//":"//str(ix2)//","//str(iy1)//":"//str(iy2)//","//str(il1)//":"//str(il2)//"]"
	print(imsec)
        imdelete(file_cont, ver-, >& "dev$null")
        delete("imcombine.log", ver-, >& "dev$null")
        imcombine (fmusecube//imsec, file_cont, combine=comb_type, project+, logfile="imcombine.log")
	imstat (file_cont, fields = "image,npix,mean,stddev,min,max,midpt")

	date

end
