procedure muse_maps(musecube)

string  musecube  = "cartwheel_muse_3d_ori"     {prompt="Input muse datacube to measure flux"}
string  filtfil   = "photometry_sbands.in"      {prompt="File containing filter info for sbands"}
int     x1        = 0            		{prompt="xmin of the subimage"}
int     x2        = 0            		{prompt="xmax of the subimage"}
int     y1        = 0            		{prompt="ymin of the subimage"}
int     y2        = 0            		{prompt="ymax of the subimage"}
bool    inter     = yes                           {prompt="Fit the continuum interactively?"}
string  linelist  = "muse_neblines.dat"       {prompt="List containing lines to be measured"}
string  parfil    = "mfluxanal.par"               {prompt="File with Physical parameters to be calculated"}
string  veltype   = "9090"                        {prompt="Velocity for Doppler correction [none|mean|median|value in km/s]"}
real    snrgood   = 3.0                           {prompt="Minimum SNR of a GOOD line"}
real    errfwhm   = 2.0                           {prompt="Good FWHM range is FW_Hb/errfw to FW_Hb*errfw for a GOOD line"}
real    fwhmlin   = 3.0                          {prompt="Expected FWHM of a resolved line [Ang]"}
string  dispkey   = "CDELT1"                       {prompt="Dispersion Keyword [CDELT1/CD1_1 etc]"}
bool    skip_calc =no                            {prompt="Skip mfluxcalc?"}
bool    skip_anal =no                            {prompt="Skip mfluxanal?"}
bool    resetpar  =yes                           {prompt="Reset mfluxcalc and mfluxanal parameters?"}
int     ilevel  = 1                     {prompt="Interaction level"}


begin
	string  filin, sbandin, specfil, spefil, imsec, sbandout, name, filtname[99]
	string	dummy, filout, filters, vel_type, parfile, datfil, s1
	real	flux_fact, fluxtot, rms
	int	nx, ny, ns, ix, iy, is, ix1, ix2, iy1, iy2, jf, nfil
	int	ix0, iy0, nx0, ny0, ilev
	bool	do_sband, ask, parset, no_calc, no_anal


	filin = musecube; specfil = filin//"_sci"
	sbandin = filtfil; ilev = ilevel
	ask = inter; no_calc = skip_calc; no_anal = skip_anal
        parset = resetpar; vel_type = veltype

	flux_fact = 1.0E-20
	ix1 = x1; ix2 = x2; iy1 = y1; iy2 = y2

	if (imaccess(specfil)){print ("Using file ", specfil)} else {
	  #imdelete(specfil, ver-, >& "dev$null")
	  print ("Reading extension 1 file and creating ", specfil )
	  imcopy (filin//"[1]", specfil, verb-)
	}

	if (access(sbandin)){
		do_sband=yes; filters = " bands: "; tinfo(sbandin, ttout-); nfil = tinfo.nrows
		for (jf = 1; jf <= nfil; jf +=1) { 
		  fields (sbandin, fields=1, lines=jf) | scan (filtname[jf])
		  filters = filters//" "//filtname[jf]
		}
		print (filters)
	}

        datfil = filin//"_mflux_lines.dat"
        delete(datfil, ver-, >& "dev$null")

        parfile = parfil
        if (!access(parfile)){print (" PARFIL=", parfile, " not found. Enter a valid filname: "); scan(parfile)}
        if (parset){
          mfluxcalc.snrgood=snrgood.; mfluxcalc.errfwhm=errfwhm; mfluxcalc.lammin=INDEF; mfluxcalc.lammax=INDEF
          mfluxcalc.skyshi=no; mfluxcalc.sample="*"; mfluxcalc.fwhmlin=fwhmlin; mfluxcalc.dispkey=dispkey
          mfluxanal.fibnum=1; mfluxanal.parfil=parfile
        }


	imhead (specfil); hselect (specfil, "NAXIS1 NAXIS2 NAXIS3", yes) | scan (nx, ny, ns)

	if (ix1 <= 0){ix1 = 1}; if (iy1 <= 0){iy1 = 1}
	if (ix2 <= 0 || ix2 > nx){ix2 = nx}; if (iy2 <= 0 || iy2 > ny){iy2 = ny}
	nx0 = ix2 - ix1 + 1; ny0 = iy2 - iy1 + 1
	imsec = "["//str(ix1)//":"//str(ix2)//","//str(iy1)//":"//str(iy2)//",1]"

	filout = filin//"_map"
	print ("Output file ", filout, " contains maps in ", filters)
	imdelete(filout, ver-, >& "dev$null")
	imexpr ("a*0.0", filout, specfil//imsec, dim=str(nx0)//","//str(ny0)//","//str(nfil), verb-)
	date
	for (jf = 1; jf <= nfil; jf +=1) { hedit(filout, "SBAND"//str(jf), filtname[jf], add+, verif-, show-, update+) }
	
        for (ix=ix1; ix <= ix2; ix +=1) {
	  ix0 = ix - ix1 + 1
          for (iy=iy1; iy <= iy2; iy +=1) {
	    iy0 = iy - iy1 + 1
		imsec = "["//str(ix)//":"//str(ix)//","//str(iy)//":"//str(iy)//",*]"
		spefil = filin//"_"//str(ix)//"_"//str(iy)
		imdelete(spefil, ver-, >& "dev$null")
		imcopy (specfil//imsec, spefil, verb-)
		imstat (spefil, fields="stddev", format-) | scan (rms)

		if (do_sband && rms > 0.0){
			sbandout = filin//"_"//str(ix)//"_"//str(iy)//"_sbands.dat"
			delete (sbandout, ver-, >& "dev$null")
			#sbands (spefil, sbandout, "photometry_sbands.in")
			sbands (spefil, sbandout, sbandin)
			for (jf = 1; jf <= nfil; jf +=1) {
				fields (sbandout, fields="2,3", lines="1-") | match (filtname[jf]) | scan (dummy, fluxtot)
              imreplace (filout//"["//str(ix0)//":"//str(ix0)//","//str(iy0)//":"//str(iy0)//","//str(jf)//"]", fluxtot)
			}  #jf
		delete (sbandout, ver-, >& "dev$null")
		} #do_sband

	
        if (!no_calc){
		sarith (spefil, "*", flux_fact, spefil, clobber+)
                delete(spefil//"_ap1_1_fluxobs.dat", ver-, >& "dev$null")
		#hselect (spefil, "CD1_1 CD3_3", yes)
                mfluxcalc (spefil, inter=ask, linelis=linelist, veltype=vel_type, graphics="stdvdm", ilevel=ilev)
                match ("Velocity", spefil//"_ap1_1_fluxobs.dat") | scan (s1, s1, s1, s1, s1, s1, s1, s1, vel_type)
                delete(spefil//"_ap1_1_fluxobs.dat", ver-, >& "dev$null")
		#hselect (spefil, "CD1_1 CD3_3", yes)
                mfluxcalc (spefil, inter=ask, linelis=linelist, veltype=vel_type, dispkey = "CDELT1", graphics="stdvdm", ilevel=ilev)
                }
		#splot (spefil)
		imdelete(spefil, ver-, >& "dev$null")
		imdelete(spefil//"_ap*.fits", ver-, >& "dev$null")
		imdelete(spefil//"_dop*.fits", ver-, >& "dev$null")
	  }  #iy
	}  #ix
	date

end
