procedure omspsum(spec2d, skyrms1d)

#Brief explanation:
#Given a 2D spectra and the sky spectra (usually outputs of omskysub), sums the
#columns over the [ap] range specified in the slitfil, which can be the output
#of omslits or a user edited file (see example file head.dat). 4 output 1d files are
#written for each extracted spectrum. Name conventions
#spec2d_##outsuf.fits (summed), spec2d_##outsuf_snr.fits (SNR spectrum), 
#spec2d_##outsuf_rms.fits (rms spectrum), and spec2d_##outsuf_clean.fits (cleaned spectrum),  
# ##=identification number of slitlet, outsuf=user asigned suffix
#Alternatively, if 2d spectrum is not sky-subtracted (zerosky-), it is subtracted.
#
#-------------------------------------------------------------------------
#The task has a number of control parameters which are explained below
#zerosky - Sky is already subtracted? - If yes, then no further sky subtraction is carried out
#          
#-------------------------------------------------------------------------

string  spec2d   = "gtc14a_mos1abc_obj"         {prompt="2D slitlet spectrum to extract"}
string  skyrms1d = "gtc14a_mos1abc_sky1d"       {prompt="Sky and rms 1d spectra for each slitlet"}
bool    zerosky  = yes                          {prompt="Sky is already subtracted?"}
string  slitfil= "omslit"                       {prompt="Slit information file [omslit/filename]"}
string  slitno = "all"                          {prompt="Slit numbers to slice [all, #]"}
string  outsuf = "_spsum"                          {prompt="Output file suffix"}
real    krms   = 1.0                            {prompt="How many sigmas is considered good?"}
real    fwhm_pix = 4.5                            {prompt="FWHM of Skyline in pixels (spectral resolution)"}
#real    fwhm   = 20.0                            {prompt="FWHM of Skyline in Ang (spectral resolution)"}
real    boxsiz = 0.0                            {prompt="Size (pix) for boxcar smooth [0 or -ve ==> no smooth]"}
bool    delete = yes                          	{prompt="Delete all intemediate files?"}
bool    contfit = yes           		{prompt="Fit continumm?"}
string  sample  = "*"   			{prompt="Sample range in Ang for analysis 3700:7600"}
bool    inter   = no            		{prompt="Fit continumm interactively?"}
string  function= "spline3"     		{prompt="Function to be fit to the continuum"}
int     order   = 3             		{prompt="Order of the function to be fit"}
bool    calc_rms= no            		{prompt="Calculate theoretical rms?"}
string  sensfil = "gtc_sens_curve"        {prompt="Sensitivity function"}
string  expkey  ="EXPTIME"                {prompt="Header keyword name for exposure time/frame"}
string  ncombkey="NCOMBINE"               {prompt="Header keyword name for Number of images combined"}
string  gainkey ="GAIN"                {prompt="Header keyword name for GAIN parameter"}
real    slitwid = 0.0                           {prompt="Slitwidth in arcsec [0==> flux_unit = flux_sum]"}
real    imscale = 0.254                         {prompt="Image scale in arcsec/pixel"}

begin
        string  filin, sec[999], slitid, stype, s1, s2, suffix, filsky, headfil
        string  cslit, fil_ori, fil_det, fil_snr, fil_rms, fil_sky, fil_cle, fil_sub 
	string	fil_tmp, fil_smo, fil_ran, fil_out, sperange, fil2_rms, dimens, fil2_sky
	string	spsumpar, spsuminp, sphead, filsens, filcal, keyexp, keynim, keygain, objcal
	string	funit, tfil_tmp
        real    slitx, slity, slityx, slito, sfac, kkrms, xbox, pix1, sigma, gauss_decay_2pix
        real    exptime, xmin, xmax, lam1, lamdelt, lam2, gain, multfact, gauss_decay_1pix
	real	slitwidth, imscal, area
        int     i1, nslits, ix1, ix2, islit, sslit, nslice, is, is1, is2, isl, nap, ix, nx
	int	ny, ncomb
        bool    calcrms, cont_fit, ask, delint, skyfree, slice[999]


        filin = spec2d; filsky = skyrms1d; skyfree = zerosky; suffix = outsuf
        slitid = strlwr(slitno); headfil = slitfil; kkrms = krms; xbox = boxsiz; delint = delete
	ask = inter; sperange = sample; cont_fit = contfit
	filsens = sensfil; calcrms = calc_rms
	keyexp = expkey; keynim = ncombkey; keygain = gainkey
	slitwidth = slitwid; imscal = imscale
	area = 1.0; funit = "erg/cm2/sec/Ang"
	if (slitwidth != 0.0){area = slitwidth*imscal; funit = "erg/cm2/sec/Ang/arcsec2"}

	#---------------------------------------------------------
	#Define the maximum factor by which flux can vary between adjacent pixels 
	#for a spectral resolution of FWFM_pix 
	#---------------------------------------------------------
	sigma = fwhm_pix/2.3548
	gauss_decay_2pix = 1.1*exp(0.5*(2.0/sigma)**2)
	gauss_decay_1pix = 1.1*exp(0.5*(1.0/sigma)**2)

	#---------------------------------------------------------
	#Calculate the expected rms in the spectrum based on poisson statistics
	#---------------------------------------------------------
	if (calcrms){
        lamdelt = 0.0
        hselect (filin, "NAXIS2 CRVAL2 CDELT2 CRPIX2", yes) | scan (ny, lam1, lamdelt, pix1)
        if (lamdelt == 0.0){hselect (filin, "CD2_2", yes) | scan (lamdelt)}
        lam1 = lam1 + real(-pix1)*lamdelt
        lam2 = lam1 + real(ny-1)*lamdelt

        exptime = 1.0; ncomb = 1
        hselect (filin, keyexp, yes) | scan (exptime)
        hselect (filin, keynim, yes) | scan (ncomb)
        hselect (filin, keygain, yes) | scan (gain)
        multfact = exptime*ncomb*gain
        print(filin, " ", exptime, ncomb, gain, multfact, lam1, lam2, lamdelt, gauss_decay_2pix)

	filcal = filsens//"_cgs"; imdelete(filcal, ver-, >& "dev$null")
        imdelete(filsens//"_scp", ver-, >& "dev$null")
        scopy (filsens, filsens//"_scp", w1=lam1, w2=lam2, rebin+)
        imexpr ("10**(-0.4*a)", filcal, filsens//"_scp", verb-)
        objcal = filin//suffix//"_invcgs"; imdelete(objcal, ver-, >& "dev$null")
        imexpr ("a/b", objcal, multfact, filcal, verb-)
	} else {
          print(filin, " ", gauss_decay_2pix, gauss_decay_1pix)
	}

	#read the slit definition file
        if (slitid != "all") {
        is = strstr("-", slitid)
        if (is != 0) {
        is1 = int(substr(slitid, 1, is - 1)); is2 = int(substr(slitid, is + 1, strlen(slitid)))
        for (isl = is1; isl <= is2; isl = isl + 1) { slice[isl] = yes }
        } else {
        files(slitid) | wc | scan(nslice)
        for (i = 1; i <= nslice; i = i + 1) {
        files(slitid) | fields (fields=1, lines=i) | scan(isl)
        slice[isl] = yes
        #print("Slit no ", i, " to slice = ", isl)
        }
        }
        }
        if (headfil == "omslit"){
        omslits (filin, verb-)
        nslits = omslits.nslitlets
        list = omslits.out
        } else {
                tinfo (headfil, ttout-)
                nslits = tinfo.nrows
                list = headfil
        }

	#Prepare the output file format and header
        fil_out = filin//suffix
        hselect (filin, "NAXIS2", expr+) | scan (ny)
        imdelete (fil_out, ver-, >& "dev$null")
        #dimens = str(nslits)//","//str(ny)//",7"; imexpr ("a*0", fil_out, filin, dims=dimens, verb-)
        dimens = str(nslits)//","//str(ny)//",8"; imexpr ("a*0", fil_out, filin, dims=dimens, verb-)
	hedit(fil_out, "SPSUMSKY", filsky, add+, ver-, show-)
	hedit(fil_out, "SPSUM_1", "CLEAN+RMS", add+, ver-, show-)
	hedit(fil_out, "SPSUM_2", "SKY SUBTRACTED", add+, ver-, show-)
	hedit(fil_out, "SPSUM_3", "SKY RMS", add+, ver-, show-)
	hedit(fil_out, "SPSUM_4", "CONTINUUM FIT", add+, ver-, show-)
	hedit(fil_out, "SPSUM_5", "SNR", add+, ver-, show-)
	hedit(fil_out, "SPSUM_6", "SPSUM BEFORE SKY SUBTARTION", add+, ver-, show-)
	hedit(fil_out, "SPSUM_7", "SKY SUMMED OVER OBJECT PIXELS", add+, ver-, show-)
	hedit(fil_out, "SPSUM_8", "CLEAN", add+, ver-, show-)
	hedit(fil_out, "SPSUMZ", skyfree, add+, ver-, show-)
	hedit(fil_out, "SP_UNIT", funit, add+, ver-, show-)

        i1 = 0
        for (i = 1; i <= nslits; i = i + 1) {
        if (headfil == "omslit"){
        i1 = fscan(list, islit, ix1, ix2, slitx, slity, slityx, stype, slito)
        #2 145 206 275.85513433214 1506.322089713 1492.5674821309 rectangular  179.8389
        cslit = str(islit); sslit = i   #; sslit = islit
        } else {
                fields (headfil, fields="1-5", lines=i) | scan (islit, ix1, ix2, cslit, sslit)
                stype = "rectangular"
        }
        if (slitid == "all") {slice[i] = yes}
        if (stype != "circular" && slice[i]) {
        nap  = ix2 - ix1 + 1
        ix = ix1 + nap/2
        sec[i] = "["//str(ix)//",*]"
        print(i, islit, nap, " ",sec[i], " ", cslit, " ", sslit)
	if (i == nslits){print("i  apno  width_pix  center  slitlet_id  sky_apno")}
	spsumpar = str(i)//" "//str(ix)//" "//str(nap)
	spsuminp = str(islit)//" "//str(ix1)//" "//str(ix2)//" "//cslit//" "//str(sslit)
	sphead = spsumpar//" "//spsuminp

        fil_ori = filin//"_"//cslit//suffix
        fil_sub = filin//"_"//cslit//suffix//"_sub"
        fil_det = filin//"_"//cslit//suffix//"_det"
        fil_snr = filin//"_"//cslit//suffix//"_snr"
        fil_rms = filin//"_"//cslit//suffix//"_rms"
        fil_sky = filin//"_"//cslit//suffix//"_sky"
        fil_tmp = filin//"_tmp"//cslit//suffix
        fil_smo = filin//"_"//cslit//suffix//"_smo"
        #fil_out = filin//"_"//cslit//suffix//"_clean"
        fil_ran = filin//"_"//cslit//suffix//"_random"
        fil2_sky = filin//"_"//cslit//suffix//"_sky2"

        imdelete (fil_ori, ver-, >& "dev$null")
        imdelete (fil_sub, ver-, >& "dev$null")
        imdelete (fil_snr, ver-, >& "dev$null")
        imdelete (fil_rms, ver-, >& "dev$null")
        imdelete (fil_sky, ver-, >& "dev$null")
        imdelete (fil_tmp, ver-, >& "dev$null")
        #imdelete (fil_smo, ver-, >& "dev$null")
        #imdelete (fil_out, ver-, >& "dev$null")
        imdelete (fil_ran, ver-, >& "dev$null")
        imdelete (fil2_sky, ver-, >& "dev$null")

	#---------------------------------------------------------
	#Extract the spectrum at pixel "ix" summing all values in the spatial direction of width "nap"

        nsum = nap
        sarith (filin, "copy", "", fil_ori, apertur=ix, verbose-)
	#---------------------------------------------------------

	#---------------------------------------------------------
	#Read the sky spectrum and scale it to "nap" pixels.
	#If required, subtract the sky spectrum from that of the object

        nsum = 1
        sarith (filsky, "*", nap, fil_sky, aper=sslit, bands=1, verbose-)
        if (!skyfree){
                imexpr ("a-b", fil_sub, fil_ori, fil_sky, verb-)
                #imdelete (fil_ori, ver-, >& "dev$null")
                #imrename (fil_tmp, fil_ori, verbo-)
        } else {imcopy (fil_ori, fil_sub, verbo-)}
	#---------------------------------------------------------

	#---------------------------------------------------------
	#Read the sky noise spectrum and calculate the noise in "nap" pixels.
	#If the sky spectrum is from a zone different from that of the object,
	#re-calculate rms summing in quadrature the rms of the two sky spectra,
	#and the difference between the two sky spectra
	#---------------------------------------------------------

        sarith (filsky, "*", sqrt(nap), fil_rms, aper=sslit, bands=2, verbose-)
	if (islit != sslit){
          sarith (filsky, "*", nap, fil2_sky, aper=islit, bands=1, verbose-)
          fil2_rms = filin//"_"//cslit//suffix//"_rms2"
          sarith (filsky, "*", sqrt(nap), fil2_rms, aper=islit, bands=2, verbose-)
	  ##imexpr ("sqrt(a**2+b**2)", fil_tmp, fil_rms, fil2_rms, verb-) 
	  ##imexpr ("sqrt(a**2+b**2+(c-d)**2)", fil_tmp, fil_rms, fil2_rms, fil_sky, fil2_sky, verb-) 
	  #imexpr ("sqrt(a**2+(b-c)**2)", fil_tmp, fil_rms, fil_sky, fil2_sky, verb-) 
          #imdelete (fil2_rms, ver-, >& "dev$null")
          #imdelete (fil_rms, ver-, >& "dev$null")
          #imrename (fil_tmp, fil_rms, verbo-)
	  ##imexpr ("abs(a-b)>c*d ? sqrt(d**2+(a-b)**2 : d", fil_rms, fil_sky, fil2_sky, kkrms, fil_tmp, verb-)
	  ##imexpr ("abs(a-b)>c*d ? sqrt(d**2+(a-b)**2 : d", fil_rms, fil_sky, fil2_sky, kkrms, fil_tmp, verb-)
	}

	#Broaden the noise spectrum to take into account the skyline profile differences.
	imexpr ("max(a,shift(a,1),shift(a,-1),shift(a,2),shift(a,-2))", fil_rms//"_1", fil_rms, verb-)
	#imexpr ("max(a,shift(a,1),shift(a,-1),shift(a,2),shift(a,-2),shift(a,3),shift(a,-3))", fil_rms//"_1", fil_rms, verb-)
        imdelete (fil_rms, ver-, >& "dev$null")
	imrename (fil_rms//"_1", fil_rms, verbo-)
	
	#---------------------------------------------------------
	#Fit and subtract the continuum from the sky-subtracted object spectrum
	#---------------------------------------------------------

        imcopy(fil_sub, fil_tmp, verbo-)
	if (cont_fit){
          omsperms (fil_tmp, sample=sperange, inter=ask, function=function, order=order)
	} else {
          imdelete (fil_tmp//"_csub", ver-, >& "dev$null")
          imdelete (fil_tmp//"_cont", ver-, >& "dev$null")
	  imcopy(fil_tmp, fil_tmp//"_csub", verbo-)
	  imexpr ("a*0.0", fil_tmp//"_cont", fil_tmp, verb-)
	}

	#---------------------------------------------------------
	#Remove spikes in sky-subtracted spectra by rejecting all pixels
	#that have a steeper rise than that expected from the spectral resolution.
	#---------------------------------------------------------
        imdelete (fil_det//"_csub", ver-, >& "dev$null")
        imdelete (fil_smo//"_csub", ver-, >& "dev$null")
        imdelete (fil_smo//"_csub1", ver-, >& "dev$null")
        imdelete (fil_smo//"_csub2", ver-, >& "dev$null")

	#imexpr ("a>b*shift(a,2) ? shift(a,2) : a", fil_smo//"_csub1", fil_tmp//"_csub", gauss_decay_2pix, verb-)
	##imexpr ("a>b*shift(a,1) ? shift(a,1) : a", fil_smo//"_csub2", fil_smo//"_csub1", gauss_decay_1pix, verb-)
	#for (j = 1; j <= 1; j = j + 1) {
        #imdelete (fil_smo//"_csub2", ver-, >& "dev$null")
	#imexpr ("a>b*shift(a,1) ? shift(a,1) : a", fil_smo//"_csub2", fil_smo//"_csub1", gauss_decay_1pix, verb-)
	##imexpr ("a>b*shift(a,1) ? shift(a,1) : a", fil_smo//"_csub1", fil_smo//"_csub2", gauss_decay_1pix, verb-)
        #imdelete (fil_smo//"_csub1", ver-, >& "dev$null")
	##imcopy (fil_smo//"_csub1", fil_smo//"_csub2", ver-)
	#imexpr ("a>b*shift(a,2) ? shift(a,2) : a", fil_smo//"_csub1", fil_smo//"_csub2", gauss_decay_2pix, verb-)
	#}
	##imexpr ("a>b*shift(a,1) ? shift(a,1) : a", fil_smo//"_csub2", fil_smo//"_csub1", gauss_decay_1pix, verb-)

	imexpr ("a>b*shift(a,-1) ? 0.0 : a", fil_smo//"_csub1", fil_tmp//"_csub", gauss_decay_1pix, verb-)
	imexpr ("a>b*shift(a,-1) ? 0.0 : a", fil_smo//"_csub", fil_smo//"_csub1", gauss_decay_1pix, verb-)
	#imcopy (fil_tmp//"_csub", fil_smo//"_csub", verb-)

	#---------------------------------------------------------
	#Smooth the spectrum using boxcar
	#---------------------------------------------------------
	#xbox = 0.0
	if (xbox > 0.0){
        imdelete (fil_smo//"_csub1", ver-, >& "dev$null")
        boxcar (fil_smo//"_csub", fil_smo//"_csub1", xbox, 1, boundary="nearest", constant = 0.)
        imdelete (fil_smo//"_csub", ver-, >& "dev$null")
        imrename (fil_smo//"_csub1", fil_smo//"_csub", verbo-)
        ##boxcar (fil_tmp//"_csub", fil_smo//"_csub", xbox, 1, boundary="nearest", constant = 0.)
	}

	#---------------------------------------------------------
	#Calculate the noise expected from Possion statistics if asked to do so
	#and add it quadratically the noise from sky
	#---------------------------------------------------------
	if (calcrms){
          imdelete(fil_ori//"_therms", ver-, >& "dev$null")
          imexpr ("sqrt((a + 0.2*b)*c)/c", fil_ori//"_therms", fil_tmp//"_cont", fil_sky, objcal, verb-)
          #imexpr ("sqrt((a + 0.5*b)*c)/c", fil_ori//"_therms", fil_tmp//"_cont", fil_sky, objcal, verb-)
          #imexpr ("sqrt((a + 1.0*b)*c)/c", fil_ori//"_therms", fil_smo//"_csub", fil_sky, objcal, verb-)
          #imexpr ("sqrt((a + 1.0*b)*c)/c", fil_ori//"_therms", filobj//"_cont", fil_sky, objcal, verb-)
          fil2_rms = filin//"_"//cslit//suffix//"_rms2"
          imdelete (fil2_rms, ver-, >& "dev$null")
	  imexpr ("sqrt(a**2+b**2)", fil2_rms, fil_rms, fil_ori//"_therms", verb-) 
          imdelete (fil_rms, ver-, >& "dev$null")
          imrename (fil2_rms, fil_rms, verbo-)
	}

	#---------------------------------------------------------
	#On the spike-free and smoothed spectrum, detect pixels above k*sigma
	#Store also the SNR spectrum
	#---------------------------------------------------------

        #boxcar (fil_ori, fil_smo, xbox, 1, boundary="nearest", constant = 0.)
        imexpr ("a>0 ? a/b : 0.0", fil_snr, fil_sub, fil_rms, verb-)
        #imexpr ("a>c*b ? d : 0.0", fil_det//"_csub", fil_smo//"_csub", fil_rms, kkrms, fil_tmp//"_csub", verb-)

        imexpr ("a>c*b ? d : 0.0", fil_det//"_csub", fil_smo//"_csub", fil_rms, kkrms, fil_tmp//"_csub", verb-)

	#---------------------------------------------------------
	#Add only the "good" pixels to the continuum spectrum to prepare a "clean" spectrm
	#Add "box" noise of maximum amplitude "rms" on the clean spectrum 
	#---------------------------------------------------------

        hselect (fil_ori, "NAXIS1", expr+) | scan(nx)
        urand (nx, 1, seed=1.0) | rtext (output=fil_ran, header-, dim=nx)
        fil_cle = filin//"_"//cslit//suffix//"_clean"
        imdelete (fil_cle, ver-, >& "dev$null")
        imdelete (fil_cle//"_rms", ver-, >& "dev$null")
        imexpr ("a+b", fil_cle, fil_det//"_csub", fil_tmp//"_cont", fil_rms, fil_ran, verb-)
        imexpr ("a+b+c*2.0*(d-0.5)", fil_cle//"_rms", fil_det//"_csub", fil_tmp//"_cont", fil_rms, fil_ran, verb-)
        #imexpr ("a+b+c*2.0*(d-0.5)", fil_out, fil_det//"_csub", fil_tmp//"_cont", fil_rms, fil_ran, verb-)

	#---------------------------------------------------------
	#Store useful outputs in to a single file and optionally delete intermediate files 
	#---------------------------------------------------------
	
	imexpr ("a*b", fil_rms//"_k", fil_rms, kkrms, verb-)

        if (slitwidth != 0.0){
	  area = slitwidth*imscal*real(nap)
	  tfil_tmp = "tfil_tmp_sb"
	  imexpr ("a/b", tfil_tmp, fil_cle, area, verb-) 
	  imdel (fil_cle, ver-, >& "dev$null"); imrename (tfil_tmp, fil_cle, verbo-)
	  imexpr ("a/b", tfil_tmp, fil_cle//"_rms", area, verb-) 
	  imdel (fil_cle//"_rms", ver-, >& "dev$null"); imrename (tfil_tmp, fil_cle//"_rms", verbo-)
	  imexpr ("a/b", tfil_tmp, fil_sub, area, verb-) 
	  imdel (fil_sub, ver-, >& "dev$null"); imrename (tfil_tmp, fil_sub, verbo-)
	  imexpr ("a/b", tfil_tmp, fil_rms//"_k", area, verb-) 
	  imdel (fil_rms//"_k", ver-, >& "dev$null"); imrename (tfil_tmp, fil_rms//"_k", verbo-)
	  imexpr ("a/b", tfil_tmp, fil_tmp//"_cont", area, verb-) 
	  imdel (fil_tmp//"_cont", ver-, >& "dev$null"); imrename (tfil_tmp, fil_tmp//"_cont", verbo-)
	  imexpr ("a/b", tfil_tmp, fil_ori, area, verb-) 
	  imdel (fil_ori, ver-, >& "dev$null"); imrename (tfil_tmp, fil_ori, verbo-)
	  imexpr ("a/b", tfil_tmp, fil_sky, area, verb-) 
	  imdel (fil_sky, ver-, >& "dev$null"); imrename (tfil_tmp, fil_sky, verbo-)
	}

	imcopy(fil_cle//"_rms", fil_out//"["//str(i)//",*,1]", ver-)
	imcopy(fil_sub, fil_out//"["//str(i)//",*,2]", ver-)
	imcopy(fil_rms//"_k", fil_out//"["//str(i)//",*,3]", ver-)
	imcopy(fil_tmp//"_cont", fil_out//"["//str(i)//",*,4]", ver-)
	imcopy(fil_snr, fil_out//"["//str(i)//",*,5]", ver-)
	imcopy(fil_ori, fil_out//"["//str(i)//",*,6]", ver-)
	imcopy(fil_sky, fil_out//"["//str(i)//",*,7]", ver-)
	imcopy(fil_cle, fil_out//"["//str(i)//",*,8]", ver-)
	hedit(fil_out, "SPSUM"//str(i), sphead, add+, ver-, show-)
        imdelete (fil_cle, ver-, >& "dev$null")
        imdelete (fil_cle//"_rms", ver-, >& "dev$null")
        imdelete (fil_sub, ver-, >& "dev$null")
        imdelete (fil_ori, ver-, >& "dev$null")
        imdelete (fil_rms, ver-, >& "dev$null")
        imdelete (fil_snr, ver-, >& "dev$null")
        imdelete (fil_rms//"_k", ver-, >& "dev$null")

        if (delint){
        #imdelete (fil_smo, ver-, >& "dev$null")
        #imdelete (fil_rms, ver-, >& "dev$null")
        imdelete (fil_sky, ver-, >& "dev$null")
        imdelete (fil2_sky, ver-, >& "dev$null")
        imdelete (fil_tmp, ver-, >& "dev$null")
        imdelete (fil_ran, ver-, >& "dev$null")

        imdelete (fil_tmp//"_*", ver-, >& "dev$null")
        imdelete (fil_det//"_csub", ver-, >& "dev$null")
        imdelete (fil_smo//"_csub", ver-, >& "dev$null")
        }


        #}   #if (slice[i])
        }   #!= "circular"
        }   #i = 1; i <= nslits

end


