procedure plotfunc(filnam)

#-----------------------------------------------------------------------------------------------
#plotfunc v2.0 19/Sep/2018    ydm@inaoep.mx
#This task reads the apall or identify coefficients stored in the database and re-constructs
#  the polynomials. The resulting polynomial is plotted and/or stored in a datafile plotfunc.dat.
#  For apall files, additionally a ds9 plotfunc.reg file is prepared which can be overplotted on the image. 
#  The extraction window is shown in green box and the sky windows are shown in blues boxes.
#  If wrcoef = yes, and a fil1d is given or it ends with filnam.ms [fil1d=DEFALT], the important
#  coefficients are stored as image KEYWORDS APCENi APFUNi APSKYi [i=1,nap].
#  With filnam = "", and fil1d exists, and wrcoef=yes, the task reads (instead of write) the 
#  coeffients from the header. If ybeg>0 and yend>0, the polynomials are prepared for this range, 
#  otherwise, the range available in the database file is used. To plot/prepare polynomials for
#  all apertures, give recno=0. Otherwise, only the polynomial for the given recno is analyzed.
#  Stepsize corresponds the independent variable stepsize where polynomials are evaluated.
#  Stepsize<100, may result in buffer size errors. Boxes of height=Stepsize are stored in the
#  regfile if regtype=box (this option is not updated to include sky bins). 
#
#  e.g. apall gtc17aob003_object_123_wl_tlt_cal[1700:2000,*] output=gtc17aob003_object_123_wl_tlt_cal_28_30.ms
#  plotfunc gtc17aob003_object_123_wl_tlt_cal[1700:2000,*] wrcoef+
#-----------------------------------------------------------------------------------------------

string  filnam    = "gtc14a_mos1abc" 	{prompt="File Name"}
string  dsource   = "ap" 		{prompt="Data source [id/ap]"}
string  fil1d     = "DEFAULT" 		{prompt="Extracted 1d spectrum [DEFAULT=filnam.ms]"}
int     step      = 100			{prompt="Step size for aperture box"}
int     recno     = 1 			{prompt="Record number"}
string  database  = "database/" 	{prompt="Database directory name [database/]"}
string  output    = "plotfunc" 		{prompt="File to store calculated function [.reg, .dat]"}
int     ybeg      = 0 			{prompt="Min value independent variable to plot"}
int     yend      = 0 			{prompt="Max value independent variable to plot"}
string  regtype   = "polygon" 		{prompt="Data source [polygon/box]"}
bool	wrcoef    = no			{prompt="Write/read apall coefficients into/from header of fil1d?"}

begin
        string  datadir, suffix, fnam, dfile, ans, func, comm1, froot 
	string	ofil, datfil, regfil, cline, comm_left, comm_right, boxtyp, ystr
	string  bfunc, headcen[99], spe1d, headfun[99], headbg[99], dum, cdisp
	real	y, ymin, ymax, yfit, cc[9], s, zz[9], n, aa, bb
	real	center, low, high, refpix, shift, bl[9], br[9]
	int	ncoef, k, j, i, norder, id, nfunc, istep, irec, ibegin, nlines 
	int	ipix, y1, y2, i1, i2, is1, is2, is, ii, border, iln, il1, il2, ibg, kk
	int	nx, nap, idc
	bool	bgsec, plotbg
        struct  line, bsamp

	fnam = filnam; datadir = database; suffix = dsource; y1 = ybeg; y2 = yend
	irec = recno; ofil = output; boxtyp = regtype
	istep = max(100,step)

	regfil = ofil//".reg"; delete (regfil, ver-, >& "dev$null")

	plotbg = wrcoef; if (suffix == "id"){plotbg = no}
	spe1d = fil1d
	i1 = 1
	is1 = strstr("[", fnam); is2 = strstr(",", fnam)
	if (is1 == 0){froot = fnam} else {froot = substr(fnam, 1, is1-1)}
	if (is1 != is2){
	  is = strstr(":", fnam) 
	  i1 = int(substr(fnam, is1+1, is-1))
	  i2 = int(substr(fnam, is+1, is2-1))
	  fnam = substr(fnam, 1, is1-1)//"_"//str(i1)//"_"//str(i2)//"___"
	}
	print(is1, fnam)
	if (strupr(spe1d) == "DEFAULT"){spe1d = froot//".ms"}
	print(spe1d)
	if (!imaccess(spe1d) && plotbg) {print("ERROR *** 1D spectrum = ", spe1d, " does not exist"); goto exit}
	if (substr(boxtyp, 1, 1) != "b"){boxtyp = "polygon"}

	dfile = datadir//suffix//fnam
	if (!access(dfile) && !plotbg){print("ERROR *** No source for coefficient file. Run with wrcoef=yes if ", spe1d, " contains the corresponding headers"); goto exit}
	print(" Analysing file: ", dfile)
	center=0.0; low=0.0; high=0.0; shift = 0.0

	ii = 0; id = 0 
	if (access(dfile)){
	  match("begin", dfile) | wc | scan(nap)
	  idc = 0
	  list = dfile
	  while (fscan (list, line) != EOF) {
            ii = ii+1
            if (substr(line, 1, 1) != "#") {
	      fields (dfile, fields="1", lines=ii) | scan (ans) 
	      if (ans == "begin") {ibegin = ii; idc = idc + 1}
	      if (ans == "background") {
		fields (dfile, fields="2", lines=ii+3) | scan (bfunc)
		fields (dfile, fields="2", lines=ii+4) | scan (border)
		fields (dfile, fields="2-", lines=ii+5) | scan (bsamp)
		bgsec = yes; ibg = 0
	 	while (bgsec) {
		  iln = strlen(bsamp); il1 = strstr(" ", bsamp); il2 = strstr(":", bsamp)
		  if (il2 > 0) {
		    ibg = ibg + 1
		    bl[ibg] = real(substr(bsamp, 1, il2-1)); br[ibg] = real(substr(bsamp, il2+1, il1)) 
		    bsamp = substr(bsamp, il1+3, iln)
		    #print(iln, il1, il2, bl[ibg], br[ibg], bsamp)
		  } else {bgsec = no}   #il2 > 0
		}  #bgsec
	      }  #background
	      if (ans == "coefficients" || ans == "curve") {
		if (ans == "curve") {
		  fields (dfile, fields="2,3", lines=ii-15) | scan (center, refpix)
		  fields (dfile, fields="2", lines=ii-14) | scan (low)
		  fields (dfile, fields="2", lines=ii-13) | scan (high)
		  #print("Center=", center, " Low=", low, " High=", high, " Refpix=", refpix, " Step=", istep)
		  headcen[idc] = str(center)//" "//str(refpix)//" "//str(low)//" "//str(high)
        	  #for (kk=1; kk <= ibg; kk = kk + 1) {bl[kk] = bl[kk] + center; br[kk] = br[kk] + center }  
		} #curve
		if (ans == "coefficients") {
		  fields (dfile, fields="2", lines=ibegin+5) | scan (nlines)
        	  cline=str(ibegin+6)//"-"//str(ibegin+5+nlines)
		  }
		  fields (dfile, fields="2", lines=ii) | scan (ncoef) 
		  ncoef = ncoef - 4
		  fields (dfile, fields="1", lines=ii+1) | scan (nfunc) 
		  fields (dfile, fields="1", lines=ii+2) | scan (norder) 
		  fields (dfile, fields="1", lines=ii+3) | scan (ymin) 
		  fields (dfile, fields="1", lines=ii+4) | scan (ymax) 
		  if (y1 != 0) {ymin = real(y1)}
		  if (y2 != 0) {ymax = real(y2)}

        for (k=1; k <= ncoef; k = k + 1) {fields (dfile, fields="1", lines=ii+4+k) | scan (cc[k])}

	if (suffix == "ap"){
	headbg[idc] = str(ibg)//" "//bfunc//" "//str(border)//" " 
	for (kk=1; kk <= ibg; kk = kk + 1) {headbg[idc] = headbg[idc]//" "//str(bl[kk])//" "//str(br[kk])}
	}

	headfun[idc] = str(nfunc)//" "//str(norder)//" "//str(ncoef)
        for (k = 1; k <= ncoef; k = k+1) {headfun[idc] = headfun[idc]//" "//str(cc[k])}

	      }  # coefficients || curve
	}}    #  line, EOF  
	}  else {           #if dfile doen't exist obtain info from header

	if (plotbg){hselect(spe1d, "NAXIS1 NAXIS2", yes) | scan(nx, nap)}
	}   #end of reading and saving in strings the fitting parameters

	#print(nap)

	if (irec > nap){print(" Only ", nap, " aperture(s) exist(s)" )}
       	for (id=1; id <= nap; id = id + 1) {
	if (id == irec || irec == 0) {
	#print ("Apextract for ap=",id, " Center Refpix Low High: ", headcen[id])
	#print ("Apextract for ap=",id, " Polynomial coefficients [nfunc norder ncoef coeff]: ", headfun[id])
	#print ("Apextract for ap=",id, " Background bins [Nbg func order bins]: ", headbg[id])

	if (access(dfile)) {
	  print(headfun[id]) | scan(nfunc, norder, ncoef)

	if (suffix == "ap"){
	  print(headcen[id]) | scan(center, refpix, low, high)
	  print(headbg[id]) | scan(ibg, bfunc, border)
	  for (kk=1; kk <= ibg; kk = kk + 1) {
		dum = str(3+2*kk-1)//"-"//str(3+2*kk) 
		print(headbg[id]) | fields (fields=dum) | scan(bl[kk], br[kk])
		}
	}

	  for (kk=1; kk <= ncoef; kk = kk + 1) {dum = 3 + kk; print(headfun[id]) | fields (fields=dum) | scan(cc[kk])}
          #for (k=1; k <= ncoef; k = k + 1) {print(k, " ",cc[k])}
	} else {
	  hselect(spe1d, "APCEN"//str(id), yes) | fields (fields=1) | scan(center, refpix, low, high)
	  hselect(spe1d, "APFUN"//str(id), yes) | fields (fields=1) | scan(nfunc, norder, ncoef)
	  hselect(spe1d, "APSKY"//str(id), yes) | fields (fields=1) | scan(ibg, bfunc, border)
	  ibg = min(3, ibg)   #only reads the first 2 bg bins 
	  for (kk=1; kk <= ibg; kk = kk + 1) {
		dum = str(3+2*kk-1)//"-"//str(3+2*kk) 
		hselect(spe1d, "APSKY"//str(id), yes) | fields (fields=1) | fields (fields=dum) | scan(bl[kk], br[kk])
	  }
	  for (kk=1; kk <= ncoef; kk = kk + 1) {
		dum = str(3+kk) 
		hselect(spe1d, "APFUN"//str(id), yes) | fields (fields=1) | fields (fields=dum) | scan(cc[kk]) 
	  }
	  ymin = real(1.); ymax = real(nx)

	headcen[id] = str(center)//" "//str(refpix)//" "//str(low)//" "//str(high)
	if (suffix == "ap"){
	  headbg[id] = str(ibg)//" "//bfunc//" "//str(border)//" " 
	  for (kk=1; kk <= ibg; kk = kk + 1) {headbg[id] = headbg[id]//" "//str(bl[kk])//" "//str(br[kk])}
	}
	headfun[id] = str(nfunc)//" "//str(norder)//" "//str(ncoef)
        for (k = 1; k <= ncoef; k = k+1) {headfun[id] = headfun[id]//" "//str(cc[k])}

	} #end access
	if (suffix == "ap"){
	print ("Apextract for ap=",id, " Center Refpix Low High: ", center, refpix, low, high)
	print ("Apextract for ap=",id, " Background bins [Nbg func order bins]: ", ibg, bfunc, border)
	for (kk=1; kk <= ibg; kk = kk + 1) {print(bl[kk], " ", br[kk])}
	}
	print ("ID=",id, " Polynomial coefficients [nfunc norder ncoef coeff]: ", nfunc, norder, ncoef)
        for (k=1; k <= ncoef; k = k + 1) {print(k, " ",cc[k])}

	switch (nfunc) {
	case 1:
		func = "chebyshev"
	case 2:
		func = "legendre"
	case 3:
		func = "spline3"
	case 4:
		func = "spline1"
	}  


	#print("ID=", id, " Function=", func, " Line=",ii, " norder=",norder, " MinMax=[", ymin, ymax,"]")
	#print("ID=", id, " Function=", func, " norder=",norder, " MinMax=[", ymin, ymax,"]")

	datfil = ofil//".dat"; delete (datfil, ver-, >& "dev$null")
	if (suffix == "ap"){
	  print("# Region file format: DS9 version 4.1", >> regfil)
	  #print("global color=green dashlist=8 3 width=1 select=1 highlite=1 dash=0 fixed=0 edit=1 move=1 delete=1 include=1 source=1", >> regfil)
	  print("image", >> regfil)
	}

	ipix = 0
	#procedure calcfunc(func, cc, norder, ymin, ymax, istep, center, low, high, suffix)
        for (k = int(ymin)-1; k <= int(ymax)-1; k = k + 1) {
	  ipix = ipix + 1; y = real(k); if (ipix == 1 && suffix == "ap"){y = refpix}
          n = (2.0*y - (ymax + ymin))/(ymax - ymin)
          yfit = 0.0

	  if (func == "spline3" || func == "spline1"){
            s = real(norder)*(y - ymin)/(ymax - ymin); j = int(s)
            aa = real(j)+1.0 - s; bb = s - real(j)
	  }
	  if (func == "spline1"){yfit = cc[j+1]*aa + cc[j+1+1]*bb}
	  if (func == "spline3"){
            zz[1] = aa**3
            zz[2] = 1.0 + 3.0*aa*(1.0 + aa*bb)
            zz[3] = 1.0 + 3.0*bb*(1.0 + aa*bb)
            zz[4] = bb**3
            for (i = 1; i <= 4; i = i + 1) { yfit = yfit + cc[i+j]*zz[i] }
	  }

	  if (func == "chebyshev") {
            zz[1] = 1; zz[2] = n 
            for (i = 1; i <= norder; i = i + 1) {
		if (i>2){zz[i] = 2.0*n*zz[i-1] - zz[i-2]}
                yfit = yfit + cc[i]*zz[i]
            }
          }

	  if (func == "legendre") {
            zz[1] = 1; zz[2] = n 
            for (i = 1; i <= norder; i = i + 1) {
		#if (i>2){zz[i] = (2.0*real(i-3))*n*zz[i-1] - real(i-2)*zz[i-2]/real(i-1)}
		#if (i>2){zz[i] = ((2.0*real(i-3))*n*zz[i-1] - real(i-2)*zz[i-2])/real(i-1)}
		if (i>2){zz[i] = ((2.0*real(i)-3.0)*n*zz[i-1] - real(i-2)*zz[i-2])/real(i-1)}
                yfit = yfit + cc[i]*zz[i]
             }
           }

	   if (ipix == 1 && suffix == "ap"){
		shift = center - yfit
		} else {
		yfit = shift + yfit + real(i1) - 1.0
                print (y, yfit, >> datfil)

		if (suffix == "ap"){
        if (boxtyp == "box" && mod(k, istep) == 0){comm1 = "box("//str(yfit+low+high)//","//str(y)//","//str(high-low)//","//str(istep)//","//str(0)//")";   print(comm1, >> regfil) } 
	        }
	    }    #ipix!=1

	  }    #k

        graph (datfil, wx1=ymin*0.9, wx2=1.1*ymax, wy1 = INDEF, wy2=INDEF, pointmode-,szmarke=0.02, marker = "circle")

	if (suffix == "id"){
	fields (dfile, fields="1,2", lines=cline) | graph (append+,  pointmode+, szmarke=0.02, marker = "circle")} else { print("The function stored in XY table = ", datfil, " and ds9 region ", boxtyp, " file = ",regfil)}

	#----------------------  popygon generator
	if (suffix == "ap" && boxtyp != "box"){
	print("global color=green dashlist=8 3 width=1 select=1 highlite=1 dash=0 fixed=0 edit=1 move=1 delete=1 include=1 source=1", >> regfil)
	#print(k, low, high)
        comm1 = "polygon("
        comm_left = ""
        comm_right = ""
        for (k = int(ymin); k <= int(ymax)-1; k = k + istep) {
        fields (datfil, fields="2", lines=k) | scan (yfit)
	y = yfit+low; printf("%6.1f", y) | scan(ystr)
	#print(k, yfit, y, ystr)
        comm_left = comm_left//ystr//","//str(k)//","
	}
        comm_left = comm_left//ystr//","//str(ymax)//","

        for (k = int(ymax); k >= int(ymin)+istep; k = k - istep) {
        fields (datfil, fields="2", lines=k) | scan (yfit)
	y = yfit+high; printf("%6.1f", y) | scan(ystr)
	#print(k, yfit, y, ystr)
        comm_right = comm_right//ystr//","//str(k)//","
	}
	comm1 = "polygon("//comm_left//comm_right//ystr//","//str(ymin)//")"
	#print("polygon("//comm_left, >> regfil)
	#print(comm_right//str(y)//","//str(ymin)//")", >> regfil)
	print(comm1, >> regfil)
	}

	#if (suffix == "ap" && boxtyp != "box" && plotbg){
	if (suffix == "ap" && boxtyp != "box" && ibg > 0){
	print("global color=blue dashlist=8 3 width=1 select=1 highlite=1 dash=0 fixed=0 edit=1 move=1 delete=1 include=1 source=1", >> regfil)
        for (kk=1; kk <= ibg; kk = kk + 1) {
        comm1 = "polygon("
        comm_left = ""
        comm_right = ""
	low = bl[kk]; high = br[kk]
        for (k = int(ymin); k <= int(ymax)-1; k = k + istep) {
        fields (datfil, fields="2", lines=k) | scan (yfit)
	y = yfit+low; printf("%6.1f", y) | scan(ystr)
	#print(k, yfit, y, ystr)
        comm_left = comm_left//ystr//","//str(k)//","
	}
        comm_left = comm_left//ystr//","//str(ymax)//","

        for (k = int(ymax); k >= int(ymin)+istep; k = k - istep) {
        fields (datfil, fields="2", lines=k) | scan (yfit)
	y = yfit+high; printf("%6.1f", y) | scan(ystr)
	#print(k, yfit, y, ystr)
        comm_right = comm_right//ystr//","//str(k)//","
	}
	comm1 = "polygon("//comm_left//comm_right//ystr//","//str(ymin)//")"
	#print("polygon("//comm_left, >> regfil)
	#print(comm_right//str(y)//","//str(ymin)//")", >> regfil)
	print(comm1, >> regfil)
	}
	}   #kk = 1, ibg
	print(" -------------------------------- ")

	if (plotbg) {
	  hedit (spe1d, "APCEN"//str(id), headcen[id], add+, addonly-, verify-, show-, update+)
	  hedit (spe1d, "APFUN"//str(id), headfun[id], add+, addonly-, verify-, show-, update+)
	  hedit (spe1d, "APSKY"//str(id), headbg[id], add+, addonly-, verify-, show-, update+)
	}
	}  # id=irec
	}  # id

exit:
	print(" ")

end
